;; from avr
(define_attr "cc" "none,set_cz,set_z,compare,clobber"
  (const_string "clobber"))              ;clobber by default

(define_constants
  [(A_REGNUM 0)
   (X_REGNUM 1)
   (BP_REGNUM 2)
   (SP_REGNUM 3)
  ]
)

(define_mode_iterator SIF [SI SF])

(define_predicate "hicovec_reg_or_0"
  (ior
   (match_operand 0 "register_operand")
   (and (match_code "const_int")
        (match_test "op == const0_rtx"))))

(define_predicate "register_a"
  (match_operand 0 "register_operand")
{
  unsigned int regno;
  regno = REGNO(op);
  return regno == 0;
})

; ARITH Operations

(define_insn "addsi3"
  [(set (match_operand:SI 0 "register_operand" "=a")
        (plus:SI
         (match_operand:SI 1 "hicovec_reg_or_0" "a0")
         (match_operand:SI 2 "nonmemory_operand" "ai")))]
  ""
  "add %0, %1, %2	; %0 = %1 + %2"
  [(set_attr "cc" "set_cz")])

(define_insn "subsi3"
  [(set (match_operand:SI 0 "register_operand" "=a")
        (minus:SI
         (match_operand:SI 1 "hicovec_reg_or_0" "a")
         (match_operand:SI 2 "nonmemory_operand" "ai")))
         ]
  ""
  "sub %0, %1, %2	; %0 = %1 - %2"
  [(set_attr "cc" "set_cz")])

(define_insn "negsi2"
  [(set (match_operand:SI 0 "register_operand" "=a")
        (neg:SI (match_operand:SI 1 "nonmemory_operand" "ai")))]
  ""
  "sub %0, 0, %1	; %0 = -%1"
  [(set_attr "cc" "set_cz")])

  
(define_insn "cmpsi"
  [(set (cc0)
        (compare
         (match_operand:SI 0 "register_operand" "a")
         (match_operand:SI 1 "nonmemory_operand" "ai")))]
  ""
  "sub 0, %0, %1	; %0 < %1?"
  [(set_attr "cc" "compare")])


(define_insn "tstsi"
  [(set (cc0)
         (match_operand:SI 0 "register_operand" "a"))]
  ""
  "sub 0, %0, 0	; %0 != 0?"
  [(set_attr "cc" "compare")])

(define_insn "*rev_tstsi"
  [(set (cc0)
        (compare (const_int 0)  
		 (match_operand:SI 0 "register_operand" "a")))]
  ""
  "sub 0, 0, %0	; 0 != %0?"
  [(set_attr "cc" "compare")])

  
(define_insn "andsi3"
  [(set (match_operand:SI 0 "register_operand" "=a")
        (and:SI
         (match_operand:SI 1 "hicovec_reg_or_0" "a0")
         (match_operand:SI 2 "nonmemory_operand" "ai")))
         ]
  ""
  "and %0, %1, %2	; %0 = %1 & %2"
  [(set_attr "cc" "set_z")])

(define_insn "iorsi3"
  [(set (match_operand:SI 0 "register_operand" "=a")
        (ior:SI
         (match_operand:SI 1 "hicovec_reg_or_0" "a0")
         (match_operand:SI 2 "nonmemory_operand" "ai")))
         ]
  ""
  "or %0, %1, %2	; %0 = %1 | %2"
  [(set_attr "cc" "set_z")])

(define_insn "xorsi3"
  [(set (match_operand:SI 0 "register_operand" "=a")
        (xor:SI
         (match_operand:SI 1 "hicovec_reg_or_0" "a0")
         (match_operand:SI 2 "nonmemory_operand" "ai")))
         ]
  ""
  "xor %0, %1, %2	; %0 = %1 ^ %2"
  [(set_attr "cc" "set_z")])

(define_insn "one_cmplsi2"
  [(set (match_operand:SI 0 "register_operand" "=a")
        (not:SI (match_operand:SI 1 "register_operand" "a")))]
  ""
  "xor %0, %1, -1	; %0 = ~%1"
  [(set_attr "cc" "set_z")])
  

;;; Shifts. These don't seem to work at all. I think you need a
;;; define_expand for the usual ashift and so on and then define these
;;; patterns for const_int_operand

;;; The current solution works, it just always generates calls to libgcc

;; (define_insn "*ashr_1_bit"
;;   [(set (match_operand:SI 0 "register_operand" "=a")
;; 	(lshiftrt:SI (match_operand:SI 1 "register_operand" "a")
;;                      (match_operand:SI 2 "const_int_operand" "1")))]
;;   ""
;;   "lsr %0, %1"
;;   [(set_attr "cc" "clobber")])

;; (define_insn "*ashl_1_bit"
;;   [(set (match_operand:SI 0 "register_operand" "=a")
;; 	(ashift:SI (match_operand:SI 1 "register_operand" "a")
;; 		   (match_operand:SI 2 "const_int_operand" "1")))]
;;   ""
;;   "lsl %0, %1"
;;   [(set_attr "cc" "clobber")])

;; MOV Pattern. Change this to a define_expand + a lot of insns to implement the other datatypes, handle QI, HI, SI and SF at least.
(define_insn "mov<mode>"
  [(set (match_operand:SIF 0 "nonimmediate_operand" "=a, a, a, m, a, R")
        (match_operand:SIF 1 "general_operand"       "i, a, m, r, R, a"))]
  ""
  "@
	or %0, 0, %1	; %0 = %1
	or %0, %1, %1	; %0 = %1
	ld %0, %a1	; %0 = %a1
	st %a0, %1	; %a0 = %1
	mov %0, r0(0)	; %0 = SP
	mov r0(0), %1	; SP = %1"
  [(set_attr "cc" "set_z, set_z, clobber, none, none, none")])

(define_insn "nop"
  [(const_int 0)]
  ""
  "nop"
  [(set_attr "cc" "none")])


;; Branches. EQ and NE are defined here, the whole comparison stuff is more complicated and further down

(define_insn "beq"
  [(set (pc)
        (if_then_else (eq (cc0) (const_int 0))
                      (label_ref (match_operand 0 "" ""))
                      (pc)))]
  ""
  "jz %a0"
  [(set_attr "cc" "none")])

(define_insn "bne"
  [(set (pc)
        (if_then_else (ne (cc0) (const_int 0))
                      (label_ref (match_operand 0 "" ""))
                      (pc)))]
  ""
  "jnz %a0"
  [(set_attr "cc" "none")])

  
;; Jumps and Calls
  
(define_insn "jump"
  [(set (pc)
	(label_ref (match_operand 0 "" "")))]
  ""
  "jmp [0 + %l0]"
  [(set_attr "cc" "none")])

(define_insn "indirect_jump"
  [(set (pc)
        (match_operand:SI 0 "register_operand" ""))]
  ""
  "jmp [0 + %0]"
  [(set_attr "cc" "none")])

(define_insn "call"
  [(call (match_operand:SI 0 "memory_operand" "")
         (match_operand:SI 1 "const_int_operand" ""))]
  ""
  "jal A, %a0	; call %a1"
  [(set_attr "cc" "clobber")])
  
(define_insn "call_value"
  [(set (match_operand 0 "register_a" "")
	(call (match_operand:SI 1 "memory_operand" "m")
	      (match_operand:SI 2 "const_int_operand" "")))]
  ""
  "jal A, %a1	; A = call %a1"
  [(set_attr "cc" "clobber")])

;; epilogue - prologue is defined in hicovec.c. We do our stack
;; unwinding by hand, so we don't need a return pattern. The prologue
;; is simple enough to need no support from gcc, so no custom expander
;; here.

(define_insn "epilogue"
  [(return)]
  ""
  "\n\tjmp [0 + __fun_epilogue]"
  [(set_attr "cc" "clobber")])

;; Expands for the RTL Generation...

(define_expand "bge"
  [(set (pc)
        (if_then_else (ge (cc0) (const_int 0))
                      (label_ref (match_operand 0 "" ""))
                      (pc)))
]
  ""
  "")

(define_expand "bgeu"
  [(set (pc)
        (if_then_else (geu (cc0) (const_int 0))
                      (label_ref (match_operand 0 "" ""))
                      (pc)))
      ]
  ""
  "")

(define_expand "blt"
  [(set (pc)
        (if_then_else (lt (cc0) (const_int 0))
                      (label_ref (match_operand 0 "" ""))
                      (pc)))
      ]
  ""
  "")

(define_expand "bltu"
  [(set (pc)
        (if_then_else (ltu (cc0) (const_int 0))
                      (label_ref (match_operand 0 "" ""))
                      (pc)))
      ]
  ""
  "")

(define_expand "ble"
  [(set (pc)
        (if_then_else (le (cc0) (const_int 0))
                      (label_ref (match_operand 0 "" ""))
                      (pc)))
      ]
  ""
  "")

(define_expand "bleu"
  [(set (pc)
        (if_then_else (leu (cc0) (const_int 0))
                      (label_ref (match_operand 0 "" ""))
                      (pc)))
   ]
  ""
  "")

(define_expand "bgt"
  [(set (pc)
        (if_then_else (gt (cc0) (const_int 0))
                      (label_ref (match_operand 0 "" ""))
                      (pc)))
   ]
  ""
  "")

(define_expand "bgtu"
  [(set (pc)
        (if_then_else (gtu (cc0) (const_int 0))
                      (label_ref (match_operand 0 "" ""))
                      (pc)))
   ]
  ""
  "")

;; Insns for the assembly generation

(define_insn "branch"
  [(set (pc)
        (if_then_else (match_operator 1 "comparison_operator"
                        [(cc0)
                         (const_int 0)])
                      (label_ref (match_operand 0 "" ""))
                      (pc)))
   
   ]
  ""
  "*
   return ret_cond_branch (operands[1], 0);"
  [(set_attr "cc" "clobber")])

(define_insn "rvbranch"
  [(set (pc)
        (if_then_else (match_operator 1 "comparison_operator" 
	                [(cc0)
                         (const_int 0)])
                      (pc)
                      (label_ref (match_operand 0 "" ""))))]
  ""
  "*
   return ret_cond_branch (operands[1], 1);"
  [(set_attr "cc" "clobber")])

;; Local variables:
;; mode:emacs-lisp
;; End: