;#############################################################################
; all.asm - TEST OF ALL HICOVEC INSTRUCTIONS WITH THE HICOSIM
; author: Michael Wager, Luca Calchera
; assemble: scotchas -o all.o all.asm (or use already assembled file all.o)
;#############################################################################

;##### INFO: #####
; - Register X ist der Zaehler fuer die Adressen, (beginnend bei 0x1000)
;   an welche die Ueberpruefungsziffern(1 fuer OK) geschrieben werden.
; - NOTE: CONFIGURATION: N=20 K=16 !!!
;UEBERPRUEFUNGS-TABELLE:###########################################
;  (wenn hier an einer Stelle KEINE 1 steht gab es einen Fehler!)
; 0x1000 - TEST LOAD
; 0x1001 - TEST STORE
; 0x1002 - TEST SUB WITH CARRY 
; 0x1003 - TEST ADD WITH CARRY 
; 0x1004 - TEST AND OR
; 0x1005 - TEST XOR INC DEC
; 0x1006 - TEST ADD_CARRY
; 0x1007 - TEST SUB_CARRY
; 0x1008 - TEST JUMPS
; 0x1009 - TEST MOVES
; 0x100A - TEST VEC_LOAD/STORE
; 0x100B - TEST VEC_ALU
; 0x100C - TEST COOPERATION_INSTRUCTIONS
; 0x100D - TEST SCALAR SHIFT
; 0x100E - TEST SCALAR ROTATE
;##################################################################


;--------------- write the 1 --------------------------------------
.org 0x400
  WRITE:  ;write a 1 if everything is ok, and increment X
	OR A, 0, 1
	ST[0+X], A 
	INC X, X   ; increment X
	JMP [0+Y]  ; JMP back

;----------------JUMP_TEST: ---------------------------------------
.org 0x800 
  JUMPTEST:
		ADD A, A, 10 ;um 10 erhoehen
		JMP [0+Y]

;----------------JUST INCREMENT X IF A TEST FAILS -----------------
  INCREMENT:
  		INC X, X
  		JMP [0+Y]

; #################################################### START
.org 0x00 
	.dc 12

.start
		OR X, 0, 0x1000 ; start at 0x1000 = 4096, never change X!
		
; ############################ LOAD
        TESTLOAD:
		LD A, [0+0] ; 12 in A

		OR Y, 0, TESTSTORE
		SUB 0, A, 12
		
		JNZ [0+INCREMENT]	;falls was nicht stimmt muss X inkrementiert werden!

		
		JAL Y, [0+WRITE] ;########## Testadresse #1: 0x1000 TEST LD

; ############################ STORE
        TESTSTORE:	
  		OR A, 0, 0xffff
		ST [0+500], A
		LD Y, [0+500]
		INC X, X 
		SUB 0, Y, 0xfffe ;Y > 0xfffe --> setCarry 
		JNC [0+SUBWITHCARRY] 
		DEC X, X 

		JAL Y, [0+WRITE] ;########## Testadresse: 0x1001 TEST ST
		
; ############################ SUB WITH CARRY
        SUBWITHCARRY:		
		;##### teste carrybit bei subtraktion:
		OR A, 0, 30
		OR Y, 0, 20
		SUB A, A, Y
		JC [0+JUMP0] 
		INC X, X ;wenn carry nicht gesetzt wurde-> inkr X!
		JNC [0+ADDWITHCARRY] ;wenn carry nicht gesetzt wurde -> fehler

        JUMP0:
		JAL Y, [0+WRITE] ;########## Testadresse: 0x1002 TEST SUB WITH CARRY 

; ############################ ADD WITH CARRY		
        ADDWITHCARRY:
		;##### teste carrybit bei addition:
		LD Y, [0+LOADINTMAXVALUE]
		ADD A, Y, Y
		ADD A, A, 2 ;==> exakt 2 hoch 32, setzt carry bit
		JC [0+JUMP1]
		INC X, X
		JNC [0+ANDOR]

		
        JUMP1:
		JAL Y, [0+WRITE] ;########## Testadresse: 0x1003 TEST ADD WITH CARRY

; ############################ AND OR
        ANDOR:
		OR A, 0, 20 ; A=20
		OR Y, 0, 30 ; Y=30

		AND A, A, Y ; 20 AND 30 == 20
	
		OR Y, 0, XORINCDEC
		SUB 0, A, 20
		JNZ [0+INCREMENT]

		JAL Y, [0+WRITE] ;########## Testadresse: 0x1004 TEST AND-OR

; ############################ XOR INC DEC	
        XORINCDEC:
		OR A, 0, 99
		INC A, A
		SUB Y, A, 100

		OR A, 0, 3
		DEC A, A

		OR Y, 0, ADDCARRY
		SUB 0, A, 2
		JNZ [0+INCREMENT]

		JAL Y, [0+WRITE] ;########## Testadresse: 0x1005 TEST XOR INC DEC
	
; ############################ ADD-CARRY	
        ADDCARRY:
		SEC
		ADC A, 0, 5
		INC X, X 
		SUB A, A, 5 ;if A > 5 --> setCarry
		JNC [0+SUBCARRY]
		DEC X, X 
		
		JAL Y, [0+WRITE] ;########## Testadresse: 0x1006 TEST ADD_CARRY	
	
; ############################ SUB-CARRY	
        SUBCARRY:
		CLC
		ADD A, 0, 7
		SBC A, A, 4
		INC X, X 
		SUB A, A, 1 ;if A > 1 setCarry
		JNC [0+JUMPS]
		DEC X, X 
		JAL Y, [0+WRITE]  ;########## Testadresse: 0x1007 TEST SUB_CARRY


; ############################JUMPS
        JUMPS:
		OR A, 0, 0
		JAL Y, [0+JUMPTEST] ; -> speichert returnadresse in Y
		ADD Y, Y, 2     ;return-Adresse in Y; MANUELL!
		JNZ [0+JUMPTEST]

		ADD Y, Y, 3     ;return-Adresse in Y
		SEZ
		JZ [0+JUMPTEST]

		ADD Y, Y, 3     ;return-Adresse in Y
		SEC
		JC [0+JUMPTEST]

		ADD Y, Y, 3     ;return-Adresse in Y
		CLC
		JNC [0+JUMPTEST]
		
		;---------ueberpruefe Wert in A, muss 50 sein, da 5 mal ADD A, A, 10
		OR Y, 0, MOVES ;um nach INC wieder an MOVES zurueckzuspringen
		SUB 0, A, 50
		JNZ [0+INCREMENT]   ;wenn wert in A nicht = 50, ueberspringe JAL Y, [0+WRITE] 

		JAL Y, [0+WRITE]	;########## Testadresse: 0x1008 TEST JUMPS	


;############################## MOVES
        MOVES:
		OR Y, 0, 1
		OR A, 0, 20
		MOV R15(Y), A
		OR A, 0, 0
		MOV A, R15(Y)

		MOVA R0, X

		VMOV R1, R0
		VMOV R<15>, R0
		VMOV R1, R<15>

		VMOL R0, R0
		VMOR R1, R0

		OR Y, 0, VECLOADSTORE
		SUB 0, A, 20     
		JNZ [0+INCREMENT]  
		
		JAL Y, [0+WRITE] ;########## Testadresse: 0x1009 TEST MOVES	

;############################# VECTOR LOAD STORE
        VECLOADSTORE:
		OR A, 0, LOADIT ; first load R1 and R2
		VLD R1, [0+A]     
		VLD R2, [0+A]		
		OR A, 0, 0x2000
		VST [0+A], R1	; test VST, check with memory start at 0x2000

		OR A, 0, 15
		MOV A, R1(A) ;check R1 an 15. Stelle, sollte = 16 sein
		OR Y, 0, VECALU
		SUB 0, A, 16
		JNZ [0+INCREMENT]

		JAL Y, [0+WRITE] ;########## Testadresse: 0x100A TEST VEC_LOAD_STORE	

;############################# VECTOR ALU
        VECALU: ;(go on)

		; now test vector ALU Instructions:
		VADD.DW R3, R1, R2
		VSUB.DW R4, R1, R2
		VAND.W R5, R1, R2
		VOR.W  R6, R1, R2
		VXOR.W R7, R1, R2
		VMUL.B R8, R1, R2

		VLSR.B R9,  R1
		VLSL.B R10, R1
		
		OR A, 0, 3
		MOV A, R10(A)

		OR Y, 0, COOPS
		SUB 0, A, 8  ; dann stimmt VLSL/R
		JNZ [0+INCREMENT]

		JAL Y, [0+WRITE] ;########## Testadresse: 0x100B TEST VEC_ALU	
		

;############################## COOPERATION-INSTRUCTIONS
        COOPS:
		ADD A, X, Y # VADD.W R0, R1, R1
			
		OR Y, 0, SHIFTS	
		SUB A, A, 4225
		JNZ [0+INCREMENT]

		JAL Y, [0+WRITE] ;########## Testadresse: 0x100C TEST COOPERATION_INSTRs	

;############################## SCALAR SHIFT
        SHIFTS:
		SUB A, 0, 1 ; "-1" == 11111111 11111111 11111111 11111111
		LSR A, A	;   nun:  01111111 11111111 11111111 11111111
		LSL A, A	;   nun:  11111111 11111111 11111111 11111110
		LSL A, A	;   nun:1.11111111 11111111 11111111 11111100 -> carry gesetzt
		JC [0+JUMP2]	
		INC X, X
		JMP [0+ROTATES]	

        JUMP2:
		JAL Y, [0+WRITE] ;########## Testadresse: 0x100D TEST SCALAR SHIFT

;############################## SCALAR ROTATE
        ROTATES:
		SUB A, 0, 2     ; "-2" == 11111111 11111111 11111111 11111110
		SEC		;war carry vorher gesetzt wird um eins nach links
		ROL A, A   	;geschoben und carryBit rechts eingefuegt
				; --> ergebnis dann "-3" == 11111111 11111111 11111111 11111101
		
		OR Y, 0, END
		ADD 0, A, 3	; "-3" + 3 == 0 --> set zero
		JNZ [0+INCREMENT]				

		JAL Y, [0+WRITE] ;########## Testadresse: 0x100E TEST SCALAR ROTATE

        END:
		NOP
		VNOP 
		HALT
		

.org 0xc00
	LOADINTMAXVALUE:
		.dc 0x7FFFFFFF    ; == 2147483647 == (2**32) -1
  
    LOADIT: ;used to load a vector register with 16 words
    	.dc 1
    	.dc 2
    	.dc 3
    	.dc 4
    	.dc 5
    	.dc 6
    	.dc 7
    	.dc 8
    	.dc 9
    	.dc 10
    	.dc 11
    	.dc 12
    	.dc 13
    	.dc 14
    	.dc 15
    	.dc 16
.end
