package hicosim;

import hicosim.components.backend.io.CommandProcessor;
import hicosim.components.frontend.MainWindow;
import hicosim.components.frontend.iListenerFrontend;
import hicosim.control.Engine;

import javax.swing.JFrame;
import javax.swing.SwingUtilities;

/** 
 * Mainprogram of the HICOSIM application.
 * It starts the console application, use --gui to start GUI.
 *  
 * @author Michael Wager 2nd July 2009 ( fred@RZ.FH-Augsburg.DE )
 * @version 1.0
 */
public class LaunchHICOSIM {

	public static void main(String[] args) {
		
		//ENGINE UND COMMAND_PROCESSOR:
		final Engine engine = new Engine();
		final CommandProcessor cmdProcessor = new hicosim.components.backend.io.CommandProcessor(engine, args);
		
		//start GUI?
		if(args.length==1 && args[0].equals("--gui")) {
			//STARTEN DES GUI_APPLIKATION:
			SwingUtilities.invokeLater(new Runnable() {
				public void run() {
					 MainWindow win = new MainWindow(engine, cmdProcessor);
					 try{
						 engine.addListener((iListenerFrontend)win);	 
					 }catch (Exception e) {
						 System.err.println("Error while trying to add frontend-listener...");
						 System.exit(-1);
					 }
					
					 win.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
//					 win.setSize(1000, 800);
					 win.setVisible(true);
				}
			});
		}
		//else start console
		else {
			try {
				cmdProcessor.processConsoleInput();
			}
			catch (Exception e) {
				System.err.println("BAD ERROR IN cmdProcessor.processConsoleInput(); --> see LaunchHICOSIM.java");
				System.exit(-1);
			}
		}

	}

}
