package hicosim.components.backend.io;

import java.util.Vector;
import hicosim.components.frontend.RunThread;
import hicosim.components.plugins.vga.VGAPlugin;
import hicosim.control.Engine;
import hicosim.control.utils.Messages;

/**
 * CommandProcessor - interpreting commands from the command line.
 * @author Michael Wager,Calchera, Weber 16th May 2009
 */
public class CommandProcessor {

	private Engine engine;
	private String[] programArgs;
	private CommandScanner scanner;
	private boolean deletebr=false;
	
	private VGAPlugin vgaPlugin=null;
	
	public CommandProcessor(Engine engine, String[] args) {
		this.engine = engine;
		this.programArgs = args;
		
		this.scanner= new CommandScanner(engine);
	}
	
	/**
	 * Mainloop of Console based Application.
	 *  
	 * @throws Exception
	 */
	public void processConsoleInput() throws Exception {
		System.out.println(Messages.initialMessage);
		System.out.println(Messages.getHelp());
		
		Command command=null;
		/**
		 * Endlosschleife zur Verarbeitung der Kommandos.
		 * Unterscheidet immer ob gui oder nicht.
		 */
		while(true) {
			//EVERY TIME PRINT:
			
			System.out.print("\n(HICOSIM) ");
			deletebr=false;//delete zurucksetzen!!!						
			command = (Command)this.scanner.scanline(null, false);
			
			switch(command) {
			
			case EXIT:
				//TODO before exit ???
				System.exit(0);
				break;
			
			case HELP:
				System.out.println(Messages.getHelp());
				break;

			case CLEAR:
				try {
					String answer = this.engine.execute("clear"); 	
					
					if(answer != null ) {
						System.out.println(answer);
					}	
				}
				catch (Exception e) {
					System.err.println(e.getMessage());
					break;
				}
							
				break;
			
			case OPEN:
				Object[] params = command.getParams();
				params.toString();
				String oFilePath = (String) params[0];
				
//				System.out.println("checking Objectfile " + oFilePath +" ...");

				try {
					String ret = this.engine.loadObjectFile(oFilePath); //ein parameter zu open: Der Pfad zur Datei; 	
					if(ret != null) { //nur wenns geklappt hat					
						System.out.println(ret);
						System.out.println("File " + params[0] +" loaded.");
					}	
				}
				catch (Exception e) {
					System.err.println(e.getMessage());
					break;
				}															 		
				break;	
				
				
			case WATCHPOINT:
				Object[] wpParams = command.getParams();
				//ohne parameter: alle watchpoints zeigen
				if(((String)wpParams[0]).equals("*")) { 
					if(engine.getAllWatchPoints().size()==0)
						System.out.println("No watchpoints set.");
					else
						System.out.println(engine.getAllWatchPoints());
					break;
				}
				//HEX:
				String str=wpParams[0].toString();
				if(str.contains("0x")) {
					str = str.replace("0x", "");
					int add = Integer.parseInt(str, 16);
					if(!engine.isWatchPoint(add)){
						engine.setWatchPoint(add);
					} else {
						engine.delWatchPoint(add);
					}
					break;
				}
				//DECIMAL:
				if(!engine.isWatchPoint(Integer.parseInt(wpParams[0].toString()))){
					engine.setWatchPoint(Integer.parseInt(wpParams[0].toString()));
					
				} else {
					engine.delWatchPoint(Integer.parseInt(wpParams[0].toString()));
				}
				
				break;
			
			case DELETE:
				if(!engine.isObjectFileLoaded()) {
					System.err.println("NO OBJECTFILE LOADED");
					break;
				}
				if("*"==(String)command.getParams()[0]){//alle breakpoints loeschen
					this.engine.deleteAllBr();
					System.out.println("Deleted all breakpoints.");
					break;
				}
				deletebr=true;				

			
			
			
			case BREAKPOINT:
				StringBuffer breaks=new StringBuffer();
				breaks.append("");
				try {
					String paramBreak = (String)command.getParams()[0];
					if(paramBreak.equals("*")) {
						Vector<Integer> breakAdds=this.engine.getBreakpoints();
						
						
					    if(breakAdds == null) 
					    	System.out.println("No breakpoints set.");
					    else {
					    	Sourcefile src=this.engine.getSourcefile();					    	
					    		
					    	for(int i=0; i<breakAdds.size(); i++) {
					    			if(this.engine.getMemory().getLabelAt(breakAdds.get(i)) != null) {
					    				if(breakAdds.get(i) != -1) {
					    					if(src.getLines()==null)
					    						breaks.append("Breakpoint #"+(i+1) + " at address 0x" +Integer.toHexString(breakAdds.get(i)) + 					    								 
					    							", Label: " +this.engine.getMemory().getLabelAt(breakAdds.get(i)) +"\n");
					    					else	
					    						breaks.append("Breakpoint #"+(i+1) + " at address 0x" +Integer.toHexString(breakAdds.get(i)) + 
					    								" in line " + src.getLineNumberAtAddress(breakAdds.get(i))  + 
					    							", Label: " +this.engine.getMemory().getLabelAt(breakAdds.get(i)) +"\n");
					    				}
					    			}
					    			else {
					    				if(breakAdds.get(i) != -1) {
					    					if(src.getLines()==null)
					    						breaks.append("Breakpoint #"+(i+1) + " at address 0x" +Integer.toHexString(breakAdds.get(i)) +"\n");					    							  
					    					else
					    						breaks.append("Breakpoint #"+(i+1) + " at address 0x" +Integer.toHexString(breakAdds.get(i)) + 				    							  
					    						" in line " + src.getLineNumberAtAddress(breakAdds.get(i)   ) +"\n");
					    						
					    				}
					    			}
					    	}
					    	if(breaks.toString().equals(""))
					    		System.out.println("No breakpoints set.");
					    	else {
					    		System.out.println(breaks.toString());
					    		break;
					    	}
					    		
					    }			    				    	
					}
					else {
						if (!(this.engine.isObjectFileLoaded())) {
							if(deletebr) System.err.println("ERROR WHILE TRYING TO DELETE BREAKPOINT. OBJECTFILE NOT LOADED.");
							else System.err.println("ERROR WHILE TRYING TO SET BREAKPOINT. OBJECTFILE NOT LOADED.");
						    break;
						}
						System.out.println(this.engine.setBreakpoint(paramBreak, deletebr));
					}
						
					
				}
				catch (Exception e) {
					System.err.println("ERROR in BREAKPOINTS: " + e.getMessage());
				}
				break;
			
			case LIST:
				Object[] params1 = command.getParams();
				String[] lines = null;
				str=null;
				String pathToSrcFile =null;
				
				//srcfile schon eingelesen:
				if(params1.length==0 || params1[0]==null) { 
					try {
						str = this.engine.getSourceCodeLines(); //Inhalt holen
						lines = str.split("\n");
						pathToSrcFile = this.engine.getSourceCodePATH();	//Pfad holen	
					}
					catch (Exception e) {
						System.err.println(e.getMessage());
						break;
					}
					
				}
				//pfad manuell angegeben:
				else if(params1.length==1) { 
					//EINLESEN:
					pathToSrcFile = (String)params1[0];
					try {
						this.engine.readSourcefile(pathToSrcFile);
						str= this.engine.getSourceCodeLines();
						lines = str.split("\n");
					}
					catch (Exception e) {
						System.err.println(e.getMessage());
						break;
					}
					
				}
				
				for (int i = 0; i < lines.length; i++) {
					System.out.println((i+1) + ": " + lines[i]);
				}								
				
				break;
			
			case GET: //INFO: "get 0xfff 0x11f" --> error cause start > end
				Object[] paramsGet = command.getParams();
				try {
					String ret1 = getMemory(paramsGet);
					
					System.out.println(ret1);
					break;
				}
				catch (Exception e) {
					System.err.println(e.getMessage());
					break;
				}
				
			case SET:
				try {
					Object[] paramsSet = command.getParams();
					String ret2 = setMemory(paramsSet);					
					System.out.println(ret2);
					break;
				}
				catch (Exception e) {
					System.err.println(e.getMessage());
					break;
				}		
			case SETR:
				try {
					Object[] paramsSetR = command.getParams();
					String reg = (String)paramsSetR[0];
					String value = (String)paramsSetR[1];
					String ret3 = 	engine.setRegister(reg +" "+ value);
					System.out.println(ret3);
					break;
				}
				catch (Exception e) {
					System.err.println(e.getMessage());
					break;
				}		
			
			case RUN:
				if (!(this.engine.isObjectFileLoaded())){
					System.err.println("ERROR WHILE TRYING TO RUN. OBJECTFILE NOT LOADED.");
				    break;
				}
				try {					
					this.engine.run();			
				}
				catch (Exception e) {
					System.err.println(e.getMessage());
					break;
				}	
				break;
				
			case STEP:
				if (!(this.engine.isObjectFileLoaded())){
					System.err.println("ERROR WHILE TRYING TO STEP. OBJECTFILE NOT LOADED.");
					break;
				}
				try {
					this.engine.step(false);	
				}
				catch (Exception e) {
					System.err.println(e.getMessage());
					break;
				}
							
				break;
				
//			case CONTINUE:
//				if (!(this.engine.isObjectFileLoaded())){
//					System.err.println("ERROR WHILE TRYING TO CONTINUE. OBJECTFILE NOT LOADED.");
//					break;
//				}
//				try {
//					System.out.println("...running...Type 'i' + <ENTER> to interrupt.");
//					this.engine.continueRun();	
//				}
//				catch (Exception e) {
//					System.err.println(e.getMessage());
//					break;
//				}			
//				break;
			
			case RESET:
				String resetParam=null;
				try {
					Object[] resetParams = command.getParams();
					resetParam = (String)resetParams[0];
				}
				catch (Exception e) {
					// ignore, this is just reset cpu
				}
				
				if(resetParam != null) {
					if(resetParam.equals("mem") || resetParam.equals("memory")) {
						try {
							command.setParams(null);							
							this.engine.resetMemory();
							System.out.println("Reset memory OK.");
						}
						catch (Exception e) {
							System.err.println(e.getMessage());
							break;
						}
					}
					else if (resetParam.equals("cpu")){
						try {
							command.setParams(null);							
							this.engine.reset();
							System.out.println("Reset cpu OK.");
						}
						catch (Exception e) {
							System.err.println(e.getMessage());
							break;
						}
					}
					else 
						System.out.println("RESET: WRONG INPUT: " + resetParam);
				}
				else {
					try {
						this.engine.reset();
						this.engine.resetMemory();
						System.out.println("Full reset OK.");	
					}
					catch (Exception e) {
						System.err.println(e.getMessage());
						break;
					}
				}				
				break;
				
			case INFO:
				Object[] params2 = command.getParams();
				String infoParam = (String)params2[0];
				
				if(infoParam.equals("*")) {
					System.out.println (Messages.getHelpInfo());
				}
				
				else if(infoParam.equals("regs")|| infoParam.equals("registers")) {
					try {
						System.out.println (this.engine.getRegs());
//						System.out.println(this.engineI.getScalarRegisters().toString());
//						System.out.println(this.engineI.getVectorRegisters().toString());
					}
					catch (Exception e) {
						System.err.println(e.getMessage());
					}
				}
				else if(infoParam.equals("ip") ) {
					try {
						System.out.println((this.engine.getIP()));
					}
					catch (Exception e) {
						System.err.println((e.getMessage()));
					}							
				}
				
				else if(infoParam.equals("labels") ) {
					try {
						String labs = this.engine.getLabels().toString();
						String ret="";
						int count=1;
						for(int i=0; i<labs.length(); i++) {
							if(labs.toCharArray()[i]==',')
								count++;
							if(count%8==0) {
								ret += "\n";
								count=1;
							}
							ret += labs.toCharArray()[i];
						}
						System.out.println(ret);
					}
					catch (Exception e) {
						System.err.println("THERE ARE NO LABELS.");
					}							
				}
				
				else if(infoParam.equals("stats") || infoParam.equals("statistics")) {
					try {
						//System.out.println(this.engine.getStats());
						System.out.println(this.engine.getProcessor().getStatistics().toString());
					}
					catch (Exception e) {
						System.err.println(e.getMessage());
					}
				}
				else if(infoParam.equals("config") || infoParam.equals("configuration")) {
					try {
						System.out.println(this.engine.getConfig());
					}
					catch (Exception e) {
						System.err.println(e.getMessage());
					}
				}
				else if(infoParam.equals("about") ) {
					System.out.println(this.engine.getAboutMessage());
				}	
				else if(infoParam.equals("reset") ) {
					System.out.println(Messages.getHelpReset());
				}	
				else 
					System.out.println("info: WRONG parameter: '" + infoParam + "'. Try help");
				
				break;
			
			case CONFIG:
				Object [] p = command.getParams(); //2 params, e.g. "K 4" or "use_shuffle 1"
				
				if(p.length==1 && ((String)p[0]).equals("*") ) {
					System.out.println(this.engine.getConfig());
					break;
				}
				String paramsConfig=p[0] + " " + p[1];
				try {
					String message = this.engine.setConfiguration(paramsConfig);
					System.out.println(message);
				}
				catch (Exception e) {
					System.err.println(e.getMessage());
				}
				break;
				
//			case WATCHMEM:
//				Object [] wP = command.getParams(); //2 params, e.g. "K 4" or "use_shuffle 1"
//				
//				
////				Object wI = ((Integer)wP[1]).intValue();
//				int start = Integer.parseInt(wP[0].toString());
//				int end = Integer.parseInt(wP[1].toString());
//				try {
//					IPlugin watchmem = new TestPlugin(start, end, this.engine);
//					watchmem.setObserver(true);
//					this.engine.addPlugin(watchmem);
//				}
//				catch (Exception e) {
//					//System.err.println(e.getMessage());
//					break;
//				}
//				
//				//IPlugin buttons = new RaceButtons(60000, 70000);
//				//buttons.setObserver(false);
//				//this.engineI.addPlugin(buttons);
//				break;
				
			
//			case BUTTONS:
//				System.out.println("Buttons mit Zugriff auf 0xFFFF bzw. 65535 dez.");
//				int startButtons = 65535;
//				int endButtons = 65536;
//				try {
//					IPlugin buttons = new RaceButtons(startButtons, endButtons);
//					this.engine.addPlugin(buttons);
//				} catch (Exception e1) { System.out.println("Fehler beim anlegen der Buttons!"); }
//				break;
				
				//MOEGLICHKEIT VGA VON KONSOLENAPP ZU STARTEN:
			case VGA:										
				Object [] paramVGA = command.getParams(); //
				if(paramVGA[0].equals("start")){
//					if(!this.engine.isObjectFileLoaded()) {
//						System.err.println("CANNOT START VGA. OBJECTFILE NOT LOADED.");
//						break;
//					}
					try {
						vgaPlugin = new VGAPlugin(320, 240, this.engine);
						vgaPlugin.setUpFrame();
					}
					catch (Exception e) {
						System.err.println("VGA-ERROR: " + e.getMessage());
					}				
				}
				else if(paramVGA[0].equals("stop")) {
					if(vgaPlugin!=null) {
						vgaPlugin.stopRenderFromConsole();
						vgaPlugin=null;
					}
				}
				else 
					System.err.println("vga: WRONG PARAMETER: " + paramVGA[0]);				
				break;
				
			case DISAS:
				try {
					System.out.println(this.engine.disas());
				}
				catch (Exception e) {
					System.err.println(e.getMessage());
					break;
				}
				
				break;
				
			//DIE COMMAND SPEZIFISCHE HILFE: ==================================HELP <cmd>
			case HELP_DELETEBR:
				System.out.println(Messages.getHelpForDelete());
				break;
			case HELP_BREAK:
				System.out.println(Messages.getHelpBreakpoint());
				break;
			case HELP_WATCHPOINT:
				System.out.println(Messages.getHelpWatchpoint());
				break;
			case HELP_GET:
				System.out.println(Messages.getHelpGet());
				break;
			case HELP_SET:
				System.out.println(Messages.getHelpSet());
				break;
			case HELP_SETR:
				System.out.println(Messages.getHelpSetR());
				break;
			case HELP_OPEN:
				System.out.println(Messages.getHelpOpen());
				break;
			case HELP_RESET:
				System.out.println(Messages.getHelpReset());
				break;
			case HELP_LIST:
				System.out.println(Messages.getHelpList());
				break;
			case HELP_INFO:
				System.out.println(Messages.getHelpInfo());
				break;
			case HELP_CONFIG:
				System.out.println(Messages.getHelpConfig());
				break;
			case HELP_VGA:
				System.out.println(Messages.getHelpVGA());
				break;
			case DUMMY:
				System.out.println("WRONG INPUT. Try help.");			
				break;
				
			
			}//END SWITCH
		}//END WHILE TRUE
	}
	/**
	 * This method is used by the gui to interpret gui-commandline commands.
	 *
	 * @param cmd - comming from gui, interpret and execute this command.
	 * 
	 * @throws Exception 
	 */
	public String processGUI_Input(String cmd) throws Exception {
		
		Command command = (Command)this.scanner.scanline(cmd, true);
		deletebr=false;//delete zurucksetzen!!!
		switch(command) {
			case EXIT:
				//TODO before exit ???
				System.exit(0);
				break;
		
			case HELP:
				//zeige Hilfe auf dem GUI-CommandInfoPanel ???
				return Messages.getHelp();
		
		case OPEN:
			Object[] params = command.getParams();
			params.toString();
			
			String pathToObjectFile = (String)params[0];
			String ret=null;
			try {
				ret = this.engine.loadObjectFile(pathToObjectFile); //ein parameter zu open: Der Pfad zur Datei;
				return "File "+pathToObjectFile+" loaded.";
			}
			catch (Exception e) {
				throw new Exception(e.getMessage());
			}
//			return ret;
		case DELETE:
			if(!engine.isObjectFileLoaded()) {
				System.err.println("NO OBJECTFILE LOADED");
				break;
			}
			if("*"==(String)command.getParams()[0]){//alle breakpoints loeschen
				this.engine.deleteAllBr();
				return "Deleted all breakpoints.";
			}
			deletebr=true;			
			
		
		case WATCHPOINT:
			Object[] wpParams = command.getParams();
			//ohne parameter: alle watchpoints zeigen
			if(((String)wpParams[0]).equals("*")) { 
				if(engine.getAllWatchPoints().size()==0)
					return("No watchpoints set.");
				else
					return engine.getAllWatchPoints().toString();
			}
			//HEX:
			String str=wpParams[0].toString();
			if(str.contains("0x")) {
				str = str.replace("0x", "");
				int add = Integer.parseInt(str, 16);
				if(!engine.isWatchPoint(add)){
					engine.setWatchPoint(add);
				} else {
					engine.delWatchPoint(add);
				}
				break;
			}
			//DECIMAL:
			if(!engine.isWatchPoint(Integer.parseInt(wpParams[0].toString()))){
				engine.setWatchPoint(Integer.parseInt(wpParams[0].toString()));
				
			} else {
				engine.delWatchPoint(Integer.parseInt(wpParams[0].toString()));
			}
			
			break;
		
		
		case BREAKPOINT:
			StringBuffer breaks=new StringBuffer();
			breaks.append("");
			try {
				String paramBreak = (String)command.getParams()[0];
				if(paramBreak.equals("*")) {
					Vector<Integer> breakAdds=this.engine.getBreakpoints();
										
				    if(breakAdds == null) 
				    	return("No breakpoints set.");
				    else {
				    	Sourcefile src=this.engine.getSourcefile();					    	
				    		
				    	for(int i=0; i<breakAdds.size(); i++) {
				    			if(this.engine.getMemory().getLabelAt(breakAdds.get(i)) != null) {
				    				if(breakAdds.get(i) != -1) {
				    					if(src.getLines()==null)
				    						breaks.append("Breakpoint #"+(i+1) + " at address 0x" +Integer.toHexString(breakAdds.get(i)) + 					    								 
				    							", Label: " +this.engine.getMemory().getLabelAt(breakAdds.get(i)) +"\n");
				    					else	
				    						breaks.append("Breakpoint #"+(i+1) + " at address 0x" +Integer.toHexString(breakAdds.get(i)) + 
				    								" in line " + src.getLineNumberAtAddress(breakAdds.get(i))  + 
				    							", Label: " +this.engine.getMemory().getLabelAt(breakAdds.get(i)) +"\n");
				    				}
				    			}
				    			else {
				    				if(breakAdds.get(i) != -1) {
				    					if(src.getLines()==null)
				    						breaks.append("Breakpoint #"+(i+1) + " at address 0x" +Integer.toHexString(breakAdds.get(i)) +"\n");					    							  
				    					else
				    						breaks.append("Breakpoint #"+(i+1) + " at address 0x" +Integer.toHexString(breakAdds.get(i)) + 				    							  
				    						" in line " + src.getLineNumberAtAddress(breakAdds.get(i)   ) +"\n");				    						
				    				}
				    			}
				    	}
				    	if(breaks.toString().equals(""))
				    		return("No breakpoints set.");
				    	else {
				    		return(breaks.toString());				    		
				    	}
				    		
				    }			    				    	
				}
				else {
					if (!(this.engine.isObjectFileLoaded())) {
						if(deletebr) 
							throw new Exception("ERROR WHILE TRYING TO DELETE BREAKPOINT. OBJECTFILE NOT LOADED.");
						else 
							throw new Exception("ERROR WHILE TRYING TO SET BREAKPOINT. OBJECTFILE NOT LOADED.");					    
					}
					return(this.engine.setBreakpoint(paramBreak, deletebr));
				}									
			}
			catch (Exception e) {
				throw new Exception("ERROR in BREAKPOINTS: " + e.getMessage());
			}
		
		case LIST: 
			Object[] params1 = command.getParams();
			String []lines = null;
			str=null;
			String pathToSrcFile =null;
			
			//srcfile schon eingelesen:
			if(params1.length==0 || params1[0]==null) { 
				str = this.engine.getSourceCodeLines(); //Inhalt holen
				pathToSrcFile = this.engine.getSourceCodePATH();	//Pfad holen
				lines = str.split("\n");
			}
			//pfad manuell angegeben:
			else if(params1.length==1) { 
				//EINLESEN:
				pathToSrcFile = (String)params1[0];
				try {
					this.engine.readSourcefile(pathToSrcFile);
					str = this.engine.getSourceCodeLines();
					lines = str.split("\n");
				}
				catch (Exception e) {
					throw new Exception (e.getMessage());
				}				
			}
			
			ret = "";	
			for (int i = 0; i < lines.length; i++) {
				ret += ((i+1) + ": " + lines[i] + "\n");
			}					
			if(this.engine.getListener()!=null)
				this.engine.getListener().displaySourceCode(str);
			
			return ret;
		
		case GET:
			Object[] paramsGet = command.getParams();
			try {
				String ret1 = getMemory(paramsGet);
				return (ret1);
			}
			catch (Exception e) {
				throw new Exception(e.getMessage());
			}
			
		case SET:
			try {
				Object[] paramsSet = command.getParams();
				String ret2 = setMemory(paramsSet);					
				return (ret2);
			}
			catch (Exception e) {
				throw new Exception(e.getMessage());
			}	
		case SETR:
			try {
				Object[] paramsSetR = command.getParams();
				String reg = (String)paramsSetR[0];
				String value = (String)paramsSetR[1];
				String ret3 = 	engine.setRegister(reg +" "+ value);
				return (ret3);				
			}
			catch (Exception e) {
				throw new Exception(e.getMessage());				
			}		
		
		case RUN: 						
			if (!(this.engine.isObjectFileLoaded())){
				throw new Exception ("ERROR WHILE TRYING TO RUN. OBJECTFILE NOT LOADED.");
			}			
			try {				
				engine.getProcessor().setIsRunning(true);
				engine.getProcessor().setStepping(false);
				RunThread thread = new RunThread(engine);
				thread.start();
			}
			catch (Exception e) {
				throw new Exception(e.getMessage());
			}
			return "";
			
		case STEP:
			if (!(this.engine.isObjectFileLoaded())){
				throw new Exception ("ERROR WHILE TRYING TO STEP. OBJECTFILE NOT LOADED.");
			}
			try {
				this.engine.step(false);
			}
			catch (Exception e) {
				throw new Exception(e.getMessage());
			}			
			return "";
			
		case RESET:
			String resetParam=null;
			try {
				Object[] resetParams = command.getParams();
				resetParam = (String)resetParams[0];
				command.setParams(null);
			}
			catch (Exception e) {
				// ignore, this is just reset cpu
			}
			
			if(resetParam != null) {
				if(resetParam.equals("mem") || resetParam.equals("memory")) {
					try {
						this.engine.resetMemory();
						return("Reset memory OK.");
					}
					catch (Exception e) {
						throw new Exception(e.getMessage());
					}
				}
				else if (resetParam.equals("cpu")){
					try {
						command.setParams(null);							
						this.engine.reset();
						return("Reset cpu OK.");
					}
					catch (Exception e) {
						throw new Exception(e.getMessage());
				    }
				}	
				else 
					return ("RESET: WRONG INPUT: " + resetParam);
			}
			else {
				try {
					this.engine.reset();
					this.engine.resetMemory();
					return("Full reset OK.");
				}
				catch (Exception e) {
					throw new Exception(e.getMessage());
				}
			}
			
		case INFO:
			Object[] params2 = command.getParams();
			String infoParam = (String)params2[0];
			
			if(infoParam.equals("*")) {
				return(Messages.getHelpInfo());
			}
			
			else if(infoParam.equals("regs") || infoParam.equals("registers")) {
				try {
					return (this.engine.getRegs());
				}
				catch (Exception e) {
					throw new Exception(e.getMessage());
				}				
			}
			else if(infoParam.equals("ip") ) {
				try {
					return "" + ((this.engine.getIP()));
				}
				catch (Exception e) {
					throw new Exception((e.getMessage()));
				}							
			}
//			{0=LABEL, 8=LABEL,...}
			else if(infoParam.equals("labels") ) {
				try {
					String labs = this.engine.getLabels().toString();
					ret="";
					int count=1;
					for(int i=0; i<labs.length(); i++) {
						if(labs.toCharArray()[i]==',')
							count++;
						if(count%8==0) {
							ret += "\n";
							count=1;
						}
						ret += labs.toCharArray()[i];
					}
					return ret;
				}
				catch (Exception e) {
					throw new Exception(e.getMessage());
				}							
			}
			
			else if(infoParam.equals("stats") || infoParam.equals("statistics")) {
				try {
//					return (this.engine.getStats());
					return this.engine.getProcessor().getStatistics().toString();
				}
				catch (Exception e) {
					throw new Exception(e.getMessage());
				}	
			}
			else if(infoParam.equals("config") || infoParam.equals("configuration")) {
				try {
					return (this.engine.getConfig());
				}
				catch (Exception e) {
					throw new Exception(e.getMessage());
				}					
			}
			else if(infoParam.equals("reset") ) {
				return (Messages.getHelpReset());
			}	
			else if(infoParam.equals("about") ) {
				return this.engine.getAboutMessage();
			}	
			else 
				return ("info: WRONG parameter: '" + infoParam + "'. Try help");
			
		
		case CONFIG:
			Object [] p = command.getParams(); //2 params, e.g. "K 4" or "use_shuffle 1"
			if(p.length==1 && ((String)p[0]).equals("*") ) {
				return (this.engine.getConfig());
			}
			String paramsConfig=p[0] + " " + p[1];
			try {
				String message = this.engine.setConfiguration(paramsConfig);
				//message hier nicht relevant
			}
			catch (Exception e) {
				throw new Exception(e.getMessage());
			}
			break;
			
		case DISAS:
			try {
				//if in gui console "disas" is typed, update the disas VIEW too
				String dis = this.engine.disas();
				this.engine.getListener().displayDisassembledCode(dis);
				return (dis);
			}
			catch (Exception e) {
				throw new Exception(e.getMessage());
			}
//		case WATCHMEM:
//			Object [] wP = command.getParams(); //2 params, e.g. "K 4" or "use_shuffle 1"
//			
//			//			Object wI = ((Integer)wP[1]).intValue();
//			int start = Integer.parseInt(wP[0].toString());
//			int end = Integer.parseInt(wP[1].toString());
//			try {
//				IPlugin watchmem = new TestPlugin(start, end, this.engine); 
//				watchmem.setObserver(true);
//				this.engine.addPlugin(watchmem);
//			}
//			catch (Exception e) {
//				//System.err.println(e.getMessage());
//				break;
//			}
//			break;
			
//		case BUTTONS:
//			engine.displayConsole("Buttons mit Zugriff auf 0xFFFF bzw. 65535 dez.", false);
//			int startButtons = 65535;
//			int endButtons = 65536;
//			try {
//				IPlugin buttons = new RaceButtons(startButtons, endButtons);
//				this.engine.addPlugin(buttons);
//			} catch (Exception e1) { throw new Exception("Fehler beim anlegen der Buttons!"); }
//			break;
			
			//VGA STARTEN NICHT MEHR ON GUI CONSOLE MOEGLICH 
		case VGA:										
			Object [] paramVGA = command.getParams(); //
			if(paramVGA[0].equals("start")){
//				if(!this.engine.isObjectFileLoaded()) {
//					throw new Exception("CANNOT START VGA. OBJECTFILE NOT LOADED.");
//				}
				try {
					this.engine.displayConsole("start VGA plugin not possible from GUI console.", true);
					return "";
				}
				catch (Exception e) {
					throw new Exception("start VGA plugin not possible from GUI console.");
				}				
			}
			else if(paramVGA[0].equals("stop")) {
				this.engine.displayConsole("start VGA plugin not possible from GUI console.", true);
				return "";
			}
			else 
				throw new Exception("start VGA plugin not possible from GUI console.");				
//			break;
			
			//DIE COMMAND SPEZIFISCHE HILFE: ==================================HELP <cmd>
		case HELP_BREAK:
			return (Messages.getHelpBreakpoint());
		case HELP_WATCHPOINT:
			return(Messages.getHelpWatchpoint());
		case HELP_DELETEBR:
			return (Messages.getHelpForDelete());
		case HELP_GET:
			return (Messages.getHelpGet());
		case HELP_SET:
			return (Messages.getHelpSet());
		case HELP_SETR:
			return (Messages.getHelpSetR());
		case HELP_OPEN:
			return (Messages.getHelpOpen());
		case HELP_RESET:
			return (Messages.getHelpReset());
		case HELP_LIST:
			return (Messages.getHelpList());
		case HELP_INFO:
			return (Messages.getHelpInfo());
		case HELP_CONFIG:
			return (Messages.getHelpConfig());
		case HELP_VGA:
			return (Messages.getHelpVGA());
			
		case DUMMY:
			throw new Exception("WRONG INPUT. Try help.");			
			
		}
		//duerfte nie passieren
		return null;
	}
	
	/**
	 * This method returns the memorycontent as Stringrepresentation depending on the given parameter.
	 * 
	 * @param paramsGet - parameter to the "GET" command e.g. new Object[] {0x0, 0xf} //startAddress, endAddress
	 * 						or: new Object[] {0x0} //get value on that address
	 * @throws Exception
	 * @return - String memory repr.
	 */
	public String getMemory(Object[] paramsGet)throws Exception {
		
		String add=null, startAdd=null, endAdd=null;
		try {
							
			int address1=0, startAddress=0, endAddress=0;
			String label=null;
			boolean one=false;
			//an einer Adresse oder von start-Ende:
			if(paramsGet.length==1) {
				add = (String) paramsGet[0]; //address
				one=true;
			}
			else if(paramsGet.length==2) {				
				startAdd = (String) paramsGet[0];	
				endAdd	  = (String) paramsGet[1];
			}
	
			//===========================================NUR GET i :
			if(one) {
				if(add.contains("0x")) {
					add = add.replace("0x", "");
					address1 = Integer.parseInt(add, 16);
				}
		
				else { //dezimal?:
					boolean intB=false;
					try {
						address1 = Integer.parseInt(add);	
						intB=true;
					}
					catch (Exception e) {/*ignore*/ }
				
			
					if(!intB) { //dann ein label
						String zahl="";
						boolean isOffset=false;
						int offset = 0, index=0;
											
						try {
							label=""; //checken ob "br label+offset"
							add=add.replaceAll("\\s*", ""); //ALLE LEERZEICHEN WEG
					
							if(add.contains("+")) {
								isOffset=true;
								add=add.replace("+"," ");									
								String [] arr = add.split(" ");
								label = arr[0];
								offset = Integer.parseInt(arr[1]);
								address1=this.engine.getAddressOfLabel(label) + offset;
							}
							else {
								address1=this.engine.getAddressOfLabel(add);
							}
																																																
						}
						catch (Exception e) {
							throw new Exception(e.getMessage());						
						}
					}
				}
		
				int content=0;
				String wp = "";
				try {
					content = this.engine.getMemoryContentAt(address1);
					if(engine.isWatchPoint(address1)){
						wp = "*";
					} else {
						wp = " ";
					}
				}
				catch (Exception e) {
					throw new Exception(e.getMessage());
				}					
				return (wp + address1 +": " +  content );
		
			}
			//=========================================== GET start-end :
			else { 			//entweder beide hex oder beide dezimal:
				if(startAdd.contains("0x") && endAdd.contains("0x")) {							
					startAdd = startAdd.replace("0x", "");	
					endAdd = endAdd.replace("0x", "");
					startAddress = Integer.parseInt(startAdd, 16);
					endAddress = Integer.parseInt(endAdd, 16);
				}
				else {		//beide dezimal:
					startAddress = Integer.parseInt(startAdd);
					endAddress = Integer.parseInt(endAdd);
				}
				String ret = "";
				String wp = "";
				ret += ("MEMORYCONTENT BETWEEN: " + startAdd +" - " +  endAdd + ":\n");
				int tmp=1;
				if(startAddress>endAddress) {
					return ("get: Start-address is bigger than end-address!");
				}
				for(int i=startAddress; i<= endAddress; i++) {
					try {
						int value = this.engine.getMemoryContentAt(i);
						if(engine.isWatchPoint(i)){
							wp = "*";
						} else {
							wp = " ";
						}
						ret += wp + "0x" + Integer.toHexString(i) +": " + value + "\t" ;						
					}
					catch (Exception e) {
						ret += "0x" + Integer.toHexString(i) +": 0\t";
					}	
					if(tmp%8 == 0) //alle 8 newLine
						ret += "\n";
					tmp++;
				}													
				return(ret);
			}
		}
		catch (Exception e) {
			throw new Exception("ERROR WHILE TRYING TO READ MEMORY --> " + ((paramsGet.length==1) ? add : startAdd + " - " + endAdd) );
		}			
	} //END getMemory()
	
	/**
	 * This method writes to memory depending on the given parameter.
	 * 
	 * @param paramsSet - parameter to the "SET" command e.g. new Object[] {0x10, 223} //address, value
	 * @throws Exception
	 * @return - String info
	 */
	public String setMemory (Object[] paramsSet) throws Exception{
		
		/**
		 * set 0 19
		 * or set 0x0 19
		 * or set 0x3 0x3
		 * or set LABEL 19/0x3
		 * or set LABEL+offset 19/0x3
		 */
		String add = (String) paramsSet[0]; //address
		String val = (String) paramsSet[1];	//value
		boolean found1=false, found2=false;
		int content=0;
		int address1=0;
	
		try {
			try {
				address1 = Integer.parseInt(add);
				found1=true;
			}
			catch(Exception e) { /*keine integer zahl, ignore...*/ }
			try {
				if(val.contains("0x")) {
					val=val.replace("0x", "");
					content = Integer.parseInt(val, 16);
					found2=true;
				}
				else {
					content = Integer.parseInt(val);
					found2=true;
				}
			}
			catch(Exception e) { /*keine integer zahl, ignore...*/ }
		
			if (found1 && found2) {
				try {
					this.engine.setMemory(address1, content);
					return ("SET MEMORY ("+content+") at address " + address1);
				}
				catch (Exception e) {
					throw new Exception(e.getMessage());					
				}
			}
			else if(!found1 && found2) {
				//is Addresse in hex angegeben?
				if(add.contains("0x")) {
					add = add.replace("0x", "");
					address1 = Integer.parseInt(add, 16);
				
					try {
						this.engine.setMemory(address1, content);
						return("SET MEMORY ("+content+") at address " + address1);
					}
					catch (Exception e) {
						throw new Exception(e.getMessage());
					}
				}
				else { //SONST LABEL oder LABEL+value
					try {
						boolean isOffset=false;
						int index=0, value=0, address=0;
						String zahl="";
						String label=""; //checken ob "set label+zahl"
						
						add=add.replaceAll("\\s*", ""); //ALLE LEERZEICHEN WEG
						
						if(add.contains("+")) {
							isOffset=true;
							add=add.replace("+", " ");
							String arr[] = add.split(" ");
							label = arr[0];
							value = Integer.parseInt(arr[1]);
						}
						else {
							label = add;
						}							
						try {
							address = this.engine.getAddressOfLabel(label) + value;
							this.engine.setMemory(address, content);
							return("SET MEMORY ("+content+") at address " + address);
						}
						catch(Exception e) {
							throw new Exception(e.getMessage());							
						}
						
					}
					catch(Exception e) {
						throw new Exception("WRONG INPUT: set " + add + " " + val);
					}						
				}
			}
		}
		catch(Exception e2) {
			throw new Exception("ERROR WHILE TRYING TO SET MEMORY CONTENT:(="+val+") at "+ add);
		}
		//should not come here:
		throw new Exception("ERROR WHILE TRYING TO SET MEMORY CONTENT:(="+val+") at "+ add);
		
	}//END setMemory()

} //END CLASS COMMAND_PROCESSOR
