package hicosim.components.backend.io;

import hicosim.control.Engine;

import java.io.BufferedReader;
import java.io.InputStreamReader;

/**
 * This class is used by the CommandProcessor to scan the Strings
 * comming from the console or the GUI-console.
 * @author Michael Wager, 16th May 2009
 *
 */
public class CommandScanner {
	
	private BufferedReader reader = null;
	private Engine engine = null;
	
	private String lastCommand="";
			
	/**
	 * Set up a Buffered Reader.
	 */
	public CommandScanner(Engine engine){
		this.reader = new BufferedReader(new InputStreamReader(System.in));
		this.engine = engine;
	}
			
	/**
	 *  Scan a line and check command
	 *  	SCAN A LINE FROM STDIN OR, IF GUI=TRUE FROM GUI.
	 *  	--> USED BY CommandProcessor for commandline commands, or by GUI for gui-commands  		
	 *  
	 * @return Command - see enum Command
	 * 
	 * @throws Exception
	 */
			
    public Command scanline(String cmd, boolean gui) throws Exception {	//lese Zeile von Standarteingabe ein
    	String splittedInputLine[] = null;    	
    	
    	if(!gui) {
    		try {
    			cmd = reader.readLine();    
    		}
    		catch (Exception e) {
    			System.exit(-1); //maybe ctrl - c
			}
    		splittedInputLine = cmd.split(" ");
    		
    		//------->>   wenn <enter>, letzten Kommand ausfuehren, sonst merken
    		if (splittedInputLine.length==1 && cmd.equals("")) {    	
    			cmd = lastCommand;
    			splittedInputLine= cmd.split(" ");
    			System.out.println(cmd);
			}
    		else {
    			this.lastCommand=cmd;    			
    		}
    	}
    	else {
    		//von gui uebergebenen command nutzen:
    		splittedInputLine = cmd.split(" ");
    	}
    	    	
    	try {    		    	
    		
    		//splittedInputLine= cmd.split(" ");
    	
    		// Befehle OHNE Parameter wie exit, help etc... 
    		if (splittedInputLine.length==1) {    			
    			if(splittedInputLine[0].equals("exit") || splittedInputLine[0].equals("q") || splittedInputLine[0].equals("quit")) {
    				return Command.EXIT;
    			}
    			else if (splittedInputLine[0].equals("help") || splittedInputLine[0].equals("h")) {
    				return Command.HELP;
    			}
    			else if (splittedInputLine[0].equals("list") || splittedInputLine[0].equals("l")) {
    				return Command.LIST;
    			}
    			else if (splittedInputLine[0].equals("clear") || splittedInputLine[0].equals("cl")) {
    				return Command.CLEAR;
    			}
    			else if (splittedInputLine[0].equals("run") || splittedInputLine[0].equals("r") ) { 
    				return Command.RUN; 
    			}
    			else if (splittedInputLine[0].equals("step") || splittedInputLine[0].equals("s") ) {
    				return Command.STEP; 
    			}
    			else if (splittedInputLine[0].equals("delete") || splittedInputLine[0].equals("del") ) { 
    				Command cDel = Command.DELETE;
    				cDel.setParams( new Object[]{"*"} );
    				return cDel; 
    			}
    			else if (splittedInputLine[0].equals("break") || splittedInputLine[0].equals("br") ) { 
    				Command cBr = Command.BREAKPOINT;
    				cBr.setParams( new Object[]{"*"} );
    				return cBr; 
    			}
    			else if (splittedInputLine[0].equals("watchpoint") || splittedInputLine[0].equals("wp") ) { 
    				Command cW = Command.WATCHPOINT;
    				cW.setParams( new Object[]{"*"} );
    				return cW; 
    			}
    			else if (splittedInputLine[0].equals("reset") ) { 
    				return Command.RESET; 
    			}
    			else if (splittedInputLine[0].equals("disas") || splittedInputLine[0].equals("d")) { 
    				return Command.DISAS; 
    			}
    			else if (splittedInputLine[0].equals("buttons") || splittedInputLine[0].equals("bu")) { 
    				return Command.BUTTONS; 
    			}
    			//wenn info ohne parameter, gib hilfe meldung
    			if(splittedInputLine[0].equals("info") || splittedInputLine[0].equals("i")) {
    				Command cINFO = Command.INFO;
    				cINFO.setParams( new Object[] {"*"} );
    				return cINFO;
    			}
    			//wenn config ohne parameter, gib configuration
    			if(splittedInputLine[0].equals("config")) {
    				Command cConf = Command.CONFIG;
    				cConf.setParams( new Object[] {"*"} );
    				return cConf;
    			}
    			
    		}
    	
    		//Befehle MIT Parameter:
    		else if (splittedInputLine.length > 1) { 
    			//path dabei:
    			if ( splittedInputLine[0].equals("open") || splittedInputLine[0].equals("o") ) {
    				Command cOpen = Command.OPEN;
    				cOpen.setParams( new Object[] {splittedInputLine[1].equals("") ? splittedInputLine[2] : splittedInputLine[1]} );
    				return cOpen;
    			}
    			
    			if ( splittedInputLine[0].equals("delete") || splittedInputLine[0].equals("del") ) {
    				Command cDel = Command.DELETE;
    				
    				if(cmd.contains("del") && !cmd.contains("ete")) 
    					cmd = cmd.replaceAll("del \\s*", "");
    				if(cmd.contains("delete")) 
    					cmd = cmd.replaceAll("delete \\s*", "");

    				cDel.setParams(new Object[] { cmd });
    				return cDel;
    			}
    			
    			if ( splittedInputLine[0].equals("break") || splittedInputLine[0].equals("br") ) {
    				Command cBreak = Command.BREAKPOINT;
    				
    				if(cmd.contains("br") && !cmd.contains("eak")) 
    					cmd = cmd.replaceAll("br \\s*", "");
    				if(cmd.contains("break")) 
    					cmd = cmd.replaceAll("break \\s*", "");

    				cBreak.setParams(new Object[] { cmd });
    				return cBreak;
    			}
    			
    			else if (splittedInputLine[0].equals("watchpoint") || splittedInputLine[0].equals("wp") ) { 
    				Command cWatch = Command.WATCHPOINT;
    				cWatch.setParams( new Object[] {splittedInputLine[1]} );
    				return cWatch; 
    			}
    			else if (splittedInputLine[0].equals("list") || splittedInputLine[0].equals("l") ) { 
    				Command cList = Command.LIST;
    				cList.setParams( new Object[] {splittedInputLine[1]} );
    				return cList; 
    			}
    			else if (splittedInputLine[0].equals("info") || splittedInputLine[0].equals("i") ) { 
    				Command cINFO = Command.INFO;
    				cINFO.setParams( new Object[] {splittedInputLine[1]} );
    				return cINFO; 
    			}
    			else if (splittedInputLine[0].equals("set") ) { 
    				Command cSET = Command.SET;
    				cSET.setParams( new Object[] {splittedInputLine[1], splittedInputLine[2]} );
    				return cSET; 
    			}
    			else if (splittedInputLine[0].equals("setr") ) { 
    				Command cSETR = Command.SETR;
    				cSETR.setParams( new Object[] {splittedInputLine[1], splittedInputLine[2]} );
    				return cSETR; 
    			}
    			else if (splittedInputLine[0].equals("get") ) { 
    				Command cGET = Command.GET;
    				if(splittedInputLine.length==2)
    					cGET.setParams( new Object[] { splittedInputLine[1] } );
    				else if(splittedInputLine.length==3) //startAdd-endAdd
    					cGET.setParams( new Object[] {splittedInputLine[1], splittedInputLine[2]} );
    				return cGET; 
    			}
    			//CONFIGURE CPU:
    			else if (splittedInputLine[0].equals("config") ) {
    				Command cCONFIG = Command.CONFIG;
    				cCONFIG.setParams( new Object[] {splittedInputLine[1], splittedInputLine[2]} );
    				return cCONFIG;
    			}    			
    			
    			if ( splittedInputLine[0].equals("reset")  ) {
    				Command cReset = Command.RESET;
    				cReset.setParams( new Object[] { splittedInputLine[1] } );
    				return cReset;
    			}
    			else if (splittedInputLine[0].equals("watchmem") ) {
    				Command cWATCHMEM = Command.WATCHMEM;
    				cWATCHMEM.setParams( new Object[] {splittedInputLine[1], splittedInputLine[2]} );
    				return cWATCHMEM;
    			}
    			else if (splittedInputLine[0].equals("vga") ) { 
    				Command cVGA = Command.VGA;
    				cVGA .setParams( new Object[] {splittedInputLine[1]} );
    				return cVGA ; 
    			}
    			
    			//SPEZIELLE HILFE:
    			else if (splittedInputLine[0].equals("help") || splittedInputLine[0].equals("h")) {
    				
    				if(splittedInputLine[1].equals("break") || splittedInputLine[1].equals("br") )
    					return Command.HELP_BREAK;  
    				if(splittedInputLine[1].equals("watchpoint") || splittedInputLine[1].equals("wp") )
    					return Command.HELP_WATCHPOINT;  
    				else if(splittedInputLine[1].equals("delete") || splittedInputLine[1].equals("del") )
    					return Command.HELP_DELETEBR;
    				else if(splittedInputLine[1].equals("set"))
    					return Command.HELP_SET;
    				else if(splittedInputLine[1].equals("setr"))
    					return Command.HELP_SETR;
    				else if(splittedInputLine[1].equals("get"))
    					return Command.HELP_GET;
    				else if(splittedInputLine[1].equals("open"))
    					return Command.HELP_OPEN;
    				else if(splittedInputLine[1].equals("reset"))
    					return Command.HELP_RESET;
    				else if(splittedInputLine[1].equals("list"))
    					return Command.HELP_LIST;
    				else if(splittedInputLine[1].equals("info") || splittedInputLine[1].equals("i"))
    					return Command.HELP_INFO;
    				else if(splittedInputLine[1].equals("config"))
    					return Command.HELP_CONFIG;
    				else if(splittedInputLine[1].equals("vga"))
    					return Command.HELP_VGA;
    			
    			}
    			
    		}
    	
    		//SONST HILFE AUSGEBEN:
    		else {
    			return Command.HELP;
    		}
    	}
    	catch (Exception e) {
			engine.displayConsole("COMMAND_SCANNER: ERROR WHILE TRYING TO INTERPRET: " + cmd, true);
		}
    	
    	//wenn er bis hier kommt stimmt was nicht:
    	return Command.DUMMY; //einfach ignoroieren
    					
	}

	}//END CLASS COMMAND_SCANNER
