package hicosim.components.frontend;

import hicosim.control.Engine;
import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.LayoutManager;

import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JScrollBar;
import javax.swing.JScrollPane;
import javax.swing.JTable;

import hicosim.components.plugins.WatchMemPlugin;

;

/**
 *
 * @author Daniel Obermueller 1st June 2009
 *
 * This class extends the super class JPanel and inherits an instance of the
 * memory watch plugin, which is displaying the memory and uses the passed
 * Engine-Object "engine".
 *
 */
public class JPanelMemoryOverview extends JPanel {

	private static final long serialVersionUID = 1L;
	private Engine engine;
	private WatchMemPlugin plugin;

	/**
	 * @param engine
	 * @throws Exception
	 */
	public JPanelMemoryOverview(Engine engine) throws Exception {
		super();
		this.engine = engine;
		initialize();
	}

	/**
	 * @param arg0
	 * @param engine
	 * @throws Exception
	 */
	public JPanelMemoryOverview(boolean arg0, Engine engine) throws Exception {
		super(arg0);
		this.engine = engine;
		initialize();
	}

	/**
	 * @param arg0
	 * @param arg1
	 * @param engine
	 * @throws Exception
	 */
	public JPanelMemoryOverview(LayoutManager arg0, boolean arg1, Engine engine)
			throws Exception {
		super(arg0, arg1);
		this.engine = engine;
		initialize();
	}

	/**
	 * @param arg0
	 * @param engine
	 * @throws Exception
	 */
	public JPanelMemoryOverview(LayoutManager arg0, Engine engine)
			throws Exception {
		super(arg0);
		this.engine = engine;
		initialize();
	}

	/**
	 * This method initializes this.
	 *
	 * @throws Exception
	 */
	private void initialize() throws Exception {
		plugin = new WatchMemPlugin(0, 17, engine);
		plugin.setObserver(true);
		engine.addPlugin(plugin);
		setLayout(new BorderLayout());
		add(new JScrollPane(plugin.getTable()), BorderLayout.CENTER);
		add(plugin.getInputPanel(), BorderLayout.SOUTH);
	}

	/**
	 * This method is used to update the instruction pointer of the memory watch
	 * plugin.
	 *
	 * @param int
	 *            instructionPointer
	 */
	public void updateIP(int instructionPointer) {
		plugin.updateIp(instructionPointer);
	}

	/**
	 * This method is used to delete the memory watch plugin from the plugin
	 * manager of the Engine "engine".
	 */
	public void deleteMemViewPlugin() {
		engine.getMemory().getPluginManager().delPlugin(plugin);
	}
}
