package hicosim.components.frontend.dialogues;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.FlowLayout;
import java.awt.Frame;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Panel;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JRadioButton;
import javax.swing.JTextField;

/**
 * @author Daniel Obermueller 23rd May 2009
 */
/**
 * This class extends the super class JDialog and provides two JTextFields, two
 * JRadioButtons and two JButtons. It is used to retrieve data of user inputs.
 */
public class CustomJDialog extends JDialog implements ActionListener {

	private static final long serialVersionUID = 1L;

	private GridBagConstraints gridBagConstraints = null;

	private Panel panelWithJTextFields = null;
	private JTextField jTextFieldStartIndex = null;
	private JTextField jTextFieldEndIndex = null;

	private Panel panelWithJRadioButtons = null;
	private JRadioButton jRadioButtonColumns = null;
	private JRadioButton jRadioButtonRows = null;

	private Panel panelWithJButtons = null;
	private JButton jButtonOk = null;
	private JButton jButtonCancel = null;

	private String[] content = null;

	public CustomJDialog(Frame frame, String title) {
		super(frame, title, true);
		initialize();
	}

	/**
	 * @author Daniel Obermueller 23rd May 2009
	 */
	/**
	 * This method initializes the Panel "panelWithJRadioButtons".
	 *
	 * @return javax.swing.JPanel
	 */
	private Panel getPanelWithJRadioButtons() {
		if (panelWithJRadioButtons == null) {
			panelWithJRadioButtons = new Panel();
			panelWithJRadioButtons.setLayout(new GridBagLayout());
			/*
			 * The ButtonGroup is needed in order to provide, that only one
			 * button at a time can be selected.
			 */
			ButtonGroup buttonGroup = new ButtonGroup();
			buttonGroup.add(getJRadioButtonColumns());
			buttonGroup.add(getJRadioButtonRows());
			addComponent(panelWithJTextFields, new JLabel("columns:"),
					gridBagConstraints, 0, 2);
			addComponent(panelWithJTextFields, getJRadioButtonColumns(),
					gridBagConstraints, 1, 2);
			addComponent(panelWithJTextFields, new JLabel("rows:"),
					gridBagConstraints, 0, 3);
			addComponent(panelWithJTextFields, getJRadioButtonRows(),
					gridBagConstraints, 1, 3);
		}
		return panelWithJRadioButtons;
	}

	/**
	 * @author Daniel Obermueller 23rd May 2009
	 */
	/**
	 * This method initializes the JRadioButton "jRadioButtonColumns".
	 *
	 * @return javax.swing.JRadioButton
	 */
	private JRadioButton getJRadioButtonColumns() {
		if (jRadioButtonColumns == null) {
			jRadioButtonColumns = new JRadioButton("columns");
			jRadioButtonColumns.setSelected(true);
		}
		return jRadioButtonColumns;
	}

	/**
	 * @author Daniel Obermueller 23rd May 2009
	 */
	/**
	 * This method initializes the JRadioButton "jRadioButtonRows".
	 *
	 * @return javax.swing.JRadioButton
	 */
	private JRadioButton getJRadioButtonRows() {
		if (jRadioButtonRows == null) {
			jRadioButtonRows = new JRadioButton("rows");
		}
		return jRadioButtonRows;
	}

	/**
	 * @author Daniel Obermueller 23rd May 2009
	 */
	/**
	 * This method initializes the Panel "panelWithJTextFields".
	 *
	 * @return javax.swing.JPanel
	 */
	private Panel getPanelWithJTextFields() {
		if (panelWithJTextFields == null) {
			panelWithJTextFields = new Panel();
			panelWithJTextFields.setLayout(new GridBagLayout());
			addComponent(panelWithJTextFields, new JLabel("Start index:"),
					gridBagConstraints, 0, 0);
			addComponent(panelWithJTextFields, getJTextFieldStartIndex(),
					gridBagConstraints, 1, 0);
			addComponent(panelWithJTextFields, new JLabel("End index:"),
					gridBagConstraints, 0, 1);
			addComponent(panelWithJTextFields, getJTextFieldEndIndex(),
					gridBagConstraints, 1, 1);
		}
		return panelWithJTextFields;
	}

	/**
	 * @author Daniel Obermueller 23rd May 2009
	 */
	/**
	 * This method initializes the JTextField "jTextFieldStartIndex".
	 *
	 * @return javax.swing.JTextField
	 */
	private JTextField getJTextFieldStartIndex() {
		if (jTextFieldStartIndex == null) {
			jTextFieldStartIndex = new JTextField(20);
		}
		return jTextFieldStartIndex;
	}

	/**
	 * @author Daniel Obermueller 23rd May 2009
	 */
	/**
	 * This method initializes the JTextField "jTextFieldEndIndex".
	 *
	 * @return javax.swing.JTextField
	 */
	private JTextField getJTextFieldEndIndex() {
		if (jTextFieldEndIndex == null) {
			jTextFieldEndIndex = new JTextField(20);
		}
		return jTextFieldEndIndex;
	}

	/**
	 * @author Daniel Obermueller 23rd May 2009
	 */
	/**
	 * This method initializes the Panel "panelWithJButtons".
	 *
	 * @return javax.swing.JPanel
	 */
	private Panel getPanelWithJButtons() {
		if (panelWithJButtons == null) {
			panelWithJButtons = new Panel();
			panelWithJButtons.setLayout(new FlowLayout());
			panelWithJButtons.add(getJButtonOk());
			panelWithJButtons.add(getJButtonCancel());
		}
		return panelWithJButtons;
	}

	/**
	 * @author Daniel Obermueller 23rd May 2009
	 */
	/**
	 * This method initializes the JButton "jButtonOk".
	 *
	 * @return javax.swing.JButton
	 */
	private JButton getJButtonOk() {
		if (jButtonOk == null) {
			jButtonOk = new JButton("OK");
			jButtonOk.addActionListener(this);
		}
		return jButtonOk;
	}

	/**
	 * @author Daniel Obermueller 23rd May 2009
	 */
	/**
	 * This method initializes the JButton "jButtonCancel".
	 *
	 * @return javax.swing.JButton
	 */
	private JButton getJButtonCancel() {
		if (jButtonCancel == null) {
			jButtonCancel = new JButton("CANCEL");
			jButtonCancel.addActionListener(this);
		}
		return jButtonCancel;
	}

	/**
	 * @author Daniel Obermueller 23rd May 2009
	 */
	/**
	 * This method initializes this.
	 */
	private void initialize() {
		gridBagConstraints = new GridBagConstraints();
		gridBagConstraints.anchor = GridBagConstraints.WEST;
		getContentPane().add(getPanelWithJButtons(), BorderLayout.SOUTH);
		getContentPane().add(getPanelWithJTextFields(), BorderLayout.NORTH);
		getContentPane().add(getPanelWithJRadioButtons(), BorderLayout.CENTER);
		pack();
		setResizable(false);
		setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
	}

	/**
	 * @author Daniel Obermueller 23rd May 2009
	 */
	/**
	 * This method is using GridBagConstraints in order to add a component to
	 * the passed Panel "panel" at the passed coordinates "x" and "y".
	 */
	private void addComponent(Panel panel, Component component,
			GridBagConstraints gridBagConstraints, int x, int y) {
		gridBagConstraints.gridx = x;
		gridBagConstraints.gridy = y;
		panel.add(component, gridBagConstraints);
	}

	/**
	 * @author Daniel Obermueller 23rd May 2009
	 */
	/**
	 * This method is invoked when the JButton "jButtonOk" or JButton
	 * "jButtonCancel" is pressed.
	 *
	 */
	public void actionPerformed(ActionEvent evt) throws NumberFormatException {
		if (evt.getSource() == jButtonOk) {
			storeContentsOfJTextFields();
			dispose();
		} else {
			dispose();
		}
	}

	/**
	 * @author Daniel Obermueller 23rd May 2009
	 */
	/**
	 * This method returns the selected JRadioButton, represented as a
	 * String-Object.
	 *
	 * @return String
	 */
	private String getSelectedJRadioButton() {
		if (jRadioButtonRows.isSelected())
			return "rows";
		return "columns";
	}

	/**
	 * @author Daniel Obermueller 23rd May 2009
	 */
	/**
	 * This method stores the contents of the two JTextFields
	 * "jTextFieldStartIndex" and "jTextFieldEndIndex" and the selected
	 * JRadioButton in a private attribute, which is a String-Array.
	 */
	private void storeContentsOfJTextFields() throws NumberFormatException {
		content = new String[3];
		content[0] = getSelectedJRadioButton();
		content[1] = jTextFieldStartIndex.getText().trim();
		content[2] = jTextFieldEndIndex.getText().trim();
	}

	/**
	 * @author Daniel Obermueller 23rd May 2009
	 */
	/**
	 * This method sets the visibility of the JDialog to "true" and returns the
	 * input values of the two JTextFields plus the selected JRadioButton as an
	 * Object-Array, after the JDialog is disposed.
	 *
	 * @return String[]
	 */
	public String[] prompt(){
		setVisible(true);
		return content;
	}

	public static void main(String[] args) {
		JFrame f = new JFrame("JFrame");
		CustomJDialog d;
		d = new CustomJDialog(f, "JDialog");
		Object[] numbers = d.prompt();
		for (int i = 0; i < numbers.length; i++)
			System.out.println(numbers[i]);
		System.exit(0);
	}
}
