package hicosim.components.plugins;

/* Fuer spaeteres Highlighting */
//import java.awt.Color;

import java.awt.Color;
import java.awt.Component;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Vector;

import hicosim.components.backend.processor.*;

import hicosim.control.Engine;

import hicosim.control.utils.*;

import javax.swing.*;
import javax.swing.table.*;

/**
 *
 * This plugin displays a modifiable section of the hicovec memory in a JTable.
 * The memory range can be set by using two JButtons or by direct input.
 *
 * This plugin is intended for observing the memory in the hicovec gui.
 *
 * @author Weber
 *
 *
 *
 */

public class WatchMemPlugin implements IPlugin {

	private IPluginManager pluginMem;
	private Engine engine;

	private int startAt;
	private int endAt;
	private int stepRange;

	private int ip;

	private boolean dispHex;
	private boolean observer;

	private JTable tabelle;
	private JPanel inputPanel;
	private JTextField txtF;

	/* columns layout in jtable */
	private final int CO_ADR = 0;
	private final int CO_DEZ = 1;
	private final int CO_HEX = 2;
	private final int CO_BIN = 3;
	private final int CO_DISAS = 4;
	private final int CO_WP = 5;

	private MemViewTableModel model;


	/* constructor */
	public WatchMemPlugin(int startAt, int endAt, Engine eng) throws Exception {

		this.startAt = startAt;
		this.endAt = endAt;
		this.stepRange = (endAt - startAt + 1);
		this.dispHex = false;

		this.engine = eng;

		this.ip = engine.getIP();

		initializeView();


	}

	/**
	 * Method is called by pluginManager every time a memory-value is written
	 *
	 * @param adr int
	 * @param value int
	 */
	public void notifyPlugin(int adr, int value) {


		try {
			model.setValueDirectAt(Integer.toString(value),
					(adr - startAt), CO_DEZ);

			model.setValueDirectAt("0x" + Integer.toHexString(value),
					(adr - startAt), CO_HEX);

			model.setValueDirectAt(Integer.toBinaryString(value),
					(adr - startAt), CO_BIN);

		}

		catch (Exception e) {
			this.engine.displayConsole(
					"ERROR IN WATCHMEM_PLUGIN.notifyPlugin()" + "\n"
					+ e.getMessage() + "\n" + e.getCause(), true);
		}

		// ################### EDIT FUE DISAS ANZEIGE
		try {
			model.setValueDirectAt(
							engine.getMemory()
									.getDisassembledInstructionAt(adr) == null ?
							"NONE" : engine.getMemory()
									.getDisassembledInstructionAt(adr),
							(adr - startAt), 4);
		}

		catch (Exception e) {
			model.setValueDirectAt("NONE", (adr - startAt), 4);
		}

		boolean wp = this.engine.getMemory().isWatchPoint(adr);
		model.setValueDirectAt(wp, (adr - startAt), 5);

	}

	public String toString() {
		return "WatchMemPlugin";
	}

	private void initializeView() throws Exception {

		String[][] rowData = new String[endAt - startAt + 1][2];

		for (int i = 0; i < (this.endAt - this.startAt + 1); i++) {
			if (!dispHex) {
				rowData[i][0] = Integer.toString(startAt + i);
			} else {
				rowData[i][0] = Integer.toHexString(startAt + i);
			}

			rowData[i][1] = "";
		}

		String[] columnNames = { "Adresse", "Wert Dezimal", "Wert Hex",
				"Wert Binaer", "DISAS", "WP" };

		model = new MemViewTableModel(rowData, columnNames);

		this.tabelle = new JTable(model);
		tabelle.setModel(model);
		TableColumn col = tabelle.getColumnModel().getColumn(CO_WP);
		col.setPreferredWidth(20);


		inputPanel = new JPanel();
		txtF = new JTextField(10);

		JButton btUp = new JButton("Up");
		JButton btDwn = new JButton("Down");

		btUp.setFont(new Font("Default", 0, 9));
		btDwn.setFont(new Font("Default", 0, 9));

		inputPanel.add(txtF);
		inputPanel.add(btUp);
		inputPanel.add(btDwn);

		ActionListener txtInput = new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				int tmpInt;
				try {
				tmpInt = parseTxtInput(txtF.getText()); }
				catch (Exception e1){
					engine.displayConsole(memViewError(), true);
					tmpInt = 0;

					return;
				}
				startAt = tmpInt;
				endAt = tmpInt + stepRange - 1;

				txtF.setText("");

				updateRange();
				reFresh();
			}
		};

		ActionListener btPressDwn = new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				startAt = startAt + stepRange;
				endAt = endAt + stepRange;

				updateRange();
				reFresh();
			}
		};

		ActionListener btPressUp = new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {

				if((startAt - stepRange) > 0){
					startAt = startAt - stepRange;
					endAt = endAt - stepRange;
				} else {
					startAt = 0;
					endAt = startAt + stepRange - 1;

				}

				updateRange();
				reFresh();
			}
		};

		txtF.addActionListener(txtInput);

		btUp.addActionListener(btPressUp);
		btDwn.addActionListener(btPressDwn);


		/* neuer DefaultCellRenderer */
		tabelle.setDefaultRenderer(Object.class,
				new DefaultTableCellRenderer() {
					/**
					 *
					 */
					private static final long serialVersionUID = 1L;

					public Component getTableCellRendererComponent(
							JTable table, Object value, boolean isSelected,
							boolean hasFocus, int row, int column) {
						Component component = super
								.getTableCellRendererComponent(table, value,
										isSelected, hasFocus, row, column);

						if ((startAt <= ip) && (endAt > ip) && row == (ip - startAt)) {
							component.setFont(component.getFont().deriveFont(
									Font.BOLD));
							component.setForeground(Color.RED);
						} else {
							component.setFont(component.getFont().deriveFont(
									Font.PLAIN));
							component.setForeground(Color.BLACK);
						}



						return component;
					}
				});



	}



	/*
	 * Manuelles refrehen der Anzeige - noetig z.B. wenn der Speicher ueber
	 * die loadObjectFile-Methode beschrieben wurde
	 *
	 */

	public void reFresh() {
		updateTable();


		for (int i = 0; i < stepRange; i++) {
			if (!dispHex) {
				model.setValueDirectAt(
				Integer.toString(startAt + i), i, 0);
			} else {
				model.setValueDirectAt(
						"0x" + Integer.toHexString(startAt + i), i, CO_ADR);
			}

			model.setValueDirectAt(
			Integer.toString(getMemory(startAt + i)), i, CO_DEZ);

			model.setValueDirectAt(
			"0x" + Integer.toHexString(getMemory(startAt + i)), i, CO_HEX);

			model.setValueDirectAt(
			Integer.toBinaryString(getMemory(startAt + i)), i, CO_BIN);

			// ################### EDIT FUER DISAS ANZEIGE

			try {
				model.setValueDirectAt(
						engine.getMemory().getDisassembledInstructionAt(
								startAt + i) == null ?
						"NONE" : engine.getMemory()
								.getDisassembledInstructionAt(startAt + i),
						(i), CO_DISAS);
			}

			catch (Exception exc) {
				model.setValueDirectAt("NONE", (i), CO_DISAS);
			}

			boolean wp = this.engine.getMemory().isWatchPoint(startAt + i);
			model.setValueDirectAt(wp, i, CO_WP);
		}
	}

	public void updateIp(int ip){

		this.ip = ip;

		if(startAt <= ip && endAt > ip){
			tabelle.setRowSelectionInterval((ip-startAt), (ip-startAt));

		} else {
			tabelle.removeRowSelectionInterval(0, (model.getRowCount() - 1));
		}
	}

	public JTable getTable() {
		return this.tabelle;
	}

	public JPanel getInputPanel() {
		return this.inputPanel;
	}

	public void setStartAt(int newStartAt) {
		this.startAt = newStartAt;
	}

	public void setEndAt(int newEndAt) {
		this.endAt = newEndAt;
	}

	public int getStartAt() {
		return this.startAt;
	}

	public int getEndAt() {
		return this.endAt;
	}

	public void updateRange() {
		pluginMem.setNewRange(this, startAt, endAt );
	}

	private int getMemory(int adr) {
		try {
			return pluginMem.pluginGetMemory(this, adr);
		}
		catch (Exception e1) {
			System.err.println(e1.toString());
			engine.displayConsole("Error! Memory access failed!", true);
			return 0;
		}
	}

	private void setMemory(int adr, int value) {
		try {
			pluginMem.pluginSetMemory(this, adr, value);
		}
		catch (Exception e1) {
			System.err.println(e1.toString());
			engine.displayConsole("Error! Memory access failed!", true);
		}
	}

	public void setPluginManager(IPluginManager newPluginManager) {
		this.pluginMem = newPluginManager;
	}

	public void setObserver(boolean observe) {
		this.observer = observe;
	}

	public boolean isObserver() {
		return this.observer;
	}

	private int parseTxtInput(String tmpTxt) {
		String[] tmpSplit = new String[2];

		if (tmpTxt.contains("-")) {
			tmpSplit = tmpTxt.split("-");

		} else if (tmpTxt.contains(" ")) {
				tmpSplit = tmpTxt.split(" ");
		} else {
			tmpSplit[0] = tmpTxt;
			tmpSplit[1] = tmpTxt;
		}

		int[] tmpInt = new int[2];

		/* Label erkennen */
		if (Character.isLetter(tmpSplit[0].charAt(0))){
			try {
				tmpInt[0] = engine.getAddressOfLabel(tmpSplit[0]);
				return tmpInt[0];
			} catch (Exception e1){
				this.engine.displayConsole("Wrong Input! Label not found!", true);
				return 0;
			}
		}



		
		for (int i = 0; i < 2; i++) {
			if (tmpSplit[i].length() > 2
					&& tmpSplit[i].substring(0, 2).equalsIgnoreCase("0x")) {

				tmpInt[i] = BitUtils.hex2Decimal(tmpSplit[i].substring(2,
						tmpSplit[i].length()));

				dispHex = true;
			} else {
					tmpInt[i] = Integer.valueOf(tmpSplit[i]);
					dispHex = false;
				}
		} 
		

		if (tmpSplit[0].equalsIgnoreCase(tmpSplit[1])) {
			return tmpInt[0];
		} else {
			if (tmpInt[0] > tmpInt[1]) {
				return 0;
			}
			
			stepRange = (tmpInt[1] - tmpInt[0]) + 1;
		}

		return tmpInt[0];

	}


	/* Return  parsed integer */
	private int parseTableInput(String value, int column){

		int retI = 0;

		switch(column){
			case 1: // Column 2 = Dez.Wert
				retI = Integer.parseInt(value);
				break;
			case 2: // Hex
				String[] tmpS = value.split("x");
				retI = BitUtils.hex2Decimal(tmpS[1]);
				break;
			case 3: // Binaer
				retI = (int)BitUtils.binary2Decimal(value);
				break;
		}

		return retI;
	}

	private void updateTable() {
		
		/* Wenn stepRange nicht mit rowCount uebereinstimmt */
		if (stepRange != model.getRowCount()) {
			/* Wenn es mehr Zeilen als Adressen gibt */
			if (model.getRowCount() > stepRange) {
				/* Zeilen muessen herausgenommen werden */
				while (model.getRowCount() > stepRange) {
					model.removeRow(0);
				}
			}
		}

		if (stepRange != model.getRowCount()) {
			/* Wenn es weniger Zeilen als Adressen gibt */
			if (model.getRowCount() < stepRange) {
				/* Zeilen muessen addiert werden */
				while (model.getRowCount() < stepRange) {
					Vector<String> tmpV = new Vector<String>();
					tmpV.add("");
					model.addRow(tmpV);
				}
			}
		}
	}

	
	
	protected String memViewError(){
		return "Wrong Input! Try DEZ/HEX, DEZ/Hex-DEZ/HEX or DEZ/HEX DEZ/HEX, LABEL\n " +
				" example: 0x200 0x204 shows memory from 0x200 to 0x204";
	}
	/* TableModel */
	class MemViewTableModel extends DefaultTableModel {

		private static final long serialVersionUID = 1L;

		public MemViewTableModel(String[][] rowData, String[] columnNames){
			super(rowData, columnNames);
		}

		public int getColumnCount() {
			return super.getColumnCount();
		}


		public int getRowCount() {
			return super.getRowCount();
		}

		public Class<?> getColumnClass(int columnIndex){
			if(columnIndex == 5){
				return Boolean.class;

			}	else {
				return String.class;
			}

		}

		public Object getValueAt(int rowIndex, int columnIndex) {
			return super.getValueAt(rowIndex, columnIndex);
		}

		public boolean isCellEditable(int rowIndex, int columnIndex){
			/* if row = memory address or disassembler: forbid editing */
//			if(columnIndex == CO_ADR || columnIndex == 4){
//				return false;
//			}
			/* if column = memory address: forbid editing */
			if(columnIndex == CO_ADR){
				return false;
			}
			/* if an opcode is set at given address: forbid editing */
//			if(!model.getValueAt(rowIndex, 4).toString().equalsIgnoreCase("NONE")){
//				return false;
//			}
			return true;
		}

		public void setValueAt(Object aValue, int rowIndex, int columnIndex){
			if(columnIndex == CO_DEZ || columnIndex == CO_HEX || columnIndex == CO_BIN){
				int ret;
				try {
				 ret = parseTableInput(aValue.toString(), columnIndex);
				} catch (Exception e1){
					System.err.println("Error setting memory!");
					return;
				}
				setMemory((startAt + rowIndex), ret);
			}

			if(columnIndex == CO_WP){
				if(aValue.toString() == "true"){
//					System.out.println("true!");
					engine.getMemory().setWatchPoint(startAt + rowIndex);

				} else {
					engine.getMemory().delWatchPoint(startAt + rowIndex);
				}

				setValueDirectAt(aValue, rowIndex, columnIndex);
			}

		}

		/* Methode schreibt direkt in die Tabelle per Aufruf der Superklasse */
		public void setValueDirectAt(Object aValue, int rowIndex, int columnIndex){
		try {
			super.setValueAt(aValue, rowIndex, columnIndex);
			} catch (ArrayIndexOutOfBoundsException e1){
				System.err.println("ArrayIndexOutOfBound in MemView! rowIndex=" + rowIndex + ", colIndex= " + columnIndex);


			}
		}

	}

}
