
package hicosim.components.plugins.vga;


/**
 * Class to look up codes for colors and signs.
 * 
 * @author Michael Wager, 19th May 2009
 *
 */
public class LookUpTable {

	private static int [] arr = new int[16];
	
	public static void setUp() {
		arr[0] = 0xffffffff; //new int[]{255, 255, 255, 255}; //white
		arr[1] = 0xff00ffff; //new int[]{255, 0, 255, 255}; //cyan
		arr[2] = 0xffff00ff; //new int[]{255, 255, 0, 255};// magenta	
		arr[3] = 0xff0000CD; //new int[]{255, 0, 0, 205};// blue
		arr[4] = 0xffffff00; //new int[]{255, 255, 255, 0};// yellow
		arr[5] = 0xff00ff00; //new int[]{255, 0, 255, 0};// green
		arr[6] = 0xffff0000; //new int[]{255, 255, 0, 0};// red
		arr[7] = 0xff696969; //new int[]{255, 105, 105, 105};// --dunkelgrau / darkgrey
		arr[8] = 0xffE0E0E0; //new int[]{255, 0xE0,0xE0,0xE0};// --hellgrau /  light colored grey
		arr[9] = 0xff008080; //new int[]{255, 0, 0x80, 0x80};// --dunkelcyan ..
		arr[10] =0xff690036; //new int[]{255, 0x69, 0, 0x36}; //--dunkelmagenta / darkmagenta
		arr[11] =0xff00007B; // new int[]{255, 0, 0, 0x7B}; //--dunkelblau / darkblue
		arr[12] =0xff808000; // new int[]{255, 0x80, 0x80, 0};// --dunkelgelb / darkyellow
		arr[13] =0xff007B00; // new int[]{255, 0, 0x7B, 0}; //--dunkelgruen / darkgreen
		arr[14] =0xff7B0000; // new int[]{255, 0x7B, 0, 0}; //--dunkelrot / darkred
		arr[15] =0xff000000; // new int[]{255, 0, 0, 0};//schwarz / black
		
		signs[65] = A;
		signs[66] = B;
		signs[67] = C;
		signs[68] = D;
		signs[69] = E;
		signs[70] = F;
		signs[71] = G;
		signs[72] = H;
		signs[73] = I;
		signs[74] = J;
		signs[75] = K;
		signs[76] = L;
		signs[77] = M;
		signs[78] = N;
		signs[79] = O;
		signs[80] = P;
		signs[81] = Q;
		signs[82] = R;
		signs[83] = S;
		signs[84] = T;
		signs[85] = U;
		signs[86] = V;
		signs[87] = W;
		signs[88] = X;
		signs[89] = Y;
		signs[90] = Z;
		
		signs[48] = Null;
		signs[49] = eins;
		signs[50] = zwei;
		signs[51] = drei;
		signs[52] = vier;
		signs[53] = fuenf;
		signs[54] = sechs;
		signs[55] = sieben;
		signs[56] = acht;
		signs[57] = neun;
		
		signs[116] = tree;
		signs[107] = klee;
		signs[98] = bomb;
		
		signs[35] = dummy; //35 = raute
		signs[32] = leer;
		
		signs[46] = poi;
		signs[58] = poi2;
		signs[95] = ustr;
		signs[45] = mstr;
		
		signs[33] = loud;
		signs[63] = ask;
	}
	
	/**
	 * This method returns the color for the given 
	 * int value "code" as an integer array. "code" can be between 0 and 15, 
	 * else return black.
	 * @param code - int
	 * @return Color - int array
	 */
	//Color Look Up Table(CLUT) (siehe Dokumentation.pdf-Scotchas Seite 20)
	//8.3.1 Adressen der VGA-Zeilen/ASCII-Zeichen -> seite 65,66
	public static int CLUT(int code) {
		return arr[code];
	}
	
	/**
	 * This method returns a String which
	 * represents a 8x8 Pixel-Sign. This is needed
	 * to render 8x8 Signs/Letters.
	 * @param ascii_code - int
	 * @return  - String 
	 */
	// ALUT (Ascii LookUp Table), ScotchDoku, site 22
	public static String ALUT(int ascii_code) {
		return signs[ascii_code]; 
	}
	
	//-------------------- 8x8 pixel string representation of signs -----------------------------
	
	final static String [] signs = new String [128];

	
	final static String A = 
		"00011000"+
		"00111100"+
		"01100110"+
		"01111110"+
		"01100110"+
		"01100110"+
		"01100110"+
		"00000000";
	
	final static String B =
		"01111110"+
		"01100110"+
		"01100110"+
		"01111110"+
		"01100110"+
		"01100110"+
		"01111110"+
		"00000000";
	final static String C =
		"01111110"+
		"01100000"+
		"01100000"+
		"01100000"+
		"01100000"+
		"01100000"+
		"01111110"+
		"00000000";
		
	final static String D = 
		"01111100"+
		"01100110"+
		"01100110"+
		"01100110"+
		"01100110"+
		"01100110"+
		"01111100"+
		"00000000";
		
		final static String E =
			"01111110"+
			"01100000"+
			"01100000"+
			"01111110"+
			"01100000"+
			"01100000"+
			"01111110"+
			"00000000";
			
		final static String F = 
			"01111110"+
			"01100000"+
			"01100000"+
			"01111110"+
			"01100000"+
			"01100000"+
			"01100000"+
			"00000000";
			
		final static String G = 
			"01111110"+
			"01100000"+
			"01100000"+
			"01101110"+
			"01100110"+
			"01100110"+
			"01111110"+
			"00000000";
			
		final static String H = 
			"01100110"+
			"01100110"+
			"01111110"+
			"01111110"+
			"01100110"+
			"01100110"+
			"01100110"+
			"00000000";
			
		final static String I = 
			"00011000"+
			"00011000"+
			"00011000"+
			"00011000"+
			"00011000"+
			"00011000"+
			"00011000"+
			"00000000";		
		final static String J = 
			"00111110"+
			"00000110"+
			"00000110"+
			"00000110"+
			"00000110"+
			"00000110"+
			"00011110"+
			"00000000";		
		final static String K = 
			"01100110"+
			"01101100"+
			"01111000"+
			"01110000"+
			"01111000"+
			"01101100"+
			"01100110"+
			"00000000";		
		final static String L = 
			"01100000"+
			"01100000"+
			"01100000"+
			"01100000"+
			"01100000"+
			"01100000"+
			"01111110"+
			"00000000";	
		final static String M = 
			"01100110"+
			"01111110"+
			"01111110"+
			"01100110"+
			"01100110"+
			"01100110"+
			"01100110"+
			"00000000";	
		final static String N = 
			"01100110"+
			"01100110"+
			"01110110"+
			"01111110"+
			"01101110"+
			"01100110"+
			"01100110"+
			"00000000";		
		final static String O = 
			"01111110"+
			"01100110"+
			"01100110"+
			"01100110"+
			"01100110"+
			"01100110"+
			"01111110"+
			"00000000";	
		final static String P = 
			"01111110"+
			"01100110"+
			"01111110"+
			"01100000"+
			"01100000"+
			"01100000"+
			"01100000"+
			"00000000";	
		final static String Q = 
			"01111110"+
			"01100110"+
			"01100110"+
			"01100110"+
			"01100110"+
			"01100110"+
			"01111111"+
			"00000000";	
		final static String R = 
			"01111110"+
			"01100110"+
			"01100110"+
			"01111110"+
			"01111000"+
			"01101100"+
			"01100110"+
			"00000000";	
		final static String S = 
			"01111110"+
			"01100000"+
			"01100000"+
			"01111110"+
			"00000110"+
			"00000110"+
			"01111110"+
			"00000000";
		final static String T = 
			"01111110"+
			"00011000"+
			"00011000"+
			"00011000"+
			"00011000"+
			"00011000"+
			"00011000"+
			"00000000";
		final static String U = 
			"01100110"+
			"01100110"+
			"01100110"+
			"01100110"+
			"01100110"+
			"01100110"+
			"01111110"+
			"00000000";
		final static String V =  
			"01100110"+
			"01100110"+
			"01100110"+
			"01100110"+
			"01100110"+
			"01100110"+
			"01111110"+
			"00011000";
			
		final static String W = 
			"01100110"+
			"01100110"+
			"01100110"+
			"01100110"+
			"01111110"+
			"01111110"+
			"01100110"+
			"00000000";
		final static String X = 
			"01100110"+
			"01100110"+
			"01111110"+
			"01111110"+
			"01111110"+
			"01100110"+
			"01100110"+
			"00000000";
		final static String Y = 
			"01100110"+
			"01100110"+
			"01100110"+
			"01111110"+
			"00011000"+
			"00011000"+
			"00011000"+
			"00000000";
		final static String Z = 
			"01111110"+
			"00000110"+
			"00001100"+
			"00011000"+
			"00110000"+
			"01100000"+
			"01111110"+
			"00000000";
		
		final static String Null =
			"01111110"+
			"01100110"+
			"01100110"+
			"01101110"+
			"01100110"+
			"01100110"+
			"01111110"+
			"00000000";	
		final static String eins = 
			"00011000"+
			"00111000"+
			"01011000"+
			"10011000"+
			"00011000"+
			"00011000"+
			"00011000"+
			"00000000";		
			final static String zwei = 
				"01111110"+
				"00000110"+
				"00000110"+
				"01111110"+
				"01100000"+
				"01100000"+
				"01111110"+
				"00000000";
			final static String drei = 
				"01111110"+
				"00000110"+
				"00000110"+
				"00111110"+
				"00000110"+
				"00000110"+
				"01111110"+
				"00000000";		
			final static String vier = 
				"00011100"+
				"00111000"+
				"01110000"+
				"11111110"+
				"11111110"+
				"00011000"+
				"00011000"+
				"00000000";		
			final static String fuenf = 
				"01111110"+
				"01111000"+
				"01100000"+
				"01111110"+
				"00000110"+
				"01100110"+
				"01111110"+
				"00000000";	
				
			final static String sechs = 
				"01111110"+
				"01100000"+
				"01100000"+
				"01111110"+
				"01100110"+
				"01100110"+
				"01111110"+
				"00000000";		
				
			final static String sieben = 
				"01111111"+
				"00000110"+
				"00001100"+
				"00011000"+
				"00110000"+
				"01100000"+
				"11000000"+
				"00000000";		
				
			final static String acht = 
				"01111110"+
				"01100110"+
				"01111110"+
				"01111110"+
				"01100110"+
				"01100110"+
				"01111110"+
				"00000000";	
				
			final static String neun = 
				"01111110"+
				"01100110"+
				"01100110"+
				"01111110"+
				"00000110"+
				"00000110"+
				"01111110"+
				"00000000";
	
		
	final static String tree =
		"00011000"+
		"00011000"+
		"00111100"+
		"01111110"+
		"11111111"+
		"00011000"+
		"00011000"+
		"00000000";
	final static String klee =
		"00111100"+
		"00011000"+
		"11011011"+
		"11111111"+
		"11011011"+
		"00011000"+
		"00011100"+
		"00000100";   //---> 0x77777577 == Vergleichszeile im Race(label "KLEECMP")
					//HAUPTSACHE DIE STIMMT, DENN DIESE WIRD IMMER VERGLICHEN OB SIE 
					//GENAU UEBER DEM AUTO STEHT!
		
	final static String bomb =
		"00000110"+
		"00001000"+
		"00010000"+
		"00110100"+
		"01101110"+
		"01111110"+
		"00111100"+
		"00111000";	 //---> 0x77FFF777 == Vergleichszeile im Race(label "BOMBCMP")
			//HAUPTSACHE DIE STIMMT, DENN DIESE WIRD IMMER VERGLICHEN OB SIE 
			//GENAU UEBER DEM AUTO STEHT!
		
	final static String poi =
	"00000000"+
	"00000000"+
	"00000000"+
	"00000000"+
	"00000000"+
	"00000000"+
	"00011000"+
	"00000000";	
	
	final static String poi2 =
		"00000000"+
		"00000000"+
		"00011000"+
		"00000000"+
		"00000000"+
		"00011000"+
		"00000000"+
		"00000000";	
	final static String ustr =
		"00000000"+
		"00000000"+
		"00000000"+
		"00000000"+
		"00000000"+
		"00000000"+
		"01111110"+
		"00000000";	
	final static String mstr =
		"00000000"+
		"00000000"+
		"00000000"+
		"01111110"+
		"00000000"+
		"00000000"+
		"00000000"+
		"00000000";	
	final static String loud =
		"00011000"+
		"00011000"+
		"00011000"+
		"00011000"+
		"00011000"+
		"00000000"+
		"00011000"+
		"00000000";	
	final static String ask =
	"01111110"+
	"00000110"+
	"01111110"+
	"01100000"+
	"01100000"+
	"01111110"+
	"00000000"+
	"00011000";
	final static String leer =
		"00000000"+
		"00000000"+
		"00000000"+
		"00000000"+
		"00000000"+
		"00000000"+
		"00000000"+
		"00000000";
		
	final static String dummy =
		"11111111"+
		"11111111"+
		"11111111"+
		"11111111"+
		"11111111"+
		"11111111"+
		"11111111"+
		"11111111";
	
}
