
package testPackage;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;


/**
 * This class is intended to read ".vhd" config files and therefore provides a
 * method "read()".
 *
 * @author Daniel Obermueller 9th May 2009
 *
 */
public class ConfigFileReader {
	private ArrayList<Object[]> list;
	private BufferedReader bufferedReader;

	/**
	 * @param String
	 *            path
	 * @throws Exception
	 *
	 */
	public ConfigFileReader(String path) throws Exception {
		list = new ArrayList<Object[]>();
		try {
			bufferedReader = new BufferedReader(new FileReader(new File(path)));
		} catch (FileNotFoundException e) {
			throw new Exception("The config file " + path + " does not exist!");
		}
	}


	/**
	 * This method is reading a ".vhd" config file and returns the configuration
	 * as an ArrayList<Object[]>, whereas the Object[] contains all required
	 * information of a constant.
	 *
	 * @author Daniel Obermueller 9th May 2009
	 * @return ArrayList<Object[]>
	 * @throws Exception
	 */
	public ArrayList<Object[]> read() throws Exception {
		try {
			String line;
			/* Determine the start of the configuration. */
			while ((line = bufferedReader.readLine()) != null) {
				if (line.contains("package cfg is"))
					break;
			}
			/* Receive the configuration. */
			while ((line = bufferedReader.readLine().trim().toLowerCase()) != null) {
				/* If the end of the configuration is reached, stop. */
				if (line.contains("end cfg;"))
					break;
				/*
				 * Else if the line is empty or is a comment, continue with the
				 * next line.
				 */
				else if (line.trim().equals("") || line.startsWith("--"))
					continue;
				/* Else if the line contains a comment, remove it. */
				else if (line.matches(".*--.*"))
					line = line.substring(0, line.indexOf("--")).trim();
				/*
				 * Else if the line is not matching the standard convention,
				 * throw an appropriate exception.
				 */
				else if (!line
						.matches("constant \\s*[a-z]*\\s*:\\s*[a-z]*\\s*:=\\s*.*;"))
					throw new Exception("Invalid expression! " + line);
				line = line.replace(";", "");
				String[] information = line.split(":");
				Object[] data = new Object[3];
				/* Save the configuration in an ArrayList<Object[]>. */
				for (int i = 0; i < information.length; i++) {
					information[i] = information[i].replace("constant", "")
							.replace("=", "").trim();
					data[i] = information[i];
				}
				list.add(data);
			}
		} catch (IOException e) {
			e.printStackTrace();
		}
		return list;
	}

	/**
	 * @param String[] args
	 * @throws Exception
	 */
	public static void main(String[] args) throws Exception {
		// String line = "constant n:integer:=2; -- amount of vector registers
		// ";
		// if (line.matches(".*--.*"))
		// line = line.substring(0, line.indexOf("--")).trim();
		// System.out.println(line
		// .matches("constant \\s*[a-z]*\\s*:\\s*[a-z]*\\s*:=\\s*.*;"));
		ConfigFileReader reader = new ConfigFileReader(System.getProperty("user.dir") + "/config.vhd");
		ArrayList<Object[]> list = reader.read();
		for (Object[] o : list) {
			for (int i = 0; i < o.length; i++)
				System.out.print(o[i] + " ");
			System.out.println();
		}
	}

}
