------------------------------------------------------------------
-- PROJECT:     Der Hicovec Prozessor in Aktion
--
-- ENTITY:		Audio
--
-- PURPOSE:    	PWM Audio Hardware
-- 
-- System to
-- Connect,
-- Operate ,
-- Transfer data and
-- Compute with the
-- Hicovec 
--
-- Prof. Gundolf Kiefer
-- Hochschule Augsburg
--
-- TEAM:
--
-- Andreas Mueller
-- Soeren Ruehm
--
-- 
-- DATE CREATE: 2009-11-28
-- LAST CHANGE: 2010-01-10 FROM: Andreas Mueller
-- VERSION:     0.1
-----------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;
use IEEE.numeric_std.all;
use work.cfg.all;

entity audio is

	port (
		clk: std_logic;
		-- bus interface...
		adr: in std_logic_vector (31 downto 0);
		data: inout std_logic_vector (31 downto 0);
		adressed: out std_logic;
		rd, wr: in std_logic;
		ready: out std_logic;
		-- I/O ports...
		AUDIO_L: out std_logic;
		AUDIO_R: out std_logic
	);
end audio;

architecture RTL of audio is
 	signal counter: std_logic_vector (24 downto 0);
	signal freqdiv: std_logic_vector (24 downto 0);
	signal s_audio_out: std_logic;
	signal enable: std_logic;

begin
	--Adressierung
	
	adressed <= '1' 
	when audio_base_adr (31 downto 4) = adr (31 downto 4)
	else '0';
	

	data <= "0000000" & counter(24 downto 0)
	when rd = '1'
	else "ZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZ";
	

 
	--Datenprozess
	process (clk)
	begin
		if clk'event and clk = '1' then
			ready <= '0';
			if rd = '1' then
				ready <= '1';
			end if;
			if wr = '1' then
				--Basisadresse
				if adr(0) = '0' then
				--if audio_base_adr(31 downto 0) = adr (31 downto 0) then
					freqdiv <= data(24 downto 0);
				elsif adr(0) = '1' then
				--Basisadresse +1
				--if audio_base_adr(31 downto 1) & '1' = adr (31 downto 0) then					
					enable <= data(0);
				end if;
				ready <= '1';
			end if;
		end if;
	end process;

	
	--PWM Prozess
	process (clk, freqdiv)
		begin
--			if reset = '1' then
--				counter <= (others => '0');
--				s_audio_out <= '0';
--			elsif enable = '1' then
			if enable = '1' then
				if clk'event and clk = '1' then
					counter <= counter - '1';
					if counter(24) = '1' then
						counter <= freqdiv(24 downto 0);
						s_audio_out <= not s_audio_out;
					end if;
				end if;
			end if;
	end process;
	
	-- Glue Logic
	AUDIO_L <= s_audio_out;
	AUDIO_R <= s_audio_out;

	
end;

