------------------------------------------------------------------
-- PROJECT:     Der Hicovec Prozessor in Aktion
--
-- ENTITY:      BUSLOGIC
--
-- PURPOSE:     Buslogic
-- 
-- System to
-- Connect,
-- Operate ,
-- Transfer data and
-- Compute with the
-- Hicovec 
--
-- Prof. Gundolf Kiefer
-- Hochschule Augsburg
--
-- TEAM:
--
-- Jakob Golus
-- David Lucinkiewicz
-- Andreas Mueller
-- Florian Richter	
-- Sebastian Rigorth
-- Soeren Ruehm
--
-- 
-- DATE CREATE: 2008-10-28
-- LAST CHANGE: 2010-02-24 FROM: Andreas Mueller
-- VERSION:     0.1
-----------------------------------------------------------------


library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;
use work.cfg.all;

entity buslogic is
	generic (slaves: integer);          -- number of slaves
	port (
		ma_rd, ma_wr: in std_logic;     -- read/write from master
		ma_ready: out std_logic;        -- ready to master
		sl_adressed, sl_ready: in std_logic_vector (0 to slaves-1);  -- signals from slaves
		sl_rd, sl_wr: out std_logic_vector (0 to slaves-1)           -- read/write to slaves
	);
end buslogic;

architecture RTL of buslogic is
begin

-- Daisy chain (combinatorial)...
	daisy_chain: process (ma_rd, ma_wr, sl_adressed, sl_ready)
	variable sl_select: std_logic_vector (0 to slaves-1);
	variable selected_ready, any_ready: std_logic;
	begin

-- check, if some module has 'ready' raised...
	any_ready := '0';
	for n in 0 to slaves-1 loop any_ready := any_ready or sl_ready(n); end loop;

-- determine selected module...
	if any_ready = '1' then
      -- If one slave has the 'ready' raised, that one is selected with priority. 
      -- Otherwise, hazards (1->0->1) may occur on the 'ma_ready' signal that may confuse
      -- the master, which may interpret the 0->1 transition as a completion of the following
      -- transaction.
		sl_select := sl_ready;
	else 
		sl_select := sl_adressed;
	end if;
	for n in slaves-2 downto 0 loop	-- pass 1: "00100100" -> "11111100"
		sl_select(n) := sl_select(n) or sl_select(n+1);
	end loop;
	for n in 0 to slaves-2 loop	-- pass 2: "11111100" -> "00000100"
		sl_select(n) := sl_select(n) and not sl_select(n+1);
	end loop;
--    sl_select(1) := sl_adressed(1);
--    sl_select(0) := not sl_select(1);
-- generate 'sl_rd', 'sl_wr' and 'ma_ready'...
	selected_ready := '0';
	for n in 0 to slaves-1 loop
		sl_rd(n) <= ma_rd and sl_select(n);
		sl_wr(n) <= ma_wr and sl_select(n);
		selected_ready := selected_ready or (sl_ready(n) and sl_select(n));
	end loop;
	ma_ready <= selected_ready and (ma_rd or ma_wr);
end process;
end RTL;
