------------------------------------------------------------------
-- PROJECT:	Der Hicovec Prozessor in Aktion
--
-- ENTITY:	CFG
--
-- PURPOSE:	Configuration-File
-- 
-- System to
-- Connect,
-- Operate ,
-- Transfer data and
-- Compute with the
-- Hicovec 
--
-- Prof. Gundolf Kiefer
-- Hochschule Augsburg
--
-- TEAM:
--
-- Jakob Golus
-- David Lucinkiewicz
-- Andreas Mueller
-- Florian Richter	
-- Sebastian Rigorth
-- Soeren Ruehm
--
--
-- DATE CREATE: 2009-01-02
-- LAST CHANGE: 2010-01-10 FROM: Andreas Mueller
-- VERSION:	0.1
-----------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

package cfg is
-- HiCoVec Configuration

	constant n: integer := 16;                      -- amount of vector registers 
	constant k: integer := 16;                     	-- amount words per vector register (even numbers)

	constant use_scalar_mult : boolean := true;     -- allow multiplications in scalar alu
	constant use_vector_mult : boolean := true;    -- allow multiplications in vector alu

	constant use_shuffle : boolean := false;        -- use shuffle unit
	constant max_shuffle_width : integer := 0;      -- max. shuffle width (dividable by 4)

	constant use_vectorshift : boolean := true;     -- allow shift of vector registers (vmol, vmor)
	constant vectorshift_width : integer := 4;     -- width of vectorshift in bit

-- I/O Configuration
	constant slaves: integer := 7;   		-- Anzahl der I/O Slaves
-- Basisadressen
	--Slave 0: BLOCKRAM 0x00000000 - 0x00001FFF						12341234123412341234123412341234
	constant blockram_base_adr: std_logic_vector(31 downto 0) 	:= "00000000000000000000000000000000";
	--Slave 1: VGA 0x00002000 - 0x00003FFFF
	constant vga_base_adr: std_logic_vector(31 downto 0) 		:= "00000000000000000010000000000000";
	--Slave 2: DEBUG_IO 0xFFFF0010 - 0xFFFF0017
	constant debug_io_base_adr: std_logic_vector(31 downto 0) 	:= "11111111111111110000000000010000";
	--Slave 3: IOMISC 0x0000FFFF
	constant iomisc_base_adr: std_logic_vector(31 downto 0) 	:= "00000000000000001111111111111111";
	--Slave 4: LCD 0x0000FFF0
	constant lcd_base_adr: std_logic_vector(31 downto 0) 		:= "00000000000000001111111111110000";
	--Slave 5: ASCII 0x00004000 - 0x00004400
	constant ascii_base_adr: std_logic_vector(31 downto 0) 		:= "00000000000000000100000000000000";
	--Slave 6: Audio 0x00004500	- 0x0000450FF	
	constant audio_base_adr: std_logic_vector(31 downto 0) 		:= "00000000000000000100010100000000";
	

end cfg;
