 /*Uebersetzer fuer Prozessor*/

%{
	#include <stdio.h>
	#include <string.h>
	#include <stdlib.h>
	#include <getopt.h>
	#include "commit.h"
	#include "functions.h"
	#include "config.h"
	#include "errors_and_warnings.h"
	
	extern char* yytext;
	extern int yylineno;
	extern FILE *yyin;
	extern int yydebug;
/*
Diese Daten werden gebraucht um die Sections richtig zu bilden
Das array lines[] ist vor allem für den Simulator wichtig um zu sehen welche Adresse jede Zeile hat

MERKLIST 	--> hier stehen die ganzen Labels, mit den Adressen
CODELIST	--> bei jedem ORG wird sich die Startadresse und die Länge gemerkt umd die 0x1 Sektions zu bilden
COMMENTLIST	--> Kommentarliste für die Sektion
lines[]		--> hier wird zu jeder Zeile im Quelltext die Adresse gemerkt, haupsächlich für den Simulator wichtig

*/
	MERKERLIST *first_merker=NULL;	
	CODELIST *first_code=NULL;	
	WERT_RETURN *first_expression=NULL;	
	COMMENTLIST *first_comment=NULL;
	int adress=0;
	int memIndex=0;
	int lineIndex=0;
	unsigned int memory[68000];
	short int lines[68000];
	int syntaxError=0;
	int start=0;				
%}

%union 
{
	VALUE i;
	STRINGS s;
	PERMUTATION p;
	MARK l;
	COMMAND_RETURN cmdRet;
	EXPRESSION_RETURN *exprRet;
	WERT_RETURN *wertRet;
	ENDV end;
	int anz;
	int x;

}

%token <i> ZAHL
%token <i> HEX ZERO
%token <i> REGISTER REGAXY VREG
%token <i> BYTE WORD DOUBLEWORD QUADWORD 
%token <i> ALPHAVALUE
%token <s> STRING
%token <l> MERKER    // z.B. DATA1:
%token <l> LABEL     // z.B. DATA1
%token <end> END
%token <p> PERMUT

%token <x>ORG START DC ADD SUB SAL SAR AND OR XOR NOT LDIL LDIH LD ST JMP HALT JZ JNZ ADC SBC INC LSL ROL DEC MUL LSR ROR JAL JC JNC CLZ SEZ CLC SEC NOP VNOP MOV MOVA VLD VST VMOV VMOL VMOR VADD VSUB VAND VOR VXOR VMUL VLSL VLSR VSHUF EQU


%token KOMMA KLAMMERAUF KLAMMERZU CLAMP_OPEN CLAMP_CLOSE RAUTE PUNKT ECK_AUF ECK_ZU

%left OROP
%left NOTOP
%left ANDOP
%left BITSHIFT_L BITSHIFT_R
%left ADDOP SUBOP
%left MULOP DIVOP

%type <cmdRet> command befehlswort
%type <exprRet> expression
%type <wertRet> wert
%type <i> regaxy0 regaxyi vregl vregh breitebw breitevoll


%start programm

%%

programm:	sections END		{CODELIST *dummy;
					 if(first_code!=NULL)
					 {
					 	for(dummy=first_code;dummy->next!=NULL;dummy=dummy->next);
					 	dummy->length=adress-dummy->startAdress;
					 }
					 else
					 {
						Err_NoCode();
					 }
					 first_comment=$2.commentl;
					}
		;
/*------------------------------------------------------------------------------------------------*/
sections:	sections code
		|code
		;
/*------------------------------------------------------------------------------------------------*/                   
code:		code MERKER befehlswort		{first_merker=add_Merker_To_List(first_merker,$2.mark,adress);          // ...
						 adressToLine($3.line,lines,&lineIndex,adress);                        // LOOP1: LD A, Y+0
						 first_expression=codeAndExpr(&$3,first_expression,memIndex,memory);   //...
						 adress+=$3.wortAnz;	
						 memIndex+=$3.wortAnz;
						/*
						adress   --> stellt die aktuelle wirkliche Adresse im Speicher im HiCoVec dar
						memIndex --> ist die aktuelle Adresse im Speicherabbildarray memory[]
						*/
						}
		|code befehlswort		{adressToLine($2.line,lines,&lineIndex,adress);                         //...
						 first_expression=codeAndExpr(&$2,first_expression,memIndex,memory);   // LD A, Y+0
						 adress+=$2.wortAnz;	                                               //...
						 memIndex+=$2.wortAnz;
						}
		|ORG HEX MERKER befehlswort	{first_code=addToCodelist($2.value,first_code,adress);		       // ....
						 adress=$2.value;						      //        ORG $30
						 first_merker=add_Merker_To_List(first_merker,$3.mark,adress);        //  DATA: DC 10
						 adressToLine($4.line,lines,&lineIndex,adress);                       //        DC 20
						 first_expression=codeAndExpr(&$4,first_expression,memIndex,memory);  //  ....      
						 adress+=$4.wortAnz;
						 memIndex+=$4.wortAnz;
						}
		|ORG HEX befehlswort		{first_code=addToCodelist($2.value,first_code,adress);			// ...
						 adress=$2.value;							// ORG $20
						 adressToLine($3.line,lines,&lineIndex,adress);				// ADD A,A,B   
						 first_expression=codeAndExpr(&$3,first_expression,memIndex,memory);    // ....
						 adress+=$3.wortAnz;
						 memIndex+=$3.wortAnz;
						}
		|error befehlswort		{yyerrok;
						 syntaxError=1;
						}
		|error ORG HEX			{yyerrok;
						 syntaxError=1;
						}
		|error MERKER			{yyerrok;
						 syntaxError=1;
						}
		;

befehlswort:	command
		{
		$$=$1;
		}
		|command RAUTE command
		{	//Syntaxprüfung/Logikprüfung muss noch richtig eingebaut werden; nur Testweise...
			
			if ($1.isSkalar == 1 && $3.isVektor == 1)
				{
				$$ = uniteCommands(&$1,&$3); // Achtung: Skalarbefehl muss erster Parameter sein in der Argumentenliste
				}
			else if ($1.isVektor == 1 && $3.isSkalar == 1) // Abfrage: Ist cmdRet ein Vektorbefehl?
				$$ = uniteCommands(&$3,&$1); 
			else
			{
				syntaxError=1;
				Err_ParallelFailure($1.line);
			}
		}
		;
command:	
		
		LD REGAXY KOMMA KLAMMERAUF regaxy0 ADDOP regaxyi KLAMMERZU
						{
						if ($7.regex == false )  //ask Flag
						  $$=NewCommand_ScalarRegDirectValue($1,0x20,$2.value,$5.value,0,$7.wertret,1,2);
						else
						  $$=NewCommand_ScalarAndReg($1,0x20,$2.value,$5.value,$7.value,NULL,1,0);
						} 			
		|ST KLAMMERAUF regaxy0 ADDOP regaxyi KLAMMERZU KOMMA REGAXY 	
						{
						if ($5.regex == false )  //ask Flags
						 $$=NewCommand_ScalarRegDirectValue($1,0x28,0x0,$3.value,0,$5.wertret,1,2);
						else
						  if ($8.value == 0x1) // ST Anweisung nur nach A zulassen sonst Syntax falsch !
						  $$=NewCommand_ScalarAndReg($1,0x28,0x0,$3.value,$5.value,NULL,1,0); // Destination mit Null gef�llt eventl. ein Sonderzeichen nehmen ??
						  else {printf("Grammatik Fehler, nur A zugelassen", yylineno); exit(1);}
						}
		
		|ADD regaxy0 KOMMA regaxy0 KOMMA regaxyi 	
						{
						if ($6.regex == false )  //ask Flags if expression
						  $$=NewCommand_ScalarRegDirectValue($1,0x10,$2.value,$4.value,0,$6.wertret,1,2);
						else
						 $$=NewCommand_ScalarAndReg($1,0x10,$2.value,$4.value,$6.value,NULL,1,0); 
						}
		
		|ADC regaxy0 KOMMA regaxy0 KOMMA regaxyi 	
						{
						if ($6.regex == false )  //ask Flags if expression
						  $$=NewCommand_ScalarRegDirectValue($1,0x11,$2.value,$4.value,0,$6.wertret,1,2);
						else
						 $$=NewCommand_ScalarAndReg($1,0x11,$2.value,$4.value,$6.value,NULL,1,0); 
						}
						
		|SUB regaxy0 KOMMA regaxy0 KOMMA regaxyi 	
						{
						if ($6.regex == false )  //ask Flags if expression
						  $$=NewCommand_ScalarRegDirectValue($1,0x12,$2.value,$4.value,0,$6.wertret,1,2);
						else
						 $$=NewCommand_ScalarAndReg($1,0x12,$2.value,$4.value,$6.value,NULL,1,0); 
						}
		
		|SBC regaxy0 KOMMA regaxy0 KOMMA regaxyi 	
						{
						if ($6.regex == false )  //ask Flags if expression
						  $$=NewCommand_ScalarRegDirectValue($1,0x13,$2.value,$4.value,0,$6.wertret,1,2);
						else
						 $$=NewCommand_ScalarAndReg($1,0x13,$2.value,$4.value,$6.value,NULL,1,0); 
						}
		
		|INC regaxy0 KOMMA regaxy0	
						{
						 $$=NewCommand_ScalarAndReg($1,0x14,$2.value,$4.value,0x0,NULL,1,0); 
						}
					
		|AND regaxy0 KOMMA regaxy0 KOMMA regaxyi 	
						{
						if ($6.regex == false )  //ask Flags if expression
						  $$=NewCommand_ScalarRegDirectValue($1,0x18,$2.value,$4.value,0,$6.wertret,1,2);
						else
						 $$=NewCommand_ScalarAndReg($1,0x18,$2.value,$4.value,$6.value,NULL,1,0); 
						}
						
		|XOR regaxy0 KOMMA regaxy0 KOMMA regaxyi 	
						{
						if ($6.regex == false )  //ask Flags if expression
						  $$=NewCommand_ScalarRegDirectValue($1,0x1A,$2.value,$4.value,0,$6.wertret,1,2);
						else
						 $$=NewCommand_ScalarAndReg($1,0x1A,$2.value,$4.value,$6.value,NULL,1,0); 
						}
						
		|LSL regaxy0 KOMMA regaxy0 	
						{
						 $$=NewCommand_ScalarAndReg($1,0x1C,$2.value,$4.value,0x0,NULL,1,0); 
						}
						
		|ROL regaxy0 KOMMA regaxy0	
						{
						 $$=NewCommand_ScalarAndReg($1,0x1D,$2.value,$4.value,0x0,NULL,1,0); 
						}
						
		|DEC regaxy0 KOMMA regaxy0	
						{
						 $$=NewCommand_ScalarAndReg($1,0x16,$2.value,$4.value,0x0,NULL,1,0); 
						}
						
		|OR regaxy0 KOMMA regaxy0 KOMMA regaxyi 	
						{
						if ($6.regex == false )  //ask Flags if expression
						  $$=NewCommand_ScalarRegDirectValue($1,0x19,$2.value,$4.value,0,$6.wertret,1,2);
						else
						 $$=NewCommand_ScalarAndReg($1,0x19,$2.value,$4.value,$6.value,NULL,1,0); 
						}
						
		|MUL regaxy0 KOMMA regaxy0 KOMMA regaxyi 	
						{
						if ($6.regex == false )  //ask Flags if expression
						  $$=NewCommand_ScalarRegDirectValue($1,0x1B,$2.value,$4.value,0,$6.wertret,1,2);
						else
						 $$=NewCommand_ScalarAndReg($1,0x1B,$2.value,$4.value,$6.value,NULL,1,0); 
						}
						
		|LSR regaxy0 KOMMA regaxy0	
						{
						 $$=NewCommand_ScalarAndReg($1,0x1E,$2.value,$4.value,0x0,NULL,1,0); 
						}
						
		|ROR regaxy0 KOMMA regaxy0
						{
						 $$=NewCommand_ScalarAndReg($1,0x1F,$2.value,$4.value,0x0,NULL,1,0); 
						}
		
		// Folgendes Problem bei JMP: Da (au�er bei JAL) nur 2 Register ben�tzt werden, hab ich bei der �bergabe immer nur dd -- tt bzw. dd -- 00000-nnnnnnnnnnn... �bergeben
		// da beim direktwert ja das ss nicht beachtet wird, hab ich angenommen dass bei JUMP Befehlen also dd und tt ben�tz werden
		// Das Problem besteht bei den ALU mit nur zwei Register auch , aber da hier keine Direktadressen m�glich sind, habe ich angenommen das dd Zielregister und ss quelle
	    // Neue �berlegung, das m�sste egal sein ob man bei ss und tt reinschreibt, da cpu immer addiert --> trotzdem abkl�ren
		|JMP KLAMMERAUF regaxy0 ADDOP regaxyi KLAMMERZU 	
						{
						if ($5.regex == false )  //ask Flags if expression
						  $$=NewCommand_ScalarRegDirectValue($1,0x08,0x0,$3.value,0,$5.wertret,1,2);
						else
						 $$=NewCommand_ScalarAndReg($1,0x08,0x0,$3.value,$5.value,NULL,1,0);
						}
		|JAL regaxy0 KOMMA KLAMMERAUF regaxy0 ADDOP regaxyi KLAMMERZU 	
						{
						if ($7.regex == false )  //ask Flags if expression
						  $$=NewCommand_ScalarRegDirectValue($1,0x08,$2.value,$5.value,0,$7.wertret,1,2);
						else
						 $$=NewCommand_ScalarAndReg($1,0x08,$2.value,$5.value,$7.value,NULL,1,0);
						}
						
		|JZ KLAMMERAUF regaxy0 ADDOP regaxyi KLAMMERZU 	
						{
						if ($5.regex == false )  //ask Flags if expression
						  $$=NewCommand_ScalarRegDirectValue($1,0x0F,$3.value,0x0,0,$5.wertret,1,2);
						else
						 $$=NewCommand_ScalarAndReg($1,0x0F,$3.value,0x0,$5.value,NULL,1,0); 
						}
						
		|JNZ KLAMMERAUF regaxy0 ADDOP regaxyi KLAMMERZU 	
						{
						if ($5.regex == false )  //ask Flags if expression
						  $$=NewCommand_ScalarRegDirectValue($1,0x0E,$3.value,0x0,0,$5.wertret,1,2);
						else
						 $$=NewCommand_ScalarAndReg($1,0x0E,$3.value,0x0,$5.value,NULL,1,0);
						}
						
		|JC KLAMMERAUF regaxy0 ADDOP regaxyi KLAMMERZU 	
						{
						if ($5.regex == false )  //ask Flags if expression
						  $$=NewCommand_ScalarRegDirectValue($1,0x0D,$3.value,0x0,0,$5.wertret,1,2);
						else
						 $$=NewCommand_ScalarAndReg($1,0x0D,$3.value,0x0,$5.value,NULL,1,0); 
						}
						
		|JNC KLAMMERAUF regaxy0 ADDOP regaxyi KLAMMERZU 	
						{
						if ($5.regex == false )  //ask Flags if expression
						  $$=NewCommand_ScalarRegDirectValue($1,0x0C,$3.value,0x0,0,$5.wertret,1,2);
						else
						 $$=NewCommand_ScalarAndReg($1,0x0C,$3.value,0x0,$5.value,NULL,1,0); 
						}
		|CLZ {$$=NewCommand_ScalarAndReg($1,0x30,0x0,0x2,0x0,NULL,1,0);   }
		
		|SEZ {$$=NewCommand_ScalarAndReg($1,0x30,0x0,0x2,0x2,NULL,1,0);   }
		
		|CLC {$$=NewCommand_ScalarAndReg($1,0x30,0x0,0x1,0x0,NULL,1,0);   }
		
		|SEC {$$=NewCommand_ScalarAndReg($1,0x30,0x0,0x1,0x1,NULL,1,0);   }

		|HALT{$$=NewCommand_ScalarAndReg($1,0x0A,0x0,0x0,0x0,NULL,1,0);}
		
		|NOP {$$=NewCommand_ScalarAndReg($1,0x00,0x0,0x0,0x0,NULL,1,0);}
		
		|VNOP{$$=NewCommand_VectorAndReg($1,0x00,0x0,0x0,0x0,NULL,1,0);}
		// mov befehle , gleiches problem nicht sicher welche quellregister zu nehmen sind....
		|MOV vregl CLAMP_OPEN regaxy0 CLAMP_CLOSE KOMMA regaxy0 
			{
			$$=NewCommand_MoveCmds($1,0x2E,0x0,$4.value,$7.value,0x84,$2.value,0x0,NULL,1,2);
			}
		|MOV regaxy0 KOMMA vregl CLAMP_OPEN regaxy0 CLAMP_CLOSE 
			{
			$$=NewCommand_MoveCmds($1,0x2F,$2.value,$6.value,0x0,0x85,0x0,$4.value,NULL,1,2);
			}
		|MOVA vregl KOMMA regaxy0
			{
			$$=NewCommand_MoveCmds($1,0x2D,0x0,0x0,$4.value,0x86,$2.value,0x0,NULL,1,2);
			}
		|VLD vregl KOMMA KLAMMERAUF regaxy0 ADDOP REGAXY KLAMMERZU
			{
			$$=NewCommand_MoveCmds($1,0x24,0x0,$5.value,$7.value,0x82,$2.value,0x0,NULL,1,2);
			}
		|VST KLAMMERAUF regaxy0 ADDOP REGAXY KLAMMERZU KOMMA vregl
			{
			$$=NewCommand_MoveCmds($1,0x2C,0x0,$3.value,$5.value,0x83,0x0,$8.value,NULL,1,2);
			}
		|VMOV vregl KOMMA vregl
			{
			$$=NewCommand_VectorAndReg($1,0x21,$2.value,$4.value,0x0,NULL,1,2);
			}
		|VMOV vregh KOMMA vregl
			{
			$$=NewCommand_VectorRegDirectValue($1,0x23,0x0,$4.value,0,$2.wertret,1,2);
			}
		|VMOV vregl KOMMA vregh
			{
			$$=NewCommand_VectorRegDirectValue($1,0x22,$2.value,0x0,0,$4.wertret,1,2);
			}
		|VMOL vregl KOMMA vregl
			{
			$$=NewCommand_VectorAndReg($1,0x28,$2.value,$4.value,0x0,NULL,1,0);
			}
		|VMOR vregl KOMMA vregl
			{
			$$=NewCommand_VectorAndReg($1,0x2C,$2.value,$4.value,0x0,NULL,1,0);
			}
		|VADD PUNKT breitevoll vregl KOMMA vregl KOMMA vregl
			{
			$$=NewCommand_VectorAlu($1,$3.value,0x0,$4.value,$6.value,$8.value,NULL,1,0);
			}
		|VSUB PUNKT breitevoll vregl KOMMA vregl KOMMA vregl
			{
			$$=NewCommand_VectorAlu($1,$3.value,0x2,$4.value,$6.value,$8.value,NULL,1,0);
			}
		|VAND PUNKT breitevoll vregl KOMMA vregl KOMMA vregl
			{
			$$=NewCommand_VectorAlu($1,$3.value,0x8,$4.value,$6.value,$8.value,NULL,1,0);
			}
		|VOR PUNKT breitevoll vregl KOMMA vregl KOMMA vregl
			{
			$$=NewCommand_VectorAlu($1,$3.value,0x9,$4.value,$6.value,$8.value,NULL,1,0);
			}
		|VXOR PUNKT breitevoll vregl KOMMA vregl KOMMA vregl
			{
			$$=NewCommand_VectorAlu($1,$3.value,0xA,$4.value,$6.value,$8.value,NULL,1,0);
			}
		|VLSL PUNKT breitevoll vregl KOMMA vregl
			{
			$$=NewCommand_VectorAlu($1,$3.value,0xC,$4.value,$6.value,0x0,NULL,1,0);
			}
		|VLSR PUNKT breitevoll vregl KOMMA vregl
			{
			$$=NewCommand_VectorAlu($1,$3.value,0xE,$4.value,$6.value,0x0,NULL,1,0);
			}
		|VMUL PUNKT breitebw vregl KOMMA vregl KOMMA vregl
			{
			$$=NewCommand_VectorAlu($1,$3.value,0xB,$4.value,$6.value,$8.value,NULL,1,0);
			}
		|VSHUF vregl KOMMA vregl KOMMA vregl KOMMA PERMUT
			{

			convertToPerm(&$8);
			$$=NewCommand_shuffle($1, $8.b,$8.s,$2.value,$4.value,$6.value,$8.p, NULL,1,0);

			}
		|DC wert			{int exAnz;            // Hier wird die verkettete Liste von wert durchlaufen und die Listenelemente gezählt
						 WERT_RETURN *help;   //  dann wird eine Struktur vom TYP COMMAND_RETURN zurückgegebn
						 for(help=$2,exAnz=0;help!=NULL;help=help->next,exAnz++);
						 $$=newCommand($1,0,$2,exAnz,1,1,1);
						}

		|START                          {
						 if(start==0)
	                                                 {
	                                                        start=$1;
	                                                 }
	                                                 else if(start>0)
	                                                 {
	                                                        start=-1;
	                                                 }
	                                                 else
	                                                 {
	                                                        start--;
	                                                 }
	                                                 $$=newCommand($1,0,NULL,0,1,1,1);
	                                         }
		
		;
/*------------------------------------------------------------------------------------------------*/


regaxy0:	REGAXY {$$=$1;}
		| ZERO {$$=$1;}
;
regaxyi:	REGAXY {
			$1.regex = true;  // true wenn keine expression
			$$=$1;
				   }
		| expression  { //dummy zeiger wird schon hier erstellt und �bergeben , das spart codezeilen, zugriff jedoch etwas komplizierter z.B.  $7.wertret->expression->value
			$$.regex = false;
			WERT_RETURN *dummy;
			dummy=(WERT_RETURN*)malloc(sizeof(WERT_RETURN));
			dummy->expression=$1;
			dummy->next=NULL;
			dummy->load=0; //bedeutet welche art der expr. 0 bedeutet Bit 0- Bit 15
			$$.wertret=dummy;
					  }
;
vregl:		VREG ZAHL
		{
			$$.line=$1.line;
			$$.regex=false;
			$$.value=$2.value;
			$$.wertret = NULL;
		}
		| VREG ZERO
		{
			$$.line=$1.line;
			$$.regex=false;
			$$.value=$2.value;
			$$.wertret = NULL;
		}

;
vregh:		VREG ECK_AUF expression ECK_ZU
		{
			$$.line=$1.line;
			$$.regex=false;
			$$.value=0;

			WERT_RETURN *dummy;
			dummy=(WERT_RETURN*)malloc(sizeof(WERT_RETURN));
			dummy->expression=$3;
			dummy->next=NULL;
			dummy->load=1; //8-Bit Direktwert für Expression im Vektorregister
			$$.wertret = dummy;
		}

;
breitebw:	BYTE   {$$=$1;}
		| WORD {$$=$1;} 	
;
breitevoll:	BYTE   {$$=$1;}
		| WORD {$$=$1;} 
		| DOUBLEWORD {$$=$1;} 
		| QUADWORD {$$=$1;} 	
;

/*
bei DC 20,30,4+6
hier wird eine verkettete Liste vom Typ WERT_RETURN zurückgegeben.
Jeder Eintrag in der Liste enthält einen ZAHLENWERT und den Zeiger auf nächste Element
*/
wert:		wert KOMMA expression		{WERT_RETURN *dummy;						
						 for(dummy=$1;dummy->next!=NULL;dummy=dummy->next);
						 dummy->next=(WERT_RETURN*)malloc(sizeof(WERT_RETURN));
						 dummy=dummy->next;
						 dummy->next=NULL;
						 dummy->expression=$3;
						 dummy->load=2;
						 $$=$1;
						}	
		|expression			{$$=(WERT_RETURN*)malloc(sizeof(WERT_RETURN));
						 $$->next=NULL;
						 $$->expression=$1;
						 $$->load=2;
						}
		|STRING				{

						int i;
						WERT_RETURN *dummyWertReturn = NULL , *currentWertReturn  = NULL, *firstWertReturn = NULL;
						EXPRESSION_RETURN *dummyExprReturn = NULL;

						/* 	Mit Hilfe der Schleife wird jedes Element im String in den ASCII-Wert umgewandelt und in eine Expression geschrieben.
							Damit die gleiche Struktur gewährleistet ist, wird eine verkettete Liste aufgebaut
						*/
							
						for (i=1; i< strlen($1.string)-1;i++)	// Achtung Schleifenzähler muss bei 1 beginnen und bei Strlen-1 aufhören, weil sonst die Anführungszeichen
											// mitinterpretiert werden		
						{
							if (firstWertReturn == NULL)	// Handelt es sich um das erste Element der verketteten Liste
							{
								dummyWertReturn = (WERT_RETURN*)malloc(sizeof(WERT_RETURN));
								dummyWertReturn->next = NULL;
								firstWertReturn = dummyWertReturn;	// Achtung firstWertReturn-Zeiger muss immer auf das erste Element zeigen
								currentWertReturn = dummyWertReturn;
							}
							else				// Sonstige Elemente
							{
								dummyWertReturn = (WERT_RETURN*)malloc(sizeof(WERT_RETURN));
								currentWertReturn->next = dummyWertReturn;
								dummyWertReturn->next = NULL;
								currentWertReturn = dummyWertReturn;
							}
							dummyExprReturn = newExpression(E_VAL,(int)$1.string[i],NULL,NULL,NULL);
						 	currentWertReturn->expression = dummyExprReturn;	// Aktuell erzeugte Expression mit der WertReturn-Struktur verknüpfen
							currentWertReturn->load = 2;
						}
						// Hier wird noch das Abschlussterminalzeichen gesetzt auf 0xFC
						dummyWertReturn = (WERT_RETURN*)malloc(sizeof(WERT_RETURN));
						currentWertReturn->next = dummyWertReturn;
						dummyWertReturn->next = NULL;
						currentWertReturn = dummyWertReturn;
						dummyExprReturn = newExpression(E_VAL,0xFC,NULL,NULL,NULL);
						currentWertReturn->expression = dummyExprReturn;	// Aktuell erzeugte Expression mit der WertReturn-Struktur verknüpfen
						currentWertReturn->load = 2;
						$$=firstWertReturn;
						}	

		|STRING	CLAMP_OPEN expression KOMMA expression CLAMP_CLOSE			{

						int i, background, frontColor, sign;
						WERT_RETURN *dummyWertReturn = NULL , *currentWertReturn  = NULL, *firstWertReturn = NULL;
						EXPRESSION_RETURN *dummyExprReturn = NULL;
						frontColor = $3->value;
						background = $5->value;
						
						/* 	Mit Hilfe der Schleife wird jedes Element im String in den ASCII-Wert umgewandelt und in eine Expression geschrieben.
							Damit die gleiche Struktur gewährleistet ist, wird eine verkettete Liste aufgebaut
						*/
							
						for (i=1; i< strlen($1.string)-1;i++)	// Achtung Schleifenzähler muss bei 1 beginnen und bei Strlen-1 aufhören, weil sonst die Anführungszeichen
											// mitinterpretiert werden		
						{
							if (firstWertReturn == NULL)	// Handelt es sich um das erste Element der verketteten Liste
							{
								dummyWertReturn = (WERT_RETURN*)malloc(sizeof(WERT_RETURN));
								dummyWertReturn->next = NULL;
								firstWertReturn = dummyWertReturn;	// Achtung firstWertReturn-Zeiger muss immer auf das erste Element zeigen
								currentWertReturn = dummyWertReturn;
							}
							else				// Sonstige Elemente
							{
								dummyWertReturn = (WERT_RETURN*)malloc(sizeof(WERT_RETURN));
								currentWertReturn->next = dummyWertReturn;
								dummyWertReturn->next = NULL;
								currentWertReturn = dummyWertReturn;
							}
							sign = (background<<12)+(frontColor<<8)+(int)$1.string[i]; /* 	Hintergrundfarbe h, Schriftfarbe s und ASCII-Wert a werden in sign-Variable geschrieben
															untersten 16 Bit: hhhhssss-aaaaaaaa */
							dummyExprReturn = newExpression(E_VAL,sign,NULL,NULL,NULL);
						 	currentWertReturn->expression = dummyExprReturn;	// Aktuell erzeugte Expression mit der WertReturn-Struktur verknüpfen
							currentWertReturn->load = 2;
						}
						// Hier wird noch das Abschlussterminalzeichen gesetzt auf 0xFC						dummyWertReturn = (WERT_RETURN*)malloc(sizeof(WERT_RETURN));
						currentWertReturn->next = dummyWertReturn;
						dummyWertReturn->next = NULL;
						currentWertReturn = dummyWertReturn;
						dummyExprReturn = newExpression(E_VAL,0xFC,NULL,NULL,NULL);
						currentWertReturn->expression = dummyExprReturn;	// Aktuell erzeugte Expression mit der WertReturn-Struktur verknüpfen
						currentWertReturn->load = 2;
						$$=firstWertReturn;
						}		
	;


/*------------------------------------------------------------------------------------------------*/
expression:	expression ADDOP expression		{$$=newExpression(E_ADDOP,0,NULL,$1,$3);}
		|expression SUBOP expression		{$$=newExpression(E_SUBOP,0,NULL,$1,$3);}
		|expression MULOP expression		{$$=newExpression(E_MULOP,0,NULL,$1,$3);}
		|expression DIVOP expression		{$$=newExpression(E_DIVOP,0,NULL,$1,$3);}
		|expression BITSHIFT_L expression	{$$=newExpression(E_BITSHIFT_L,0,NULL,$1,$3);}
		|expression BITSHIFT_R expression	{$$=newExpression(E_BITSHIFT_R,0,NULL,$1,$3);}
		|expression ANDOP expression		{$$=newExpression(E_ANDOP,0,NULL,$1,$3);}
		|expression OROP expression		{$$=newExpression(E_OROP,0,NULL,$1,$3);}
		|CLAMP_OPEN expression CLAMP_CLOSE	{$$=newExpression(E_CLAMP,0,NULL,$2,NULL);}
		|NOTOP expression			{$$=newExpression(E_NOTOP,0,NULL,$2,NULL);}
		|ZAHL			{$$=newExpression(E_VAL,$1.value,NULL,NULL,NULL);}
		|ZERO			{$$=newExpression(E_VAL,$1.value,NULL,NULL,NULL);}
		|HEX			{$$=newExpression(E_VAL,$1.value,NULL,NULL,NULL);}
		|ALPHAVALUE		{$$=newExpression(E_VAL,$1.value,NULL,NULL,NULL);}
		|LABEL			{$$=newExpression(E_LAB,0,$1.mark,NULL,NULL);}
		;
	
%%

yyerror(char *msg)
{
	Err_syntax(msg,yylineno,yytext);
}

int main(int argc, char **argv)
{


	int option;
	int visible=0;
	char srcfile[128];
	char outfile[128];	
	int inhalt=0xFF;
	int option_index=0;
	int noStart=0;
	FILE* temp; // will be used for the temporary gcc output


	if(argc==1)
	{
		printf("%s\n",HELPTEXT);
		return 1;
	}

	static struct option long_options[]=
	{
		{"nosection", no_argument, 0 ,'n'},
		{"info",no_argument,0,'i'},
		{"lines",no_argument,0,'l'},
		{"labels",no_argument,0,'m'},
		{"comments",no_argument,0,'c'},
		{"verbose",no_argument,0,'v'},
		{"preprocessor",no_argument,0,'p'},
		{"outfile",required_argument,0,'o'},
		{"start",no_argument,0,'s'},
		{"help",no_argument,0,'h'},
		{0,0,0,0}
	};
	//Copy default name for the output file
	strcpy(outfile,DefaultOutFile);
	
	//get options


	while (1)
	{
		option_index=0;
		option = getopt_long(argc, argv, "hvo:nicplms",long_options,&option_index);
		if(option==-1)
		{	break;}

		switch (option)
		{
			//write only the register-section and the code-section into the output-file
			case 'n':	inhalt=0;
					break;
			//write the info section
			case 'i':	if(inhalt!=0)
					{
						if((inhalt&0xFF)==0xFF)
						{	inhalt=0x8;}
						else
						{	inhalt=inhalt|0x8;}
					}
					break;
			//write the comment-section
			case 'c':	if(inhalt!=0)
					{
						if((inhalt&0xFF)==0xFF)
						{	inhalt=0x4;}
						else
						{	inhalt=inhalt|0x4;}
					}
					break;
			//write the line-section
			case 'l':	if(inhalt!=0)
					{
						if((inhalt&0xFF)==0xFF)
						{	inhalt=0x2;}
						else
						{	inhalt=inhalt|0x2;}
					}
					break;
			//write the label-section
			case 'm':	if(inhalt!=0)
					{
						if((inhalt&0xFF)==0xFF)
						{	inhalt=0x1;}
						else
						{	inhalt=inhalt|0x1;}
					}
					break;
			//verbose-mode print out at terminal
			case 'v':	visible=1;
					break;
			//output-file name 
			case 'o':	strcpy(outfile,optarg);
					//check if destination-file end with DESTINATION_FILE_END, if not then ERROR
					if(strcmp(outfile+(strlen(outfile)-strlen(DESTINATION_FILE_END)),DESTINATION_FILE_END)!=0)
					{	Err_DestFile();	}
					break;
			//no warning if no .start and no register section
			case 's':	noStart=1;
					break;
			case 'h':	printf("%s\n",HELPTEXT);
					return 1;
			// preprocessor to define constants with the #define-directive in the sourcefile
			case 'p':	{
					char *gcc=NULL;
					gcc=(char *)malloc(strlen(GCC)+strlen(GCC_OPTIONS));
					char renameCommand[50];
					int i;
					

					strcpy (gcc,GCC);
					strcat (gcc,GCC_OPTIONS); 	// connecting the gcc path with its options

					for (i=0;i<argc;i++)	 	// loop to find the sourcefilename
						if (strcmp(argv[i]+(strlen(argv[i])-strlen(SOURCE_FILE_END)),SOURCE_FILE_END)==0)
						{
							gcc=(char *)realloc(gcc,strlen(GCC)+strlen(GCC_OPTIONS)+strlen(argv[i])+1);
							strcat(gcc,argv[i]);
							break;
						}	
					system(gcc);
					// The gcc compiler doesn't support to use the inputfile as outputfile so we have to copy the file
					strcpy(srcfile,"tmp.asm");
					}
					break;	
			//unknown option
			case '?':	Err_UnknownOption(optopt);
					break;
			default:	Err_Out();
		}
	}
	//look if there is one sourcefile in the programm call
	if((argc-optind)==1)
	{
		//check if sourcefile end with SOURCE_FILE_END, if not then ERROR
		//if(strcmp(argv[optind]+(strlen(argv[optind])-strlen(SOURCE_FILE_END)),SOURCE_FILE_END)!=0)
		//{	Err_SrcFile();}
		// Preprocessing was permfored? 
		if(strcmp(srcfile,"tmp.asm") != 0 )
			strcpy(srcfile,argv[optind]);
		//open assembler-file, if it couldn't open the file then ERROR
		if((yyin=fopen(srcfile,"r"))==NULL)
		{	Err_ReadSrc(argv[optind]);}
	}
	//look if there is no source-file in the programm call then ERROR
	else if(optind==argc)
	{	Err_NoSrc();}
	//if there are more than one sourcefiles in the programm call then ERROR
	else
	{	Err_TooMuchSourceFiles(optind,argc,argv);}


	//call of the Parser
	yyparse();
	
	//check if there was an parse error
	if(syntaxError==1) return 1;
	//check if .start exists if no .start exists then WARNING if too much exist then ERROR
	//if(errorStartCheck(start,noStart)<0) return 1;
	//check if there is a memory-adress used twice then WARNING
	if(errorMemoryCheck(first_code)>0) return 1;
	
	//visible functions
	if(visible==1 || DEBUG==1)
	{
		printMerker(first_merker);	//print labels
		printCode(first_code);		//print section-adresses
		if(DEBUG==1) {printMemory(first_code, memory);}	//memory-dump
		printLines(lineIndex, lines);	//print lines with their memory-adresses
		printExpressions(first_expression);	//print memory-adresses, where an expression is
		printComments(first_comment);	//print comments with line number
	}
	
	//caculate  the expressions and write to memory
	calculateExpressions(first_expression, memory, first_merker);
	
	//Debug functions
	if(visible==1 || DEBUG==1) printMemory(first_code, memory); //memory-dump with calculated expressions
	

	//write all informations into the output-file
	makeObjectDat(first_merker, lineIndex, lines, first_comment, first_code, memory,start,outfile,inhalt);
	
	//say that the Compiler worked without an error
	return 0;
}
