/*! debug.c
 *  In this file the functions are defined which are used to debug the compiler
 *
 *  @author Eggenmueller Bernd , Juergen Schmidt
 *	$Revision: 1.0 $
 *	@date 25.04.2007
 */
#include <stdio.h>
#include "commit.h"

/*! This function prints the list of the merker onto the display
 *
 *	@param MERKERLIST* first pointer to the first Merker in the Merkerlist
 *
 *	@return void
 */
void printMerker(MERKERLIST *first)
{	
	printf("LABELLIST:\n");
	for(;first!=NULL;first=first->next)
	{
		printf("length: %d\tname: %s\tadress: 0x%x\n",first->length,first->name,first->adress);
	}
}

/*! This function prints the list of the code onto the display
 *
 *	@param CODELIST* first first element of the Codelist
 *
 *	@return void
 */
void printCode(CODELIST *first)
{	
	printf("CODELIST:\n");
	for(;first!=NULL;first=first->next)
	{
		printf("start adress: 0x%x\tlength:%d\n",first->startAdress,first->length);
	}
}

/*! This function prints the list of the memory with adresses onto the display
 *
 *	@param CODELIST* first first element of the Codelist
 *	@param short int* memeory list of the memory content
 *
 *	@return void
 */
void printMemory(CODELIST *first,unsigned int *memory)
{
	int i,ind=0;
	printf("MEMORY-ARRAY:\n");
	printf("adress\t:  content\n");
	for(;first!=NULL;first=first->next)
	{
		for(i=0;i<first->length;i++,ind++)	
		{
			printf("0x%x\t:  0x%x\n", first->startAdress+i, memory[ind]);
		}
	}
}

/*! This function prints the lines and the adresses which belongs to the lines onto the display
 *
 *	@param int index number of the lines
*	@param short int* lines and the adresse which belongs to the lines
 *
 *	@return void
 */
void printLines(int index, short int *lines)
{
	int i;
	printf("LINE-ARRAY:\n");
	for(i=0;i<index;i++)
	{
		printf("%d: 0x%x\n",i+1,lines[i]);
	}
}

/*! This function prints the adress where the expression in the memory is onto the display
 *
 *	@param WERT_RETURN* firstExpr list of the expressions which must be calculated
 *
 *	@return void
 */
void printExpressions(WERT_RETURN *firstExpr)
{
	printf("EXPRESSION-LIST:\n");
	WERT_RETURN *help;
	for(help=firstExpr;help!=NULL;help=help->next)
	{
		printf("Memory-Platz: %x\n",help->memory);
	}
}

/*! This function prints the comments and the line where the comment is onto the display
 *
 *	@param COMMENTLIST* first list of the comments
 *
 *	@return void
 */
void printComments(COMMENTLIST *first)
{
	COMMENTLIST *help;
	printf("COMMENTLIST:\n");
	for(help=first;help!=NULL;help=help->next)
	{
		printf("ZEILE: %d COMMENT: %s\n", help->line, help->comment);
	}
}

/*! This function prints the list of the sections onto the display
 *
 *	@param int* secArray array of the whole sections
 *	@param int secAnz number of the sections
 *
 *	@return void
 */
void printSectionTab(int *secArray,int secAnz)
{
	int i;
	printf("SECTIONTAB:\n");
	for(i=0;i<secAnz*2;i=i+2)
	{	switch(secArray[i])
		{	case 0x1:	printf("Typ: Code\tLength: %d\n",secArray[i+1]);
					break;
			case 0x2:	printf("Typ: Label\tLength: %d\n",secArray[i+1]);
					break;
			case 0x3:	printf("Typ: Line\tLength: %d\n",secArray[i+1]);
					break;
			case 0x4:	printf("Typ: Breakpoint\tLength: %d\n",secArray[i+1]);
					break;
			case 0x5:	printf("Typ: Comment\tLength: %d\n",secArray[i+1]);
					break;
			case 0x6:	printf("Typ: Info\tLength: %d\n",secArray[i+1]);
					break;
			case 0x7:	printf("Typ: Register\tLength: %d\n",secArray[i+1]);
					break;
			default:	printf("No such Section\n");
		}
	}	
}


