/*! error.c
 *  In this file are the functions which check things on error
 *
 *  @author Eggenmueller Bernd , Juergen Schmidt
 *	$Revision: 1.0 $
 *	@date 25.04.2007
 */
#include <stdlib.h>
#include "functions.h"
#include "commit.h"
#include "errors_and_warnings.h"

void sortMem(int **mem,int anz);

/*! This function checks if the directive .start is set or not
 *
 *	@param int start the line at which start is declered, if too much the number of start is negativ, if there is no start is 0
 *	@param int noStartWarning says if there should be a warning (=0) if there is no .start or no Warning (!=0)
 *
 *	@return int returns if there is an error 1 or not 0
 */
int errorStartCheck(int start,int noStartWarning)
{
	//check if no .start is set and if there should be a warning
	if(start==0 && noStartWarning==0)
	{	War_NoStart();}
	//check if there are to much .start are set
	else if(start<0)
	{	Err_TooMochStart(start);}
	else
	{	/*one .start is set*/	}
	return start;	
}

/*! This function checks the list of the code if codesections use the same memory adress
 *
 *	@param CODELIST* first pointer to the first code in the Codelist
 *
 *	@return int returns the number of errors
 */
int errorMemoryCheck(CODELIST *first)
{
	int anz,i,error;
	int *mem;
	CODELIST *help=NULL;
	for(help=first,anz=0;help!=NULL;help=help->next,anz++);
	mem=(int*)malloc(sizeof(int)*2*anz);
	for(help=first,anz=0;help!=NULL;help=help->next,anz++)
	{	
		mem[2*anz]=help->startAdress;
		mem[2*anz+1]=mem[2*anz]+help->length-1;
	}
	sortMem(&mem,anz);
	error=0;
	for(i=1;i<anz;i++)
	{
		if(mem[2*i-1]>=mem[2*i])
		{
			War_SameAdresses(mem[2*(i-1)],mem[2*i-1],mem[2*i],mem[2*i+1]);
			error++;
		}
	}
	free(mem);
	return error;
}

/*! This function sorts an array of the memory with the bubble-sort-algorithm adresses that the lowest will be at first
 *
 *	@param int** mem array of the memory spaces
 *	@param int anz number of the memory spaces
 *
 *	@return void
 */
void sortMem(int **mem,int anz)
{
	int i,j,help;
	for(i=anz-1;i>=0;i--)
	{
		for(j=1;j<=i;j++)
		{
			if((*mem)[(j-1)*2]>(*mem)[j*2])
			{
				help=(*mem)[(j-1)*2];
				(*mem)[(j-1)*2]=(*mem)[j*2];
				(*mem)[j*2]=help;
				help=(*mem)[(j-1)*2+1];
				(*mem)[(j-1)*2+1]=(*mem)[j*2+1];
				(*mem)[j*2+1]=help;
			}
		}
	}
}
