
/*! parsefunctions.c
 *  In this file are the functions which 
 *
 *  @author Eggenmueller Bernd , Juergen Schmidt
 *	$Revision: 1.0 $
 *	@date 25.04.2007
 */
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "functions.h"
#include "commit.h"

/*!This functions extracts from a string the three permutationvariables p, s and b.
 *
 *	@param PERMUT *permut Pointer to the permutation structure, which will be converted
*/
void convertToPerm(PERMUTATION* permut)
{
	int buffer = 0; 	// temporärer Puffer
	int i,j;
	//lt. Permutationsbeschreibung: nnnnnnnn
	for (i=15,j=0; i>7; i--,j++)
	{
		//printf("%i\t%c\t\t%i\n", j, permut->permutation[i],buffer); 	// for debug only
		buffer = buffer + ( (int) pow(2,j) * ( (int)permut->permutation[i] - 48 )); // 2 hoch j * mal 0 oder 1, je nach was gerade beim Indizes steht ( ASCII - 48 = realer Zahlenwert)
	}

	permut->p = buffer;
	buffer = 0; // buffer wieder löschen ...
	//lt. Permutationsbeschreibung: ssss
	for (j=0; i>3; i--, j++)
	{
		buffer = buffer + ( (int) pow(2,j) * ( (int)permut->permutation[i] - 48 ));
	}

        permut->s = buffer;
	buffer = 0;
	//lt. Permutationsbeschreibung: ww
	for (j=0; i>1; i--, j++)
	{
		buffer = buffer + ( (int) pow(2,j) * ( (int)permut->permutation[i] - 48 ));
	}

	permut->b = buffer;
	
} 


/*!This functions creates a instructionword from a scalarcommand and a vectorcommand
 *
 *	@param COMMAND_RETURN *cmdRet1 Pointer to the Scalarcommand
 *	@param COMMAND_RETURN *cmdRet2 Pointer to the Vektorcommand

 *
 *	@return EXPRESSION_RETURN* Returns the combined structure of both commands
  */
COMMAND_RETURN uniteCommands(COMMAND_RETURN *cmdRet1, COMMAND_RETURN *cmdRet2)
{
	COMMAND_RETURN dummy;

	dummy.flag = 0; 						// damit wird in des Memoryarray keine expression behandelt
	dummy.wortAnz =1; 						// wird benötigt, um die Breite im memoryArray anzugeben (1 == 1 									   int Variable)
	dummy.line = cmdRet1->line; 					// cmdRet1->line == cmdRet2->line
	dummy.code = (cmdRet1->code +cmdRet2->code) & 0xffffffff;	// Zusammenfuegen des Codes: WICHTIG: Skalareinheit muss cmdRet1 									   sein, Vektoreinheit cmdRet2
	dummy.expr = NULL;		// hier kann keine Expression auftreten, da 32 Bit von der Vektor und Skalareinheit benötigt werden

	return dummy; 
} 

/*! This function creates an part of an expression with the values,operations and labels
 *
 *	@param enum operations operation is the number of the Operation which should calculate
 *	@param int value value of the expression part
 *	@param char *merker label name which is in the expression
 *	@param EXPRESSION_RETURN *left pointer to the left side of an expression
 *	@param EXPRESSION_RETURN *right pointer to the right side of an expression
 *
 *	@return EXPRESSION_RETURN* pointer returns a pointer to the expression part
 */
EXPRESSION_RETURN* newExpression(				enum operations operation,
								int value,
								char *merker,
								EXPRESSION_RETURN *left,
								EXPRESSION_RETURN *right)
{
	EXPRESSION_RETURN *new=NULL;
	new=(EXPRESSION_RETURN*)malloc(sizeof(EXPRESSION_RETURN));
	new->operation=operation;
	new->value=value;
	if(merker!=NULL)  // ok, ein LABEL wurde erkannt, das Label muss in die Adresse aufgelößt werden
	{
		strcpy(new->label,merker);
	}
	new->left=left;
	new->right=right;
	return new;
}

/*! This function save the command which is in cmd5bit, reg1_3bit, reg2_3bit, reg3_3bit, last2bit, into the 16Bit format,
 *   save the line in which the command is in the source file, save the word number in x*16Bit which the command needs in
 *   the memmory and it save if there is an command which has a expression which must be write into the low 8 Bit
 *
 *	@param int line the line in which the command is in the source file
 *	@param int cmd_5bit value which should be written into the Bit15 - Bit11 (xxxxx00000000000)
 *	@param int reg1_3bit value which should be written into the Bit10 - Bit8  (00000xxx00000000)  
 *	@param int reg2_3bit value which should be written into the Bit7 - Bit5  (00000000xxx00000) 
 *	@param int reg3_3bit value which should be written into the Bit4 - Bit2  (00000000000xxx00) 
 *	@param int last2bit  value which should be written into the Bit1 - Bit0  (00000000000000xx) 
 *	@param WERT_RETURN *wert pointer to an list of expressions if there are expressions
 *	@param int wortAnz number of the word which are need in the memory
 *	@param int low8 if there is an expression which must be written into the low 8 Bit at the end of the parsing
 *
 *	@return COMMAND_RETURN returns it all in the format of COMMAND_RETURN
  */
COMMAND_RETURN newCommand_Reg(				  int line,
							  int cmd_5bit,
							  int reg1_3bit, 
							  int reg2_3bit, 
							  int reg3_3bit,
							  int last2bit,
							  WERT_RETURN *wert,
							  int wortAnz,
							  int low8)
{
	
	unsigned int code=0;
	code=((0x1f&cmd_5bit)<<11)+((0x7&reg1_3bit)<<8)+((0x7&reg2_3bit)<<5)+((0x7&reg3_3bit)<<2)+(0x3&last2bit);
	return newCommand(line,code,wert,wortAnz,low8,0,0);
}

/*!	#########1. Scalarcommand without direct value(LD,ST,ALU,JMPs,FLAGs, HALT, NOP)###########
 *	This function save the command which is in cmd6bit(Opcode), reg1_2bit, reg2_2bit, reg3_2bit into the 12Bit format,
 *   	save the line in which the command is in the source file, save the word number in x*16Bit which the command needs in
 *   	the memmory and it save if there is an command which has a expression which must be write into the low 8 Bit
 *
 *	@param int line the line in which the command is in the source file
 *	@param int cmd_6bit value which should be written into the Bit31 - Bit26 (xxxxxx00000000000000000000000000)
 *	@param int reg1_2bit value which should be written into the Bit25 - Bit24 (000000xx000000000000000000000000)
 *	@param int reg2_2bit value which should be written into the Bit23 - Bit22 (00000000xx0000000000000000000000)
 *	@param int reg3_2bit value which should be written into the Bit21 - Bit20 (0000000000xx00000000000000000000)
 *	@param WERT_RETURN *wert pointer to an list of expressions if there are expressions
 *	@param int wortAnz number of the word which are need in the memory
 *	@param int low8 if there is an expression which must be written into the low 8 Bit at the end of the parsing
 *
 *	@return COMMAND_RETURN returns it all in the format of COMMAND_RETURN
  */
COMMAND_RETURN NewCommand_ScalarAndReg(			  int line,
							  int cmd_6bit,
							  int reg1_2bit,
							  int reg2_2bit,
							  int reg3_2bit,
							  WERT_RETURN *wert,
							  int wortAnz,
							  int low8)
{ // hier auch besser mit 3F
	unsigned int code=0;
	code=(((0x3f&cmd_6bit)<<26)+((0x3&reg1_2bit)<<24)+((0x3&reg2_2bit)<<22)+((0x3&reg3_2bit)<<20));
	return newCommand(line,code,wert,wortAnz,low8,1,0);
}

/*!	#########2. Vectorinstruction without direct value(VNOP,VMOV, VMOL/R)#############
 *	This function save the command which is in cmd8bit(Opcode), reg1_4bit, reg2_4bit, reg3_4bit into the 20Bit format,
 *   	save the line in which the command is in the source file, save the word number in x*16Bit which the command needs in
 *   	the memmory and it save if there is an command which has a expression which must be write into the low 8 Bit
 *
 *	@param int line the line in which the command is in the source file
 *	@param int cmd_8bit value which should be written into the Bit12 - Bit19 (0000000000000xxxxxxxx00000000000)
 *	@param int reg1_4bit value which should be written into the Bit8 - Bit11 (00000000000000000000xxxx00000000)
 *	@param int reg2_4bit value which should be written into the Bit4 - Bit7 (000000000000000000000000xxxx0000)
 *	@param int reg3_4bit value which should be written into the Bit0 - Bit3 (0000000000000000000000000000xxxx)
 *	@param WERT_RETURN *wert pointer to an list of expressions if there are expressions
 *	@param int wortAnz number of the word which are need in the memory
 *	@param int low8 if there is an expression which must be written into the low 8 Bit at the end of the parsing
 *
 *	@return COMMAND_RETURN returns it all in the format of COMMAND_RETURN
  */
  
 
COMMAND_RETURN NewCommand_VectorAndReg(			  int line,
							  int cmd_8bit,
							  int reg1_4bit,
							  int reg2_4bit,
							  int reg3_4bit,
							  WERT_RETURN *wert,
							  int wortAnz,
							  int low8)
{
	unsigned int code=0;
	code=(((0xff&cmd_8bit)<<12)+((0xf&reg1_4bit)<<8)+((0xf&reg2_4bit)<<4)+(0xf&reg3_4bit));
	return newCommand(line,code,wert,wortAnz,low8,0,1);
}

/*!	##############3. Scalarcommand with direct value(LD,ST,ALU,JMPs)#############
 *	This function save the command which is in cmd8bit(Opcode), reg1_4bit, reg2_4bit, reg3_4bit into the 20Bit format,
 *   	save the line in which the command is in the source file, save the word number in x*16Bit which the command needs in
 *   	the memmory and it save if there is an command which has a expression which must be write into the low 8 Bit
 *
 *	@param int line the line in which the command is in the source file
 *	@param int cmd_6bit value which should be written into the Bit31 - Bit26 (xxxxxx00000000000000000000000000)
 *	@param int reg1_2bit value which should be written into the Bit25 - Bit24 (000000xx000000000000000000000000)
 *	@param int reg2_2bit value which should be written into the Bit23 - Bit22 (00000000xx0000000000000000000000)
 *	@param int direct_value direct_value which should be written into the Bit15 - Bit0 (0000000000000000xxxxxxxxxxxxxxxx)
 *	@param WERT_RETURN *wert pointer to an list of expressions if there are expressions
 *	@param int wortAnz number of the word which are need in the memory
 *	@param int low8 if there is an expression which must be written into the low 8 Bit at the end of the parsing
 *
 *	@return COMMAND_RETURN returns it all in the format of COMMAND_RETURN
  */
COMMAND_RETURN NewCommand_ScalarRegDirectValue(		  int line,
							  int cmd_6bit, // mit 3F verunden nicht mit FF --> sicherer
							  int reg1_2bit,
							  int reg2_2bit,
							  int direct_value, // Achtung darf nur ein Direktwert sein es hat  gar nichts mit einer Expression zu tun hat. Da später die Bits bei der Expression-Berechnung wieder manipuliert werden
							  WERT_RETURN *wert,
							  int wortAnz,
							  int low8)
{
	unsigned int code=0;
	code=(((0x3f&cmd_6bit)<<26)+((0x3&reg1_2bit)<<24)+((0x3&reg2_2bit)<<22)+((0x00)<<14)+(0xffff&direct_value));

	return newCommand(line,code,wert,wortAnz,low8,1,1); 
}

/*!	##############4. Movecommands (MOV, MOVA, VLD, VST)#############
 *	This function save the command which is in cmd8bit(Opcode), reg1_4bit, reg2_4bit, reg3_4bit into the 20Bit format,
 *   	save the line in which the command is in the source file, save the word number in x*16Bit which the command needs in
 *   	the memmory and it save if there is an command which has a expression which must be write into the low 8 Bit
 *
 *	@param int line the line in which the command is in the source file
 *	@param int scCmd_6bit value which should be written into the Bit31 - Bit26  (xxxxxx00000000000000000000000000)
 *	@param int scReg1_2bit value which should be written into the Bit25 - Bit24 (000000xx000000000000000000000000)
 *	@param int scReg2_2bit value which should be written into the Bit23 - Bit22 (00000000xx0000000000000000000000)
 *	@param int scReg3_2bit value which should be written into the Bit21 - Bit20 (0000000000xx00000000000000000000)
 *	@param int veCMD_8bit  value which should be written into the Bit19 - Bit12 (000000000000xxxxxxxx000000000000)
 *	@param int veReg1_4Bit  value which should be written into the Bit11 - Bit8 (00000000000000000000xxxx00000000)
 *	@param int veReg2_4Bit  value which should be written into the Bit7 - Bit4 (0000000000000000000000000xxxx0000)
 *	@param WERT_RETURN *wert pointer to an list of expressions if there are expressions 
 *	@param int wortAnz number of the word which are need in the memory
 *	@param int low8 if there is an expression which must be written into the low 8 Bit at the end of the parsing
 *
 *	@return COMMAND_RETURN returns it all in the format of COMMAND_RETURN
  */
COMMAND_RETURN NewCommand_MoveCmds(			  int line,
							  int scCmd_6bit,
							  int scReg1_2bit,
							  int scReg2_2bit,
							  int scReg3_2bit,
							  int veCMD_8bit,
							  int veReg1_4Bit,
							  int veReg2_4Bit,
							  WERT_RETURN *wert,
							  int wortAnz,
							  int low8)
{	
	unsigned int code=0;
	code=(((0x3f&scCmd_6bit)<<26)+((0xf&scReg1_2bit)<<24)+((0xf&scReg2_2bit)<<22)+((0xf&scReg3_2bit)<<20)+((0xff&veCMD_8bit)<<12)+((0xf&veReg1_4Bit)<<8)+((0xf&veReg2_4Bit)<<4)+(0x0));
	return newCommand(line,code,wert,wortAnz,low8,1,1); // letzen zwei bit müsste doch 0 0  sein
}

/*!	##############5. Vectorinstructions with direct value (VMOV)#############
 *	This function save the command which is in cmd8bit(Opcode), reg1_4bit, reg2_4bit, reg3_4bit into the 20Bit format,
 *   	save the line in which the command is in the source file, save the word number in x*16Bit which the command needs in
 *   	the memmory and it save if there is an command which has a expression which must be write into the low 8 Bit
 *
 *	@param int line the line in which the command is in the source file
 *	@param int cmd_8bit value which should be written into the Bit19 - Bit12  (000000000000xxxxxxxx000000000000)
 *	@param int reg1_2bit value which should be written into the Bit11 - Bit8 (00000000000000000000xxxx00000000)
 *	@param int reg2_2bit value which should be written into the Bit7 - Bit4 (000000000000000000000000xxxx0000)
 *	@param int direct_value direct_value which should be written into the Bit27 - Bit20 (0000xxxxxxxx00000000000000000000)
 *	@param WERT_RETURN *wert pointer to an list of expressions if there are expressions 
 *	@param int wortAnz number of the word which are need in the memory
 *	@param int low8 if there is an expression which must be written into the low 8 Bit at the end of the parsing
 *
 *	@return COMMAND_RETURN returns it all in the format of COMMAND_RETURN
  */
COMMAND_RETURN NewCommand_VectorRegDirectValue(		  int line,	
							  int cmd_8bit,
							  int regV_4bit,
							  int regR_4bit,
							  int direct_value,
							  WERT_RETURN *wert,
							  int wortAnz,
							  int low8)
{
	unsigned int code=0;
	code=(((0x0)<<28)+((0xff&direct_value)<<20)+((0xff&cmd_8bit)<<12)+((0xf&regV_4bit)<<8)+((0xf&regR_4bit)<<4)+(0x00));
	return newCommand(line,code,wert,wortAnz,low8,1,1); // letzen zwei bit müsste doch 0 0 sein
}

/*!	#########6. Vectoraluinstructions (VALU)#############
 *	This function save the command which is in cmd8bit(Opcode), reg1_4bit, reg2_4bit, reg3_4bit into the 20Bit format,
 *   	save the line in which the command is in the source file, save the word number in x*16Bit which the command needs in
 *   	the memmory and it save if there is an command which has a expression which must be write into the low 8 Bit
 *
 *	@param int line the line in which the command is in the source file
 *	@param int width value which should be written into the Bit17 - Bit16    (-----------01xx----------------)
 *	@param int cmd_4bit value which should be written into the Bit15 - Bit12 (-----------01--xxxx------------)
 *	@param int reg1_4bit value which should be written into the Bit11 - Bit8 (-----------01------xxxx--------)
 *	@param int reg2_4bit value which should be written into the Bit7 - Bit4  (-----------01----------xxxx----)
 *	@param int reg3_4bit value which should be written into the Bit3 - Bit0  (-----------01--------------xxxx)
 *	@param WERT_RETURN *wert pointer to an list of expressions if there are expressions
 *	@param int wortAnz number of the word which are need in the memory
 *	@param int low8 if there is an expression which must be written into the low 8 Bit at the end of the parsing
 *
 *	@return COMMAND_RETURN returns it all in the format of COMMAND_RETURN
  */
COMMAND_RETURN NewCommand_VectorAlu(			                  int line,
							  int width,	
							  int cmdx_4bit,
							  int regR_4bit, 
							  int regV_4bit, 
							  int regW_4bit,
							  WERT_RETURN *wert,
							  int wortAnz,
							  int low8)
{
	unsigned int code=0;
	code=(((0x1)<<18)+((0x3&width)<<16)+((0xff&cmdx_4bit)<<12)+((0xf&regR_4bit)<<8)+((0xf&regV_4bit)<<4)+(0xf&regW_4bit));
	return newCommand(line,code,wert,wortAnz,low8,0,1);
}

/*!	#########7. Shuffleoperation (VSHUF)#############
 *	This function save the command which is in cmd8bit(Opcode), reg1_4bit, reg2_4bit, reg3_4bit into the 20Bit format,
 *   	save the line in which the command is in the source file, save the word number in x*16Bit which the command needs in
 *   	the memmory and it save if there is an command which has a expression which must be write into the low 8 Bit
 *
 *	@param int line the line in which the command is in the source file
 *	@param int width value which should be written into the Bit17 - Bit16    (------------11xx----------------)
 *	@param int regA_4bit value which should be written into the Bit15 - Bit12 (------------11--xxxx------------)
 *	@param int reg1_4bit value which should be written into the Bit11 - Bit8 (------------11------xxxx--------)
 *	@param int reg2_4bit value which should be written into the Bit7 - Bit4  (------------11----------xxxx----)
 *	@param int reg3_4bit value which should be written into the Bit3 - Bit0  (------------11--------------xxxx)
 *	@param int reg3_4bit value which should be written into the Bit27 - Bit20(----xxxxxxxx11------------------)
 *	@param WERT_RETURN *wert pointer to an list of expressions if there are expressions
 *	@param int wortAnz number of the word which are need in the memory
 *	@param int low8 if there is an expression which must be written into the low 8 Bit at the end of the parsing
 *
 *	@return COMMAND_RETURN returns it all in the format of COMMAND_RETURN
  */
COMMAND_RETURN NewCommand_shuffle(			  int line,
							  int width,	
							  int regA_4bit,
							  int reg1_4bit, 
							  int reg2_4bit, 
							  int reg3_4bit,
							  int permutation,
							  WERT_RETURN *wert,
							  int wortAnz,
							  int low8)
{	
	unsigned int code=0;
	code=(((0x0)<<28)+((0xff&permutation)<<20)+((0x3)<<18)+((0x3&width)<<16)+((0xff&regA_4bit)<<12)+((0xf&reg1_4bit)<<8)+((0xf&reg2_4bit)<<4)+(0xf&reg3_4bit));
	return newCommand(line,code,wert,wortAnz,low8,1,1); // letzen zwei bit müsste doch 0 0 sein
}

/*! This function save the command which is in cmd_5bit, reg_3bit and value_8bitinto the 16Bit format, save the line in
 *   which the command is in the source file, save the word number in x*16Bit which the command needs in the memmory
 *   and it save if there is an command which has a expression which must be write into the low 8 Bit
 *
 *	@param int line the line in which the command is in the source file
 *	@param int cmd_5bit value which should be written into the Bit15 - Bit11 (xxxxx00000000000)
 *	@param int reg_3bit value which should be written into the Bit10 - Bit8  (00000xxx00000000)  
 *	@param int value_8bit value which should be written into the Bit7 - Bit0 (00000000xxxxxxxx)
 *	@param WERT_RETURN *wert pointer to an list of expressions if there are expressions
 *	@param int wortAnz number of the word which are need in the memory
 *	@param int low8 if there is an expression which must be written into the low 8 Bit at the end of the parsing
 *
 *	@return COMMAND_RETURN returns it all in the format of COMMAND_RETURN
  */
COMMAND_RETURN newCommand_LDI(int line,
							  int cmd_5bit,
							  int reg_3bit, 
							  int value_8bit,
							  WERT_RETURN *wert,
							  int wortAnz,
							  int low8)
{
	unsigned int code=0;
	code=((0x1f&cmd_5bit)<<11)+((0x7&reg_3bit)<<8)+(0xff&value_8bit);
	return newCommand(line,code,wert,wortAnz,low8,0,0);
}

/*! This function save the command which is in code_32Bit into the 32Bit format, save the line in which the command is in the source file,
 *   save the word number in x*16Bit which the command needs in the memmory and it save if there is an command which
 *   has a expression which must be write into the low 8 Bit
 *
 *	@param int line the line in which the command is in the source file
 *	@param int code_32bit value which should be written into the Bit7 - Bit0 (00000000xxxxxxxx)
 *	@param WERT_RETURN *wert pointer to an list of expressions if there are expressions
 *	@param int wortAnz number of the word which are need in the memory
 *	@param int low8 if there is an expression which must be written into the low 8 Bit at the end of the parsing
 *  	@param int Skalar, shows which unit is used in the instruction word (first twelve bits) 
 *	@param int Vektor, shows which unit is used in the instruction word (first twelve bits)
 *	@return COMMAND_RETURN returns it all in the format of COMMAND_RETURN
 */
COMMAND_RETURN newCommand(int line,
					unsigned int code_32bit,
					WERT_RETURN *wert,
					int wortAnz,
					int low8,
					int Skalar,
					int Vektor	 	)
{
	COMMAND_RETURN new;

	new.wortAnz=wortAnz;
	new.code=(0xffffffff & code_32bit);
	new.flag=low8;
	new.line=line;
	new.expr=wert;
	new.isSkalar=Skalar;
	new.isVektor=Vektor;

	return new;
}
