------------------------------------------------------------------
-- PROJECT:     Der Hicovec Prozessor in Aktion
--
-- ENTITY:	VGA_RAM
--
-- PURPOSE:     8192*32Bit Blockram for VGA
-- 
-- System to
-- Connect,
-- Operate ,
-- Transfer data and
-- Compute with the
-- Hicovec 
--
-- Prof. Gundolf Kiefer
-- Hochschule Augsburg
--
-- TEAM:
--
-- Jakob Golus
-- David Lucinkiewicz
-- Andreas Mueller
-- Florian Richter	
-- Sebastian Rigorth
-- Soeren Ruehm
--
-- 
-- DATE CREATE: 2008-12-19
-- LAST CHANGE: 2010-02-24 FROM: Andreas Mueller
-- VERSION:     0.1
-----------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;
use work.cfg.all;

entity vga_ram is
	generic (width: integer := 13);
	port (
		clk: std_logic; -- normal
		CLK_50M: std_logic;
		adr: in std_logic_vector (31 downto 0);
		data: inout std_logic_vector (31 downto 0);
		adressed: out std_logic;
		rd, wr: in std_logic;
		ready: out std_logic;
		vga_out: out std_logic_vector (31 downto 0);
		vga_adr: in std_logic_vector (31 downto 0);
		ascii_wr_in: in std_logic;
		ascii_data_in: in std_logic_vector (31 downto 0)
	);
end vga_ram;


architecture syn of vga_ram is
	constant words: integer := (2 ** width);
	type ram_type is array (0 to words-1) of std_logic_vector (31 downto 0);
	signal RAM : ram_type := (
	  16#0000# => X"F7000000",
  16#0001# => X"FFFFFFFF",
  16#0002# => X"FFFFFFFF",
  16#0003# => X"FFFFFFFF",
  16#0004# => X"FFFFFFFF",
  16#0005# => X"FF878FFF",
  16#0006# => X"FFFFFFFF",
  16#0007# => X"FFFFFFFF",
  16#0008# => X"FFFFFFFF",
  16#0009# => X"7FFFFFFF",
  16#000a# => X"77777777",
  16#000b# => X"77777777",
  16#000c# => X"77777777",
  16#000d# => X"77777777",
  16#000e# => X"87777777",
  16#000f# => X"FFFFFFFF",
  16#0010# => X"FFFFFFFF",
  16#0011# => X"FFFFFFFF",
  16#0012# => X"FFFFFFFF",
  16#0013# => X"00000078",
  16#0014# => X"77777777",
  16#0015# => X"FFFFFF77",
  16#0016# => X"FFFFFFFF",
  16#0017# => X"FFFFFFFF",
  16#0018# => X"FFFFFFFF",
  16#0019# => X"FFFFFFFF",
  16#001a# => X"FFFFFFFF",
  16#001b# => X"00000708",
  16#001c# => X"77777000",
  16#001d# => X"88888877",
  16#001e# => X"88888888",
  16#001f# => X"FFF88888",
  16#0020# => X"FFFFFFFF",
  16#0021# => X"888FFFFF",
  16#0022# => X"78888877",
  16#0023# => X"87777777",
  16#0024# => X"FFFFFFFF",
  16#0025# => X"FFFFFFFF",
  16#0026# => X"FFFFFFFF",
  16#0027# => X"00FFFFFF",
  16#0028# => X"80000000",
  16#0029# => X"FFFFFFFF",
  16#002a# => X"FFFFFFFF",
  16#002b# => X"FFFFFFFF",
  16#002c# => X"FFFFFFFF",
  16#002d# => X"8878FFFF",
  16#002e# => X"FFFFFFF8",
  16#002f# => X"FFFFFFFF",
  16#0030# => X"FFFFFFFF",
  16#0031# => X"8FFFFFFF",
  16#0032# => X"77777777",
  16#0033# => X"77777777",
  16#0034# => X"77777777",
  16#0035# => X"77777777",
  16#0036# => X"77777777",
  16#0037# => X"FFFFFFFF",
  16#0038# => X"FFFFFFFF",
  16#0039# => X"FFFFFFFF",
  16#003a# => X"788FFFFF",
  16#003b# => X"00000000",
  16#003c# => X"77777700",
  16#003d# => X"FFFFF877",
  16#003e# => X"FFFFFFFF",
  16#003f# => X"FFFFFFFF",
  16#0040# => X"FFFFFFFF",
  16#0041# => X"FFFFFFFF",
  16#0042# => X"FFFFFFFF",
  16#0043# => X"00007778",
  16#0044# => X"77770000",
  16#0045# => X"88888777",
  16#0046# => X"88888888",
  16#0047# => X"FFF88888",
  16#0048# => X"FFFFFFFF",
  16#0049# => X"FFFFFFFF",
  16#004a# => X"78887788",
  16#004b# => X"77777777",
  16#004c# => X"FFFFFFFF",
  16#004d# => X"FFFFFFFF",
  16#004e# => X"FFFFFFFF",
  16#004f# => X"00FFFFFF",
  16#0050# => X"80000000",
  16#0051# => X"FFFFFFFF",
  16#0052# => X"FFFFFFFF",
  16#0053# => X"FFFFFFFF",
  16#0054# => X"FFFFFFFF",
  16#0055# => X"8878FFFF",
  16#0056# => X"FFFFF887",
  16#0057# => X"FFFFFFFF",
  16#0058# => X"FFFFFFFF",
  16#0059# => X"FFFFFFFF",
  16#005a# => X"77777777",
  16#005b# => X"77777777",
  16#005c# => X"77777777",
  16#005d# => X"77777777",
  16#005e# => X"77777777",
  16#005f# => X"FFFFFFF8",
  16#0060# => X"FFFFFFFF",
  16#0061# => X"FFFFFFFF",
  16#0062# => X"00078FFF",
  16#0063# => X"00000000",
  16#0064# => X"77770000",
  16#0065# => X"FFFF8777",
  16#0066# => X"FFFFFFFF",
  16#0067# => X"FFFFFFFF",
  16#0068# => X"FFFFFFFF",
  16#0069# => X"FFFFFFFF",
  16#006a# => X"FFFFFFFF",
  16#006b# => X"0007777F",
  16#006c# => X"77770000",
  16#006d# => X"88887777",
  16#006e# => X"88888888",
  16#006f# => X"FF888888",
  16#0070# => X"FFFFFFFF",
  16#0071# => X"FFFFFFFF",
  16#0072# => X"877888FF",
  16#0073# => X"77777778",
  16#0074# => X"FFFFFFF8",
  16#0075# => X"FFFFFFFF",
  16#0076# => X"FFFFFFFF",
  16#0077# => X"00FFFFFF",
  16#0078# => X"70000000",
  16#0079# => X"FFFFFFFF",
  16#007a# => X"FFFFFFFF",
  16#007b# => X"FFFFFFFF",
  16#007c# => X"FFFFFFFF",
  16#007d# => X"888FFFFF",
  16#007e# => X"FF887788",
  16#007f# => X"FFFFFFFF",
  16#0080# => X"FFFFFFFF",
  16#0081# => X"FFFFFFFF",
  16#0082# => X"77777778",
  16#0083# => X"77777777",
  16#0084# => X"77777777",
  16#0085# => X"77777777",
  16#0086# => X"77777777",
  16#0087# => X"FFFFFFF7",
  16#0088# => X"FFFFFFFF",
  16#0089# => X"FFFFFFFF",
  16#008a# => X"00000078",
  16#008b# => X"00000000",
  16#008c# => X"70000000",
  16#008d# => X"FFFF7777",
  16#008e# => X"FFFFFFFF",
  16#008f# => X"FFFFFFFF",
  16#0090# => X"FFFFFFFF",
  16#0091# => X"FFFFFFFF",
  16#0092# => X"FFFFFFFF",
  16#0093# => X"0077778F",
  16#0094# => X"77700000",
  16#0095# => X"88877777",
  16#0096# => X"88888888",
  16#0097# => X"FF888888",
  16#0098# => X"FFFFFFFF",
  16#0099# => X"FFFFFFFF",
  16#009a# => X"888FFFFF",
  16#009b# => X"77777877",
  16#009c# => X"FFFFFFF8",
  16#009d# => X"FFFFFFFF",
  16#009e# => X"FFFFFFFF",
  16#009f# => X"00FFFFFF",
  16#00a0# => X"00000000",
  16#00a1# => X"FFFFFFF8",
  16#00a2# => X"FFFFFFFF",
  16#00a3# => X"FFFFFFFF",
  16#00a4# => X"FFFFFFFF",
  16#00a5# => X"888FFFFF",
  16#00a6# => X"88788888",
  16#00a7# => X"FFFFFFF8",
  16#00a8# => X"FFFFFFFF",
  16#00a9# => X"FFFFFFFF",
  16#00aa# => X"7777777F",
  16#00ab# => X"77777777",
  16#00ac# => X"77777777",
  16#00ad# => X"77777777",
  16#00ae# => X"77777777",
  16#00af# => X"FFFFFF87",
  16#00b0# => X"FFFFFFFF",
  16#00b1# => X"788FFFFF",
  16#00b2# => X"00000000",
  16#00b3# => X"00000000",
  16#00b4# => X"00000000",
  16#00b5# => X"FFF80707",
  16#00b6# => X"FFFFFFFF",
  16#00b7# => X"FFFFFFFF",
  16#00b8# => X"FFFFFFFF",
  16#00b9# => X"FFFFFFFF",
  16#00ba# => X"FFFFFFFF",
  16#00bb# => X"077777FF",
  16#00bc# => X"77000000",
  16#00bd# => X"88877777",
  16#00be# => X"88888888",
  16#00bf# => X"F8888888",
  16#00c0# => X"FFFFFFFF",
  16#00c1# => X"FFFFFFFF",
  16#00c2# => X"FFFFFFFF",
  16#00c3# => X"77777788",
  16#00c4# => X"FFFFFF87",
  16#00c5# => X"FFFFFFFF",
  16#00c6# => X"FFFFFFFF",
  16#00c7# => X"00FFFFFF",
  16#00c8# => X"00000000",
  16#00c9# => X"FFFFFFF7",
  16#00ca# => X"FFFFFFFF",
  16#00cb# => X"FFFFFFFF",
  16#00cc# => X"FFFFFFFF",
  16#00cd# => X"88FFFFFF",
  16#00ce# => X"78888888",
  16#00cf# => X"FFFF8887",
  16#00d0# => X"FFFFFFFF",
  16#00d1# => X"FFFFFFFF",
  16#00d2# => X"7777778F",
  16#00d3# => X"77777777",
  16#00d4# => X"77777777",
  16#00d5# => X"77777777",
  16#00d6# => X"77777777",
  16#00d7# => X"FFFFF877",
  16#00d8# => X"FFFFFFFF",
  16#00d9# => X"00078FFF",
  16#00da# => X"00000000",
  16#00db# => X"00000000",
  16#00dc# => X"00000000",
  16#00dd# => X"FFF70000",
  16#00de# => X"FFFFFFFF",
  16#00df# => X"FFFFFFFF",
  16#00e0# => X"FFFFFFFF",
  16#00e1# => X"FFFFFFFF",
  16#00e2# => X"FFFFFFFF",
  16#00e3# => X"777778FF",
  16#00e4# => X"70000000",
  16#00e5# => X"88777777",
  16#00e6# => X"88888888",
  16#00e7# => X"F8888888",
  16#00e8# => X"FFFFFFFF",
  16#00e9# => X"FFFFFFFF",
  16#00ea# => X"FFFFFFFF",
  16#00eb# => X"77788FFF",
  16#00ec# => X"FFFFFF87",
  16#00ed# => X"FFFFFFFF",
  16#00ee# => X"FFFFFFFF",
  16#00ef# => X"00FFFFFF",
  16#00f0# => X"00000000",
  16#00f1# => X"FFFFFF80",
  16#00f2# => X"FFFFFFFF",
  16#00f3# => X"FFFFFFFF",
  16#00f4# => X"FFFFFFFF",
  16#00f5# => X"88FFFFFF",
  16#00f6# => X"88888888",
  16#00f7# => X"F8887778",
  16#00f8# => X"FFFFFFFF",
  16#00f9# => X"FFFFFFFF",
  16#00fa# => X"777777FF",
  16#00fb# => X"77777777",
  16#00fc# => X"77777777",
  16#00fd# => X"77777777",
  16#00fe# => X"77777777",
  16#00ff# => X"FFFFF777",
  16#0100# => X"FFFFFFFF",
  16#0101# => X"00000078",
  16#0102# => X"00000000",
  16#0103# => X"00000000",
  16#0104# => X"00000000",
  16#0105# => X"FF800000",
  16#0106# => X"FFFFFFFF",
  16#0107# => X"FFFFFFFF",
  16#0108# => X"FFFFFFFF",
  16#0109# => X"FFFFFFFF",
  16#010a# => X"FFFFFFFF",
  16#010b# => X"77777FFF",
  16#010c# => X"00000007",
  16#010d# => X"87777777",
  16#010e# => X"88888888",
  16#010f# => X"F8888888",
  16#0110# => X"FFFFFFFF",
  16#0111# => X"FFFFFFFF",
  16#0112# => X"FFFFFFFF",
  16#0113# => X"88FFFFFF",
  16#0114# => X"FFFFF877",
  16#0115# => X"FFFFFFFF",
  16#0116# => X"FFFFFFFF",
  16#0117# => X"00FFFFFF",
  16#0118# => X"00000000",
  16#0119# => X"FFFFFF70",
  16#011a# => X"FFFFFFFF",
  16#011b# => X"FFFFFFFF",
  16#011c# => X"FFFFFFFF",
  16#011d# => X"78FFFFFF",
  16#011e# => X"88888888",
  16#011f# => X"87778888",
  16#0120# => X"FFFFFF88",
  16#0121# => X"FFFFFFFF",
  16#0122# => X"777778FF",
  16#0123# => X"77777777",
  16#0124# => X"77777777",
  16#0125# => X"77777777",
  16#0126# => X"77777777",
  16#0127# => X"FFFF8777",
  16#0128# => X"788FFFFF",
  16#0129# => X"00000000",
  16#012a# => X"00000000",
  16#012b# => X"00000000",
  16#012c# => X"00000000",
  16#012d# => X"FF700000",
  16#012e# => X"FFFFFFFF",
  16#012f# => X"FFFFFFFF",
  16#0130# => X"FFFFFFFF",
  16#0131# => X"FFFFFFFF",
  16#0132# => X"FFFFFFFF",
  16#0133# => X"77778FFF",
  16#0134# => X"00000077",
  16#0135# => X"77777770",
  16#0136# => X"88888888",
  16#0137# => X"88888888",
  16#0138# => X"FFFFFFFF",
  16#0139# => X"FFFFFFFF",
  16#013a# => X"FFFFFFFF",
  16#013b# => X"FFFFFFFF",
  16#013c# => X"FFFFF88F",
  16#013d# => X"FFFFFFFF",
  16#013e# => X"FFFFFFFF",
  16#013f# => X"00FFFFFF",
  16#0140# => X"00000000",
  16#0141# => X"FFFFFF70",
  16#0142# => X"FFFFFFFF",
  16#0143# => X"FFFFFFFF",
  16#0144# => X"FFFFFFFF",
  16#0145# => X"8FFFFFFF",
  16#0146# => X"88888888",
  16#0147# => X"77778888",
  16#0148# => X"FFF88877",
  16#0149# => X"FFFFFFFF",
  16#014a# => X"77777FFF",
  16#014b# => X"77777777",
  16#014c# => X"77777777",
  16#014d# => X"77777777",
  16#014e# => X"77777777",
  16#014f# => X"FFF87777",
  16#0150# => X"000788FF",
  16#0151# => X"00000000",
  16#0152# => X"00000000",
  16#0153# => X"00000000",
  16#0154# => X"00000000",
  16#0155# => X"F8000000",
  16#0156# => X"FFFFFFFF",
  16#0157# => X"FFFFFFFF",
  16#0158# => X"FFFFFFFF",
  16#0159# => X"FFFFFFFF",
  16#015a# => X"FFFFFFFF",
  16#015b# => X"77778FFF",
  16#015c# => X"00000077",
  16#015d# => X"77777770",
  16#015e# => X"88888887",
  16#015f# => X"88888888",
  16#0160# => X"FFFFFFFF",
  16#0161# => X"FFFFFFFF",
  16#0162# => X"FFFFFFFF",
  16#0163# => X"FFFFFFFF",
  16#0164# => X"FFFFFFFF",
  16#0165# => X"FFFFFFFF",
  16#0166# => X"FFFFFFFF",
  16#0167# => X"00FFFFFF",
  16#0168# => X"00000000",
  16#0169# => X"FFFFF800",
  16#016a# => X"FFFFFFFF",
  16#016b# => X"FFFFFFFF",
  16#016c# => X"FFFFFFFF",
  16#016d# => X"8FFFFFFF",
  16#016e# => X"88888887",
  16#016f# => X"77788888",
  16#0170# => X"88777777",
  16#0171# => X"FFFFFFFF",
  16#0172# => X"77778FFF",
  16#0173# => X"77777777",
  16#0174# => X"77777777",
  16#0175# => X"77777777",
  16#0176# => X"77777777",
  16#0177# => X"8FFF8777",
  16#0178# => X"77777077",
  16#0179# => X"00000007",
  16#017a# => X"00000000",
  16#017b# => X"00000000",
  16#017c# => X"00000000",
  16#017d# => X"F0000000",
  16#017e# => X"FFFFFFFF",
  16#017f# => X"FFFFFFFF",
  16#0180# => X"FFFFFFFF",
  16#0181# => X"FFFFFFFF",
  16#0182# => X"FFFFFFFF",
  16#0183# => X"7777FFFF",
  16#0184# => X"00000777",
  16#0185# => X"77777700",
  16#0186# => X"88888887",
  16#0187# => X"88888888",
  16#0188# => X"FFFFFFFF",
  16#0189# => X"FFFFFFFF",
  16#018a# => X"FFFFFFFF",
  16#018b# => X"FFFFFFFF",
  16#018c# => X"FF8FFFFF",
  16#018d# => X"FF88FFFF",
  16#018e# => X"FFFFFFFF",
  16#018f# => X"00FFFFFF",
  16#0190# => X"00000000",
  16#0191# => X"FFFFF700",
  16#0192# => X"FFFFFFFF",
  16#0193# => X"FFFFFFFF",
  16#0194# => X"FFFFFFFF",
  16#0195# => X"FFFFFFFF",
  16#0196# => X"88888888",
  16#0197# => X"78888888",
  16#0198# => X"77777777",
  16#0199# => X"FFFF8887",
  16#019a# => X"7777FFFF",
  16#019b# => X"77777777",
  16#019c# => X"77777777",
  16#019d# => X"77777777",
  16#019e# => X"88777777",
  16#019f# => X"08FFFFF8",
  16#01a0# => X"77777777",
  16#01a1# => X"00000000",
  16#01a2# => X"00000000",
  16#01a3# => X"00000000",
  16#01a4# => X"00000000",
  16#01a5# => X"80000000",
  16#01a6# => X"FFFFFFFF",
  16#01a7# => X"FFFFFFFF",
  16#01a8# => X"FFFFFFFF",
  16#01a9# => X"FFFFFFFF",
  16#01aa# => X"FFFFFFFF",
  16#01ab# => X"7778FFFF",
  16#01ac# => X"00007777",
  16#01ad# => X"77777000",
  16#01ae# => X"88888877",
  16#01af# => X"88888888",
  16#01b0# => X"FFFFFFF8",
  16#01b1# => X"FFFFFFFF",
  16#01b2# => X"FFFFFFFF",
  16#01b3# => X"FFFFFFFF",
  16#01b4# => X"877FFFFF",
  16#01b5# => X"877777F8",
  16#01b6# => X"FFFFF888",
  16#01b7# => X"00FFFFFF",
  16#01b8# => X"00000000",
  16#01b9# => X"FFFF8000",
  16#01ba# => X"FFFFFFFF",
  16#01bb# => X"FFFFFFFF",
  16#01bc# => X"FFFFFFFF",
  16#01bd# => X"FFFFFFFF",
  16#01be# => X"88888888",
  16#01bf# => X"78888888",
  16#01c0# => X"77777777",
  16#01c1# => X"88877777",
  16#01c2# => X"8888FFFF",
  16#01c3# => X"77777777",
  16#01c4# => X"77777777",
  16#01c5# => X"77777777",
  16#01c6# => X"FFFF8888",
  16#01c7# => X"7FFFFFFF",
  16#01c8# => X"77777777",
  16#01c9# => X"00000000",
  16#01ca# => X"00000000",
  16#01cb# => X"00000000",
  16#01cc# => X"00000000",
  16#01cd# => X"00000000",
  16#01ce# => X"FFFFFFFF",
  16#01cf# => X"FFFFFFFF",
  16#01d0# => X"FFFFFFFF",
  16#01d1# => X"FFFFFFFF",
  16#01d2# => X"FFFFFFFF",
  16#01d3# => X"777FFFFF",
  16#01d4# => X"00077777",
  16#01d5# => X"77770000",
  16#01d6# => X"88888777",
  16#01d7# => X"88888888",
  16#01d8# => X"FFFFFFF8",
  16#01d9# => X"FFFFFFFF",
  16#01da# => X"FFFFFFFF",
  16#01db# => X"FFFFFFFF",
  16#01dc# => X"778FFFFF",
  16#01dd# => X"77777777",
  16#01de# => X"88888877",
  16#01df# => X"00FFF888",
  16#01e0# => X"00000000",
  16#01e1# => X"FFFF8000",
  16#01e2# => X"FFFFFFFF",
  16#01e3# => X"FFFFFFFF",
  16#01e4# => X"FFFFFFFF",
  16#01e5# => X"FFFFFFFF",
  16#01e6# => X"88888878",
  16#01e7# => X"88888888",
  16#01e8# => X"77777788",
  16#01e9# => X"77777777",
  16#01ea# => X"FFFF8877",
  16#01eb# => X"88888FFF",
  16#01ec# => X"88888888",
  16#01ed# => X"FFF88888",
  16#01ee# => X"FFFFFFFF",
  16#01ef# => X"8FFFFFFF",
  16#01f0# => X"77777777",
  16#01f1# => X"00000000",
  16#01f2# => X"00000000",
  16#01f3# => X"00000000",
  16#01f4# => X"00000000",
  16#01f5# => X"00000000",
  16#01f6# => X"FFFFFFF7",
  16#01f7# => X"FFFFFFFF",
  16#01f8# => X"FFFFFFFF",
  16#01f9# => X"FFFFFFFF",
  16#01fa# => X"FFFFFFFF",
  16#01fb# => X"778FFFFF",
  16#01fc# => X"00777777",
  16#01fd# => X"77770000",
  16#01fe# => X"88887777",
  16#01ff# => X"88888888",
  16#0200# => X"FFFFFF88",
  16#0201# => X"FFFFFFFF",
  16#0202# => X"FFFFFFFF",
  16#0203# => X"FFFFFFFF",
  16#0204# => X"7FFFFFFF",
  16#0205# => X"77777777",
  16#0206# => X"88888887",
  16#0207# => X"00888888",
  16#0208# => X"00000000",
  16#0209# => X"FFFF7000",
  16#020a# => X"FFFFFFFF",
  16#020b# => X"FFFFFFFF",
  16#020c# => X"FFFFFFFF",
  16#020d# => X"FFFFFFFF",
  16#020e# => X"8888888F",
  16#020f# => X"88888888",
  16#0210# => X"77777777",
  16#0211# => X"77777777",
  16#0212# => X"FFFF7777",
  16#0213# => X"FFFFFFFF",
  16#0214# => X"FFFFFFFF",
  16#0215# => X"FFFFFFFF",
  16#0216# => X"FFFFFFFF",
  16#0217# => X"FFFFFFFF",
  16#0218# => X"77077708",
  16#0219# => X"00000000",
  16#021a# => X"00000000",
  16#021b# => X"00000000",
  16#021c# => X"00000000",
  16#021d# => X"00000000",
  16#021e# => X"FFFFFF80",
  16#021f# => X"FFFFFFFF",
  16#0220# => X"FFFFFFFF",
  16#0221# => X"FFFFFFFF",
  16#0222# => X"FFFFFFFF",
  16#0223# => X"778FFFFF",
  16#0224# => X"07777777",
  16#0225# => X"77700000",
  16#0226# => X"88877777",
  16#0227# => X"88888888",
  16#0228# => X"FFFFFF88",
  16#0229# => X"FFFFFFFF",
  16#022a# => X"FFFFFFFF",
  16#022b# => X"FFFFFFFF",
  16#022c# => X"8FFFFFFF",
  16#022d# => X"77777777",
  16#022e# => X"88888877",
  16#022f# => X"00888888",
  16#0230# => X"00000000",
  16#0231# => X"FFF80000",
  16#0232# => X"FFFFFFFF",
  16#0233# => X"FFFFFFFF",
  16#0234# => X"FFFFFFFF",
  16#0235# => X"FFFFFFFF",
  16#0236# => X"8888878F",
  16#0237# => X"88888888",
  16#0238# => X"77777888",
  16#0239# => X"77777777",
  16#023a# => X"FFF87777",
  16#023b# => X"FFFFFFFF",
  16#023c# => X"FFFFFFFF",
  16#023d# => X"FFFFFFFF",
  16#023e# => X"FFFFFFFF",
  16#023f# => X"FFFFFFFF",
  16#0240# => X"0070777F",
  16#0241# => X"00000000",
  16#0242# => X"00000000",
  16#0243# => X"00000000",
  16#0244# => X"00000000",
  16#0245# => X"00000000",
  16#0246# => X"FFFFFF80",
  16#0247# => X"FFFFFFFF",
  16#0248# => X"FFFFFFFF",
  16#0249# => X"FFFFFFFF",
  16#024a# => X"FFFFFFFF",
  16#024b# => X"77FFFFFF",
  16#024c# => X"77777777",
  16#024d# => X"77000000",
  16#024e# => X"88877777",
  16#024f# => X"88888888",
  16#0250# => X"FFFFFF88",
  16#0251# => X"FFFFFFFF",
  16#0252# => X"FFFFFFFF",
  16#0253# => X"FFFFFFFF",
  16#0254# => X"8FFFFFFF",
  16#0255# => X"77777777",
  16#0256# => X"88887777",
  16#0257# => X"00888888",
  16#0258# => X"00000000",
  16#0259# => X"FFF70000",
  16#025a# => X"FFFFFFFF",
  16#025b# => X"FFFFFFFF",
  16#025c# => X"FFFFFFFF",
  16#025d# => X"FFFFFFFF",
  16#025e# => X"888888FF",
  16#025f# => X"88888888",
  16#0260# => X"77778888",
  16#0261# => X"77777777",
  16#0262# => X"FFF77777",
  16#0263# => X"FFFFFFFF",
  16#0264# => X"FFFFFFFF",
  16#0265# => X"FFFFFFFF",
  16#0266# => X"FFFFFFFF",
  16#0267# => X"FFFFFFFF",
  16#0268# => X"0000708F",
  16#0269# => X"00000000",
  16#026a# => X"00000000",
  16#026b# => X"00000000",
  16#026c# => X"00000000",
  16#026d# => X"00000000",
  16#026e# => X"FFFFFF00",
  16#026f# => X"FFFFFFFF",
  16#0270# => X"FFFFFFFF",
  16#0271# => X"FFFFFFFF",
  16#0272# => X"FFFFFFFF",
  16#0273# => X"78FFFFFF",
  16#0274# => X"77777777",
  16#0275# => X"70000007",
  16#0276# => X"88777777",
  16#0277# => X"88888888",
  16#0278# => X"FFFFF888",
  16#0279# => X"FFFFFFFF",
  16#027a# => X"FFFFFFFF",
  16#027b# => X"FFFFFFFF",
  16#027c# => X"FFFFFFFF",
  16#027d# => X"77777778",
  16#027e# => X"88887777",
  16#027f# => X"00888888",
  16#0280# => X"00000000",
  16#0281# => X"FFF00000",
  16#0282# => X"FFFFFFFF",
  16#0283# => X"FFFFFFFF",
  16#0284# => X"FFFFFFFF",
  16#0285# => X"FFFFFFFF",
  16#0286# => X"888888FF",
  16#0287# => X"88888888",
  16#0288# => X"77788888",
  16#0289# => X"77777777",
  16#028a# => X"FF877777",
  16#028b# => X"FFFFFFFF",
  16#028c# => X"FFFFFFFF",
  16#028d# => X"FFFFFFFF",
  16#028e# => X"FFFFFFFF",
  16#028f# => X"FFFFFFFF",
  16#0290# => X"000007FF",
  16#0291# => X"00000000",
  16#0292# => X"00000000",
  16#0293# => X"00000000",
  16#0294# => X"00000000",
  16#0295# => X"00000000",
  16#0296# => X"FFFFF800",
  16#0297# => X"FFFFFFFF",
  16#0298# => X"FFFFFFFF",
  16#0299# => X"FFFFFFFF",
  16#029a# => X"FFFFFFFF",
  16#029b# => X"7FFFFFFF",
  16#029c# => X"77777777",
  16#029d# => X"00000007",
  16#029e# => X"87777777",
  16#029f# => X"88888888",
  16#02a0# => X"FFFFF888",
  16#02a1# => X"FFFFFFFF",
  16#02a2# => X"FFFFFFFF",
  16#02a3# => X"FFFFFFFF",
  16#02a4# => X"FFFFFFFF",
  16#02a5# => X"77777778",
  16#02a6# => X"88777777",
  16#02a7# => X"00888888",
  16#02a8# => X"00000000",
  16#02a9# => X"FF800000",
  16#02aa# => X"66666666",
  16#02ab# => X"FFFFF666",
  16#02ac# => X"FFFFFFFF",
  16#02ad# => X"FFFFFFFF",
  16#02ae# => X"666668FF",
  16#02af# => X"88866666",
  16#02b0# => X"77888888",
  16#02b1# => X"77777777",
  16#02b2# => X"66777777",
  16#02b3# => X"66666666",
  16#02b4# => X"FFFFFFFF",
  16#02b5# => X"FFFFFFFF",
  16#02b6# => X"FFFFFFFF",
  16#02b7# => X"FFFFFFFF",
  16#02b8# => X"000008FF",
  16#02b9# => X"00000000",
  16#02ba# => X"60000000",
  16#02bb# => X"66666666",
  16#02bc# => X"00000006",
  16#02bd# => X"00000000",
  16#02be# => X"FFFFF000",
  16#02bf# => X"FFFFFFFF",
  16#02c0# => X"FFFFFFFF",
  16#02c1# => X"FFFFFFFF",
  16#02c2# => X"FFFFFFFF",
  16#02c3# => X"8FFFFFFF",
  16#02c4# => X"77777777",
  16#02c5# => X"00000077",
  16#02c6# => X"77777777",
  16#02c7# => X"88888888",
  16#02c8# => X"FFFFF888",
  16#02c9# => X"FFFFFFFF",
  16#02ca# => X"FFFFFFFF",
  16#02cb# => X"FFFFFFFF",
  16#02cc# => X"FFFFFFFF",
  16#02cd# => X"7777777F",
  16#02ce# => X"87777777",
  16#02cf# => X"00888888",
  16#02d0# => X"00000000",
  16#02d1# => X"66600000",
  16#02d2# => X"66666666",
  16#02d3# => X"FF666666",
  16#02d4# => X"FFFFFFFF",
  16#02d5# => X"FFFFFFFF",
  16#02d6# => X"66666666",
  16#02d7# => X"66666666",
  16#02d8# => X"88888888",
  16#02d9# => X"77777777",
  16#02da# => X"66666777",
  16#02db# => X"66666666",
  16#02dc# => X"FFFFF666",
  16#02dd# => X"666FFFFF",
  16#02de# => X"66666666",
  16#02df# => X"66666666",
  16#02e0# => X"66666666",
  16#02e1# => X"00000066",
  16#02e2# => X"66660000",
  16#02e3# => X"66666666",
  16#02e4# => X"00006666",
  16#02e5# => X"00000000",
  16#02e6# => X"66666666",
  16#02e7# => X"FFF66666",
  16#02e8# => X"666666FF",
  16#02e9# => X"F6666666",
  16#02ea# => X"FFFFFFFF",
  16#02eb# => X"8FFFFFFF",
  16#02ec# => X"77777777",
  16#02ed# => X"00000777",
  16#02ee# => X"77777770",
  16#02ef# => X"88888888",
  16#02f0# => X"FFFF8888",
  16#02f1# => X"FFFFFFFF",
  16#02f2# => X"FFFFFFFF",
  16#02f3# => X"FFFFFFFF",
  16#02f4# => X"FFFFFFFF",
  16#02f5# => X"7777778F",
  16#02f6# => X"77777777",
  16#02f7# => X"00888888",
  16#02f8# => X"00000000",
  16#02f9# => X"66666000",
  16#02fa# => X"66666666",
  16#02fb# => X"66666666",
  16#02fc# => X"FFFFFFFF",
  16#02fd# => X"6FFFFFFF",
  16#02fe# => X"66666666",
  16#02ff# => X"66666666",
  16#0300# => X"88888886",
  16#0301# => X"77777778",
  16#0302# => X"66666677",
  16#0303# => X"66666666",
  16#0304# => X"FFFF6666",
  16#0305# => X"666FFFFF",
  16#0306# => X"66666666",
  16#0307# => X"66666666",
  16#0308# => X"66666666",
  16#0309# => X"00000066",
  16#030a# => X"66666000",
  16#030b# => X"66666666",
  16#030c# => X"00066666",
  16#030d# => X"00000000",
  16#030e# => X"66666666",
  16#030f# => X"FFF66666",
  16#0310# => X"666666FF",
  16#0311# => X"F6666666",
  16#0312# => X"FFFFFFFF",
  16#0313# => X"FFFFFFFF",
  16#0314# => X"77777777",
  16#0315# => X"00007777",
  16#0316# => X"77777700",
  16#0317# => X"88888887",
  16#0318# => X"FFFF8888",
  16#0319# => X"FFFFFFFF",
  16#031a# => X"FFFFFFFF",
  16#031b# => X"FFFFFFFF",
  16#031c# => X"FFFFFFFF",
  16#031d# => X"777777FF",
  16#031e# => X"77777777",
  16#031f# => X"00888887",
  16#0320# => X"00000000",
  16#0321# => X"66666600",
  16#0322# => X"66666666",
  16#0323# => X"66666666",
  16#0324# => X"FFFFFFF6",
  16#0325# => X"666FFFFF",
  16#0326# => X"66666666",
  16#0327# => X"66666666",
  16#0328# => X"88888666",
  16#0329# => X"77777788",
  16#032a# => X"66666666",
  16#032b# => X"66666666",
  16#032c# => X"FF666666",
  16#032d# => X"666FFFFF",
  16#032e# => X"66666666",
  16#032f# => X"66666666",
  16#0330# => X"66666666",
  16#0331# => X"00000066",
  16#0332# => X"66666660",
  16#0333# => X"66666666",
  16#0334# => X"06666666",
  16#0335# => X"00000000",
  16#0336# => X"66666666",
  16#0337# => X"FFF66666",
  16#0338# => X"666666FF",
  16#0339# => X"F6666666",
  16#033a# => X"FFFFFFFF",
  16#033b# => X"FFFFFFFF",
  16#033c# => X"77777778",
  16#033d# => X"00077777",
  16#033e# => X"77777000",
  16#033f# => X"88888877",
  16#0340# => X"FFF88888",
  16#0341# => X"FFFFFFFF",
  16#0342# => X"FFFFFFFF",
  16#0343# => X"FFFFFFFF",
  16#0344# => X"FFFFFFFF",
  16#0345# => X"777778FF",
  16#0346# => X"77777777",
  16#0347# => X"00888877",
  16#0348# => X"00000000",
  16#0349# => X"66666660",
  16#034a# => X"66666666",
  16#034b# => X"66666666",
  16#034c# => X"FFFFFF66",
  16#034d# => X"6666FFFF",
  16#034e# => X"66666666",
  16#034f# => X"66666666",
  16#0350# => X"88886666",
  16#0351# => X"67778888",
  16#0352# => X"66666666",
  16#0353# => X"66666666",
  16#0354# => X"F6666666",
  16#0355# => X"666FFFFF",
  16#0356# => X"66666666",
  16#0357# => X"66666666",
  16#0358# => X"66666666",
  16#0359# => X"00000066",
  16#035a# => X"66666666",
  16#035b# => X"66666666",
  16#035c# => X"66666666",
  16#035d# => X"00000000",
  16#035e# => X"66666666",
  16#035f# => X"FFF66666",
  16#0360# => X"666666FF",
  16#0361# => X"F6666666",
  16#0362# => X"FFFFFFFF",
  16#0363# => X"FFFFFFFF",
  16#0364# => X"77777778",
  16#0365# => X"00077777",
  16#0366# => X"77777000",
  16#0367# => X"88888777",
  16#0368# => X"FFF88888",
  16#0369# => X"FFFFFFFF",
  16#036a# => X"FFFFFFFF",
  16#036b# => X"FFFFFFFF",
  16#036c# => X"FFFFFFFF",
  16#036d# => X"777778FF",
  16#036e# => X"77777777",
  16#036f# => X"00888777",
  16#0370# => X"00000000",
  16#0371# => X"66666660",
  16#0372# => X"66666666",
  16#0373# => X"66666666",
  16#0374# => X"FFFFF666",
  16#0375# => X"66666FFF",
  16#0376# => X"66666666",
  16#0377# => X"66666666",
  16#0378# => X"88866666",
  16#0379# => X"67788888",
  16#037a# => X"66666666",
  16#037b# => X"66666666",
  16#037c# => X"F6666666",
  16#037d# => X"666FFFFF",
  16#037e# => X"66666666",
  16#037f# => X"66666666",
  16#0380# => X"66666666",
  16#0381# => X"60000066",
  16#0382# => X"66666666",
  16#0383# => X"66666666",
  16#0384# => X"66666666",
  16#0385# => X"00000006",
  16#0386# => X"66666666",
  16#0387# => X"FFF66666",
  16#0388# => X"666666FF",
  16#0389# => X"F6666666",
  16#038a# => X"FFFFFFFF",
  16#038b# => X"FFFFFFFF",
  16#038c# => X"7777777F",
  16#038d# => X"00777777",
  16#038e# => X"77770000",
  16#038f# => X"88888777",
  16#0390# => X"FFF88888",
  16#0391# => X"FFFFFFFF",
  16#0392# => X"FFFFFFFF",
  16#0393# => X"FFFFFFFF",
  16#0394# => X"FFFFFFFF",
  16#0395# => X"77778FFF",
  16#0396# => X"77777777",
  16#0397# => X"00887777",
  16#0398# => X"00000000",
  16#0399# => X"66666666",
  16#039a# => X"66666666",
  16#039b# => X"66666666",
  16#039c# => X"FFFFF666",
  16#039d# => X"66666FFF",
  16#039e# => X"66666666",
  16#039f# => X"66666666",
  16#03a0# => X"88866666",
  16#03a1# => X"66888888",
  16#03a2# => X"66666666",
  16#03a3# => X"66666666",
  16#03a4# => X"66666666",
  16#03a5# => X"666FFFFF",
  16#03a6# => X"66666666",
  16#03a7# => X"66666666",
  16#03a8# => X"66666666",
  16#03a9# => X"60000066",
  16#03aa# => X"66666666",
  16#03ab# => X"66666666",
  16#03ac# => X"66666666",
  16#03ad# => X"00000006",
  16#03ae# => X"66666666",
  16#03af# => X"FFF66666",
  16#03b0# => X"666666FF",
  16#03b1# => X"F6666666",
  16#03b2# => X"FFFFFFFF",
  16#03b3# => X"FFFFFFFF",
  16#03b4# => X"7777778F",
  16#03b5# => X"07777777",
  16#03b6# => X"77700000",
  16#03b7# => X"88887777",
  16#03b8# => X"FF888888",
  16#03b9# => X"FFFFFFFF",
  16#03ba# => X"FFFFFFFF",
  16#03bb# => X"FFFFFFFF",
  16#03bc# => X"FFFFFFFF",
  16#03bd# => X"77778FFF",
  16#03be# => X"77777777",
  16#03bf# => X"00877777",
  16#03c0# => X"00000000",
  16#03c1# => X"66666666",
  16#03c2# => X"66666666",
  16#03c3# => X"66666666",
  16#03c4# => X"FFFFF666",
  16#03c5# => X"666666FF",
  16#03c6# => X"66666666",
  16#03c7# => X"66666666",
  16#03c8# => X"88666666",
  16#03c9# => X"66688888",
  16#03ca# => X"66666666",
  16#03cb# => X"66666666",
  16#03cc# => X"66666666",
  16#03cd# => X"666FFFF6",
  16#03ce# => X"66666666",
  16#03cf# => X"66666666",
  16#03d0# => X"66666666",
  16#03d1# => X"66000066",
  16#03d2# => X"66666666",
  16#03d3# => X"66666666",
  16#03d4# => X"66666666",
  16#03d5# => X"00000066",
  16#03d6# => X"66666666",
  16#03d7# => X"FFF66666",
  16#03d8# => X"666666FF",
  16#03d9# => X"F6666666",
  16#03da# => X"FFFFFFFF",
  16#03db# => X"FFFFFFFF",
  16#03dc# => X"7777778F",
  16#03dd# => X"77777777",
  16#03de# => X"77000000",
  16#03df# => X"88877777",
  16#03e0# => X"FF888888",
  16#03e1# => X"FFFFFFFF",
  16#03e2# => X"FFFFFFFF",
  16#03e3# => X"FFFFFFFF",
  16#03e4# => X"FFFFFFFF",
  16#03e5# => X"7777FFFF",
  16#03e6# => X"77777777",
  16#03e7# => X"00777777",
  16#03e8# => X"60000000",
  16#03e9# => X"66666666",
  16#03ea# => X"66666666",
  16#03eb# => X"66666666",
  16#03ec# => X"FFFF6666",
  16#03ed# => X"666666FF",
  16#03ee# => X"66666666",
  16#03ef# => X"66666666",
  16#03f0# => X"88666666",
  16#03f1# => X"66688888",
  16#03f2# => X"66666666",
  16#03f3# => X"66666666",
  16#03f4# => X"66666666",
  16#03f5# => X"666FFFF6",
  16#03f6# => X"66666666",
  16#03f7# => X"66666666",
  16#03f8# => X"66666666",
  16#03f9# => X"66000066",
  16#03fa# => X"66666666",
  16#03fb# => X"66666666",
  16#03fc# => X"66666666",
  16#03fd# => X"00000066",
  16#03fe# => X"66666666",
  16#03ff# => X"FFF66666",
  16#0400# => X"666666FF",
  16#0401# => X"F6666666",
  16#0402# => X"FFFFFFFF",
  16#0403# => X"FFFFFFFF",
  16#0404# => X"777777FF",
  16#0405# => X"77777777",
  16#0406# => X"77000000",
  16#0407# => X"88877777",
  16#0408# => X"FF888888",
  16#0409# => X"FFFFFFFF",
  16#040a# => X"FFFFFFFF",
  16#040b# => X"FFFFFFFF",
  16#040c# => X"FFFFFFFF",
  16#040d# => X"7778FFFF",
  16#040e# => X"77777777",
  16#040f# => X"00777777",
  16#0410# => X"60000000",
  16#0411# => X"66666666",
  16#0412# => X"6FFF6666",
  16#0413# => X"66666666",
  16#0414# => X"FFFF6666",
  16#0415# => X"666666FF",
  16#0416# => X"86666666",
  16#0417# => X"66666668",
  16#0418# => X"88666666",
  16#0419# => X"66688888",
  16#041a# => X"66666666",
  16#041b# => X"6666FF66",
  16#041c# => X"66666666",
  16#041d# => X"666FFFF6",
  16#041e# => X"66666666",
  16#041f# => X"66666666",
  16#0420# => X"66666666",
  16#0421# => X"66000066",
  16#0422# => X"66666666",
  16#0423# => X"66600666",
  16#0424# => X"66666666",
  16#0425# => X"00000066",
  16#0426# => X"66666666",
  16#0427# => X"FFF66666",
  16#0428# => X"666666FF",
  16#0429# => X"F6666666",
  16#042a# => X"FFFFFFFF",
  16#042b# => X"FFFFFFFF",
  16#042c# => X"777778FF",
  16#042d# => X"77777777",
  16#042e# => X"70000007",
  16#042f# => X"88777777",
  16#0430# => X"F8888888",
  16#0431# => X"FFFFFFFF",
  16#0432# => X"FFFFFFFF",
  16#0433# => X"FFFFFFFF",
  16#0434# => X"FFFFFFFF",
  16#0435# => X"7778FFFF",
  16#0436# => X"77777777",
  16#0437# => X"FF777777",
  16#0438# => X"60077888",
  16#0439# => X"66666666",
  16#043a# => X"FFFFF666",
  16#043b# => X"66666666",
  16#043c# => X"FFFF6666",
  16#043d# => X"6666666F",
  16#043e# => X"78666666",
  16#043f# => X"66666688",
  16#0440# => X"86666666",
  16#0441# => X"66668888",
  16#0442# => X"66666666",
  16#0443# => X"666FF876",
  16#0444# => X"66666666",
  16#0445# => X"666FFF66",
  16#0446# => X"66666666",
  16#0447# => X"66666666",
  16#0448# => X"66666666",
  16#0449# => X"66600066",
  16#044a# => X"66666666",
  16#044b# => X"66000066",
  16#044c# => X"66666666",
  16#044d# => X"00000666",
  16#044e# => X"66666666",
  16#044f# => X"FFF66666",
  16#0450# => X"666666FF",
  16#0451# => X"86666666",
  16#0452# => X"87888887",
  16#0453# => X"FFFFFFF8",
  16#0454# => X"777778FF",
  16#0455# => X"77777777",
  16#0456# => X"00000077",
  16#0457# => X"87777777",
  16#0458# => X"F8888888",
  16#0459# => X"FFFFFFFF",
  16#045a# => X"FFFFFFFF",
  16#045b# => X"FFFFFFFF",
  16#045c# => X"FFFFFFFF",
  16#045d# => X"778FFFFF",
  16#045e# => X"77777777",
  16#045f# => X"FF777777",
  16#0460# => X"68FFFFFF",
  16#0461# => X"66666666",
  16#0462# => X"FFFFF666",
  16#0463# => X"66666666",
  16#0464# => X"FFFF6666",
  16#0465# => X"6666666F",
  16#0466# => X"78666666",
  16#0467# => X"66666688",
  16#0468# => X"86666666",
  16#0469# => X"66668888",
  16#046a# => X"66666666",
  16#046b# => X"666FF876",
  16#046c# => X"66666666",
  16#046d# => X"FFFFFF66",
  16#046e# => X"666FFFFF",
  16#046f# => X"66666666",
  16#0470# => X"FFFFFF66",
  16#0471# => X"66600008",
  16#0472# => X"66666666",
  16#0473# => X"66000066",
  16#0474# => X"66666666",
  16#0475# => X"00000666",
  16#0476# => X"66666666",
  16#0477# => X"FFF66666",
  16#0478# => X"666666FF",
  16#0479# => X"06666666",
  16#047a# => X"77777777",
  16#047b# => X"88887777",
  16#047c# => X"7888FFF8",
  16#047d# => X"77777777",
  16#047e# => X"00000777",
  16#047f# => X"77777777",
  16#0480# => X"88888888",
  16#0481# => X"FFFFFFFF",
  16#0482# => X"FFFFFFFF",
  16#0483# => X"FFFFFFFF",
  16#0484# => X"FFFFFFFF",
  16#0485# => X"778FFFFF",
  16#0486# => X"77777777",
  16#0487# => X"FF777777",
  16#0488# => X"6FFFFFFF",
  16#0489# => X"66666666",
  16#048a# => X"FFFFF666",
  16#048b# => X"66666666",
  16#048c# => X"FFFF6666",
  16#048d# => X"6666666F",
  16#048e# => X"8F666666",
  16#048f# => X"66666687",
  16#0490# => X"86666666",
  16#0491# => X"66668888",
  16#0492# => X"66666666",
  16#0493# => X"666F8776",
  16#0494# => X"66666666",
  16#0495# => X"FFFFFF66",
  16#0496# => X"666FFFFF",
  16#0497# => X"66666666",
  16#0498# => X"FFFFFF66",
  16#0499# => X"6660000F",
  16#049a# => X"66666666",
  16#049b# => X"66000066",
  16#049c# => X"66666666",
  16#049d# => X"00000666",
  16#049e# => X"66666666",
  16#049f# => X"77866666",
  16#04a0# => X"66666607",
  16#04a1# => X"76666666",
  16#04a2# => X"77777770",
  16#04a3# => X"77777777",
  16#04a4# => X"FFFFF877",
  16#04a5# => X"77777888",
  16#04a6# => X"00000777",
  16#04a7# => X"77777770",
  16#04a8# => X"88888888",
  16#04a9# => X"FFFFFFFF",
  16#04aa# => X"FFFFFFFF",
  16#04ab# => X"FFFFFFFF",
  16#04ac# => X"FFFFFFFF",
  16#04ad# => X"77FFFFFF",
  16#04ae# => X"77777777",
  16#04af# => X"FF777777",
  16#04b0# => X"6FFFFFFF",
  16#04b1# => X"66666666",
  16#04b2# => X"FFFF8666",
  16#04b3# => X"66666666",
  16#04b4# => X"FFFF6666",
  16#04b5# => X"6666666F",
  16#04b6# => X"8F666666",
  16#04b7# => X"66666687",
  16#04b8# => X"86666666",
  16#04b9# => X"66668888",
  16#04ba# => X"66666666",
  16#04bb# => X"666F8776",
  16#04bc# => X"66666666",
  16#04bd# => X"FFFFFF66",
  16#04be# => X"666FFFFF",
  16#04bf# => X"66666666",
  16#04c0# => X"FFFFFF66",
  16#04c1# => X"6660008F",
  16#04c2# => X"66666666",
  16#04c3# => X"66000066",
  16#04c4# => X"66666666",
  16#04c5# => X"00000666",
  16#04c6# => X"66666666",
  16#04c7# => X"00066666",
  16#04c8# => X"66666600",
  16#04c9# => X"06666666",
  16#04ca# => X"77777770",
  16#04cb# => X"77777777",
  16#04cc# => X"FFFFF877",
  16#04cd# => X"7788FFFF",
  16#04ce# => X"00007707",
  16#04cf# => X"77777700",
  16#04d0# => X"88888887",
  16#04d1# => X"FFFFFFF8",
  16#04d2# => X"FFFFFFFF",
  16#04d3# => X"FFFFFFFF",
  16#04d4# => X"FFFFFFFF",
  16#04d5# => X"78FFFFFF",
  16#04d6# => X"77777777",
  16#04d7# => X"FF777777",
  16#04d8# => X"6FFFFFFF",
  16#04d9# => X"66666666",
  16#04da# => X"FFF78666",
  16#04db# => X"66666666",
  16#04dc# => X"FFFF6666",
  16#04dd# => X"6666666F",
  16#04de# => X"8F666666",
  16#04df# => X"66666687",
  16#04e0# => X"86666666",
  16#04e1# => X"66668888",
  16#04e2# => X"66666666",
  16#04e3# => X"66687776",
  16#04e4# => X"66666666",
  16#04e5# => X"FFFFFF66",
  16#04e6# => X"666FFFFF",
  16#04e7# => X"66666666",
  16#04e8# => X"FFFFFF66",
  16#04e9# => X"666000FF",
  16#04ea# => X"66666666",
  16#04eb# => X"66000066",
  16#04ec# => X"66666666",
  16#04ed# => X"00000666",
  16#04ee# => X"66666666",
  16#04ef# => X"00066666",
  16#04f0# => X"66666600",
  16#04f1# => X"06666666",
  16#04f2# => X"77777770",
  16#04f3# => X"77777777",
  16#04f4# => X"FFFFF777",
  16#04f5# => X"FFFFFFFF",
  16#04f6# => X"00000788",
  16#04f7# => X"77777000",
  16#04f8# => X"88888777",
  16#04f9# => X"FFFFFFF8",
  16#04fa# => X"FFFFFFFF",
  16#04fb# => X"FFFFFFFF",
  16#04fc# => X"FFFFFFFF",
  16#04fd# => X"78FFFFFF",
  16#04fe# => X"77777777",
  16#04ff# => X"FF777777",
  16#0500# => X"6FFFFFFF",
  16#0501# => X"66666666",
  16#0502# => X"FFF86666",
  16#0503# => X"66666666",
  16#0504# => X"FFFF6666",
  16#0505# => X"6666666F",
  16#0506# => X"FF666666",
  16#0507# => X"66666678",
  16#0508# => X"86666666",
  16#0509# => X"66668888",
  16#050a# => X"66666666",
  16#050b# => X"66687776",
  16#050c# => X"66666666",
  16#050d# => X"FFFFFF66",
  16#050e# => X"666FFFFF",
  16#050f# => X"66666666",
  16#0510# => X"FFFFFF66",
  16#0511# => X"666008FF",
  16#0512# => X"66666666",
  16#0513# => X"66000066",
  16#0514# => X"66666666",
  16#0515# => X"70000666",
  16#0516# => X"66666666",
  16#0517# => X"00066666",
  16#0518# => X"66666600",
  16#0519# => X"06666666",
  16#051a# => X"77777700",
  16#051b# => X"77777777",
  16#051c# => X"FFFF8777",
  16#051d# => X"FFFFFFFF",
  16#051e# => X"00788FFF",
  16#051f# => X"77777000",
  16#0520# => X"88887777",
  16#0521# => X"FFFFFFF8",
  16#0522# => X"FFFFFFFF",
  16#0523# => X"FFFFFFFF",
  16#0524# => X"FFFFFFFF",
  16#0525# => X"7FFFFFFF",
  16#0526# => X"77777777",
  16#0527# => X"FF777777",
  16#0528# => X"6FFFFFFF",
  16#0529# => X"66666666",
  16#052a# => X"88706666",
  16#052b# => X"FFFFFFFF",
  16#052c# => X"FFFFFFFF",
  16#052d# => X"6666666F",
  16#052e# => X"FF666666",
  16#052f# => X"66666678",
  16#0530# => X"86666666",
  16#0531# => X"66668888",
  16#0532# => X"66666666",
  16#0533# => X"66687776",
  16#0534# => X"66666666",
  16#0535# => X"FFFFFF66",
  16#0536# => X"666FFFFF",
  16#0537# => X"66666666",
  16#0538# => X"FFFFFF66",
  16#0539# => X"666008FF",
  16#053a# => X"66666666",
  16#053b# => X"66000066",
  16#053c# => X"66666666",
  16#053d# => X"FF870666",
  16#053e# => X"66666666",
  16#053f# => X"00066666",
  16#0540# => X"66666600",
  16#0541# => X"06666666",
  16#0542# => X"77777000",
  16#0543# => X"77777777",
  16#0544# => X"FFFF8777",
  16#0545# => X"FFFFFFFF",
  16#0546# => X"78FFFFFF",
  16#0547# => X"77780007",
  16#0548# => X"88887777",
  16#0549# => X"FFFFFF88",
  16#054a# => X"FFFFFFFF",
  16#054b# => X"FFFFFFFF",
  16#054c# => X"FFFFFFFF",
  16#054d# => X"8FFFFFFF",
  16#054e# => X"77777777",
  16#054f# => X"FF777777",
  16#0550# => X"6FFFFFFF",
  16#0551# => X"66666666",
  16#0552# => X"00066666",
  16#0553# => X"FFFFF887",
  16#0554# => X"FFFFFFFF",
  16#0555# => X"6666666F",
  16#0556# => X"FF666666",
  16#0557# => X"66666678",
  16#0558# => X"86666666",
  16#0559# => X"66668888",
  16#055a# => X"66666666",
  16#055b# => X"66678886",
  16#055c# => X"66666666",
  16#055d# => X"FFFFFF66",
  16#055e# => X"666FFFFF",
  16#055f# => X"66666666",
  16#0560# => X"FFFFFF66",
  16#0561# => X"66607FFF",
  16#0562# => X"66666666",
  16#0563# => X"66000066",
  16#0564# => X"66666666",
  16#0565# => X"FFFFF666",
  16#0566# => X"66666666",
  16#0567# => X"00066666",
  16#0568# => X"66666600",
  16#0569# => X"06666666",
  16#056a# => X"77770000",
  16#056b# => X"77777777",
  16#056c# => X"FFFF7777",
  16#056d# => X"FFFFFFFF",
  16#056e# => X"FFFFFFFF",
  16#056f# => X"88FFFF8F",
  16#0570# => X"88877777",
  16#0571# => X"FFFFFF88",
  16#0572# => X"FFFFFFFF",
  16#0573# => X"FFFFFFFF",
  16#0574# => X"FFFFFFFF",
  16#0575# => X"FFFFFFFF",
  16#0576# => X"77777777",
  16#0577# => X"FF777777",
  16#0578# => X"6FFFFFFF",
  16#0579# => X"66666666",
  16#057a# => X"06666666",
  16#057b# => X"FFF87000",
  16#057c# => X"FFFFFFFF",
  16#057d# => X"6666666F",
  16#057e# => X"FF666666",
  16#057f# => X"6666668F",
  16#0580# => X"86666666",
  16#0581# => X"66668888",
  16#0582# => X"66666666",
  16#0583# => X"66678886",
  16#0584# => X"66666666",
  16#0585# => X"FFFFFF66",
  16#0586# => X"666FFFFF",
  16#0587# => X"66666666",
  16#0588# => X"FFFFFF66",
  16#0589# => X"66608FFF",
  16#058a# => X"66666666",
  16#058b# => X"66000066",
  16#058c# => X"66666666",
  16#058d# => X"FFFFF666",
  16#058e# => X"66666666",
  16#058f# => X"00066666",
  16#0590# => X"66666600",
  16#0591# => X"06666666",
  16#0592# => X"77700000",
  16#0593# => X"77777777",
  16#0594# => X"FFFF7777",
  16#0595# => X"FFFFFFFF",
  16#0596# => X"FFFFFFFF",
  16#0597# => X"FFFFFFFF",
  16#0598# => X"8877788F",
  16#0599# => X"FFFFFF88",
  16#059a# => X"FFFFFFFF",
  16#059b# => X"FFFFFFFF",
  16#059c# => X"FFFFFFFF",
  16#059d# => X"FFFFFFFF",
  16#059e# => X"77777778",
  16#059f# => X"FF777777",
  16#05a0# => X"FFFFFFFF",
  16#05a1# => X"66666666",
  16#05a2# => X"66666666",
  16#05a3# => X"87000000",
  16#05a4# => X"FFFFFFFF",
  16#05a5# => X"6666666F",
  16#05a6# => X"FF666666",
  16#05a7# => X"6666668F",
  16#05a8# => X"86666666",
  16#05a9# => X"66668888",
  16#05aa# => X"66666666",
  16#05ab# => X"66688886",
  16#05ac# => X"66666666",
  16#05ad# => X"FFFFFF66",
  16#05ae# => X"666FFFFF",
  16#05af# => X"66666666",
  16#05b0# => X"FFFFFF66",
  16#05b1# => X"6667FFFF",
  16#05b2# => X"66666666",
  16#05b3# => X"66000066",
  16#05b4# => X"66666666",
  16#05b5# => X"FFFFF666",
  16#05b6# => X"66666666",
  16#05b7# => X"00066666",
  16#05b8# => X"66666600",
  16#05b9# => X"06666666",
  16#05ba# => X"77000000",
  16#05bb# => X"77777777",
  16#05bc# => X"FFF87777",
  16#05bd# => X"FFFFFFFF",
  16#05be# => X"FFFFFFFF",
  16#05bf# => X"FFFFFFFF",
  16#05c0# => X"8788FFFF",
  16#05c1# => X"FFFFF888",
  16#05c2# => X"FFFFFFFF",
  16#05c3# => X"FFFFFFFF",
  16#05c4# => X"FFFFFFFF",
  16#05c5# => X"FFFFFFFF",
  16#05c6# => X"77777778",
  16#05c7# => X"FF777777",
  16#05c8# => X"FFFFFFFF",
  16#05c9# => X"66666666",
  16#05ca# => X"66666666",
  16#05cb# => X"00000066",
  16#05cc# => X"FFFFF877",
  16#05cd# => X"6666666F",
  16#05ce# => X"FF666666",
  16#05cf# => X"6666668F",
  16#05d0# => X"86666666",
  16#05d1# => X"66668888",
  16#05d2# => X"66666666",
  16#05d3# => X"66688886",
  16#05d4# => X"66666666",
  16#05d5# => X"FFFFFF66",
  16#05d6# => X"666FFFFF",
  16#05d7# => X"66666666",
  16#05d8# => X"FFFFFF66",
  16#05d9# => X"6668FFFF",
  16#05da# => X"66666666",
  16#05db# => X"66000066",
  16#05dc# => X"66666666",
  16#05dd# => X"FFFFF666",
  16#05de# => X"66666666",
  16#05df# => X"66666666",
  16#05e0# => X"66666666",
  16#05e1# => X"06666666",
  16#05e2# => X"70000000",
  16#05e3# => X"77777777",
  16#05e4# => X"FFF87777",
  16#05e5# => X"FFFFFFFF",
  16#05e6# => X"FFFFFFFF",
  16#05e7# => X"FFFFFFFF",
  16#05e8# => X"8FFFFFFF",
  16#05e9# => X"FFFFF888",
  16#05ea# => X"FFFFFFFF",
  16#05eb# => X"FFFFFFFF",
  16#05ec# => X"FFFFFFFF",
  16#05ed# => X"FFFFFFFF",
  16#05ee# => X"7777777F",
  16#05ef# => X"FF777777",
  16#05f0# => X"FFFFFFFF",
  16#05f1# => X"66666666",
  16#05f2# => X"66666666",
  16#05f3# => X"70700666",
  16#05f4# => X"FFF87000",
  16#05f5# => X"6666666F",
  16#05f6# => X"FF666666",
  16#05f7# => X"666666FF",
  16#05f8# => X"86666666",
  16#05f9# => X"66668888",
  16#05fa# => X"66666666",
  16#05fb# => X"66688886",
  16#05fc# => X"66666666",
  16#05fd# => X"FFFFFF66",
  16#05fe# => X"666FFFFF",
  16#05ff# => X"66666666",
  16#0600# => X"FFFFFF66",
  16#0601# => X"666FFFFF",
  16#0602# => X"66666666",
  16#0603# => X"66000066",
  16#0604# => X"66666666",
  16#0605# => X"FFFFF666",
  16#0606# => X"66666666",
  16#0607# => X"66666666",
  16#0608# => X"66666666",
  16#0609# => X"06666666",
  16#060a# => X"00000000",
  16#060b# => X"77777777",
  16#060c# => X"FFF77777",
  16#060d# => X"FFFFFFFF",
  16#060e# => X"FFFFFFFF",
  16#060f# => X"FFFFFFFF",
  16#0610# => X"FFFFFFFF",
  16#0611# => X"FFFFF88F",
  16#0612# => X"FFFFFFFF",
  16#0613# => X"FFFFFFFF",
  16#0614# => X"FFFFFFFF",
  16#0615# => X"FFFFFFFF",
  16#0616# => X"7777778F",
  16#0617# => X"FF777777",
  16#0618# => X"FFFFFFFF",
  16#0619# => X"6666666F",
  16#061a# => X"66666666",
  16#061b# => X"00006666",
  16#061c# => X"87700700",
  16#061d# => X"6666666F",
  16#061e# => X"FF666666",
  16#061f# => X"666666FF",
  16#0620# => X"86666666",
  16#0621# => X"66668888",
  16#0622# => X"66666666",
  16#0623# => X"66688886",
  16#0624# => X"66666666",
  16#0625# => X"FFFFFF66",
  16#0626# => X"666FFFFF",
  16#0627# => X"66666666",
  16#0628# => X"FFFFFF66",
  16#0629# => X"666FFFFF",
  16#062a# => X"66666666",
  16#062b# => X"66870066",
  16#062c# => X"66666666",
  16#062d# => X"FFFFF666",
  16#062e# => X"66666666",
  16#062f# => X"66666666",
  16#0630# => X"66666666",
  16#0631# => X"06666666",
  16#0632# => X"00000000",
  16#0633# => X"77777770",
  16#0634# => X"FF877777",
  16#0635# => X"FFFFFFFF",
  16#0636# => X"FFFFFFFF",
  16#0637# => X"FFFFFFFF",
  16#0638# => X"FFFFFFFF",
  16#0639# => X"FFFFFFFF",
  16#063a# => X"FFFFFFFF",
  16#063b# => X"FFFFFFFF",
  16#063c# => X"FFFFFFFF",
  16#063d# => X"FFFFFFFF",
  16#063e# => X"7777778F",
  16#063f# => X"FF777777",
  16#0640# => X"FFFFFFFF",
  16#0641# => X"666666FF",
  16#0642# => X"66666666",
  16#0643# => X"00666666",
  16#0644# => X"00070000",
  16#0645# => X"66666667",
  16#0646# => X"FF666666",
  16#0647# => X"877778FF",
  16#0648# => X"88888888",
  16#0649# => X"66668888",
  16#064a# => X"66666666",
  16#064b# => X"66688886",
  16#064c# => X"66666666",
  16#064d# => X"FFFFFF66",
  16#064e# => X"666FFFFF",
  16#064f# => X"66666666",
  16#0650# => X"FFFFFF66",
  16#0651# => X"666FFFFF",
  16#0652# => X"66666666",
  16#0653# => X"FFFFF866",
  16#0654# => X"FFFFFFFF",
  16#0655# => X"FFFFFFFF",
  16#0656# => X"66666666",
  16#0657# => X"66666666",
  16#0658# => X"66666666",
  16#0659# => X"06666666",
  16#065a# => X"00000000",
  16#065b# => X"77777700",
  16#065c# => X"FF877777",
  16#065d# => X"FFFFFFFF",
  16#065e# => X"FFFFFFFF",
  16#065f# => X"FFFFFFFF",
  16#0660# => X"FFFFFFFF",
  16#0661# => X"FFFFFFFF",
  16#0662# => X"FFFFFFFF",
  16#0663# => X"FFFFFFFF",
  16#0664# => X"FFFFFFFF",
  16#0665# => X"FFFFFFFF",
  16#0666# => X"777777FF",
  16#0667# => X"FF777777",
  16#0668# => X"FFFFFFFF",
  16#0669# => X"666666FF",
  16#066a# => X"66666666",
  16#066b# => X"06666666",
  16#066c# => X"77000000",
  16#066d# => X"66666660",
  16#066e# => X"FF666666",
  16#066f# => X"77777FFF",
  16#0670# => X"88888888",
  16#0671# => X"66668888",
  16#0672# => X"66666666",
  16#0673# => X"66688886",
  16#0674# => X"66666666",
  16#0675# => X"FFFFFF66",
  16#0676# => X"666FFFFF",
  16#0677# => X"66666666",
  16#0678# => X"FFFFFF66",
  16#0679# => X"666FFFFF",
  16#067a# => X"66666666",
  16#067b# => X"FFFFFF66",
  16#067c# => X"FFFFFFFF",
  16#067d# => X"FFFFFFFF",
  16#067e# => X"66666666",
  16#067f# => X"66666666",
  16#0680# => X"66666666",
  16#0681# => X"06666666",
  16#0682# => X"00000000",
  16#0683# => X"77777000",
  16#0684# => X"FF777777",
  16#0685# => X"FFFFFFFF",
  16#0686# => X"FFFFFFFF",
  16#0687# => X"FFFFFFFF",
  16#0688# => X"FFFFFFFF",
  16#0689# => X"F88FFFFF",
  16#068a# => X"FFFFFFFF",
  16#068b# => X"FFFFFFFF",
  16#068c# => X"FFFFFFFF",
  16#068d# => X"FFFFFFFF",
  16#068e# => X"777778FF",
  16#068f# => X"FF777777",
  16#0690# => X"FFFFFFFF",
  16#0691# => X"6666FFFF",
  16#0692# => X"66666666",
  16#0693# => X"66666666",
  16#0694# => X"07000000",
  16#0695# => X"66666660",
  16#0696# => X"FF666666",
  16#0697# => X"77778FFF",
  16#0698# => X"88888888",
  16#0699# => X"66668888",
  16#069a# => X"66666666",
  16#069b# => X"66688886",
  16#069c# => X"66666666",
  16#069d# => X"FFFFFF66",
  16#069e# => X"666FFFFF",
  16#069f# => X"66666666",
  16#06a0# => X"FFFFFF66",
  16#06a1# => X"666FFFFF",
  16#06a2# => X"66666666",
  16#06a3# => X"FFFFFF66",
  16#06a4# => X"FFFFFFFF",
  16#06a5# => X"FFFFFFFF",
  16#06a6# => X"66666666",
  16#06a7# => X"66666666",
  16#06a8# => X"66666666",
  16#06a9# => X"06666666",
  16#06aa# => X"00000000",
  16#06ab# => X"77770000",
  16#06ac# => X"FF777777",
  16#06ad# => X"FFFFFFFF",
  16#06ae# => X"FFFFFFFF",
  16#06af# => X"FFFFFFFF",
  16#06b0# => X"FFFFFFFF",
  16#06b1# => X"888FFFFF",
  16#06b2# => X"FFFFFFF8",
  16#06b3# => X"FFFFFFFF",
  16#06b4# => X"FFFFFFFF",
  16#06b5# => X"FFFFFFFF",
  16#06b6# => X"77777FFF",
  16#06b7# => X"FF777777",
  16#06b8# => X"FFFFFFFF",
  16#06b9# => X"666FFFFF",
  16#06ba# => X"66666666",
  16#06bb# => X"66666666",
  16#06bc# => X"70000006",
  16#06bd# => X"66666667",
  16#06be# => X"FF666666",
  16#06bf# => X"77778FFF",
  16#06c0# => X"88888887",
  16#06c1# => X"66668888",
  16#06c2# => X"66666666",
  16#06c3# => X"66688886",
  16#06c4# => X"66666666",
  16#06c5# => X"FFFFFF66",
  16#06c6# => X"666FFFFF",
  16#06c7# => X"66666666",
  16#06c8# => X"FFFFFF66",
  16#06c9# => X"666FFFFF",
  16#06ca# => X"66666666",
  16#06cb# => X"FFFFFF66",
  16#06cc# => X"FFFFFFFF",
  16#06cd# => X"FFFFFFFF",
  16#06ce# => X"66666666",
  16#06cf# => X"66666666",
  16#06d0# => X"66666666",
  16#06d1# => X"06666666",
  16#06d2# => X"00000000",
  16#06d3# => X"77770000",
  16#06d4# => X"F8777777",
  16#06d5# => X"FFFFFFFF",
  16#06d6# => X"FFFFFFFF",
  16#06d7# => X"FFFFFFFF",
  16#06d8# => X"FFFFFFFF",
  16#06d9# => X"88FFFFFF",
  16#06da# => X"FFFF8888",
  16#06db# => X"FFFFFFFF",
  16#06dc# => X"FFFFFFFF",
  16#06dd# => X"FFFFFFFF",
  16#06de# => X"77777FFF",
  16#06df# => X"FF777777",
  16#06e0# => X"FFFFFFFF",
  16#06e1# => X"66FFFFFF",
  16#06e2# => X"66666666",
  16#06e3# => X"66666666",
  16#06e4# => X"77000066",
  16#06e5# => X"66666667",
  16#06e6# => X"FF666666",
  16#06e7# => X"7777FFFF",
  16#06e8# => X"88888887",
  16#06e9# => X"66668888",
  16#06ea# => X"66666666",
  16#06eb# => X"66688886",
  16#06ec# => X"66666666",
  16#06ed# => X"FFFFFF66",
  16#06ee# => X"666FFFFF",
  16#06ef# => X"66666666",
  16#06f0# => X"FFFFFF66",
  16#06f1# => X"666FFFFF",
  16#06f2# => X"66666666",
  16#06f3# => X"FFFFFF66",
  16#06f4# => X"FFFFFFFF",
  16#06f5# => X"FFFFFFFF",
  16#06f6# => X"66666666",
  16#06f7# => X"66666666",
  16#06f8# => X"66666666",
  16#06f9# => X"06666666",
  16#06fa# => X"00000000",
  16#06fb# => X"77700000",
  16#06fc# => X"87777777",
  16#06fd# => X"FFFFFFFF",
  16#06fe# => X"FFFFFFFF",
  16#06ff# => X"FFFFFFFF",
  16#0700# => X"FFFFFFFF",
  16#0701# => X"88FFFFFF",
  16#0702# => X"FF888888",
  16#0703# => X"FFFFFFFF",
  16#0704# => X"FFFFFFFF",
  16#0705# => X"FFFFFFFF",
  16#0706# => X"77778FFF",
  16#0707# => X"FF777777",
  16#0708# => X"FFFFFFFF",
  16#0709# => X"FFFFFFFF",
  16#070a# => X"66666666",
  16#070b# => X"66666666",
  16#070c# => X"00000666",
  16#070d# => X"66666660",
  16#070e# => X"F8666666",
  16#070f# => X"7778FFFF",
  16#0710# => X"88888877",
  16#0711# => X"66668888",
  16#0712# => X"66666666",
  16#0713# => X"66688886",
  16#0714# => X"66666666",
  16#0715# => X"FFFFFF66",
  16#0716# => X"666FFFFF",
  16#0717# => X"66666666",
  16#0718# => X"FFFFFF66",
  16#0719# => X"666FFFFF",
  16#071a# => X"66666666",
  16#071b# => X"FFFFFF66",
  16#071c# => X"FFFFFFFF",
  16#071d# => X"FFFFFFFF",
  16#071e# => X"66666666",
  16#071f# => X"66666666",
  16#0720# => X"66666666",
  16#0721# => X"06666666",
  16#0722# => X"00000000",
  16#0723# => X"77000000",
  16#0724# => X"87777777",
  16#0725# => X"FFFFFFFF",
  16#0726# => X"FFFFFFFF",
  16#0727# => X"FFFFFFFF",
  16#0728# => X"FFFFFFFF",
  16#0729# => X"88FFFFFF",
  16#072a# => X"88888888",
  16#072b# => X"FFFFFFF8",
  16#072c# => X"FFFFFFFF",
  16#072d# => X"FFFFFFFF",
  16#072e# => X"7777FFFF",
  16#072f# => X"FF777777",
  16#0730# => X"FFFFFFFF",
  16#0731# => X"FFFFFFFF",
  16#0732# => X"6666666F",
  16#0733# => X"66666666",
  16#0734# => X"70000666",
  16#0735# => X"66666667",
  16#0736# => X"70666666",
  16#0737# => X"7778FF87",
  16#0738# => X"88888877",
  16#0739# => X"66668888",
  16#073a# => X"66666666",
  16#073b# => X"66688886",
  16#073c# => X"66666666",
  16#073d# => X"FFFFFF66",
  16#073e# => X"666FFFFF",
  16#073f# => X"66666666",
  16#0740# => X"FFFFFF66",
  16#0741# => X"666FFFFF",
  16#0742# => X"66666666",
  16#0743# => X"FFFFFF66",
  16#0744# => X"FFFFFFFF",
  16#0745# => X"FFFFFFFF",
  16#0746# => X"66666666",
  16#0747# => X"66666666",
  16#0748# => X"66666666",
  16#0749# => X"06666666",
  16#074a# => X"00000000",
  16#074b# => X"70000000",
  16#074c# => X"77777777",
  16#074d# => X"FFFFFFFF",
  16#074e# => X"FFFFFFFF",
  16#074f# => X"FFFFFFFF",
  16#0750# => X"FFFFFFFF",
  16#0751# => X"88FFFFFF",
  16#0752# => X"88888888",
  16#0753# => X"FFFF8888",
  16#0754# => X"FFFFFFFF",
  16#0755# => X"FFFFFFFF",
  16#0756# => X"7778FFFF",
  16#0757# => X"FF777777",
  16#0758# => X"FFFFFFFF",
  16#0759# => X"FFFFFFFF",
  16#075a# => X"66666FFF",
  16#075b# => X"66666666",
  16#075c# => X"70006666",
  16#075d# => X"66666667",
  16#075e# => X"77666666",
  16#075f# => X"88FF8777",
  16#0760# => X"88888777",
  16#0761# => X"66668888",
  16#0762# => X"66666666",
  16#0763# => X"66688886",
  16#0764# => X"66666666",
  16#0765# => X"FFFFFF66",
  16#0766# => X"666FFFFF",
  16#0767# => X"66666666",
  16#0768# => X"FFFFFF66",
  16#0769# => X"666788FF",
  16#076a# => X"66666666",
  16#076b# => X"FFFFFF66",
  16#076c# => X"FFFFFFFF",
  16#076d# => X"FFFFFFFF",
  16#076e# => X"66666666",
  16#076f# => X"7FF66666",
  16#0770# => X"66666600",
  16#0771# => X"06666666",
  16#0772# => X"00000000",
  16#0773# => X"70000000",
  16#0774# => X"77777770",
  16#0775# => X"FFFFFFFF",
  16#0776# => X"FFFFFFFF",
  16#0777# => X"FFFFFFFF",
  16#0778# => X"FFFFFFFF",
  16#0779# => X"8FFFFFFF",
  16#077a# => X"88888888",
  16#077b# => X"F8888888",
  16#077c# => X"FFFFFFFF",
  16#077d# => X"FFFFFFFF",
  16#077e# => X"7778FFFF",
  16#077f# => X"FF777777",
  16#0780# => X"FFFFFFFF",
  16#0781# => X"FFFFFFFF",
  16#0782# => X"6666FFFF",
  16#0783# => X"66666666",
  16#0784# => X"00006666",
  16#0785# => X"66666660",
  16#0786# => X"77666666",
  16#0787# => X"66666677",
  16#0788# => X"86666666",
  16#0789# => X"66668888",
  16#078a# => X"66666666",
  16#078b# => X"66688886",
  16#078c# => X"66666666",
  16#078d# => X"FFFFFF66",
  16#078e# => X"666FFFFF",
  16#078f# => X"66666666",
  16#0790# => X"8FFFFF66",
  16#0791# => X"66677778",
  16#0792# => X"66666666",
  16#0793# => X"66FFFF66",
  16#0794# => X"66666666",
  16#0795# => X"FFFFF666",
  16#0796# => X"66666666",
  16#0797# => X"8FF66666",
  16#0798# => X"66666600",
  16#0799# => X"06666666",
  16#079a# => X"00000000",
  16#079b# => X"00000000",
  16#079c# => X"77777700",
  16#079d# => X"FFFFFFF8",
  16#079e# => X"FFFFFFFF",
  16#079f# => X"FFFFFFFF",
  16#07a0# => X"FFFFFFFF",
  16#07a1# => X"8FFFFFFF",
  16#07a2# => X"88888888",
  16#07a3# => X"88888888",
  16#07a4# => X"FFFFFF88",
  16#07a5# => X"FFFFFFFF",
  16#07a6# => X"777FFFFF",
  16#07a7# => X"FF777777",
  16#07a8# => X"FFFFFFFF",
  16#07a9# => X"FFFFFFFF",
  16#07aa# => X"66FFFFFF",
  16#07ab# => X"66666666",
  16#07ac# => X"00006666",
  16#07ad# => X"66666660",
  16#07ae# => X"77666666",
  16#07af# => X"66666677",
  16#07b0# => X"86666666",
  16#07b1# => X"66668888",
  16#07b2# => X"66666666",
  16#07b3# => X"66688886",
  16#07b4# => X"66666666",
  16#07b5# => X"FFFFFF66",
  16#07b6# => X"666FFFFF",
  16#07b7# => X"66666666",
  16#07b8# => X"7788FF66",
  16#07b9# => X"66677777",
  16#07ba# => X"66666666",
  16#07bb# => X"66FFFF66",
  16#07bc# => X"66666666",
  16#07bd# => X"FFFFF666",
  16#07be# => X"66666666",
  16#07bf# => X"8FF66666",
  16#07c0# => X"66666600",
  16#07c1# => X"06666666",
  16#07c2# => X"00000000",
  16#07c3# => X"00000000",
  16#07c4# => X"07777000",
  16#07c5# => X"FFFFFFF8",
  16#07c6# => X"FFFFFFFF",
  16#07c7# => X"FFFFFFFF",
  16#07c8# => X"FFFFFFFF",
  16#07c9# => X"8FFFFFFF",
  16#07ca# => X"88888888",
  16#07cb# => X"88888888",
  16#07cc# => X"FF888888",
  16#07cd# => X"FFFFFFFF",
  16#07ce# => X"778FFFFF",
  16#07cf# => X"FF777777",
  16#07d0# => X"FFFFFFFF",
  16#07d1# => X"FFFFFFFF",
  16#07d2# => X"6FFFFFFF",
  16#07d3# => X"66666666",
  16#07d4# => X"00066666",
  16#07d5# => X"66666660",
  16#07d6# => X"77666666",
  16#07d7# => X"66666677",
  16#07d8# => X"F6666666",
  16#07d9# => X"66668888",
  16#07da# => X"66666666",
  16#07db# => X"66688886",
  16#07dc# => X"66666666",
  16#07dd# => X"FFFFFF66",
  16#07de# => X"666FFFFF",
  16#07df# => X"66666666",
  16#07e0# => X"77777866",
  16#07e1# => X"66677777",
  16#07e2# => X"66666666",
  16#07e3# => X"66FFFF66",
  16#07e4# => X"66666666",
  16#07e5# => X"FFFFF666",
  16#07e6# => X"66666666",
  16#07e7# => X"FFF66666",
  16#07e8# => X"66666600",
  16#07e9# => X"06666666",
  16#07ea# => X"00000000",
  16#07eb# => X"00000000",
  16#07ec# => X"77770000",
  16#07ed# => X"FFFFFFF7",
  16#07ee# => X"FFFFFFFF",
  16#07ef# => X"FFFFFFFF",
  16#07f0# => X"FFFFFFFF",
  16#07f1# => X"8FFFFFFF",
  16#07f2# => X"88888888",
  16#07f3# => X"88888888",
  16#07f4# => X"88888888",
  16#07f5# => X"FFFFFF88",
  16#07f6# => X"778FFFFF",
  16#07f7# => X"FF777777",
  16#07f8# => X"6FFFFFFF",
  16#07f9# => X"66666666",
  16#07fa# => X"FFFFF666",
  16#07fb# => X"66666666",
  16#07fc# => X"00066666",
  16#07fd# => X"66666660",
  16#07fe# => X"77666666",
  16#07ff# => X"66666677",
  16#0800# => X"F6666666",
  16#0801# => X"6666FFFF",
  16#0802# => X"66666666",
  16#0803# => X"66688886",
  16#0804# => X"66666666",
  16#0805# => X"FFFFFF66",
  16#0806# => X"666FFFFF",
  16#0807# => X"66666666",
  16#0808# => X"77777766",
  16#0809# => X"66677777",
  16#080a# => X"66666666",
  16#080b# => X"66FFFF66",
  16#080c# => X"66666666",
  16#080d# => X"FFFFF666",
  16#080e# => X"66666666",
  16#080f# => X"FFF66666",
  16#0810# => X"66666607",
  16#0811# => X"06666666",
  16#0812# => X"00000000",
  16#0813# => X"00000000",
  16#0814# => X"77770000",
  16#0815# => X"FFFFFFF7",
  16#0816# => X"FFFFFFFF",
  16#0817# => X"FFFFFFFF",
  16#0818# => X"FFFFFFFF",
  16#0819# => X"FFFFFFFF",
  16#081a# => X"88888888",
  16#081b# => X"88888888",
  16#081c# => X"88888888",
  16#081d# => X"F8888888",
  16#081e# => X"88FFFFFF",
  16#081f# => X"FF777777",
  16#0820# => X"6FFFFFFF",
  16#0821# => X"66666666",
  16#0822# => X"FFFFF666",
  16#0823# => X"66666666",
  16#0824# => X"00066666",
  16#0825# => X"66666660",
  16#0826# => X"77666666",
  16#0827# => X"66666677",
  16#0828# => X"F6666666",
  16#0829# => X"6666FFFF",
  16#082a# => X"66666666",
  16#082b# => X"66688886",
  16#082c# => X"66666666",
  16#082d# => X"FFFFFF66",
  16#082e# => X"666FFFFF",
  16#082f# => X"66666666",
  16#0830# => X"77777766",
  16#0831# => X"66677777",
  16#0832# => X"66666666",
  16#0833# => X"66FFFF66",
  16#0834# => X"66666666",
  16#0835# => X"FFFFF666",
  16#0836# => X"66666666",
  16#0837# => X"FFF66666",
  16#0838# => X"66666608",
  16#0839# => X"06666666",
  16#083a# => X"00000000",
  16#083b# => X"00000000",
  16#083c# => X"77700000",
  16#083d# => X"FFFFFF87",
  16#083e# => X"FFFFFFFF",
  16#083f# => X"FFFFFFFF",
  16#0840# => X"FFFFFFFF",
  16#0841# => X"FFFFFFFF",
  16#0842# => X"88888888",
  16#0843# => X"88888888",
  16#0844# => X"88888888",
  16#0845# => X"88888888",
  16#0846# => X"FFF88888",
  16#0847# => X"FF88FFFF",
  16#0848# => X"6FFFFFFF",
  16#0849# => X"66666666",
  16#084a# => X"FFFFF666",
  16#084b# => X"6666666F",
  16#084c# => X"00066666",
  16#084d# => X"66666660",
  16#084e# => X"77666666",
  16#084f# => X"66666677",
  16#0850# => X"F6666666",
  16#0851# => X"6666FFFF",
  16#0852# => X"66666666",
  16#0853# => X"66688886",
  16#0854# => X"66666666",
  16#0855# => X"FFFFFF66",
  16#0856# => X"6668FFFF",
  16#0857# => X"66666666",
  16#0858# => X"77777766",
  16#0859# => X"66677777",
  16#085a# => X"66666666",
  16#085b# => X"66FFFF66",
  16#085c# => X"66666666",
  16#085d# => X"FFFFF666",
  16#085e# => X"66666666",
  16#085f# => X"FFF66666",
  16#0860# => X"66666608",
  16#0861# => X"06666666",
  16#0862# => X"00000000",
  16#0863# => X"00000000",
  16#0864# => X"77000000",
  16#0865# => X"FFFFFF80",
  16#0866# => X"FFFFFFFF",
  16#0867# => X"FFFFFFFF",
  16#0868# => X"FFFFFFFF",
  16#0869# => X"FFFFFFFF",
  16#086a# => X"88888888",
  16#086b# => X"88888888",
  16#086c# => X"88888888",
  16#086d# => X"88888888",
  16#086e# => X"FF877888",
  16#086f# => X"FFFFFFFF",
  16#0870# => X"6FFFFFFF",
  16#0871# => X"66666666",
  16#0872# => X"FFFFF666",
  16#0873# => X"6666666F",
  16#0874# => X"00066666",
  16#0875# => X"66666660",
  16#0876# => X"77666666",
  16#0877# => X"66666677",
  16#0878# => X"F6666666",
  16#0879# => X"6666FFFF",
  16#087a# => X"66666666",
  16#087b# => X"666FFFF6",
  16#087c# => X"66666666",
  16#087d# => X"8FFFFF66",
  16#087e# => X"66677888",
  16#087f# => X"66666666",
  16#0880# => X"77777766",
  16#0881# => X"66677777",
  16#0882# => X"66666666",
  16#0883# => X"66FFFF66",
  16#0884# => X"66666666",
  16#0885# => X"FFFFF666",
  16#0886# => X"66666666",
  16#0887# => X"FFF66666",
  16#0888# => X"6666660F",
  16#0889# => X"06666666",
  16#088a# => X"00000000",
  16#088b# => X"00000000",
  16#088c# => X"70000000",
  16#088d# => X"FFFFFF80",
  16#088e# => X"FFFFFFFF",
  16#088f# => X"FFFFFFFF",
  16#0890# => X"FFFFFFFF",
  16#0891# => X"FFFFFFFF",
  16#0892# => X"88888878",
  16#0893# => X"88888888",
  16#0894# => X"88888888",
  16#0895# => X"88888888",
  16#0896# => X"FF888888",
  16#0897# => X"FFFFFFFF",
  16#0898# => X"6FFFFFFF",
  16#0899# => X"66666666",
  16#089a# => X"FFFFF666",
  16#089b# => X"6666666F",
  16#089c# => X"00066666",
  16#089d# => X"66666660",
  16#089e# => X"77666666",
  16#089f# => X"66666677",
  16#08a0# => X"F6666666",
  16#08a1# => X"6666FFFF",
  16#08a2# => X"66666666",
  16#08a3# => X"666FFFF6",
  16#08a4# => X"66666666",
  16#08a5# => X"78888866",
  16#08a6# => X"66677777",
  16#08a7# => X"66666666",
  16#08a8# => X"77777766",
  16#08a9# => X"66677777",
  16#08aa# => X"66666666",
  16#08ab# => X"66FFFF66",
  16#08ac# => X"66666666",
  16#08ad# => X"FFFFF666",
  16#08ae# => X"66666666",
  16#08af# => X"FFF66666",
  16#08b0# => X"6666667F",
  16#08b1# => X"06666666",
  16#08b2# => X"00000000",
  16#08b3# => X"00000000",
  16#08b4# => X"70000000",
  16#08b5# => X"FFFFFF70",
  16#08b6# => X"FFFFFFFF",
  16#08b7# => X"FFFFFFFF",
  16#08b8# => X"FFFFFFFF",
  16#08b9# => X"FFFFFFFF",
  16#08ba# => X"8888888F",
  16#08bb# => X"88888888",
  16#08bc# => X"88888888",
  16#08bd# => X"88888888",
  16#08be# => X"FF788888",
  16#08bf# => X"FFFFFFFF",
  16#08c0# => X"6FFFFFFF",
  16#08c1# => X"66666666",
  16#08c2# => X"FFFFF666",
  16#08c3# => X"6666666F",
  16#08c4# => X"00066666",
  16#08c5# => X"66666660",
  16#08c6# => X"77666666",
  16#08c7# => X"66666677",
  16#08c8# => X"F6666666",
  16#08c9# => X"6666FFFF",
  16#08ca# => X"66666666",
  16#08cb# => X"666FFFF6",
  16#08cc# => X"66666666",
  16#08cd# => X"88888866",
  16#08ce# => X"66677788",
  16#08cf# => X"66666666",
  16#08d0# => X"77777766",
  16#08d1# => X"66677777",
  16#08d2# => X"66666666",
  16#08d3# => X"66FFFF66",
  16#08d4# => X"66666666",
  16#08d5# => X"FFFFF666",
  16#08d6# => X"66666666",
  16#08d7# => X"FFF66666",
  16#08d8# => X"6666668F",
  16#08d9# => X"06666666",
  16#08da# => X"00000000",
  16#08db# => X"00000000",
  16#08dc# => X"00000000",
  16#08dd# => X"FFFFFF70",
  16#08de# => X"FFFFFFFF",
  16#08df# => X"FFFFFFFF",
  16#08e0# => X"FFFFFFFF",
  16#08e1# => X"FFFFFFFF",
  16#08e2# => X"8888888F",
  16#08e3# => X"88888888",
  16#08e4# => X"88888888",
  16#08e5# => X"88888888",
  16#08e6# => X"F8788888",
  16#08e7# => X"FFFFFFFF",
  16#08e8# => X"6FFFFFFF",
  16#08e9# => X"66666666",
  16#08ea# => X"FFFFF666",
  16#08eb# => X"6666666F",
  16#08ec# => X"00066666",
  16#08ed# => X"66666660",
  16#08ee# => X"77666666",
  16#08ef# => X"66666677",
  16#08f0# => X"F6666666",
  16#08f1# => X"6666FFFF",
  16#08f2# => X"66666666",
  16#08f3# => X"666FFFF6",
  16#08f4# => X"66666666",
  16#08f5# => X"88888866",
  16#08f6# => X"66677788",
  16#08f7# => X"66666666",
  16#08f8# => X"77777766",
  16#08f9# => X"66677777",
  16#08fa# => X"66666666",
  16#08fb# => X"66FFFF66",
  16#08fc# => X"66666666",
  16#08fd# => X"FFFFF666",
  16#08fe# => X"66666666",
  16#08ff# => X"FFF66666",
  16#0900# => X"6666668F",
  16#0901# => X"06666666",
  16#0902# => X"00000000",
  16#0903# => X"00000000",
  16#0904# => X"00000000",
  16#0905# => X"FFFFF807",
  16#0906# => X"FFFFFFFF",
  16#0907# => X"FFFFFFFF",
  16#0908# => X"FFFFFFFF",
  16#0909# => X"FFFFFFFF",
  16#090a# => X"8888878F",
  16#090b# => X"88888888",
  16#090c# => X"88888888",
  16#090d# => X"88888888",
  16#090e# => X"F8888888",
  16#090f# => X"FFFFFFFF",
  16#0910# => X"6FFFFFFF",
  16#0911# => X"66666666",
  16#0912# => X"FFFFF666",
  16#0913# => X"6666666F",
  16#0914# => X"00066666",
  16#0915# => X"66666660",
  16#0916# => X"77666666",
  16#0917# => X"66666677",
  16#0918# => X"F6666666",
  16#0919# => X"6666FFFF",
  16#091a# => X"66666666",
  16#091b# => X"666FFFF6",
  16#091c# => X"66666666",
  16#091d# => X"88888866",
  16#091e# => X"66677888",
  16#091f# => X"66666666",
  16#0920# => X"77777766",
  16#0921# => X"66677777",
  16#0922# => X"66666666",
  16#0923# => X"66FFFF66",
  16#0924# => X"66666666",
  16#0925# => X"FFFFF666",
  16#0926# => X"66666666",
  16#0927# => X"FFF66666",
  16#0928# => X"666666FF",
  16#0929# => X"06666666",
  16#092a# => X"00000000",
  16#092b# => X"00000000",
  16#092c# => X"00000000",
  16#092d# => X"FFFFF800",
  16#092e# => X"FFFFFFFF",
  16#092f# => X"FFFFFFFF",
  16#0930# => X"FFFFFFFF",
  16#0931# => X"FFFFFFFF",
  16#0932# => X"8888878F",
  16#0933# => X"88888888",
  16#0934# => X"88888888",
  16#0935# => X"88888888",
  16#0936# => X"F8888888",
  16#0937# => X"FFFFFFFF",
  16#0938# => X"6FFFFFFF",
  16#0939# => X"66666666",
  16#093a# => X"FFFFF666",
  16#093b# => X"6666666F",
  16#093c# => X"00066666",
  16#093d# => X"66666600",
  16#093e# => X"77666666",
  16#093f# => X"66666677",
  16#0940# => X"FF666666",
  16#0941# => X"6666FFFF",
  16#0942# => X"66666666",
  16#0943# => X"666FFFF6",
  16#0944# => X"66666666",
  16#0945# => X"88888866",
  16#0946# => X"66688888",
  16#0947# => X"66666666",
  16#0948# => X"77777766",
  16#0949# => X"66777777",
  16#094a# => X"66666666",
  16#094b# => X"66FFFF66",
  16#094c# => X"66666666",
  16#094d# => X"FFFFFF66",
  16#094e# => X"66666666",
  16#094f# => X"FFF66666",
  16#0950# => X"666666FF",
  16#0951# => X"06666666",
  16#0952# => X"00000000",
  16#0953# => X"00000000",
  16#0954# => X"00000000",
  16#0955# => X"FFFFF700",
  16#0956# => X"FFFFFFFF",
  16#0957# => X"FFFFFFFF",
  16#0958# => X"FFFFFFFF",
  16#0959# => X"FFFFFFFF",
  16#095a# => X"888887FF",
  16#095b# => X"88888888",
  16#095c# => X"88888888",
  16#095d# => X"88888888",
  16#095e# => X"88888888",
  16#095f# => X"FFFFFFFF",
  16#0960# => X"6FFFFFFF",
  16#0961# => X"66666666",
  16#0962# => X"FFFF6666",
  16#0963# => X"66666666",
  16#0964# => X"00066666",
  16#0965# => X"66666600",
  16#0966# => X"76666666",
  16#0967# => X"66666667",
  16#0968# => X"FF666666",
  16#0969# => X"666FFFFF",
  16#096a# => X"66666666",
  16#096b# => X"6666FF66",
  16#096c# => X"66666666",
  16#096d# => X"88888886",
  16#096e# => X"66688888",
  16#096f# => X"66666666",
  16#0970# => X"77777766",
  16#0971# => X"66777777",
  16#0972# => X"66666666",
  16#0973# => X"666FF666",
  16#0974# => X"66666666",
  16#0975# => X"FFFFFF66",
  16#0976# => X"66666666",
  16#0977# => X"FFF66666",
  16#0978# => X"666666FF",
  16#0979# => X"06666666",
  16#097a# => X"00000000",
  16#097b# => X"00000000",
  16#097c# => X"00000000",
  16#097d# => X"FFFFF700",
  16#097e# => X"FFFFFFFF",
  16#097f# => X"FFFFFFFF",
  16#0980# => X"FFFFFFFF",
  16#0981# => X"FFFFFFFF",
  16#0982# => X"888878FF",
  16#0983# => X"88888888",
  16#0984# => X"88888888",
  16#0985# => X"88888888",
  16#0986# => X"88888888",
  16#0987# => X"FFFFFFFF",
  16#0988# => X"FFFFFFFF",
  16#0989# => X"66666666",
  16#098a# => X"66666666",
  16#098b# => X"66666666",
  16#098c# => X"00066666",
  16#098d# => X"66666600",
  16#098e# => X"66666666",
  16#098f# => X"66666666",
  16#0990# => X"FF666666",
  16#0991# => X"666FFFFF",
  16#0992# => X"66666666",
  16#0993# => X"66666666",
  16#0994# => X"66666666",
  16#0995# => X"88888886",
  16#0996# => X"66688888",
  16#0997# => X"66666666",
  16#0998# => X"77777766",
  16#0999# => X"66777777",
  16#099a# => X"66666666",
  16#099b# => X"66666666",
  16#099c# => X"66666666",
  16#099d# => X"FFFFFF66",
  16#099e# => X"66666666",
  16#099f# => X"FFF66666",
  16#09a0# => X"666666FF",
  16#09a1# => X"06666666",
  16#09a2# => X"00000000",
  16#09a3# => X"00000000",
  16#09a4# => X"00000000",
  16#09a5# => X"FFFFF000",
  16#09a6# => X"FFFFFFFF",
  16#09a7# => X"FFFFFFFF",
  16#09a8# => X"FFFFFFFF",
  16#09a9# => X"FFFFFFFF",
  16#09aa# => X"888878FF",
  16#09ab# => X"88888888",
  16#09ac# => X"88888888",
  16#09ad# => X"88888888",
  16#09ae# => X"88888888",
  16#09af# => X"FFFFFFFF",
  16#09b0# => X"FFFFFFFF",
  16#09b1# => X"66666666",
  16#09b2# => X"66666666",
  16#09b3# => X"66666666",
  16#09b4# => X"00006666",
  16#09b5# => X"66666600",
  16#09b6# => X"66666666",
  16#09b7# => X"66666666",
  16#09b8# => X"FF666666",
  16#09b9# => X"666FFFFF",
  16#09ba# => X"66666666",
  16#09bb# => X"66666666",
  16#09bc# => X"66666666",
  16#09bd# => X"88888886",
  16#09be# => X"66688888",
  16#09bf# => X"66666666",
  16#09c0# => X"77777766",
  16#09c1# => X"66777777",
  16#09c2# => X"66666666",
  16#09c3# => X"66666666",
  16#09c4# => X"66666666",
  16#09c5# => X"FFFFFF66",
  16#09c6# => X"66666666",
  16#09c7# => X"FFF66666",
  16#09c8# => X"666666FF",
  16#09c9# => X"06666666",
  16#09ca# => X"00000000",
  16#09cb# => X"00000000",
  16#09cc# => X"00000000",
  16#09cd# => X"FFFF8000",
  16#09ce# => X"FFFFFFFF",
  16#09cf# => X"FFFFFFFF",
  16#09d0# => X"FFFFFFFF",
  16#09d1# => X"FFFFFFFF",
  16#09d2# => X"888878FF",
  16#09d3# => X"88888888",
  16#09d4# => X"88888888",
  16#09d5# => X"88888888",
  16#09d6# => X"88888888",
  16#09d7# => X"FFFFFFF8",
  16#09d8# => X"FFFFFFFF",
  16#09d9# => X"66666666",
  16#09da# => X"66666666",
  16#09db# => X"66666666",
  16#09dc# => X"00006666",
  16#09dd# => X"66666000",
  16#09de# => X"66666666",
  16#09df# => X"66666666",
  16#09e0# => X"FFF66666",
  16#09e1# => X"66FFFFFF",
  16#09e2# => X"66666666",
  16#09e3# => X"66666666",
  16#09e4# => X"66666666",
  16#09e5# => X"8888888F",
  16#09e6# => X"66688888",
  16#09e7# => X"66666666",
  16#09e8# => X"77777766",
  16#09e9# => X"67777777",
  16#09ea# => X"66666666",
  16#09eb# => X"66666666",
  16#09ec# => X"66666666",
  16#09ed# => X"FFFFFFF6",
  16#09ee# => X"66666666",
  16#09ef# => X"FFF66666",
  16#09f0# => X"666666FF",
  16#09f1# => X"06666666",
  16#09f2# => X"00000000",
  16#09f3# => X"00000000",
  16#09f4# => X"00000000",
  16#09f5# => X"FFFF8000",
  16#09f6# => X"FFFFFFFF",
  16#09f7# => X"FFFFFFFF",
  16#09f8# => X"FFFFFFFF",
  16#09f9# => X"FFFFFFFF",
  16#09fa# => X"88877FFF",
  16#09fb# => X"88888888",
  16#09fc# => X"88888888",
  16#09fd# => X"88888888",
  16#09fe# => X"88888888",
  16#09ff# => X"FFFFFFF8",
  16#0a00# => X"FFFFFFFF",
  16#0a01# => X"6666666F",
  16#0a02# => X"66666666",
  16#0a03# => X"66666666",
  16#0a04# => X"00000666",
  16#0a05# => X"66666000",
  16#0a06# => X"66666666",
  16#0a07# => X"66666666",
  16#0a08# => X"FFFF6666",
  16#0a09# => X"6FFFFFFF",
  16#0a0a# => X"66666666",
  16#0a0b# => X"66666666",
  16#0a0c# => X"F6666666",
  16#0a0d# => X"888888FF",
  16#0a0e# => X"66688888",
  16#0a0f# => X"66666666",
  16#0a10# => X"77777766",
  16#0a11# => X"67777777",
  16#0a12# => X"66666666",
  16#0a13# => X"66666666",
  16#0a14# => X"66666666",
  16#0a15# => X"FFFFFFFF",
  16#0a16# => X"66666666",
  16#0a17# => X"FFF66666",
  16#0a18# => X"666666FF",
  16#0a19# => X"06666666",
  16#0a1a# => X"00000000",
  16#0a1b# => X"00000000",
  16#0a1c# => X"00000000",
  16#0a1d# => X"FFFF7000",
  16#0a1e# => X"FFFFFFFF",
  16#0a1f# => X"FFFFFFFF",
  16#0a20# => X"FFFFFFFF",
  16#0a21# => X"FFFFFFFF",
  16#0a22# => X"88878FFF",
  16#0a23# => X"88888888",
  16#0a24# => X"88888888",
  16#0a25# => X"88888888",
  16#0a26# => X"88888888",
  16#0a27# => X"FFFFFFF8",
  16#0a28# => X"FFFFFFFF",
  16#0a29# => X"666666FF",
  16#0a2a# => X"66666666",
  16#0a2b# => X"66666666",
  16#0a2c# => X"00000666",
  16#0a2d# => X"66660000",
  16#0a2e# => X"66666666",
  16#0a2f# => X"66666666",
  16#0a30# => X"FFFF6666",
  16#0a31# => X"6FFFFFFF",
  16#0a32# => X"66666666",
  16#0a33# => X"66666666",
  16#0a34# => X"F6666666",
  16#0a35# => X"888888FF",
  16#0a36# => X"66688888",
  16#0a37# => X"66666666",
  16#0a38# => X"77777766",
  16#0a39# => X"77777777",
  16#0a3a# => X"66666666",
  16#0a3b# => X"66666666",
  16#0a3c# => X"66666666",
  16#0a3d# => X"FFFFFFFF",
  16#0a3e# => X"66666666",
  16#0a3f# => X"FFF66666",
  16#0a40# => X"666666FF",
  16#0a41# => X"06666666",
  16#0a42# => X"00000000",
  16#0a43# => X"00000000",
  16#0a44# => X"00000000",
  16#0a45# => X"FFFF7000",
  16#0a46# => X"FFFFFFFF",
  16#0a47# => X"FFFFFFFF",
  16#0a48# => X"FFFFFFFF",
  16#0a49# => X"FFFFFFFF",
  16#0a4a# => X"88778FFF",
  16#0a4b# => X"88888888",
  16#0a4c# => X"88888888",
  16#0a4d# => X"88888888",
  16#0a4e# => X"88888888",
  16#0a4f# => X"FFFFFFF8",
  16#0a50# => X"FFFFFFFF",
  16#0a51# => X"66666FFF",
  16#0a52# => X"66666666",
  16#0a53# => X"66666666",
  16#0a54# => X"00000066",
  16#0a55# => X"66600000",
  16#0a56# => X"66666666",
  16#0a57# => X"66666666",
  16#0a58# => X"FFFFFF66",
  16#0a59# => X"FFFFFFFF",
  16#0a5a# => X"66666666",
  16#0a5b# => X"66666666",
  16#0a5c# => X"FF666666",
  16#0a5d# => X"888888FF",
  16#0a5e# => X"66688888",
  16#0a5f# => X"66666666",
  16#0a60# => X"77777766",
  16#0a61# => X"77777777",
  16#0a62# => X"66666667",
  16#0a63# => X"66666666",
  16#0a64# => X"FF666666",
  16#0a65# => X"FFFFFFFF",
  16#0a66# => X"66666666",
  16#0a67# => X"FFF66666",
  16#0a68# => X"666666FF",
  16#0a69# => X"86666666",
  16#0a6a# => X"88888888",
  16#0a6b# => X"00007777",
  16#0a6c# => X"00000000",
  16#0a6d# => X"FFFF0000",
  16#0a6e# => X"FFFFFFFF",
  16#0a6f# => X"FFFFFFFF",
  16#0a70# => X"FFFFFFFF",
  16#0a71# => X"FFFFFFFF",
  16#0a72# => X"87778FFF",
  16#0a73# => X"88888888",
  16#0a74# => X"88888888",
  16#0a75# => X"88888888",
  16#0a76# => X"88888888",
  16#0a77# => X"FFFFFF88",
  16#0a78# => X"FFFFFFFF",
  16#0a79# => X"6666FFFF",
  16#0a7a# => X"66666666",
  16#0a7b# => X"66666666",
  16#0a7c# => X"00000000",
  16#0a7d# => X"66000000",
  16#0a7e# => X"66666666",
  16#0a7f# => X"66666666",
  16#0a80# => X"FFFFFF86",
  16#0a81# => X"FFFFFFFF",
  16#0a82# => X"666666FF",
  16#0a83# => X"66666666",
  16#0a84# => X"FFFF6666",
  16#0a85# => X"888888FF",
  16#0a86# => X"66688888",
  16#0a87# => X"66666666",
  16#0a88# => X"77777766",
  16#0a89# => X"77777777",
  16#0a8a# => X"66666677",
  16#0a8b# => X"66666666",
  16#0a8c# => X"FFF66666",
  16#0a8d# => X"FFFFFFFF",
  16#0a8e# => X"66666666",
  16#0a8f# => X"FFFFFFFF",
  16#0a90# => X"FF88FFFF",
  16#0a91# => X"FFFFFFFF",
  16#0a92# => X"FFFFFFFF",
  16#0a93# => X"888FFFFF",
  16#0a94# => X"00000778",
  16#0a95# => X"FFF80000",
  16#0a96# => X"FFFFFFFF",
  16#0a97# => X"FFFFFFFF",
  16#0a98# => X"FFFFFFFF",
  16#0a99# => X"FFFFFFFF",
  16#0a9a# => X"8777FFFF",
  16#0a9b# => X"88888888",
  16#0a9c# => X"88888888",
  16#0a9d# => X"88888888",
  16#0a9e# => X"88888888",
  16#0a9f# => X"FFFFFF88",
  16#0aa0# => X"FFFFFFFF",
  16#0aa1# => X"66FFFFFF",
  16#0aa2# => X"66666666",
  16#0aa3# => X"07666666",
  16#0aa4# => X"00000000",
  16#0aa5# => X"00000000",
  16#0aa6# => X"66666666",
  16#0aa7# => X"76666666",
  16#0aa8# => X"FFFFFF70",
  16#0aa9# => X"FFFFFFFF",
  16#0aaa# => X"66666FFF",
  16#0aab# => X"66666666",
  16#0aac# => X"FFFFF666",
  16#0aad# => X"88888FFF",
  16#0aae# => X"66688888",
  16#0aaf# => X"66666666",
  16#0ab0# => X"77777766",
  16#0ab1# => X"77777777",
  16#0ab2# => X"66667777",
  16#0ab3# => X"66666666",
  16#0ab4# => X"FFFFF666",
  16#0ab5# => X"FFFFFFFF",
  16#0ab6# => X"FF666666",
  16#0ab7# => X"FFFFFFFF",
  16#0ab8# => X"FFFF88FF",
  16#0ab9# => X"FFFFFFFF",
  16#0aba# => X"FFFFFFFF",
  16#0abb# => X"FFFFFFFF",
  16#0abc# => X"7788FFFF",
  16#0abd# => X"FFF80000",
  16#0abe# => X"FFFFFFFF",
  16#0abf# => X"FFFFFFFF",
  16#0ac0# => X"FFFFFFFF",
  16#0ac1# => X"FFFFFFFF",
  16#0ac2# => X"7777FFFF",
  16#0ac3# => X"88888888",
  16#0ac4# => X"88888888",
  16#0ac5# => X"88888888",
  16#0ac6# => X"88888888",
  16#0ac7# => X"FFFFFF88",
  16#0ac8# => X"FFFFFFFF",
  16#0ac9# => X"FFFFFFFF",
  16#0aca# => X"6666666F",
  16#0acb# => X"0708F666",
  16#0acc# => X"00000000",
  16#0acd# => X"00000000",
  16#0ace# => X"66666000",
  16#0acf# => X"77766666",
  16#0ad0# => X"FFFFFF77",
  16#0ad1# => X"FFFFFFFF",
  16#0ad2# => X"66FFFFFF",
  16#0ad3# => X"66666666",
  16#0ad4# => X"FFFFFFFF",
  16#0ad5# => X"88888FFF",
  16#0ad6# => X"88888888",
  16#0ad7# => X"77788888",
  16#0ad8# => X"77777777",
  16#0ad9# => X"77777777",
  16#0ada# => X"68777777",
  16#0adb# => X"66666666",
  16#0adc# => X"FFFFFFF6",
  16#0add# => X"FFFFFFFF",
  16#0ade# => X"FFFFFFFF",
  16#0adf# => X"888FFFFF",
  16#0ae0# => X"FFFF8077",
  16#0ae1# => X"FFFFFFFF",
  16#0ae2# => X"FFFFFFFF",
  16#0ae3# => X"FFFFFFFF",
  16#0ae4# => X"FFFFFFFF",
  16#0ae5# => X"FFF87888",
  16#0ae6# => X"FFFFFFFF",
  16#0ae7# => X"FFFFFFFF",
  16#0ae8# => X"FFFFFFFF",
  16#0ae9# => X"FFFFFFFF",
  16#0aea# => X"7778FFFF",
  16#0aeb# => X"88888888",
  16#0aec# => X"88888888",
  16#0aed# => X"88888888",
  16#0aee# => X"88888888",
  16#0aef# => X"70FFFF88",
  16#0af0# => X"FFF88877",
  16#0af1# => X"FFFFFFFF",
  16#0af2# => X"FFFFFFFF",
  16#0af3# => X"000FFFFF",
  16#0af4# => X"00000000",
  16#0af5# => X"00000000",
  16#0af6# => X"77000000",
  16#0af7# => X"77777777",
  16#0af8# => X"FFFFF877",
  16#0af9# => X"FFFFFFFF",
  16#0afa# => X"FFFFFFFF",
  16#0afb# => X"FFFFFFFF",
  16#0afc# => X"FFFFFFFF",
  16#0afd# => X"88888FFF",
  16#0afe# => X"88888888",
  16#0aff# => X"77888888",
  16#0b00# => X"77777777",
  16#0b01# => X"77777777",
  16#0b02# => X"F7777777",
  16#0b03# => X"FFFFFFFF",
  16#0b04# => X"FFFFFFFF",
  16#0b05# => X"FFFFFFFF",
  16#0b06# => X"FFFFFFFF",
  16#0b07# => X"0077788F",
  16#0b08# => X"FFFF7070",
  16#0b09# => X"FFFFFFFF",
  16#0b0a# => X"FFFFFFFF",
  16#0b0b# => X"FFFFFFFF",
  16#0b0c# => X"FFFFFFFF",
  16#0b0d# => X"FFFFFFFF",
  16#0b0e# => X"FFFFFFFF",
  16#0b0f# => X"FFFFFFFF",
  16#0b10# => X"FFFFFFFF",
  16#0b11# => X"FFFFFFFF",
  16#0b12# => X"7778FFFF",
  16#0b13# => X"88888888",
  16#0b14# => X"88888888",
  16#0b15# => X"88888888",
  16#0b16# => X"88888888",
  16#0b17# => X"00FFF888",
  16#0b18# => X"77000000",
  16#0b19# => X"FFFFFF88",
  16#0b1a# => X"FFFFFFFF",
  16#0b1b# => X"007FFFFF",
  16#0b1c# => X"00000000",
  16#0b1d# => X"00000000",
  16#0b1e# => X"77000000",
  16#0b1f# => X"77777777",
  16#0b20# => X"FFFFF807",
  16#0b21# => X"FFFFFFFF",
  16#0b22# => X"FFFFFFFF",
  16#0b23# => X"FFFFFFFF",
  16#0b24# => X"FFFFFFFF",
  16#0b25# => X"8888FFFF",
  16#0b26# => X"88888888",
  16#0b27# => X"78888888",
  16#0b28# => X"77777777",
  16#0b29# => X"77777777",
  16#0b2a# => X"F7777777",
  16#0b2b# => X"FFFFFFFF",
  16#0b2c# => X"FFFFFFFF",
  16#0b2d# => X"FFFFFFFF",
  16#0b2e# => X"88FFFFFF",
  16#0b2f# => X"77777777",
  16#0b30# => X"FFFF7077",
  16#0b31# => X"FFFFFFFF",
  16#0b32# => X"FFFFFFFF",
  16#0b33# => X"FFFFFFFF",
  16#0b34# => X"FFFFFFFF",
  16#0b35# => X"F8FFFFFF",
  16#0b36# => X"FFFFFFFF",
  16#0b37# => X"FFFFFFFF",
  16#0b38# => X"FFFFFFFF",
  16#0b39# => X"FFFFFFFF",
  16#0b3a# => X"7778FFFF",
  16#0b3b# => X"88888887",
  16#0b3c# => X"88888888",
  16#0b3d# => X"88888888",
  16#0b3e# => X"88888888",
  16#0b3f# => X"00FFF888",
  16#0b40# => X"00000000",
  16#0b41# => X"FF887000",
  16#0b42# => X"FFFFFFFF",
  16#0b43# => X"707FFFFF",
  16#0b44# => X"00000770",
  16#0b45# => X"00000000",
  16#0b46# => X"77000000",
  16#0b47# => X"77777777",
  16#0b48# => X"FFFFF707",
  16#0b49# => X"FFFFFFFF",
  16#0b4a# => X"FFFFFFFF",
  16#0b4b# => X"FFFFFFFF",
  16#0b4c# => X"FFFFFFFF",
  16#0b4d# => X"8888FFFF",
  16#0b4e# => X"88888888",
  16#0b4f# => X"78888888",
  16#0b50# => X"77777777",
  16#0b51# => X"77777777",
  16#0b52# => X"87777777",
  16#0b53# => X"FFFFFFFF",
  16#0b54# => X"FFFFFFFF",
  16#0b55# => X"FFFFFFFF",
  16#0b56# => X"777788FF",
  16#0b57# => X"77777777",
  16#0b58# => X"FFFF7777",
  16#0b59# => X"FFFFFFFF",
  16#0b5a# => X"FFFFFFFF",
  16#0b5b# => X"FFFFFFFF",
  16#0b5c# => X"FFFFFFFF",
  16#0b5d# => X"77FFFFFF",
  16#0b5e# => X"FFFFFF88",
  16#0b5f# => X"FFFFFFFF",
  16#0b60# => X"FFFFFFFF",
  16#0b61# => X"FFFFFFFF",
  16#0b62# => X"777FFFFF",
  16#0b63# => X"88888887",
  16#0b64# => X"88888888",
  16#0b65# => X"88888888",
  16#0b66# => X"88888888",
  16#0b67# => X"00FFF888",
  16#0b68# => X"00000000",
  16#0b69# => X"70000000",
  16#0b6a# => X"FFFFFF88",
  16#0b6b# => X"708FFFFF",
  16#0b6c# => X"00070000",
  16#0b6d# => X"00000000",
  16#0b6e# => X"70000000",
  16#0b6f# => X"77777777",
  16#0b70# => X"FFFFF707",
  16#0b71# => X"FFFFFFFF",
  16#0b72# => X"FFFFFFFF",
  16#0b73# => X"FFFFFFFF",
  16#0b74# => X"FFFFFFFF",
  16#0b75# => X"8888FFFF",
  16#0b76# => X"88888888",
  16#0b77# => X"88888888",
  16#0b78# => X"77777777",
  16#0b79# => X"77777777",
  16#0b7a# => X"87777777",
  16#0b7b# => X"FFFFFFFF",
  16#0b7c# => X"FFFFFFFF",
  16#0b7d# => X"8FFFFFFF",
  16#0b7e# => X"77777778",
  16#0b7f# => X"77777777",
  16#0b80# => X"FFF87777",
  16#0b81# => X"FFFFFFFF",
  16#0b82# => X"FFFFFFFF",
  16#0b83# => X"FFFFFFFF",
  16#0b84# => X"FFFFFFFF",
  16#0b85# => X"07FFFFFF",
  16#0b86# => X"FFF87700",
  16#0b87# => X"FFFFFFFF",
  16#0b88# => X"FFFFFFFF",
  16#0b89# => X"FFFFFFFF",
  16#0b8a# => X"777FFFFF",
  16#0b8b# => X"88888877",
  16#0b8c# => X"88888888",
  16#0b8d# => X"88888888",
  16#0b8e# => X"88888888",
  16#0b8f# => X"00FFF888",
  16#0b90# => X"00000000",
  16#0b91# => X"00000000",
  16#0b92# => X"FFF88700",
  16#0b93# => X"708FFFFF",
  16#0b94# => X"07700700",
  16#0b95# => X"00000000",
  16#0b96# => X"70000000",
  16#0b97# => X"77777777",
  16#0b98# => X"FFFF8777",
  16#0b99# => X"FFFFFFFF",
  16#0b9a# => X"FFFFFFFF",
  16#0b9b# => X"FFFFFFFF",
  16#0b9c# => X"FFFFFFFF",
  16#0b9d# => X"8888FFFF",
  16#0b9e# => X"88888888",
  16#0b9f# => X"88888888",
  16#0ba0# => X"77777778",
  16#0ba1# => X"77777777",
  16#0ba2# => X"77777777",
  16#0ba3# => X"FFFFFFFF",
  16#0ba4# => X"FFFFFFFF",
  16#0ba5# => X"7788FFFF",
  16#0ba6# => X"77777777",
  16#0ba7# => X"77777777",
  16#0ba8# => X"FFF80777",
  16#0ba9# => X"FFFFFFFF",
  16#0baa# => X"FFFFFFFF",
  16#0bab# => X"FFFFFFFF",
  16#0bac# => X"FFFFFFFF",
  16#0bad# => X"08FFFFFF",
  16#0bae# => X"87700070",
  16#0baf# => X"FFFFFFFF",
  16#0bb0# => X"FFFFFFFF",
  16#0bb1# => X"FFFFFFFF",
  16#0bb2# => X"778FFFFF",
  16#0bb3# => X"88888877",
  16#0bb4# => X"88888888",
  16#0bb5# => X"88888888",
  16#0bb6# => X"88888888",
  16#0bb7# => X"00FF8888",
  16#0bb8# => X"00000000",
  16#0bb9# => X"00000000",
  16#0bba# => X"88700000",
  16#0bbb# => X"708FFFFF",
  16#0bbc# => X"07000707",
  16#0bbd# => X"00000000",
  16#0bbe# => X"00000000",
  16#0bbf# => X"77777777",
  16#0bc0# => X"FFFF8077",
  16#0bc1# => X"FFFFFFFF",
  16#0bc2# => X"FFFFFFFF",
  16#0bc3# => X"FFFFFFFF",
  16#0bc4# => X"FFFFFFFF",
  16#0bc5# => X"888FFFFF",
  16#0bc6# => X"88888888",
  16#0bc7# => X"88888888",
  16#0bc8# => X"77777788",
  16#0bc9# => X"77777777",
  16#0bca# => X"77777777",
  16#0bcb# => X"FFFFFFFF",
  16#0bcc# => X"FFFFFFFF",
  16#0bcd# => X"7777788F",
  16#0bce# => X"77777777",
  16#0bcf# => X"77777777",
  16#0bd0# => X"FFF80777",
  16#0bd1# => X"FFFFFFFF",
  16#0bd2# => X"FFFFFFFF",
  16#0bd3# => X"FFFFFFFF",
  16#0bd4# => X"FFFFFFFF",
  16#0bd5# => X"08FFFFFF",
  16#0bd6# => X"00077000",
  16#0bd7# => X"FFFFF877",
  16#0bd8# => X"FFFFFFFF",
  16#0bd9# => X"FFFFFFFF",
  16#0bda# => X"778FFFFF",
  16#0bdb# => X"88888777",
  16#0bdc# => X"88888888",
  16#0bdd# => X"88888888",
  16#0bde# => X"88888888",
  16#0bdf# => X"00FF8888",
  16#0be0# => X"00000000",
  16#0be1# => X"00000000",
  16#0be2# => X"00000000",
  16#0be3# => X"07FFF887",
  16#0be4# => X"00777770",
  16#0be5# => X"00000000",
  16#0be6# => X"00000000",
  16#0be7# => X"77777770",
  16#0be8# => X"FFFF8077",
  16#0be9# => X"FFFFFFFF",
  16#0bea# => X"FFFFFFFF",
  16#0beb# => X"FFFFFFFF",
  16#0bec# => X"FFFFFFFF",
  16#0bed# => X"888FFFFF",
  16#0bee# => X"88888888",
  16#0bef# => X"88888888",
  16#0bf0# => X"77777788",
  16#0bf1# => X"77777777",
  16#0bf2# => X"77777777",
  16#0bf3# => X"FFFFFFF8",
  16#0bf4# => X"8FFFFFFF",
  16#0bf5# => X"77777777",
  16#0bf6# => X"77777777",
  16#0bf7# => X"77777777",
  16#0bf8# => X"FFF80777",
  16#0bf9# => X"FFFFFFFF",
  16#0bfa# => X"FFFFFFFF",
  16#0bfb# => X"FFFFFFFF",
  16#0bfc# => X"FFFFFFFF",
  16#0bfd# => X"08FFFFFF",
  16#0bfe# => X"77700000",
  16#0bff# => X"FF887707",
  16#0c00# => X"FFFFFFFF",
  16#0c01# => X"FFFFFFFF",
  16#0c02# => X"778FFFFF",
  16#0c03# => X"88888777",
  16#0c04# => X"88888888",
  16#0c05# => X"88888888",
  16#0c06# => X"88888888",
  16#0c07# => X"00FF8888",
  16#0c08# => X"00000000",
  16#0c09# => X"00000000",
  16#0c0a# => X"00000000",
  16#0c0b# => X"8FF87000",
  16#0c0c# => X"77770007",
  16#0c0d# => X"00000007",
  16#0c0e# => X"00000000",
  16#0c0f# => X"77777770",
  16#0c10# => X"FFFF7077",
  16#0c11# => X"FFFFFFFF",
  16#0c12# => X"FFFFFFFF",
  16#0c13# => X"FFFFFFFF",
  16#0c14# => X"FFFFFFFF",
  16#0c15# => X"888FFFFF",
  16#0c16# => X"88888888",
  16#0c17# => X"88888888",
  16#0c18# => X"77777888",
  16#0c19# => X"77777777",
  16#0c1a# => X"77777777",
  16#0c1b# => X"FFFFFFF8",
  16#0c1c# => X"7788FFFF",
  16#0c1d# => X"77777777",
  16#0c1e# => X"77777777",
  16#0c1f# => X"77777777",
  16#0c20# => X"FFF77777",
  16#0c21# => X"FFFFFFFF",
  16#0c22# => X"FFFFFFFF",
  16#0c23# => X"FFFFFFFF",
  16#0c24# => X"FFFFFFFF",
  16#0c25# => X"0FFFFFFF",
  16#0c26# => X"77070000",
  16#0c27# => X"87707777",
  16#0c28# => X"FFFFFFFF",
  16#0c29# => X"FFFFFFFF",
  16#0c2a# => X"77FFFFFF",
  16#0c2b# => X"88888777",
  16#0c2c# => X"88888888",
  16#0c2d# => X"88888888",
  16#0c2e# => X"88888888",
  16#0c2f# => X"00FF8888",
  16#0c30# => X"00000000",
  16#0c31# => X"00000000",
  16#0c32# => X"00000000",
  16#0c33# => X"FF800000",
  16#0c34# => X"7700788F",
  16#0c35# => X"00000770",
  16#0c36# => X"00000000",
  16#0c37# => X"77777700",
  16#0c38# => X"FFFF7777",
  16#0c39# => X"FFFFFFFF",
  16#0c3a# => X"FFFFFFFF",
  16#0c3b# => X"FFFFFFFF",
  16#0c3c# => X"FFFFFFFF",
  16#0c3d# => X"88FFFFFF",
  16#0c3e# => X"88888888",
  16#0c3f# => X"88888888",
  16#0c40# => X"77777888",
  16#0c41# => X"77777777",
  16#0c42# => X"77777777",
  16#0c43# => X"FFFFFFF8",
  16#0c44# => X"777778FF",
  16#0c45# => X"77777777",
  16#0c46# => X"77777777",
  16#0c47# => X"77777777",
  16#0c48# => X"FFF77777",
  16#0c49# => X"FFFFFFFF",
  16#0c4a# => X"FFFFFFFF",
  16#0c4b# => X"FFFFFFFF",
  16#0c4c# => X"FFFFFFFF",
  16#0c4d# => X"7FFFFFFF",
  16#0c4e# => X"77000000",
  16#0c4f# => X"77777777",
  16#0c50# => X"FFFFF887",
  16#0c51# => X"FFFFFFFF",
  16#0c52# => X"77FFFFFF",
  16#0c53# => X"88887777",
  16#0c54# => X"88888888",
  16#0c55# => X"88888888",
  16#0c56# => X"88888888",
  16#0c57# => X"00F88888",
  16#0c58# => X"00000000",
  16#0c59# => X"00000000",
  16#0c5a# => X"00000000",
  16#0c5b# => X"FF807000",
  16#0c5c# => X"0778FFFF",
  16#0c5d# => X"00000770",
  16#0c5e# => X"00000000",
  16#0c5f# => X"77777700",
  16#0c60# => X"FFF80777",
  16#0c61# => X"FFFFFFFF",
  16#0c62# => X"FFFFFFFF",
  16#0c63# => X"FFFFFFFF",
  16#0c64# => X"FFFFFFFF",
  16#0c65# => X"88FFFFFF",
  16#0c66# => X"88888888",
  16#0c67# => X"88888888",
  16#0c68# => X"77778888",
  16#0c69# => X"77777777",
  16#0c6a# => X"77777777",
  16#0c6b# => X"FFFFFFF7",
  16#0c6c# => X"77777777",
  16#0c6d# => X"77777777",
  16#0c6e# => X"77777777",
  16#0c6f# => X"77777777",
  16#0c70# => X"FFF77777",
  16#0c71# => X"FFFFFFFF",
  16#0c72# => X"FFFFFFFF",
  16#0c73# => X"FFFFFFFF",
  16#0c74# => X"FFFFFFFF",
  16#0c75# => X"7FFFFFFF",
  16#0c76# => X"77000000",
  16#0c77# => X"77777777",
  16#0c78# => X"FFF87777",
  16#0c79# => X"FFFFFFFF",
  16#0c7a# => X"07FFFFFF",
  16#0c7b# => X"88877777",
  16#0c7c# => X"88888888",
  16#0c7d# => X"88888888",
  16#0c7e# => X"88888888",
  16#0c7f# => X"00F88888",
  16#0c80# => X"00000000",
  16#0c81# => X"00000000",
  16#0c82# => X"00000000",
  16#0c83# => X"FF807000",
  16#0c84# => X"8FFFFFFF",
  16#0c85# => X"07770007",
  16#0c86# => X"70070000",
  16#0c87# => X"77777700",
  16#0c88# => X"FFF80777",
  16#0c89# => X"FFFFFFFF",
  16#0c8a# => X"FFFFFFFF",
  16#0c8b# => X"FFFFFFFF",
  16#0c8c# => X"FFFFFFFF",
  16#0c8d# => X"88FFFFFF",
  16#0c8e# => X"88888888",
  16#0c8f# => X"88888888",
  16#0c90# => X"77788888",
  16#0c91# => X"77777777",
  16#0c92# => X"77777777",
  16#0c93# => X"77FFFFF7",
  16#0c94# => X"77777777",
  16#0c95# => X"77777777",
  16#0c96# => X"77777777",
  16#0c97# => X"77777777",
  16#0c98# => X"FF877777",
  16#0c99# => X"FFFFFFFF",
  16#0c9a# => X"FFFFFFFF",
  16#0c9b# => X"FFFFFFFF",
  16#0c9c# => X"FFFFFFFF",
  16#0c9d# => X"7FFFFFFF",
  16#0c9e# => X"70000000",
  16#0c9f# => X"77777777",
  16#0ca0# => X"87777777",
  16#0ca1# => X"FFFFFFFF",
  16#0ca2# => X"08FFFFFF",
  16#0ca3# => X"88877777",
  16#0ca4# => X"88888888",
  16#0ca5# => X"88888888",
  16#0ca6# => X"88888888",
  16#0ca7# => X"00F88888",
  16#0ca8# => X"00000000",
  16#0ca9# => X"00000000",
  16#0caa# => X"00000000",
  16#0cab# => X"FF707000",
  16#0cac# => X"FFFFFFFF",
  16#0cad# => X"0700778F",
  16#0cae# => X"00700700",
  16#0caf# => X"77700770",
  16#0cb0# => X"FFF80777",
  16#0cb1# => X"FFFFFFFF",
  16#0cb2# => X"FFFFFFFF",
  16#0cb3# => X"FFFFFFFF",
  16#0cb4# => X"FFFFFFFF",
  16#0cb5# => X"88FFFFFF",
  16#0cb6# => X"88888888",
  16#0cb7# => X"88888888",
  16#0cb8# => X"77788888",
  16#0cb9# => X"77777777",
  16#0cba# => X"77777777",
  16#0cbb# => X"7777FF87",
  16#0cbc# => X"77777777",
  16#0cbd# => X"77777777",
  16#0cbe# => X"77777777",
  16#0cbf# => X"77777777",
  16#0cc0# => X"FF877777",
  16#0cc1# => X"FFFFFFFF",
  16#0cc2# => X"FFFFFFFF",
  16#0cc3# => X"FFFFFFFF",
  16#0cc4# => X"FFFFFFFF",
  16#0cc5# => X"8FFFFFFF",
  16#0cc6# => X"70000000",
  16#0cc7# => X"77777777",
  16#0cc8# => X"07777777",
  16#0cc9# => X"FFFFF887",
  16#0cca# => X"08FFFFFF",
  16#0ccb# => X"88877777",
  16#0ccc# => X"88888888",
  16#0ccd# => X"88888888",
  16#0cce# => X"88888888",
  16#0ccf# => X"00F88888",
  16#0cd0# => X"00000000",
  16#0cd1# => X"00000000",
  16#0cd2# => X"00000000",
  16#0cd3# => X"FF707000",
  16#0cd4# => X"FFFFFFFF",
  16#0cd5# => X"0078FFFF",
  16#0cd6# => X"70000070",
  16#0cd7# => X"77077007",
  16#0cd8# => X"FFF70707",
  16#0cd9# => X"FFFFFFFF",
  16#0cda# => X"FFFFFFFF",
  16#0cdb# => X"FFFFFFFF",
  16#0cdc# => X"FFFFFFFF",
  16#0cdd# => X"8FFFFFFF",
  16#0cde# => X"88888888",
  16#0cdf# => X"88888888",
  16#0ce0# => X"77888888",
  16#0ce1# => X"77777777",
  16#0ce2# => X"77777777",
  16#0ce3# => X"77777F87",
  16#0ce4# => X"77777777",
  16#0ce5# => X"77777777",
  16#0ce6# => X"77777777",
  16#0ce7# => X"77777777",
  16#0ce8# => X"FF807777",
  16#0ce9# => X"FFFFFFFF",
  16#0cea# => X"FFFFFFFF",
  16#0ceb# => X"FFFFFFFF",
  16#0cec# => X"FFFFFFFF",
  16#0ced# => X"8FFFFFFF",
  16#0cee# => X"00000000",
  16#0cef# => X"77777777",
  16#0cf0# => X"77777777",
  16#0cf1# => X"FFF87707",
  16#0cf2# => X"78FFFFFF",
  16#0cf3# => X"88777777",
  16#0cf4# => X"88888888",
  16#0cf5# => X"88888888",
  16#0cf6# => X"88888888",
  16#0cf7# => X"00888888",
  16#0cf8# => X"00000000",
  16#0cf9# => X"00000000",
  16#0cfa# => X"00000000",
  16#0cfb# => X"FF707000",
  16#0cfc# => X"FFFFFFFF",
  16#0cfd# => X"88FFFFFF",
  16#0cfe# => X"00770007",
  16#0cff# => X"77770070",
  16#0d00# => X"FFF77777",
  16#0d01# => X"FFFFFFFF",
  16#0d02# => X"FFFFFFFF",
  16#0d03# => X"FFFFFFFF",
  16#0d04# => X"FFFFFFFF",
  16#0d05# => X"8FFFFFFF",
  16#0d06# => X"88888888",
  16#0d07# => X"88888888",
  16#0d08# => X"77888888",
  16#0d09# => X"77777777",
  16#0d0a# => X"87777777",
  16#0d0b# => X"77777FFF",
  16#0d0c# => X"77777777",
  16#0d0d# => X"77777777",
  16#0d0e# => X"77777777",
  16#0d0f# => X"77777777",
  16#0d10# => X"FF877777",
  16#0d11# => X"FFFFFFFF",
  16#0d12# => X"FFFFFFFF",
  16#0d13# => X"FFFFFFFF",
  16#0d14# => X"FFFFFFFF",
  16#0d15# => X"8FFFFFFF",
  16#0d16# => X"00000000",
  16#0d17# => X"77777770",
  16#0d18# => X"77777777",
  16#0d19# => X"88700777",
  16#0d1a# => X"7FFFFFFF",
  16#0d1b# => X"88777777",
  16#0d1c# => X"88888888",
  16#0d1d# => X"88888888",
  16#0d1e# => X"88888888",
  16#0d1f# => X"00888888",
  16#0d20# => X"00000000",
  16#0d21# => X"00000000",
  16#0d22# => X"00000000",
  16#0d23# => X"F8070000",
  16#0d24# => X"FFFFFFFF",
  16#0d25# => X"FFFFFFFF",
  16#0d26# => X"0700078F",
  16#0d27# => X"77070000",
  16#0d28# => X"FF877077",
  16#0d29# => X"FFFFFFFF",
  16#0d2a# => X"FFFFFFFF",
  16#0d2b# => X"FFFFFFFF",
  16#0d2c# => X"FFFFFFFF",
  16#0d2d# => X"8FFFFFFF",
  16#0d2e# => X"88888888",
  16#0d2f# => X"88888888",
  16#0d30# => X"78888888",
  16#0d31# => X"77777777",
  16#0d32# => X"FF877077",
  16#0d33# => X"77778FFF",
  16#0d34# => X"77777777",
  16#0d35# => X"77777777",
  16#0d36# => X"77777777",
  16#0d37# => X"77777777",
  16#0d38# => X"FF877777",
  16#0d39# => X"FFFFFFFF",
  16#0d3a# => X"FFFFFFFF",
  16#0d3b# => X"FFFFFFFF",
  16#0d3c# => X"FFFFFFFF",
  16#0d3d# => X"8FFFFFFF",
  16#0d3e# => X"00000000",
  16#0d3f# => X"77777770",
  16#0d40# => X"77777777",
  16#0d41# => X"00077777",
  16#0d42# => X"7FFFFF87",
  16#0d43# => X"88777770",
  16#0d44# => X"88888888",
  16#0d45# => X"88888888",
  16#0d46# => X"88888888",
  16#0d47# => X"00888888",
  16#0d48# => X"00000000",
  16#0d49# => X"00000000",
  16#0d4a# => X"00000000",
  16#0d4b# => X"F8070000",
  16#0d4c# => X"FFFFFFFF",
  16#0d4d# => X"FFFFFFFF",
  16#0d4e# => X"00788FFF",
  16#0d4f# => X"77000770",
  16#0d50# => X"FF807770",
  16#0d51# => X"FFFFFFFF",
  16#0d52# => X"FFFFFFFF",
  16#0d53# => X"FFFFFFFF",
  16#0d54# => X"FFFFFFFF",
  16#0d55# => X"8FFFFFFF",
  16#0d56# => X"88888888",
  16#0d57# => X"88888888",
  16#0d58# => X"88888888",
  16#0d59# => X"77777777",
  16#0d5a# => X"FFFFF877",
  16#0d5b# => X"77778FFF",
  16#0d5c# => X"77777777",
  16#0d5d# => X"77777777",
  16#0d5e# => X"77777777",
  16#0d5f# => X"77777777",
  16#0d60# => X"FF777777",
  16#0d61# => X"FFFFFFFF",
  16#0d62# => X"FFFFFFFF",
  16#0d63# => X"FFFFFFFF",
  16#0d64# => X"FFFFFFFF",
  16#0d65# => X"8FFFFFFF",
  16#0d66# => X"00000000",
  16#0d67# => X"77777770",
  16#0d68# => X"77777777",
  16#0d69# => X"00707777",
  16#0d6a# => X"8FF87000",
  16#0d6b# => X"87777700",
  16#0d6c# => X"88888888",
  16#0d6d# => X"88888888",
  16#0d6e# => X"88888888",
  16#0d6f# => X"00888888",
  16#0d70# => X"00000000",
  16#0d71# => X"00000000",
  16#0d72# => X"00000000",
  16#0d73# => X"F8070000",
  16#0d74# => X"FFFFFFFF",
  16#0d75# => X"FFFFFFFF",
  16#0d76# => X"78FFFFFF",
  16#0d77# => X"77770000",
  16#0d78# => X"FF807770",
  16#0d79# => X"FFFFFFFF",
  16#0d7a# => X"FFFFFFFF",
  16#0d7b# => X"FFFFFFFF",
  16#0d7c# => X"FFFFFFFF",
  16#0d7d# => X"8FFFFFFF",
  16#0d7e# => X"88888888",
  16#0d7f# => X"88888888",
  16#0d80# => X"88888888",
  16#0d81# => X"87777777",
  16#0d82# => X"FFFFFFF8",
  16#0d83# => X"77778FFF",
  16#0d84# => X"77777777",
  16#0d85# => X"77777777",
  16#0d86# => X"77777777",
  16#0d87# => X"77777777",
  16#0d88# => X"FF777777",
  16#0d89# => X"FFFFFFFF",
  16#0d8a# => X"FFFFFFFF",
  16#0d8b# => X"FFFFFFFF",
  16#0d8c# => X"FFFFFFFF",
  16#0d8d# => X"FFFFFFFF",
  16#0d8e# => X"00000000",
  16#0d8f# => X"77777700",
  16#0d90# => X"77777777",
  16#0d91# => X"00077777",
  16#0d92# => X"F8700000",
  16#0d93# => X"8777778F",
  16#0d94# => X"88888888",
  16#0d95# => X"88888888",
  16#0d96# => X"88888888",
  16#0d97# => X"00888888",
  16#0d98# => X"00000000",
  16#0d99# => X"00000000",
  16#0d9a# => X"00000000",
  16#0d9b# => X"F8070000",
  16#0d9c# => X"FFFFFFFF",
  16#0d9d# => X"FFFFFFFF",
  16#0d9e# => X"FFFFFFFF",
  16#0d9f# => X"7700078F",
  16#0da0# => X"FF707777",
  16#0da1# => X"FFFFFFFF",
  16#0da2# => X"FFFFFFFF",
  16#0da3# => X"FFFFFFFF",
  16#0da4# => X"FFFFFFFF",
  16#0da5# => X"FFFFFFFF",
  16#0da6# => X"88888888",
  16#0da7# => X"88888888",
  16#0da8# => X"88888888",
  16#0da9# => X"FF887778",
  16#0daa# => X"FFFFFFFF",
  16#0dab# => X"7777FFFF",
  16#0dac# => X"77777777",
  16#0dad# => X"77777777",
  16#0dae# => X"77777777",
  16#0daf# => X"77777777",
  16#0db0# => X"FF777777",
  16#0db1# => X"FFFFFFFF",
  16#0db2# => X"FFFFFFFF",
  16#0db3# => X"FFFFFFFF",
  16#0db4# => X"FFFFFFFF",
  16#0db5# => X"FFFFFFFF",
  16#0db6# => X"00000007",
  16#0db7# => X"77777000",
  16#0db8# => X"77777777",
  16#0db9# => X"00077777",
  16#0dba# => X"F7000000",
  16#0dbb# => X"77888FFF",
  16#0dbc# => X"88888888",
  16#0dbd# => X"88888888",
  16#0dbe# => X"88888888",
  16#0dbf# => X"00888888",
  16#0dc0# => X"00000000",
  16#0dc1# => X"00000000",
  16#0dc2# => X"00000000",
  16#0dc3# => X"F8070000",
  16#0dc4# => X"FFFFFFFF",
  16#0dc5# => X"FFFFFFFF",
  16#0dc6# => X"FFFFFFFF",
  16#0dc7# => X"00788FFF",
  16#0dc8# => X"FF707770",
  16#0dc9# => X"FFFFFFFF",
  16#0dca# => X"FFFFFFFF",
  16#0dcb# => X"FFFFFFFF",
  16#0dcc# => X"FFFFFFFF",
  16#0dcd# => X"FFFFFFFF",
  16#0dce# => X"88888888",
  16#0dcf# => X"88888888",
  16#0dd0# => X"88888888",
  16#0dd1# => X"FFFFF887",
  16#0dd2# => X"FFFFFFFF",
  16#0dd3# => X"7777FFFF",
  16#0dd4# => X"77777777",
  16#0dd5# => X"77777777",
  16#0dd6# => X"77777777",
  16#0dd7# => X"77777777",
  16#0dd8# => X"FF777777",
  16#0dd9# => X"FFFFFFFF",
  16#0dda# => X"FFFFFFFF",
  16#0ddb# => X"FFFFFFFF",
  16#0ddc# => X"FFFFFFFF",
  16#0ddd# => X"FFFFFFFF",
  16#0dde# => X"00000007",
  16#0ddf# => X"77777000",
  16#0de0# => X"77777777",
  16#0de1# => X"00077777",
  16#0de2# => X"F7000000",
  16#0de3# => X"8FFFFFFF",
  16#0de4# => X"88888888",
  16#0de5# => X"88888888",
  16#0de6# => X"88888888",
  16#0de7# => X"00888888",
  16#0de8# => X"00000000",
  16#0de9# => X"00000000",
  16#0dea# => X"00000000",
  16#0deb# => X"F7070000",
  16#0dec# => X"FFFFFFFF",
  16#0ded# => X"FFFFFFFF",
  16#0dee# => X"FFFFFFFF",
  16#0def# => X"88FFFFFF",
  16#0df0# => X"F8770007",
  16#0df1# => X"FFFFFFFF",
  16#0df2# => X"FFFFFFFF",
  16#0df3# => X"FFFFFFFF",
  16#0df4# => X"FFFFFFFF",
  16#0df5# => X"FFFFFFFF",
  16#0df6# => X"88888888",
  16#0df7# => X"88888888",
  16#0df8# => X"88888888",
  16#0df9# => X"FFFFFFFF",
  16#0dfa# => X"FFFFFFFF",
  16#0dfb# => X"7777FFFF",
  16#0dfc# => X"77777777",
  16#0dfd# => X"77777777",
  16#0dfe# => X"77777777",
  16#0dff# => X"77777777",
  16#0e00# => X"FF777777",
  16#0e01# => X"FFFFFFFF",
  16#0e02# => X"FFFFFFFF",
  16#0e03# => X"FFFFFFFF",
  16#0e04# => X"FFFFFFFF",
  16#0e05# => X"FFFFFFFF",
  16#0e06# => X"00000007",
  16#0e07# => X"77777000",
  16#0e08# => X"77777777",
  16#0e09# => X"00077777",
  16#0e0a# => X"F0000000",
  16#0e0b# => X"FFFFFFFF",
  16#0e0c# => X"888888FF",
  16#0e0d# => X"88888888",
  16#0e0e# => X"88888888",
  16#0e0f# => X"00888888",
  16#0e10# => X"00000000",
  16#0e11# => X"00000000",
  16#0e12# => X"00000000",
  16#0e13# => X"F7070000",
  16#0e14# => X"FFFFFFFF",
  16#0e15# => X"FFFFFFFF",
  16#0e16# => X"FFFFFFFF",
  16#0e17# => X"FFFFFFFF",
  16#0e18# => X"F807788F",
  16#0e19# => X"FFFFFFFF",
  16#0e1a# => X"FFFFFFFF",
  16#0e1b# => X"FFFFFFFF",
  16#0e1c# => X"FFFFFFFF",
  16#0e1d# => X"FFFFFFFF",
  16#0e1e# => X"88888888",
  16#0e1f# => X"88888888",
  16#0e20# => X"FFF88888",
  16#0e21# => X"FFFFFFFF",
  16#0e22# => X"FFFFFFFF",
  16#0e23# => X"7778FFFF",
  16#0e24# => X"77777777",
  16#0e25# => X"77777777",
  16#0e26# => X"77777777",
  16#0e27# => X"77777777",
  16#0e28# => X"F8777777",
  16#0e29# => X"FFFFFFFF",
  16#0e2a# => X"FFFFFFFF",
  16#0e2b# => X"FFFFFFFF",
  16#0e2c# => X"FFFFFFFF",
  16#0e2d# => X"FFFFFFFF",
  16#0e2e# => X"00000007",
  16#0e2f# => X"77770000",
  16#0e30# => X"77777777",
  16#0e31# => X"00077777",
  16#0e32# => X"80000000",
  16#0e33# => X"FFFFFFFF",
  16#0e34# => X"88FFFFFF",
  16#0e35# => X"88888888",
  16#0e36# => X"88888888",
  16#0e37# => X"00888888",
  16#0e38# => X"00000000",
  16#0e39# => X"00000000",
  16#0e3a# => X"00000000",
  16#0e3b# => X"F7070000",
  16#0e3c# => X"FFFFFFFF",
  16#0e3d# => X"FFFFFFFF",
  16#0e3e# => X"FFFFFFFF",
  16#0e3f# => X"FFFFFFFF",
  16#0e40# => X"F888FFFF",
  16#0e41# => X"FFFFFFFF",
  16#0e42# => X"FFFFFFFF",
  16#0e43# => X"FFFFFFFF",
  16#0e44# => X"FFFFFFFF",
  16#0e45# => X"FFFFFFFF",
  16#0e46# => X"88888888",
  16#0e47# => X"88888888",
  16#0e48# => X"FFFFFF88",
  16#0e49# => X"FFFFFFFF",
  16#0e4a# => X"FFFFFFFF",
  16#0e4b# => X"7708FFFF",
  16#0e4c# => X"77777777",
  16#0e4d# => X"77777777",
  16#0e4e# => X"77777777",
  16#0e4f# => X"77777777",
  16#0e50# => X"F8777777",
  16#0e51# => X"FFFFFFFF",
  16#0e52# => X"FFFFFFFF",
  16#0e53# => X"FFFFFFFF",
  16#0e54# => X"FFFFFFFF",
  16#0e55# => X"FFFFFFFF",
  16#0e56# => X"00000008",
  16#0e57# => X"77770000",
  16#0e58# => X"77777777",
  16#0e59# => X"00077777",
  16#0e5a# => X"80000000",
  16#0e5b# => X"FFFFFFFF",
  16#0e5c# => X"FFFFFFFF",
  16#0e5d# => X"888888FF",
  16#0e5e# => X"88888888",
  16#0e5f# => X"00888888",
  16#0e60# => X"00000000",
  16#0e61# => X"00000000",
  16#0e62# => X"00000000",
  16#0e63# => X"F7770000",
  16#0e64# => X"FFFFFFFF",
  16#0e65# => X"FFFFFFFF",
  16#0e66# => X"FFFFFFFF",
  16#0e67# => X"FFFFFFFF",
  16#0e68# => X"FFFFFFFF",
  16#0e69# => X"FFFFFFFF",
  16#0e6a# => X"FFFFFFFF",
  16#0e6b# => X"FFFFFFFF",
  16#0e6c# => X"FFFFFFFF",
  16#0e6d# => X"FFFFFFFF",
  16#0e6e# => X"8888888F",
  16#0e6f# => X"FF888888",
  16#0e70# => X"FFFFFFFF",
  16#0e71# => X"FFFFFFFF",
  16#0e72# => X"FFFFFFFF",
  16#0e73# => X"7708FFFF",
  16#0e74# => X"77777777",
  16#0e75# => X"77777777",
  16#0e76# => X"77777777",
  16#0e77# => X"77777777",
  16#0e78# => X"F8777777",
  16#0e79# => X"FFFFFFFF",
  16#0e7a# => X"FFFFFFFF",
  16#0e7b# => X"FFFFFFFF",
  16#0e7c# => X"FFFFFFFF",
  16#0e7d# => X"FFFFFFFF",
  16#0e7e# => X"00000008",
  16#0e7f# => X"77700000",
  16#0e80# => X"77777777",
  16#0e81# => X"00777777",
  16#0e82# => X"80000000",
  16#0e83# => X"FFFFFFFF",
  16#0e84# => X"FFFFFFFF",
  16#0e85# => X"8FFFFFFF",
  16#0e86# => X"88888888",
  16#0e87# => X"00888888",
  16#0e88# => X"00000000",
  16#0e89# => X"00000000",
  16#0e8a# => X"00000000",
  16#0e8b# => X"80770000",
  16#0e8c# => X"FFFFFFFF",
  16#0e8d# => X"FFFFFFFF",
  16#0e8e# => X"FFFFFFFF",
  16#0e8f# => X"FFFFFFFF",
  16#0e90# => X"FFFFFFFF",
  16#0e91# => X"FFFFFF88",
  16#0e92# => X"FFFFFFFF",
  16#0e93# => X"FFFFFFFF",
  16#0e94# => X"FFFFFFFF",
  16#0e95# => X"FFFFFFFF",
  16#0e96# => X"8888888F",
  16#0e97# => X"FFFFFF88",
  16#0e98# => X"FFFFFFFF",
  16#0e99# => X"FFFFFFFF",
  16#0e9a# => X"FFFFFFFF",
  16#0e9b# => X"7778FFFF",
  16#0e9c# => X"77777777",
  16#0e9d# => X"77777777",
  16#0e9e# => X"77777777",
  16#0e9f# => X"77777777",
  16#0ea0# => X"F8777777",
  16#0ea1# => X"FFFFFFFF",
  16#0ea2# => X"FFFFFFFF",
  16#0ea3# => X"FFFFFFFF",
  16#0ea4# => X"FFFFFFFF",
  16#0ea5# => X"FFFFFFFF",
  16#0ea6# => X"00000008",
  16#0ea7# => X"77000000",
  16#0ea8# => X"77777777",
  16#0ea9# => X"00777777",
  16#0eaa# => X"80000000",
  16#0eab# => X"FFFFFFFF",
  16#0eac# => X"FFFFFFFF",
  16#0ead# => X"FFFFFFFF",
  16#0eae# => X"88FFFFFF",
  16#0eaf# => X"00888888",
  16#0eb0# => X"00000000",
  16#0eb1# => X"00000000",
  16#0eb2# => X"00000000",
  16#0eb3# => X"80770000",
  16#0eb4# => X"FFFFFFFF",
  16#0eb5# => X"FFFFFFFF",
  16#0eb6# => X"FFFFFFFF",
  16#0eb7# => X"FFFFFFFF",
  16#0eb8# => X"FFFFFFFF",
  16#0eb9# => X"FF887707",
  16#0eba# => X"FFFFFFFF",
  16#0ebb# => X"FFFFFFFF",
  16#0ebc# => X"FFFFFFFF",
  16#0ebd# => X"FFFFFFFF",
  16#0ebe# => X"FFF8888F",
  16#0ebf# => X"FFFFFFFF",
  16#0ec0# => X"FFFFFFFF",
  16#0ec1# => X"FFFFFFFF",
  16#0ec2# => X"FFFFFFFF",
  16#0ec3# => X"7778FFFF",
  16#0ec4# => X"77777777",
  16#0ec5# => X"77777777",
  16#0ec6# => X"77777777",
  16#0ec7# => X"77777777",
  16#0ec8# => X"F8777777",
  16#0ec9# => X"FFFFFFFF",
  16#0eca# => X"FFFFFFFF",
  16#0ecb# => X"FFFFFFFF",
  16#0ecc# => X"FFFFFFFF",
  16#0ecd# => X"FFFFFFFF",
  16#0ece# => X"00000008",
  16#0ecf# => X"77000000",
  16#0ed0# => X"77777777",
  16#0ed1# => X"00777777",
  16#0ed2# => X"70000000",
  16#0ed3# => X"FFFFFFFF",
  16#0ed4# => X"FFFFFFFF",
  16#0ed5# => X"FFFFFFFF",
  16#0ed6# => X"FFFFFFFF",
  16#0ed7# => X"00FFFFFF",
  16#0ed8# => X"00000000",
  16#0ed9# => X"00000000",
  16#0eda# => X"00000000",
  16#0edb# => X"80700000",
  16#0edc# => X"FFFFFFFF",
  16#0edd# => X"FFFFFFFF",
  16#0ede# => X"FFFFFFFF",
  16#0edf# => X"FFFFFFFF",
  16#0ee0# => X"FFFFFFFF",
  16#0ee1# => X"77000707",
  16#0ee2# => X"FFFFF888",
  16#0ee3# => X"FFFFFFFF",
  16#0ee4# => X"FFFFFFFF",
  16#0ee5# => X"FFFFFFFF",
  16#0ee6# => X"FFFFFFFF",
  16#0ee7# => X"FFFFFFFF",
  16#0ee8# => X"FFFFFFFF",
  16#0ee9# => X"FFFFFFFF",
  16#0eea# => X"FFFFFFFF",
  16#0eeb# => X"777FFFFF",
  16#0eec# => X"77777777",
  16#0eed# => X"77777777",
  16#0eee# => X"77777777",
  16#0eef# => X"77777777",
  16#0ef0# => X"F8777777",
  16#0ef1# => X"FFFFFFFF",
  16#0ef2# => X"FFFFFFFF",
  16#0ef3# => X"FFFFFFFF",
  16#0ef4# => X"FFFFFFFF",
  16#0ef5# => X"FFFFFFFF",
  16#0ef6# => X"00000008",
  16#0ef7# => X"55555550",
  16#0ef8# => X"77777555",
  16#0ef9# => X"00777777",
  16#0efa# => X"70000000",
  16#0efb# => X"FFFFFFFF",
  16#0efc# => X"FFFFFFFF",
  16#0efd# => X"FFFFFFFF",
  16#0efe# => X"FFFFFFFF",
  16#0eff# => X"00FFFFFF",
  16#0f00# => X"00000000",
  16#0f01# => X"00000000",
  16#0f02# => X"00000000",
  16#0f03# => X"80770000",
  16#0f04# => X"FFFFFFFF",
  16#0f05# => X"FFFFFFFF",
  16#0f06# => X"FFFFFFFF",
  16#0f07# => X"FFFFFFFF",
  16#0f08# => X"FFFFFFFF",
  16#0f09# => X"77777707",
  16#0f0a# => X"88877777",
  16#0f0b# => X"FFFFFFF8",
  16#0f0c# => X"FFFFFFFF",
  16#0f0d# => X"8FFFFFFF",
  16#0f0e# => X"FFFFFFFF",
  16#0f0f# => X"FFFFFFFF",
  16#0f10# => X"FFFFFFFF",
  16#0f11# => X"FFFFFFFF",
  16#0f12# => X"FFFFFFFF",
  16#0f13# => X"777FFFFF",
  16#0f14# => X"77777777",
  16#0f15# => X"55577777",
  16#0f16# => X"55555555",
  16#0f17# => X"55555555",
  16#0f18# => X"F8777775",
  16#0f19# => X"FFFFFFFF",
  16#0f1a# => X"555555FF",
  16#0f1b# => X"55555555",
  16#0f1c# => X"FF555555",
  16#0f1d# => X"FFFFFFFF",
  16#0f1e# => X"5500000F",
  16#0f1f# => X"55555555",
  16#0f20# => X"77555555",
  16#0f21# => X"00077777",
  16#0f22# => X"55555500",
  16#0f23# => X"55555555",
  16#0f24# => X"55555555",
  16#0f25# => X"FFFFFFF5",
  16#0f26# => X"FFFFFFFF",
  16#0f27# => X"00FFFFFF",
  16#0f28# => X"00000000",
  16#0f29# => X"00000000",
  16#0f2a# => X"00000000",
  16#0f2b# => X"80770000",
  16#0f2c# => X"FFFFFFFF",
  16#0f2d# => X"FFFFFFFF",
  16#0f2e# => X"FFFFFFFF",
  16#0f2f# => X"FFFFFFFF",
  16#0f30# => X"FFFFFFFF",
  16#0f31# => X"77777708",
  16#0f32# => X"77777777",
  16#0f33# => X"88887777",
  16#0f34# => X"88888888",
  16#0f35# => X"88888888",
  16#0f36# => X"FFFFFFFF",
  16#0f37# => X"FFFFFFFF",
  16#0f38# => X"FFFFFFFF",
  16#0f39# => X"FFFFFFFF",
  16#0f3a# => X"FFFFFFFF",
  16#0f3b# => X"777FFFFF",
  16#0f3c# => X"77777777",
  16#0f3d# => X"55577777",
  16#0f3e# => X"55555555",
  16#0f3f# => X"55555555",
  16#0f40# => X"F8755555",
  16#0f41# => X"FFFFFFFF",
  16#0f42# => X"555555FF",
  16#0f43# => X"55555555",
  16#0f44# => X"FF555555",
  16#0f45# => X"FFFFFFFF",
  16#0f46# => X"5550000F",
  16#0f47# => X"55555555",
  16#0f48# => X"75555555",
  16#0f49# => X"00077777",
  16#0f4a# => X"55555500",
  16#0f4b# => X"55555555",
  16#0f4c# => X"55555555",
  16#0f4d# => X"FFFFFFF5",
  16#0f4e# => X"FFFFFFFF",
  16#0f4f# => X"00FFFFFF",
  16#0f50# => X"00000000",
  16#0f51# => X"00000000",
  16#0f52# => X"00000000",
  16#0f53# => X"70770000",
  16#0f54# => X"FFFFFFFF",
  16#0f55# => X"FFFFFFFF",
  16#0f56# => X"FFFFFFFF",
  16#0f57# => X"FFFFFFFF",
  16#0f58# => X"FFFFFFFF",
  16#0f59# => X"77777708",
  16#0f5a# => X"77777777",
  16#0f5b# => X"87777777",
  16#0f5c# => X"88888888",
  16#0f5d# => X"88888888",
  16#0f5e# => X"FFFFFFF8",
  16#0f5f# => X"FFFFFFFF",
  16#0f60# => X"FFFFFFFF",
  16#0f61# => X"FFFFFFFF",
  16#0f62# => X"FFFFFFFF",
  16#0f63# => X"707FFFFF",
  16#0f64# => X"77777777",
  16#0f65# => X"55577777",
  16#0f66# => X"55555555",
  16#0f67# => X"55555555",
  16#0f68# => X"F8555555",
  16#0f69# => X"FFFFFFFF",
  16#0f6a# => X"555555FF",
  16#0f6b# => X"55555555",
  16#0f6c# => X"FF555555",
  16#0f6d# => X"FFFFFFFF",
  16#0f6e# => X"5555507F",
  16#0f6f# => X"55555555",
  16#0f70# => X"55555555",
  16#0f71# => X"00077775",
  16#0f72# => X"55555500",
  16#0f73# => X"55555555",
  16#0f74# => X"55555555",
  16#0f75# => X"FFFFFFF5",
  16#0f76# => X"FFFFFFFF",
  16#0f77# => X"00FFFFFF",
  16#0f78# => X"00000000",
  16#0f79# => X"00000000",
  16#0f7a# => X"00000000",
  16#0f7b# => X"70770000",
  16#0f7c# => X"FFFFFFFF",
  16#0f7d# => X"FFFFFFFF",
  16#0f7e# => X"FFFFFFFF",
  16#0f7f# => X"FFFFFFFF",
  16#0f80# => X"FFFFFFFF",
  16#0f81# => X"77777708",
  16#0f82# => X"77777777",
  16#0f83# => X"88877777",
  16#0f84# => X"88888888",
  16#0f85# => X"88888888",
  16#0f86# => X"FFFFFFF8",
  16#0f87# => X"FFFFFFFF",
  16#0f88# => X"FFFFFFFF",
  16#0f89# => X"FFFFFFFF",
  16#0f8a# => X"FFFFFFFF",
  16#0f8b# => X"708FFFFF",
  16#0f8c# => X"77777777",
  16#0f8d# => X"55577777",
  16#0f8e# => X"55555555",
  16#0f8f# => X"55555555",
  16#0f90# => X"F5555555",
  16#0f91# => X"FFFFFFFF",
  16#0f92# => X"555555FF",
  16#0f93# => X"55555555",
  16#0f94# => X"FF555555",
  16#0f95# => X"FFFFFFFF",
  16#0f96# => X"5555557F",
  16#0f97# => X"55555555",
  16#0f98# => X"55555555",
  16#0f99# => X"00777755",
  16#0f9a# => X"55555500",
  16#0f9b# => X"55555555",
  16#0f9c# => X"55555555",
  16#0f9d# => X"FFFFFFF5",
  16#0f9e# => X"FFFFFFFF",
  16#0f9f# => X"00FFFFFF",
  16#0fa0# => X"00000000",
  16#0fa1# => X"00000000",
  16#0fa2# => X"00000000",
  16#0fa3# => X"70770000",
  16#0fa4# => X"FFFFFFFF",
  16#0fa5# => X"FFFFFFFF",
  16#0fa6# => X"FFFFFFFF",
  16#0fa7# => X"FFFFFFFF",
  16#0fa8# => X"FFFFFFFF",
  16#0fa9# => X"7777777F",
  16#0faa# => X"77777777",
  16#0fab# => X"87777777",
  16#0fac# => X"88888888",
  16#0fad# => X"88888888",
  16#0fae# => X"FFFFFFF8",
  16#0faf# => X"FFFFFFFF",
  16#0fb0# => X"FFFFFFFF",
  16#0fb1# => X"FFFFFFFF",
  16#0fb2# => X"FFFFFFFF",
  16#0fb3# => X"708FFFFF",
  16#0fb4# => X"77777777",
  16#0fb5# => X"55577777",
  16#0fb6# => X"55555555",
  16#0fb7# => X"55555555",
  16#0fb8# => X"55555555",
  16#0fb9# => X"FFFFFFFF",
  16#0fba# => X"555555FF",
  16#0fbb# => X"55555555",
  16#0fbc# => X"FF555555",
  16#0fbd# => X"FFFFFFFF",
  16#0fbe# => X"5555555F",
  16#0fbf# => X"55555555",
  16#0fc0# => X"55555555",
  16#0fc1# => X"00777555",
  16#0fc2# => X"55555500",
  16#0fc3# => X"55555555",
  16#0fc4# => X"55555555",
  16#0fc5# => X"FFFFFFF5",
  16#0fc6# => X"FFFFFFFF",
  16#0fc7# => X"00FFFFFF",
  16#0fc8# => X"00000000",
  16#0fc9# => X"00000000",
  16#0fca# => X"00000000",
  16#0fcb# => X"77770000",
  16#0fcc# => X"FFFFFFFF",
  16#0fcd# => X"FFFFFFFF",
  16#0fce# => X"FFFFFFFF",
  16#0fcf# => X"FFFFFFFF",
  16#0fd0# => X"FFFFFFFF",
  16#0fd1# => X"7777707F",
  16#0fd2# => X"77777777",
  16#0fd3# => X"88777777",
  16#0fd4# => X"88888888",
  16#0fd5# => X"88888888",
  16#0fd6# => X"FFFFFFF8",
  16#0fd7# => X"FFFFFFFF",
  16#0fd8# => X"FFFFFFFF",
  16#0fd9# => X"FFFFFFFF",
  16#0fda# => X"FFFFFFFF",
  16#0fdb# => X"708FFFFF",
  16#0fdc# => X"77777777",
  16#0fdd# => X"55577777",
  16#0fde# => X"55555555",
  16#0fdf# => X"55555555",
  16#0fe0# => X"55555555",
  16#0fe1# => X"FFFFFFF5",
  16#0fe2# => X"555555FF",
  16#0fe3# => X"55555555",
  16#0fe4# => X"F5555555",
  16#0fe5# => X"FFFFFFFF",
  16#0fe6# => X"5555555F",
  16#0fe7# => X"55555555",
  16#0fe8# => X"55555555",
  16#0fe9# => X"00077555",
  16#0fea# => X"55555500",
  16#0feb# => X"55555555",
  16#0fec# => X"55555555",
  16#0fed# => X"FFFFFFF5",
  16#0fee# => X"FFFFFFFF",
  16#0fef# => X"00FFFFFF",
  16#0ff0# => X"00000000",
  16#0ff1# => X"00000000",
  16#0ff2# => X"00000000",
  16#0ff3# => X"77770000",
  16#0ff4# => X"FFFFFFFF",
  16#0ff5# => X"FFFFFFFF",
  16#0ff6# => X"FFFFFFFF",
  16#0ff7# => X"FFFFFFFF",
  16#0ff8# => X"FFFFFFFF",
  16#0ff9# => X"7777707F",
  16#0ffa# => X"77777777",
  16#0ffb# => X"87777777",
  16#0ffc# => X"88888888",
  16#0ffd# => X"88888888",
  16#0ffe# => X"FFFFFFF8",
  16#0fff# => X"FFFFFFFF",
  16#1000# => X"FFFFFFFF",
  16#1001# => X"FFFFFFFF",
  16#1002# => X"FFFFFFFF",
  16#1003# => X"708FFFFF",
  16#1004# => X"77777777",
  16#1005# => X"55577777",
  16#1006# => X"55555555",
  16#1007# => X"55555555",
  16#1008# => X"55555555",
  16#1009# => X"FFFFFFF5",
  16#100a# => X"5555555F",
  16#100b# => X"55555555",
  16#100c# => X"F5555555",
  16#100d# => X"FFFFFFFF",
  16#100e# => X"55555555",
  16#100f# => X"55555555",
  16#1010# => X"55555555",
  16#1011# => X"00075555",
  16#1012# => X"55555500",
  16#1013# => X"55555555",
  16#1014# => X"55555555",
  16#1015# => X"FFFFFFF5",
  16#1016# => X"FFFFFFFF",
  16#1017# => X"00FFFFFF",
  16#1018# => X"00000000",
  16#1019# => X"00000000",
  16#101a# => X"00000000",
  16#101b# => X"77770000",
  16#101c# => X"FFFFFFFF",
  16#101d# => X"FFFFFFFF",
  16#101e# => X"FFFFFFFF",
  16#101f# => X"FFFFFFFF",
  16#1020# => X"FFFFFFFF",
  16#1021# => X"7777707F",
  16#1022# => X"77777777",
  16#1023# => X"87777777",
  16#1024# => X"88888888",
  16#1025# => X"88888888",
  16#1026# => X"FFFFFF88",
  16#1027# => X"FFFFFFFF",
  16#1028# => X"FFFFFFFF",
  16#1029# => X"FFFFFFFF",
  16#102a# => X"FFFFFFFF",
  16#102b# => X"708FFFFF",
  16#102c# => X"77777777",
  16#102d# => X"55577777",
  16#102e# => X"55555555",
  16#102f# => X"55555555",
  16#1030# => X"55555555",
  16#1031# => X"FFFFFFF5",
  16#1032# => X"5555555F",
  16#1033# => X"55555555",
  16#1034# => X"F5555555",
  16#1035# => X"FFFFFFFF",
  16#1036# => X"55555555",
  16#1037# => X"55555555",
  16#1038# => X"55555555",
  16#1039# => X"00075555",
  16#103a# => X"55555500",
  16#103b# => X"55555555",
  16#103c# => X"55555555",
  16#103d# => X"FFFFFFF5",
  16#103e# => X"FFFFFFFF",
  16#103f# => X"00FFFFFF",
  16#1040# => X"00000000",
  16#1041# => X"00000000",
  16#1042# => X"00000000",
  16#1043# => X"77770000",
  16#1044# => X"FFFFFFF8",
  16#1045# => X"FFFFFFFF",
  16#1046# => X"FFFFFFFF",
  16#1047# => X"FFFFFFFF",
  16#1048# => X"FFFFFFFF",
  16#1049# => X"7777708F",
  16#104a# => X"77777777",
  16#104b# => X"87777777",
  16#104c# => X"88888888",
  16#104d# => X"88888888",
  16#104e# => X"FFFFFF88",
  16#104f# => X"FFFFFFFF",
  16#1050# => X"FFFFFFFF",
  16#1051# => X"FFFFFFFF",
  16#1052# => X"FFFFFFFF",
  16#1053# => X"708FFFFF",
  16#1054# => X"77777777",
  16#1055# => X"55577777",
  16#1056# => X"55555555",
  16#1057# => X"55555555",
  16#1058# => X"55555555",
  16#1059# => X"FFFFFF55",
  16#105a# => X"5555555F",
  16#105b# => X"55555555",
  16#105c# => X"F5555555",
  16#105d# => X"FFFFFFFF",
  16#105e# => X"55555555",
  16#105f# => X"50055555",
  16#1060# => X"55555555",
  16#1061# => X"00075555",
  16#1062# => X"55555500",
  16#1063# => X"55555555",
  16#1064# => X"55555555",
  16#1065# => X"FFFFFFF5",
  16#1066# => X"FFFFFFFF",
  16#1067# => X"00FFFFFF",
  16#1068# => X"00000000",
  16#1069# => X"00000000",
  16#106a# => X"00000000",
  16#106b# => X"77770000",
  16#106c# => X"FFFFFFF8",
  16#106d# => X"FFFFFFFF",
  16#106e# => X"FFFFFFFF",
  16#106f# => X"FFFFFFFF",
  16#1070# => X"FFFFFFFF",
  16#1071# => X"7777708F",
  16#1072# => X"77777777",
  16#1073# => X"87777777",
  16#1074# => X"88888888",
  16#1075# => X"88888888",
  16#1076# => X"FFFFFF88",
  16#1077# => X"FFFFFFFF",
  16#1078# => X"FFFFFFFF",
  16#1079# => X"FFFFFFFF",
  16#107a# => X"FFFFFFFF",
  16#107b# => X"77EFFFFF",
  16#107c# => X"77777777",
  16#107d# => X"55577777",
  16#107e# => X"55555555",
  16#107f# => X"55577755",
  16#1080# => X"55555555",
  16#1081# => X"FFFFFF55",
  16#1082# => X"5555555F",
  16#1083# => X"55555555",
  16#1084# => X"F5555555",
  16#1085# => X"5FFFFFFF",
  16#1086# => X"55555555",
  16#1087# => X"00005555",
  16#1088# => X"55555555",
  16#1089# => X"00055555",
  16#108a# => X"55555500",
  16#108b# => X"55555555",
  16#108c# => X"55555555",
  16#108d# => X"FFFFFFF5",
  16#108e# => X"FFFFFFFF",
  16#108f# => X"00FFFFFF",
  16#1090# => X"00000000",
  16#1091# => X"00000000",
  16#1092# => X"00000000",
  16#1093# => X"07770000",
  16#1094# => X"FFFFFFF8",
  16#1095# => X"FFFFFFFF",
  16#1096# => X"FFFFFFFF",
  16#1097# => X"FFFFFFFF",
  16#1098# => X"FFFFFFFF",
  16#1099# => X"7777708F",
  16#109a# => X"77777777",
  16#109b# => X"77777777",
  16#109c# => X"88888888",
  16#109d# => X"88888888",
  16#109e# => X"FFFFFF88",
  16#109f# => X"FFFFFFFF",
  16#10a0# => X"FFFFFFFF",
  16#10a1# => X"FFFFFFFF",
  16#10a2# => X"FFFFFFFF",
  16#10a3# => X"77FFFFFF",
  16#10a4# => X"77777777",
  16#10a5# => X"55577777",
  16#10a6# => X"55555555",
  16#10a7# => X"55777755",
  16#10a8# => X"55555555",
  16#10a9# => X"FFFFFF55",
  16#10aa# => X"5555555F",
  16#10ab# => X"555F5555",
  16#10ac# => X"F5555555",
  16#10ad# => X"5FFFFFFF",
  16#10ae# => X"55555555",
  16#10af# => X"00005555",
  16#10b0# => X"55555555",
  16#10b1# => X"00055555",
  16#10b2# => X"55555500",
  16#10b3# => X"F5555555",
  16#10b4# => X"FFFFFFFF",
  16#10b5# => X"FFFFFFFF",
  16#10b6# => X"FFFFFFFF",
  16#10b7# => X"00FFFFFF",
  16#10b8# => X"00000000",
  16#10b9# => X"00000000",
  16#10ba# => X"00000000",
  16#10bb# => X"07777000",
  16#10bc# => X"FFFFFFF8",
  16#10bd# => X"FFFFFFFF",
  16#10be# => X"FFFFFFFF",
  16#10bf# => X"FFFFFFFF",
  16#10c0# => X"FFFFFFFF",
  16#10c1# => X"7777708F",
  16#10c2# => X"77777777",
  16#10c3# => X"77777777",
  16#10c4# => X"88888888",
  16#10c5# => X"88888888",
  16#10c6# => X"FFFFFF88",
  16#10c7# => X"FFFFFFFF",
  16#10c8# => X"FFFFFFFF",
  16#10c9# => X"FFFFFFFF",
  16#10ca# => X"FFFFFFFF",
  16#10cb# => X"07FFFFFF",
  16#10cc# => X"77777777",
  16#10cd# => X"55577777",
  16#10ce# => X"55555555",
  16#10cf# => X"55777755",
  16#10d0# => X"55555555",
  16#10d1# => X"FFFFFF55",
  16#10d2# => X"5555555F",
  16#10d3# => X"555F5555",
  16#10d4# => X"F5555555",
  16#10d5# => X"5FFFFFFF",
  16#10d6# => X"55555555",
  16#10d7# => X"00005555",
  16#10d8# => X"55555555",
  16#10d9# => X"00055555",
  16#10da# => X"55555500",
  16#10db# => X"F5555555",
  16#10dc# => X"FFFFFFFF",
  16#10dd# => X"FFFFFFFF",
  16#10de# => X"FFFFFFFF",
  16#10df# => X"00FFFFFF",
  16#10e0# => X"00000000",
  16#10e1# => X"00000000",
  16#10e2# => X"00000000",
  16#10e3# => X"07777000",
  16#10e4# => X"FFFFFFF8",
  16#10e5# => X"FFFFFFFF",
  16#10e6# => X"FFFFFFFF",
  16#10e7# => X"FFFFFFFF",
  16#10e8# => X"FFFFFFFF",
  16#10e9# => X"777777FF",
  16#10ea# => X"77777777",
  16#10eb# => X"77777777",
  16#10ec# => X"33888888",
  16#10ed# => X"88888333",
  16#10ee# => X"FFFFFF88",
  16#10ef# => X"FFFFFFFF",
  16#10f0# => X"FFFFFFFF",
  16#10f1# => X"FFFFFFFF",
  16#10f2# => X"FFFFFFFF",
  16#10f3# => X"07FFFFFF",
  16#10f4# => X"77777777",
  16#10f5# => X"55577777",
  16#10f6# => X"55555555",
  16#10f7# => X"55777755",
  16#10f8# => X"55555555",
  16#10f9# => X"FFFFFF55",
  16#10fa# => X"5555555F",
  16#10fb# => X"555F5555",
  16#10fc# => X"55555555",
  16#10fd# => X"5FFFFFFF",
  16#10fe# => X"55555555",
  16#10ff# => X"00005555",
  16#1100# => X"55555555",
  16#1101# => X"00055555",
  16#1102# => X"55555500",
  16#1103# => X"F5555555",
  16#1104# => X"FFFFFFFF",
  16#1105# => X"FFFFFFFF",
  16#1106# => X"FFFFFFFF",
  16#1107# => X"00FFFFFF",
  16#1108# => X"00000000",
  16#1109# => X"00000000",
  16#110a# => X"00000000",
  16#110b# => X"07777000",
  16#110c# => X"FFFFFFF8",
  16#110d# => X"FFFFFFFF",
  16#110e# => X"FFFFFFFF",
  16#110f# => X"FFFFFFFF",
  16#1110# => X"FFFFFFFF",
  16#1111# => X"777707FF",
  16#1112# => X"77777777",
  16#1113# => X"77777777",
  16#1114# => X"33338887",
  16#1115# => X"83333333",
  16#1116# => X"FFFFFF88",
  16#1117# => X"FFFFFFFF",
  16#1118# => X"FFFFFFFF",
  16#1119# => X"FFFFFFFF",
  16#111a# => X"FFFFFFFF",
  16#111b# => X"07FFFFFF",
  16#111c# => X"77777777",
  16#111d# => X"55577777",
  16#111e# => X"55555555",
  16#111f# => X"55777755",
  16#1120# => X"55555555",
  16#1121# => X"77788855",
  16#1122# => X"55555555",
  16#1123# => X"55585555",
  16#1124# => X"55555555",
  16#1125# => X"5FFFFFFF",
  16#1126# => X"55555555",
  16#1127# => X"00005555",
  16#1128# => X"55555555",
  16#1129# => X"00055555",
  16#112a# => X"55555500",
  16#112b# => X"F5555555",
  16#112c# => X"FFFFFFFF",
  16#112d# => X"FFFFFFFF",
  16#112e# => X"FFFFFFFF",
  16#112f# => X"77FFFFFF",
  16#1130# => X"00000007",
  16#1131# => X"00000000",
  16#1132# => X"00000000",
  16#1133# => X"33333333",
  16#1134# => X"FFF33333",
  16#1135# => X"FFFFFFFF",
  16#1136# => X"FFFFFFFF",
  16#1137# => X"FFFFFFFF",
  16#1138# => X"FFFFFFFF",
  16#1139# => X"777707FF",
  16#113a# => X"77777777",
  16#113b# => X"33777777",
  16#113c# => X"33333333",
  16#113d# => X"33333333",
  16#113e# => X"FFFFFF83",
  16#113f# => X"FFFFFFFF",
  16#1140# => X"FFFFFFFF",
  16#1141# => X"FFFFFFFF",
  16#1142# => X"FFFFFFFF",
  16#1143# => X"07FFFFFF",
  16#1144# => X"77777777",
  16#1145# => X"55577777",
  16#1146# => X"55555555",
  16#1147# => X"55887755",
  16#1148# => X"55555555",
  16#1149# => X"00000755",
  16#114a# => X"55555555",
  16#114b# => X"55505555",
  16#114c# => X"55555555",
  16#114d# => X"5FFFFFFF",
  16#114e# => X"55555555",
  16#114f# => X"00005555",
  16#1150# => X"55555555",
  16#1151# => X"00055555",
  16#1152# => X"55555500",
  16#1153# => X"F5555555",
  16#1154# => X"FFFFFFFF",
  16#1155# => X"FFFFFFFF",
  16#1156# => X"FFFFFFFF",
  16#1157# => X"FFFFFFFF",
  16#1158# => X"077888FF",
  16#1159# => X"00000000",
  16#115a# => X"33300000",
  16#115b# => X"33333333",
  16#115c# => X"33333333",
  16#115d# => X"FFFFFFF3",
  16#115e# => X"FFFFFFFF",
  16#115f# => X"FFFFFFFF",
  16#1160# => X"FFFFFFFF",
  16#1161# => X"777707FF",
  16#1162# => X"77777777",
  16#1163# => X"33333377",
  16#1164# => X"33333333",
  16#1165# => X"33333333",
  16#1166# => X"FFFFF333",
  16#1167# => X"FFFFFFFF",
  16#1168# => X"FFFFFFFF",
  16#1169# => X"FFFFFFFF",
  16#116a# => X"FFFFFFFF",
  16#116b# => X"07FFFFFF",
  16#116c# => X"77777777",
  16#116d# => X"55577777",
  16#116e# => X"55555555",
  16#116f# => X"55FFFF55",
  16#1170# => X"55555555",
  16#1171# => X"77777755",
  16#1172# => X"55555555",
  16#1173# => X"55505555",
  16#1174# => X"55555555",
  16#1175# => X"5FFFFFFF",
  16#1176# => X"55555555",
  16#1177# => X"00005555",
  16#1178# => X"55555555",
  16#1179# => X"00055555",
  16#117a# => X"55555500",
  16#117b# => X"F5555555",
  16#117c# => X"FFFFFFFF",
  16#117d# => X"FFFFFFFF",
  16#117e# => X"FFFFFFFF",
  16#117f# => X"FFFFFFFF",
  16#1180# => X"FFFFFFFF",
  16#1181# => X"00007788",
  16#1182# => X"33333000",
  16#1183# => X"33333333",
  16#1184# => X"33333333",
  16#1185# => X"FFFFF333",
  16#1186# => X"FFFFFFFF",
  16#1187# => X"FFFFFFFF",
  16#1188# => X"FFFFFFFF",
  16#1189# => X"777707FF",
  16#118a# => X"77777777",
  16#118b# => X"33333333",
  16#118c# => X"33333333",
  16#118d# => X"33333333",
  16#118e# => X"FFF33333",
  16#118f# => X"FFFFFFFF",
  16#1190# => X"FFFFFFFF",
  16#1191# => X"FFFFFFFF",
  16#1192# => X"FFFFFFFF",
  16#1193# => X"07FFFFFF",
  16#1194# => X"77777777",
  16#1195# => X"55577777",
  16#1196# => X"55555555",
  16#1197# => X"55FFFF55",
  16#1198# => X"55555555",
  16#1199# => X"77777755",
  16#119a# => X"55555555",
  16#119b# => X"55505555",
  16#119c# => X"55555555",
  16#119d# => X"5FFF8870",
  16#119e# => X"55555555",
  16#119f# => X"00005555",
  16#11a0# => X"55555555",
  16#11a1# => X"00055555",
  16#11a2# => X"55555500",
  16#11a3# => X"F5555555",
  16#11a4# => X"FFFFFFFF",
  16#11a5# => X"FFFFFFFF",
  16#11a6# => X"FFFFFFFF",
  16#11a7# => X"FFFFFFFF",
  16#11a8# => X"FFFFFFFF",
  16#11a9# => X"7788FFFF",
  16#11aa# => X"33333330",
  16#11ab# => X"33333333",
  16#11ac# => X"33333333",
  16#11ad# => X"FFF33333",
  16#11ae# => X"FFFFFFFF",
  16#11af# => X"FFFFFFFF",
  16#11b0# => X"FFFFFFFF",
  16#11b1# => X"777708FF",
  16#11b2# => X"33777777",
  16#11b3# => X"33333333",
  16#11b4# => X"33333333",
  16#11b5# => X"33333333",
  16#11b6# => X"FF333333",
  16#11b7# => X"FFFFFFFF",
  16#11b8# => X"FFFFFFFF",
  16#11b9# => X"FFFFFFFF",
  16#11ba# => X"FFFFFFFF",
  16#11bb# => X"07FFFFFF",
  16#11bc# => X"77777777",
  16#11bd# => X"55577777",
  16#11be# => X"55555555",
  16#11bf# => X"55FFFF55",
  16#11c0# => X"55555555",
  16#11c1# => X"77777755",
  16#11c2# => X"55555555",
  16#11c3# => X"55505555",
  16#11c4# => X"55555555",
  16#11c5# => X"58700000",
  16#11c6# => X"55555555",
  16#11c7# => X"00005555",
  16#11c8# => X"55555555",
  16#11c9# => X"00055555",
  16#11ca# => X"55555500",
  16#11cb# => X"F5555555",
  16#11cc# => X"FFFFFFFF",
  16#11cd# => X"FFFFFFFF",
  16#11ce# => X"FFFFFFFF",
  16#11cf# => X"FFFFFFFF",
  16#11d0# => X"FFFFFFFF",
  16#11d1# => X"FFFFFFFF",
  16#11d2# => X"33333333",
  16#11d3# => X"33333333",
  16#11d4# => X"33333333",
  16#11d5# => X"FF333333",
  16#11d6# => X"FFFFFFFF",
  16#11d7# => X"FFFFFFFF",
  16#11d8# => X"FFFFFFFF",
  16#11d9# => X"770708FF",
  16#11da# => X"33377777",
  16#11db# => X"33333333",
  16#11dc# => X"33333333",
  16#11dd# => X"33333333",
  16#11de# => X"F3333333",
  16#11df# => X"FFFFFFFF",
  16#11e0# => X"FFFFFFFF",
  16#11e1# => X"FFFFFFFF",
  16#11e2# => X"FFFFFFFF",
  16#11e3# => X"07FFFFFF",
  16#11e4# => X"77777777",
  16#11e5# => X"55577777",
  16#11e6# => X"55555555",
  16#11e7# => X"55FFFF55",
  16#11e8# => X"55555555",
  16#11e9# => X"77777755",
  16#11ea# => X"55555555",
  16#11eb# => X"55505555",
  16#11ec# => X"55555555",
  16#11ed# => X"50000005",
  16#11ee# => X"55555555",
  16#11ef# => X"00005555",
  16#11f0# => X"55555555",
  16#11f1# => X"00055555",
  16#11f2# => X"55555500",
  16#11f3# => X"F5555555",
  16#11f4# => X"FFFFFFFF",
  16#11f5# => X"FFFFFFFF",
  16#11f6# => X"FFFFFFFF",
  16#11f7# => X"FFFFFFFF",
  16#11f8# => X"FFFFFFFF",
  16#11f9# => X"3FFFFFFF",
  16#11fa# => X"33333333",
  16#11fb# => X"33333333",
  16#11fc# => X"33333333",
  16#11fd# => X"33333333",
  16#11fe# => X"FFFFFFFF",
  16#11ff# => X"FFFFFFFF",
  16#1200# => X"FFFFFFFF",
  16#1201# => X"770708FF",
  16#1202# => X"33337777",
  16#1203# => X"33333333",
  16#1204# => X"33333333",
  16#1205# => X"33333333",
  16#1206# => X"33333333",
  16#1207# => X"FFFFFFFF",
  16#1208# => X"FFFFFFFF",
  16#1209# => X"FFFFFFFF",
  16#120a# => X"FFFFFFFF",
  16#120b# => X"07FFFFFF",
  16#120c# => X"77777707",
  16#120d# => X"55588777",
  16#120e# => X"55555555",
  16#120f# => X"555FFF55",
  16#1210# => X"55555555",
  16#1211# => X"77777755",
  16#1212# => X"55555555",
  16#1213# => X"55505555",
  16#1214# => X"55555555",
  16#1215# => X"50000005",
  16#1216# => X"55555555",
  16#1217# => X"00005555",
  16#1218# => X"55555555",
  16#1219# => X"00055555",
  16#121a# => X"55555500",
  16#121b# => X"55555555",
  16#121c# => X"55555555",
  16#121d# => X"FFFFFFFF",
  16#121e# => X"FFFFFFFF",
  16#121f# => X"FFFFFFFF",
  16#1220# => X"FFFFFFFF",
  16#1221# => X"33FFFFFF",
  16#1222# => X"33333333",
  16#1223# => X"33333333",
  16#1224# => X"33333333",
  16#1225# => X"33333333",
  16#1226# => X"FFFFFFFF",
  16#1227# => X"FFFFFFFF",
  16#1228# => X"FFFFFFFF",
  16#1229# => X"770708FF",
  16#122a# => X"33333777",
  16#122b# => X"33333333",
  16#122c# => X"33333333",
  16#122d# => X"33333333",
  16#122e# => X"33333333",
  16#122f# => X"FFFFFFF3",
  16#1230# => X"FFFFFFFF",
  16#1231# => X"FFFFFFFF",
  16#1232# => X"FFFFFFFF",
  16#1233# => X"08FFFFFF",
  16#1234# => X"77777777",
  16#1235# => X"555FFF88",
  16#1236# => X"55555555",
  16#1237# => X"55555555",
  16#1238# => X"55555555",
  16#1239# => X"77777775",
  16#123a# => X"55555555",
  16#123b# => X"55000555",
  16#123c# => X"55555555",
  16#123d# => X"50000005",
  16#123e# => X"55555555",
  16#123f# => X"00075555",
  16#1240# => X"55555555",
  16#1241# => X"00055555",
  16#1242# => X"55555500",
  16#1243# => X"55555555",
  16#1244# => X"55555555",
  16#1245# => X"FFFFFFFF",
  16#1246# => X"FFFFFFFF",
  16#1247# => X"FFFFFFFF",
  16#1248# => X"FFFFFFFF",
  16#1249# => X"333FFFFF",
  16#124a# => X"33333333",
  16#124b# => X"33333333",
  16#124c# => X"33333333",
  16#124d# => X"33333333",
  16#124e# => X"FFFFFFF3",
  16#124f# => X"FFFFFFFF",
  16#1250# => X"FFFFFFFF",
  16#1251# => X"700708FF",
  16#1252# => X"33333377",
  16#1253# => X"33333333",
  16#1254# => X"33333333",
  16#1255# => X"33333333",
  16#1256# => X"33333333",
  16#1257# => X"FFFFFFF3",
  16#1258# => X"FFFFFFFF",
  16#1259# => X"FFFFFFFF",
  16#125a# => X"FFFFFFFF",
  16#125b# => X"08FFFFFF",
  16#125c# => X"F8770077",
  16#125d# => X"555FFFFF",
  16#125e# => X"55555555",
  16#125f# => X"55555555",
  16#1260# => X"55555555",
  16#1261# => X"57777775",
  16#1262# => X"55555555",
  16#1263# => X"55000555",
  16#1264# => X"55555555",
  16#1265# => X"50000005",
  16#1266# => X"55555555",
  16#1267# => X"078F5555",
  16#1268# => X"55555555",
  16#1269# => X"00055555",
  16#126a# => X"55555500",
  16#126b# => X"55555555",
  16#126c# => X"55555555",
  16#126d# => X"FFFFFFFF",
  16#126e# => X"FFFFFFFF",
  16#126f# => X"FFFFFFFF",
  16#1270# => X"FFFFFFFF",
  16#1271# => X"333FFFFF",
  16#1272# => X"33333333",
  16#1273# => X"33333333",
  16#1274# => X"33333333",
  16#1275# => X"33333333",
  16#1276# => X"FFFFFF33",
  16#1277# => X"FFFFFFFF",
  16#1278# => X"FFFFFFFF",
  16#1279# => X"000008FF",
  16#127a# => X"33333337",
  16#127b# => X"33333333",
  16#127c# => X"33333333",
  16#127d# => X"33333333",
  16#127e# => X"33333333",
  16#127f# => X"FFFFFF33",
  16#1280# => X"FFFFFFFF",
  16#1281# => X"FFFFFFFF",
  16#1282# => X"FFFFFFFF",
  16#1283# => X"08FFFFFF",
  16#1284# => X"FFF88700",
  16#1285# => X"555FFFFF",
  16#1286# => X"55555555",
  16#1287# => X"55555555",
  16#1288# => X"55555555",
  16#1289# => X"57777777",
  16#128a# => X"55555555",
  16#128b# => X"55000555",
  16#128c# => X"55555555",
  16#128d# => X"50000005",
  16#128e# => X"55555555",
  16#128f# => X"8FFF5555",
  16#1290# => X"07700078",
  16#1291# => X"00000070",
  16#1292# => X"55555500",
  16#1293# => X"55555555",
  16#1294# => X"55555555",
  16#1295# => X"FFFFFFFF",
  16#1296# => X"FFFFFFFF",
  16#1297# => X"FFFFFFFF",
  16#1298# => X"FFFFFFFF",
  16#1299# => X"3333FFFF",
  16#129a# => X"33333333",
  16#129b# => X"33333333",
  16#129c# => X"33333333",
  16#129d# => X"33333333",
  16#129e# => X"FFFFF333",
  16#129f# => X"FFFFFFFF",
  16#12a0# => X"FFFFFFFF",
  16#12a1# => X"000008FF",
  16#12a2# => X"33333333",
  16#12a3# => X"33333333",
  16#12a4# => X"33333333",
  16#12a5# => X"33333333",
  16#12a6# => X"33333333",
  16#12a7# => X"FFFFFF33",
  16#12a8# => X"FFFFFFFF",
  16#12a9# => X"FFFFFFFF",
  16#12aa# => X"FFFFFFFF",
  16#12ab# => X"08FFFFFF",
  16#12ac# => X"FFFFFF87",
  16#12ad# => X"555FFFFF",
  16#12ae# => X"55555555",
  16#12af# => X"55555555",
  16#12b0# => X"FF555555",
  16#12b1# => X"57777777",
  16#12b2# => X"55555555",
  16#12b3# => X"55000555",
  16#12b4# => X"55555555",
  16#12b5# => X"50000005",
  16#12b6# => X"55555555",
  16#12b7# => X"FFFF5555",
  16#12b8# => X"700778FF",
  16#12b9# => X"00007707",
  16#12ba# => X"55555500",
  16#12bb# => X"55555555",
  16#12bc# => X"55555555",
  16#12bd# => X"FFFFFFFF",
  16#12be# => X"FFFFFFFF",
  16#12bf# => X"FFFFFFFF",
  16#12c0# => X"FFFFFFFF",
  16#12c1# => X"3333FFFF",
  16#12c2# => X"33333333",
  16#12c3# => X"33333333",
  16#12c4# => X"33333333",
  16#12c5# => X"33333333",
  16#12c6# => X"FFFFF333",
  16#12c7# => X"FFFFFFFF",
  16#12c8# => X"FFFFFFFF",
  16#12c9# => X"30007FFF",
  16#12ca# => X"33333333",
  16#12cb# => X"33333333",
  16#12cc# => X"33883333",
  16#12cd# => X"33333333",
  16#12ce# => X"33333333",
  16#12cf# => X"FFFFF333",
  16#12d0# => X"FFFFFFFF",
  16#12d1# => X"FFFFFFFF",
  16#12d2# => X"FFFFFFFF",
  16#12d3# => X"88FFFFFF",
  16#12d4# => X"FFFFFFFF",
  16#12d5# => X"555FFFFF",
  16#12d6# => X"55555555",
  16#12d7# => X"55555555",
  16#12d8# => X"8FFFF555",
  16#12d9# => X"57777777",
  16#12da# => X"55555555",
  16#12db# => X"55077555",
  16#12dc# => X"55555555",
  16#12dd# => X"50000005",
  16#12de# => X"55555555",
  16#12df# => X"FFFF5555",
  16#12e0# => X"0788FFFF",
  16#12e1# => X"00000000",
  16#12e2# => X"55555500",
  16#12e3# => X"55555555",
  16#12e4# => X"55555555",
  16#12e5# => X"FFFFFFFF",
  16#12e6# => X"FFFFFFFF",
  16#12e7# => X"FFFFFFFF",
  16#12e8# => X"FFFFFFFF",
  16#12e9# => X"33333FFF",
  16#12ea# => X"33333333",
  16#12eb# => X"33333333",
  16#12ec# => X"33333333",
  16#12ed# => X"33333333",
  16#12ee# => X"FFFF3333",
  16#12ef# => X"FFFFFFFF",
  16#12f0# => X"FFFFFFFF",
  16#12f1# => X"30007FFF",
  16#12f2# => X"33333333",
  16#12f3# => X"33333333",
  16#12f4# => X"33883333",
  16#12f5# => X"33333333",
  16#12f6# => X"33333333",
  16#12f7# => X"FFFFF333",
  16#12f8# => X"FFFFFFFF",
  16#12f9# => X"FFFFFFFF",
  16#12fa# => X"FFFFFFFF",
  16#12fb# => X"FFFFFFFF",
  16#12fc# => X"FFFFFFFF",
  16#12fd# => X"555FFFFF",
  16#12fe# => X"55555555",
  16#12ff# => X"55555555",
  16#1300# => X"8F555555",
  16#1301# => X"57777777",
  16#1302# => X"55555555",
  16#1303# => X"55777555",
  16#1304# => X"55555555",
  16#1305# => X"50000055",
  16#1306# => X"55555555",
  16#1307# => X"FFFF5555",
  16#1308# => X"8FFFFFFF",
  16#1309# => X"00000077",
  16#130a# => X"55555500",
  16#130b# => X"55555555",
  16#130c# => X"55555555",
  16#130d# => X"FFFFFFFF",
  16#130e# => X"FFFFFFFF",
  16#130f# => X"FFFFFFFF",
  16#1310# => X"FFFFFFFF",
  16#1311# => X"33333FFF",
  16#1312# => X"33333333",
  16#1313# => X"33F33333",
  16#1314# => X"33333333",
  16#1315# => X"33333333",
  16#1316# => X"FFFF3333",
  16#1317# => X"FFFFFFFF",
  16#1318# => X"FFFFFFFF",
  16#1319# => X"30707FFF",
  16#131a# => X"33333333",
  16#131b# => X"33333333",
  16#131c# => X"38888333",
  16#131d# => X"33333333",
  16#131e# => X"33333333",
  16#131f# => X"FFFFF333",
  16#1320# => X"FFFFFFFF",
  16#1321# => X"FFFFFFFF",
  16#1322# => X"FFFFFFFF",
  16#1323# => X"FF878FFF",
  16#1324# => X"FFFFFFFF",
  16#1325# => X"555FFFFF",
  16#1326# => X"55555555",
  16#1327# => X"55555555",
  16#1328# => X"55555555",
  16#1329# => X"57777777",
  16#132a# => X"55555555",
  16#132b# => X"55777555",
  16#132c# => X"55555555",
  16#132d# => X"50000055",
  16#132e# => X"55555555",
  16#132f# => X"FFFF5555",
  16#1330# => X"FFFFFFFF",
  16#1331# => X"000078FF",
  16#1332# => X"55555500",
  16#1333# => X"55555555",
  16#1334# => X"55555555",
  16#1335# => X"FFFFFFFF",
  16#1336# => X"FFFFFFFF",
  16#1337# => X"FFFFFFFF",
  16#1338# => X"FFFFFFFF",
  16#1339# => X"33333FFF",
  16#133a# => X"33333333",
  16#133b# => X"3FFF3333",
  16#133c# => X"33333333",
  16#133d# => X"33333333",
  16#133e# => X"FFFF3333",
  16#133f# => X"FFFFFFFF",
  16#1340# => X"FFFFFFFF",
  16#1341# => X"30707FFF",
  16#1342# => X"33333333",
  16#1343# => X"33333333",
  16#1344# => X"38888333",
  16#1345# => X"33333333",
  16#1346# => X"33333333",
  16#1347# => X"FFFFF333",
  16#1348# => X"FFFFFFFF",
  16#1349# => X"FFFFFFFF",
  16#134a# => X"FFFFFFFF",
  16#134b# => X"FF80078F",
  16#134c# => X"FFFFFFFF",
  16#134d# => X"555FFFFF",
  16#134e# => X"55555555",
  16#134f# => X"55555555",
  16#1350# => X"55555555",
  16#1351# => X"57777777",
  16#1352# => X"55555555",
  16#1353# => X"55777555",
  16#1354# => X"55555555",
  16#1355# => X"50000055",
  16#1356# => X"55555555",
  16#1357# => X"FFFF5555",
  16#1358# => X"FFFFFFFF",
  16#1359# => X"0788FFFF",
  16#135a# => X"55555500",
  16#135b# => X"55555555",
  16#135c# => X"55555555",
  16#135d# => X"FFFFFFFF",
  16#135e# => X"FFFFFFFF",
  16#135f# => X"FFFFFFFF",
  16#1360# => X"FFFFFFFF",
  16#1361# => X"33333FFF",
  16#1362# => X"33333333",
  16#1363# => X"FFFF3333",
  16#1364# => X"33333333",
  16#1365# => X"33333333",
  16#1366# => X"FFFF3333",
  16#1367# => X"FFFFFFFF",
  16#1368# => X"FFFFFFFF",
  16#1369# => X"30007FFF",
  16#136a# => X"33333333",
  16#136b# => X"33333333",
  16#136c# => X"38888333",
  16#136d# => X"33333333",
  16#136e# => X"33333333",
  16#136f# => X"FFFF3333",
  16#1370# => X"FFFFFFFF",
  16#1371# => X"FFFFFFFF",
  16#1372# => X"88FFFFFF",
  16#1373# => X"FF807007",
  16#1374# => X"FFFFFFFF",
  16#1375# => X"555FFFFF",
  16#1376# => X"55555555",
  16#1377# => X"555FFF55",
  16#1378# => X"55555555",
  16#1379# => X"55777775",
  16#137a# => X"55555555",
  16#137b# => X"55777555",
  16#137c# => X"55555555",
  16#137d# => X"50000055",
  16#137e# => X"55555555",
  16#137f# => X"FFFF5555",
  16#1380# => X"FFFFFFFF",
  16#1381# => X"8FFFFFFF",
  16#1382# => X"55555507",
  16#1383# => X"55555555",
  16#1384# => X"55555555",
  16#1385# => X"FFFFFFFF",
  16#1386# => X"FFFFFFFF",
  16#1387# => X"FFFFFFFF",
  16#1388# => X"FFFFFFFF",
  16#1389# => X"333333FF",
  16#138a# => X"33333333",
  16#138b# => X"FFFFF333",
  16#138c# => X"33333333",
  16#138d# => X"33333333",
  16#138e# => X"FFF33333",
  16#138f# => X"FFFFFFFF",
  16#1390# => X"FFFFFFFF",
  16#1391# => X"33007FFF",
  16#1392# => X"33333333",
  16#1393# => X"33333333",
  16#1394# => X"38888333",
  16#1395# => X"33333333",
  16#1396# => X"33333333",
  16#1397# => X"FFFF3333",
  16#1398# => X"FFFFFFFF",
  16#1399# => X"FFFFFFFF",
  16#139a# => X"7788FFFF",
  16#139b# => X"FF800077",
  16#139c# => X"FFFFFFFF",
  16#139d# => X"555FFFFF",
  16#139e# => X"55555555",
  16#139f# => X"55FFFF55",
  16#13a0# => X"55555555",
  16#13a1# => X"55777755",
  16#13a2# => X"55555555",
  16#13a3# => X"55777555",
  16#13a4# => X"55555555",
  16#13a5# => X"50000055",
  16#13a6# => X"55555555",
  16#13a7# => X"FFFF5555",
  16#13a8# => X"FFFFFFFF",
  16#13a9# => X"FFFFFFFF",
  16#13aa# => X"5555558F",
  16#13ab# => X"F5555555",
  16#13ac# => X"FFFFFFFF",
  16#13ad# => X"FFFFFFFF",
  16#13ae# => X"FFFFFFFF",
  16#13af# => X"FFFFFFFF",
  16#13b0# => X"FFFFFFFF",
  16#13b1# => X"333333FF",
  16#13b2# => X"33333333",
  16#13b3# => X"FFFFF333",
  16#13b4# => X"33333333",
  16#13b5# => X"33333333",
  16#13b6# => X"FFF33333",
  16#13b7# => X"FFFFFFFF",
  16#13b8# => X"FFFFFFFF",
  16#13b9# => X"33007FFF",
  16#13ba# => X"33333333",
  16#13bb# => X"33333333",
  16#13bc# => X"38887333",
  16#13bd# => X"33333333",
  16#13be# => X"33333333",
  16#13bf# => X"FFFF3333",
  16#13c0# => X"FFFFFFFF",
  16#13c1# => X"FFFFFFFF",
  16#13c2# => X"777788FF",
  16#13c3# => X"FF800777",
  16#13c4# => X"FFFFFFFF",
  16#13c5# => X"555FFFFF",
  16#13c6# => X"55555555",
  16#13c7# => X"55FFFF55",
  16#13c8# => X"55555555",
  16#13c9# => X"55777755",
  16#13ca# => X"55555555",
  16#13cb# => X"57777755",
  16#13cc# => X"55555555",
  16#13cd# => X"50000055",
  16#13ce# => X"55555555",
  16#13cf# => X"FFFF5555",
  16#13d0# => X"55555555",
  16#13d1# => X"FFF55555",
  16#13d2# => X"555555FF",
  16#13d3# => X"F5555555",
  16#13d4# => X"FFFFFFFF",
  16#13d5# => X"FFFFFFFF",
  16#13d6# => X"FFFFFFFF",
  16#13d7# => X"FFFFFFFF",
  16#13d8# => X"FFFFFFFF",
  16#13d9# => X"333333FF",
  16#13da# => X"33333333",
  16#13db# => X"FFFFF333",
  16#13dc# => X"33333333",
  16#13dd# => X"33333333",
  16#13de# => X"FFF33333",
  16#13df# => X"FFFFFFFF",
  16#13e0# => X"FFFFFFFF",
  16#13e1# => X"33007FFF",
  16#13e2# => X"33333333",
  16#13e3# => X"33333333",
  16#13e4# => X"38887333",
  16#13e5# => X"33333333",
  16#13e6# => X"33333333",
  16#13e7# => X"FFFF3333",
  16#13e8# => X"FFFFFFFF",
  16#13e9# => X"8FFFFFFF",
  16#13ea# => X"77777888",
  16#13eb# => X"FF800777",
  16#13ec# => X"FFFFFFFF",
  16#13ed# => X"555FFFFF",
  16#13ee# => X"55555555",
  16#13ef# => X"55FFFF55",
  16#13f0# => X"55555555",
  16#13f1# => X"55777755",
  16#13f2# => X"55555555",
  16#13f3# => X"57777755",
  16#13f4# => X"55555555",
  16#13f5# => X"50000555",
  16#13f6# => X"55555555",
  16#13f7# => X"FFFF5555",
  16#13f8# => X"55555555",
  16#13f9# => X"FFF55555",
  16#13fa# => X"555555FF",
  16#13fb# => X"F5555555",
  16#13fc# => X"FFFFFFFF",
  16#13fd# => X"FFFFFFFF",
  16#13fe# => X"FFFFFFFF",
  16#13ff# => X"FFFFFFFF",
  16#1400# => X"FFFFFFFF",
  16#1401# => X"333333FF",
  16#1402# => X"33333333",
  16#1403# => X"FFFFF333",
  16#1404# => X"33333333",
  16#1405# => X"33333333",
  16#1406# => X"FF833333",
  16#1407# => X"FFFFFFFF",
  16#1408# => X"FFFFFFFF",
  16#1409# => X"33707FFF",
  16#140a# => X"33333333",
  16#140b# => X"33333333",
  16#140c# => X"38887333",
  16#140d# => X"33333333",
  16#140e# => X"33333333",
  16#140f# => X"FFFF3333",
  16#1410# => X"FFFFFFFF",
  16#1411# => X"888FFFFF",
  16#1412# => X"77778888",
  16#1413# => X"FF800077",
  16#1414# => X"FFFFFFFF",
  16#1415# => X"555FFFFF",
  16#1416# => X"55555555",
  16#1417# => X"55FFFF55",
  16#1418# => X"55555555",
  16#1419# => X"55777755",
  16#141a# => X"55555555",
  16#141b# => X"57777755",
  16#141c# => X"55555555",
  16#141d# => X"50000555",
  16#141e# => X"55555555",
  16#141f# => X"FFFF5555",
  16#1420# => X"55555555",
  16#1421# => X"FFF55555",
  16#1422# => X"555555FF",
  16#1423# => X"F5555555",
  16#1424# => X"FFFFFFFF",
  16#1425# => X"FFFFFFFF",
  16#1426# => X"FFFFFFFF",
  16#1427# => X"FFFFFFFF",
  16#1428# => X"FFFFFFFF",
  16#1429# => X"333333FF",
  16#142a# => X"33333333",
  16#142b# => X"FFFFF333",
  16#142c# => X"33333333",
  16#142d# => X"33333333",
  16#142e# => X"87733333",
  16#142f# => X"FF88FFFF",
  16#1430# => X"FFFFFFFF",
  16#1431# => X"33008FFF",
  16#1432# => X"33333333",
  16#1433# => X"33333333",
  16#1434# => X"38887333",
  16#1435# => X"33333333",
  16#1436# => X"33333333",
  16#1437# => X"FFFF3333",
  16#1438# => X"FFFFFFFF",
  16#1439# => X"888888FF",
  16#143a# => X"77777888",
  16#143b# => X"FF800077",
  16#143c# => X"FFFFFFFF",
  16#143d# => X"555FFFFF",
  16#143e# => X"55555555",
  16#143f# => X"55FFFF55",
  16#1440# => X"55555555",
  16#1441# => X"55777755",
  16#1442# => X"55555555",
  16#1443# => X"55555555",
  16#1444# => X"55555555",
  16#1445# => X"50000555",
  16#1446# => X"55555555",
  16#1447# => X"FFFF5555",
  16#1448# => X"55555555",
  16#1449# => X"FFF55555",
  16#144a# => X"555555FF",
  16#144b# => X"F5555555",
  16#144c# => X"FFFFFFFF",
  16#144d# => X"FFFFFFFF",
  16#144e# => X"FFFFFFFF",
  16#144f# => X"FFFFFFFF",
  16#1450# => X"FFFFFFFF",
  16#1451# => X"333333FF",
  16#1452# => X"33333333",
  16#1453# => X"FFFFF333",
  16#1454# => X"33333333",
  16#1455# => X"33333333",
  16#1456# => X"07733333",
  16#1457# => X"87007777",
  16#1458# => X"FFFFFFF8",
  16#1459# => X"33007FFF",
  16#145a# => X"33333333",
  16#145b# => X"33333333",
  16#145c# => X"38887333",
  16#145d# => X"33333333",
  16#145e# => X"33333333",
  16#145f# => X"FFFF3333",
  16#1460# => X"8FFFFFFF",
  16#1461# => X"88888888",
  16#1462# => X"77777888",
  16#1463# => X"FF800077",
  16#1464# => X"FFFFFFFF",
  16#1465# => X"555FFFFF",
  16#1466# => X"55555555",
  16#1467# => X"55FFFF55",
  16#1468# => X"55555555",
  16#1469# => X"55777755",
  16#146a# => X"55555555",
  16#146b# => X"55555555",
  16#146c# => X"55555555",
  16#146d# => X"50000555",
  16#146e# => X"55555555",
  16#146f# => X"FFFF5555",
  16#1470# => X"55555555",
  16#1471# => X"FFF55555",
  16#1472# => X"555555FF",
  16#1473# => X"F5555555",
  16#1474# => X"FFFFFFFF",
  16#1475# => X"FFFFFFFF",
  16#1476# => X"FFFFFFFF",
  16#1477# => X"FFFFFFFF",
  16#1478# => X"FFFFFFFF",
  16#1479# => X"333333FF",
  16#147a# => X"33333333",
  16#147b# => X"FFFFF333",
  16#147c# => X"33333333",
  16#147d# => X"33333333",
  16#147e# => X"77733333",
  16#147f# => X"00777777",
  16#1480# => X"FFFF8870",
  16#1481# => X"33707FFF",
  16#1482# => X"33333333",
  16#1483# => X"33333333",
  16#1484# => X"38887333",
  16#1485# => X"33333333",
  16#1486# => X"33333333",
  16#1487# => X"FFFF3333",
  16#1488# => X"8888FFFF",
  16#1489# => X"88888888",
  16#148a# => X"77777888",
  16#148b# => X"FF800077",
  16#148c# => X"FFFFFFFF",
  16#148d# => X"555FFFFF",
  16#148e# => X"55555555",
  16#148f# => X"55FFFF55",
  16#1490# => X"55555555",
  16#1491# => X"55777755",
  16#1492# => X"55555555",
  16#1493# => X"55555555",
  16#1494# => X"55555555",
  16#1495# => X"50000555",
  16#1496# => X"55555555",
  16#1497# => X"FFFF5555",
  16#1498# => X"55555555",
  16#1499# => X"FFF55555",
  16#149a# => X"555555FF",
  16#149b# => X"75555555",
  16#149c# => X"FFFFFFF8",
  16#149d# => X"FFFFFFFF",
  16#149e# => X"FFFFFFFF",
  16#149f# => X"FFFFFFFF",
  16#14a0# => X"FFFFFFFF",
  16#14a1# => X"333333FF",
  16#14a2# => X"33333333",
  16#14a3# => X"3FFFF333",
  16#14a4# => X"33333333",
  16#14a5# => X"33333333",
  16#14a6# => X"77733333",
  16#14a7# => X"77777777",
  16#14a8# => X"88770007",
  16#14a9# => X"33008FFF",
  16#14aa# => X"33333333",
  16#14ab# => X"33333333",
  16#14ac# => X"38887333",
  16#14ad# => X"33333333",
  16#14ae# => X"33333333",
  16#14af# => X"FFFF3333",
  16#14b0# => X"8888888F",
  16#14b1# => X"88888888",
  16#14b2# => X"77777888",
  16#14b3# => X"FF800077",
  16#14b4# => X"FFFFFFFF",
  16#14b5# => X"555FFFFF",
  16#14b6# => X"55555555",
  16#14b7# => X"55FFFF55",
  16#14b8# => X"55555555",
  16#14b9# => X"55577755",
  16#14ba# => X"55555555",
  16#14bb# => X"55555555",
  16#14bc# => X"55555555",
  16#14bd# => X"50000555",
  16#14be# => X"55555555",
  16#14bf# => X"FFFF5555",
  16#14c0# => X"55555555",
  16#14c1# => X"FFF55555",
  16#14c2# => X"555555FF",
  16#14c3# => X"05555555",
  16#14c4# => X"FFF88700",
  16#14c5# => X"FFFFFFFF",
  16#14c6# => X"FFFFFFFF",
  16#14c7# => X"FFFFFFFF",
  16#14c8# => X"FFFFFFFF",
  16#14c9# => X"333333FF",
  16#14ca# => X"33333333",
  16#14cb# => X"3FFFF333",
  16#14cc# => X"33333333",
  16#14cd# => X"33333333",
  16#14ce# => X"77733333",
  16#14cf# => X"00777777",
  16#14d0# => X"00000000",
  16#14d1# => X"338FFF87",
  16#14d2# => X"33333333",
  16#14d3# => X"33333333",
  16#14d4# => X"38887333",
  16#14d5# => X"33333333",
  16#14d6# => X"33333333",
  16#14d7# => X"88FF3333",
  16#14d8# => X"88888888",
  16#14d9# => X"88888888",
  16#14da# => X"77777888",
  16#14db# => X"FFF00077",
  16#14dc# => X"FFFFFFFF",
  16#14dd# => X"555FFFFF",
  16#14de# => X"55555555",
  16#14df# => X"55FFFF55",
  16#14e0# => X"55555555",
  16#14e1# => X"55577755",
  16#14e2# => X"55555555",
  16#14e3# => X"55555555",
  16#14e4# => X"55555555",
  16#14e5# => X"50000555",
  16#14e6# => X"55555555",
  16#14e7# => X"FFFF5555",
  16#14e8# => X"55555555",
  16#14e9# => X"FFF55555",
  16#14ea# => X"555555FF",
  16#14eb# => X"05555555",
  16#14ec# => X"88770000",
  16#14ed# => X"FFFFFFFF",
  16#14ee# => X"FFFFFFFF",
  16#14ef# => X"FFFFFFFF",
  16#14f0# => X"FFFFFFFF",
  16#14f1# => X"333333FF",
  16#14f2# => X"33333333",
  16#14f3# => X"3FFFF333",
  16#14f4# => X"33333333",
  16#14f5# => X"33333333",
  16#14f6# => X"77773333",
  16#14f7# => X"00777777",
  16#14f8# => X"00000000",
  16#14f9# => X"33FFF800",
  16#14fa# => X"33333333",
  16#14fb# => X"33333333",
  16#14fc# => X"38887333",
  16#14fd# => X"33333333",
  16#14fe# => X"33333333",
  16#14ff# => X"88883333",
  16#1500# => X"88888888",
  16#1501# => X"88888888",
  16#1502# => X"77777888",
  16#1503# => X"FFF00077",
  16#1504# => X"FFFFFFFF",
  16#1505# => X"555FFFFF",
  16#1506# => X"55555555",
  16#1507# => X"55FFFF55",
  16#1508# => X"55555555",
  16#1509# => X"55577755",
  16#150a# => X"55555555",
  16#150b# => X"55555555",
  16#150c# => X"55555555",
  16#150d# => X"50005555",
  16#150e# => X"55555555",
  16#150f# => X"FFFF5555",
  16#1510# => X"55555555",
  16#1511# => X"FFF55555",
  16#1512# => X"555555FF",
  16#1513# => X"05555555",
  16#1514# => X"77777700",
  16#1515# => X"FFFF8887",
  16#1516# => X"FFFFFFFF",
  16#1517# => X"FFFFFFFF",
  16#1518# => X"FFFFFFFF",
  16#1519# => X"333333FF",
  16#151a# => X"33333333",
  16#151b# => X"33FFF333",
  16#151c# => X"33333333",
  16#151d# => X"33333333",
  16#151e# => X"77773333",
  16#151f# => X"00777777",
  16#1520# => X"00000000",
  16#1521# => X"33FFF800",
  16#1522# => X"33333333",
  16#1523# => X"33333333",
  16#1524# => X"38888333",
  16#1525# => X"33333333",
  16#1526# => X"33333333",
  16#1527# => X"88883333",
  16#1528# => X"88888888",
  16#1529# => X"88888888",
  16#152a# => X"77777888",
  16#152b# => X"FFF70077",
  16#152c# => X"FFFFFFFF",
  16#152d# => X"555FFFFF",
  16#152e# => X"55555555",
  16#152f# => X"55FFFF55",
  16#1530# => X"55555555",
  16#1531# => X"55577755",
  16#1532# => X"55555555",
  16#1533# => X"55555555",
  16#1534# => X"55555555",
  16#1535# => X"50005555",
  16#1536# => X"55555555",
  16#1537# => X"FFFF5555",
  16#1538# => X"55555555",
  16#1539# => X"FFF55555",
  16#153a# => X"555555FF",
  16#153b# => X"05555555",
  16#153c# => X"77777700",
  16#153d# => X"88888888",
  16#153e# => X"FFFFFFFF",
  16#153f# => X"FFFFFFFF",
  16#1540# => X"FFFFFFFF",
  16#1541# => X"333333FF",
  16#1542# => X"33333333",
  16#1543# => X"33FFF333",
  16#1544# => X"33333333",
  16#1545# => X"33333333",
  16#1546# => X"77773333",
  16#1547# => X"00777777",
  16#1548# => X"00000000",
  16#1549# => X"33FFF800",
  16#154a# => X"33333333",
  16#154b# => X"33333333",
  16#154c# => X"3FFFF333",
  16#154d# => X"33333333",
  16#154e# => X"33333333",
  16#154f# => X"88883333",
  16#1550# => X"88888888",
  16#1551# => X"88888888",
  16#1552# => X"77777888",
  16#1553# => X"FFF70077",
  16#1554# => X"FFFFFFFF",
  16#1555# => X"555FFFFF",
  16#1556# => X"55555555",
  16#1557# => X"55FFFF55",
  16#1558# => X"55555555",
  16#1559# => X"55577755",
  16#155a# => X"55555555",
  16#155b# => X"55555555",
  16#155c# => X"55555555",
  16#155d# => X"50005555",
  16#155e# => X"55555555",
  16#155f# => X"FFFF5555",
  16#1560# => X"55555555",
  16#1561# => X"FFF55555",
  16#1562# => X"555555FF",
  16#1563# => X"05555555",
  16#1564# => X"77777700",
  16#1565# => X"88888888",
  16#1566# => X"FF888888",
  16#1567# => X"FFFFFFFF",
  16#1568# => X"FFFFFFFF",
  16#1569# => X"FFFFFFFF",
  16#156a# => X"FFFFFFFF",
  16#156b# => X"333FFFFF",
  16#156c# => X"33333333",
  16#156d# => X"33333333",
  16#156e# => X"77773333",
  16#156f# => X"00777777",
  16#1570# => X"00000000",
  16#1571# => X"33FFF800",
  16#1572# => X"33333333",
  16#1573# => X"33333333",
  16#1574# => X"3FFFF333",
  16#1575# => X"33333333",
  16#1576# => X"33333333",
  16#1577# => X"88883333",
  16#1578# => X"88888888",
  16#1579# => X"88888888",
  16#157a# => X"77777888",
  16#157b# => X"FFF70007",
  16#157c# => X"FFFFFFFF",
  16#157d# => X"555FFFFF",
  16#157e# => X"55555555",
  16#157f# => X"55FFFF55",
  16#1580# => X"55555555",
  16#1581# => X"55577755",
  16#1582# => X"55555555",
  16#1583# => X"55555555",
  16#1584# => X"55555555",
  16#1585# => X"80005555",
  16#1586# => X"55555555",
  16#1587# => X"FFFF5555",
  16#1588# => X"55555555",
  16#1589# => X"FFFF5555",
  16#158a# => X"555555FF",
  16#158b# => X"55555555",
  16#158c# => X"55555555",
  16#158d# => X"88888855",
  16#158e# => X"88888888",
  16#158f# => X"FF888888",
  16#1590# => X"FFFFFFFF",
  16#1591# => X"FFFFFFFF",
  16#1592# => X"FFFFFFFF",
  16#1593# => X"333FFFFF",
  16#1594# => X"33333333",
  16#1595# => X"33333333",
  16#1596# => X"77777333",
  16#1597# => X"00077777",
  16#1598# => X"00000000",
  16#1599# => X"33FFF800",
  16#159a# => X"33333333",
  16#159b# => X"33333333",
  16#159c# => X"3FFFF333",
  16#159d# => X"33333333",
  16#159e# => X"33333333",
  16#159f# => X"88883333",
  16#15a0# => X"88888888",
  16#15a1# => X"88888888",
  16#15a2# => X"77777888",
  16#15a3# => X"FFF70007",
  16#15a4# => X"FFFFFFFF",
  16#15a5# => X"555FFFFF",
  16#15a6# => X"55555555",
  16#15a7# => X"55FFFF55",
  16#15a8# => X"55555555",
  16#15a9# => X"55577755",
  16#15aa# => X"55555555",
  16#15ab# => X"77777775",
  16#15ac# => X"55555555",
  16#15ad# => X"80005555",
  16#15ae# => X"55555555",
  16#15af# => X"5FF55555",
  16#15b0# => X"55555555",
  16#15b1# => X"FFFF5555",
  16#15b2# => X"555555FF",
  16#15b3# => X"55555555",
  16#15b4# => X"55555555",
  16#15b5# => X"88888855",
  16#15b6# => X"88888888",
  16#15b7# => X"FF888888",
  16#15b8# => X"FFFFFFFF",
  16#15b9# => X"FFFFFFFF",
  16#15ba# => X"FFFFFFFF",
  16#15bb# => X"333FFFFF",
  16#15bc# => X"33333333",
  16#15bd# => X"33333333",
  16#15be# => X"77777333",
  16#15bf# => X"00077777",
  16#15c0# => X"00000000",
  16#15c1# => X"33FFF800",
  16#15c2# => X"33333333",
  16#15c3# => X"33333333",
  16#15c4# => X"3FFFF333",
  16#15c5# => X"33333333",
  16#15c6# => X"33333333",
  16#15c7# => X"88883333",
  16#15c8# => X"88888888",
  16#15c9# => X"88888888",
  16#15ca# => X"77777888",
  16#15cb# => X"FFF70007",
  16#15cc# => X"FFFFFFFF",
  16#15cd# => X"555FFFFF",
  16#15ce# => X"55555555",
  16#15cf# => X"55FFFF55",
  16#15d0# => X"55555555",
  16#15d1# => X"55557755",
  16#15d2# => X"55555555",
  16#15d3# => X"77777775",
  16#15d4# => X"55555555",
  16#15d5# => X"80005555",
  16#15d6# => X"55555555",
  16#15d7# => X"55555555",
  16#15d8# => X"55555555",
  16#15d9# => X"FFFF5555",
  16#15da# => X"555555FF",
  16#15db# => X"55555555",
  16#15dc# => X"55555555",
  16#15dd# => X"88888855",
  16#15de# => X"88888888",
  16#15df# => X"FF888888",
  16#15e0# => X"FFFFFFFF",
  16#15e1# => X"FFFFFFFF",
  16#15e2# => X"FFFFFFFF",
  16#15e3# => X"3333FFFF",
  16#15e4# => X"33333333",
  16#15e5# => X"33333333",
  16#15e6# => X"77777333",
  16#15e7# => X"00077777",
  16#15e8# => X"00000000",
  16#15e9# => X"33FFF800",
  16#15ea# => X"33333333",
  16#15eb# => X"33333333",
  16#15ec# => X"3FFFF333",
  16#15ed# => X"33333333",
  16#15ee# => X"33333333",
  16#15ef# => X"88883333",
  16#15f0# => X"88888888",
  16#15f1# => X"88888888",
  16#15f2# => X"77777888",
  16#15f3# => X"FFF70007",
  16#15f4# => X"FFFFFFFF",
  16#15f5# => X"555FFFFF",
  16#15f6# => X"55555555",
  16#15f7# => X"55FFFF55",
  16#15f8# => X"55555555",
  16#15f9# => X"55557755",
  16#15fa# => X"55555555",
  16#15fb# => X"77777775",
  16#15fc# => X"55555555",
  16#15fd# => X"F7055555",
  16#15fe# => X"55555555",
  16#15ff# => X"55555555",
  16#1600# => X"55555555",
  16#1601# => X"FFFF5555",
  16#1602# => X"555555FF",
  16#1603# => X"55555555",
  16#1604# => X"55555555",
  16#1605# => X"88888855",
  16#1606# => X"88888888",
  16#1607# => X"FF888888",
  16#1608# => X"FFFFFFFF",
  16#1609# => X"FFFFFFFF",
  16#160a# => X"FFFFFFFF",
  16#160b# => X"3333FFFF",
  16#160c# => X"33333333",
  16#160d# => X"33333333",
  16#160e# => X"77777733",
  16#160f# => X"00077777",
  16#1610# => X"00000000",
  16#1611# => X"33FFF800",
  16#1612# => X"33333333",
  16#1613# => X"33333333",
  16#1614# => X"3FFFF333",
  16#1615# => X"33333333",
  16#1616# => X"33333333",
  16#1617# => X"88883333",
  16#1618# => X"88888888",
  16#1619# => X"88888888",
  16#161a# => X"77777888",
  16#161b# => X"FFF70007",
  16#161c# => X"FFFFFFFF",
  16#161d# => X"555FFFFF",
  16#161e# => X"55555555",
  16#161f# => X"55FFFF55",
  16#1620# => X"55555555",
  16#1621# => X"55557755",
  16#1622# => X"55555555",
  16#1623# => X"77777775",
  16#1624# => X"55555555",
  16#1625# => X"F7055555",
  16#1626# => X"5555555F",
  16#1627# => X"55555555",
  16#1628# => X"55555555",
  16#1629# => X"FFFFF555",
  16#162a# => X"555555FF",
  16#162b# => X"55555555",
  16#162c# => X"55555555",
  16#162d# => X"88888855",
  16#162e# => X"88888888",
  16#162f# => X"FF888888",
  16#1630# => X"FFFFFFFF",
  16#1631# => X"FFFFFFFF",
  16#1632# => X"FFFFFFFF",
  16#1633# => X"33333FFF",
  16#1634# => X"33333333",
  16#1635# => X"33333333",
  16#1636# => X"77777733",
  16#1637# => X"00007777",
  16#1638# => X"00000000",
  16#1639# => X"33FFF800",
  16#163a# => X"33333333",
  16#163b# => X"33333333",
  16#163c# => X"3FFFF333",
  16#163d# => X"33333333",
  16#163e# => X"33333333",
  16#163f# => X"88883333",
  16#1640# => X"88888888",
  16#1641# => X"88888888",
  16#1642# => X"77777788",
  16#1643# => X"FFF80007",
  16#1644# => X"FFFFFFFF",
  16#1645# => X"555FFFFF",
  16#1646# => X"55555555",
  16#1647# => X"55FFFF55",
  16#1648# => X"55555555",
  16#1649# => X"55557755",
  16#164a# => X"55555555",
  16#164b# => X"77777775",
  16#164c# => X"55555555",
  16#164d# => X"F7055555",
  16#164e# => X"5555555F",
  16#164f# => X"55555555",
  16#1650# => X"55555555",
  16#1651# => X"FFFFFF55",
  16#1652# => X"555555FF",
  16#1653# => X"55555555",
  16#1654# => X"55555555",
  16#1655# => X"88888855",
  16#1656# => X"88888888",
  16#1657# => X"FF888888",
  16#1658# => X"FFFFFFFF",
  16#1659# => X"FFFFFFFF",
  16#165a# => X"FFFFFFFF",
  16#165b# => X"33333FFF",
  16#165c# => X"33333333",
  16#165d# => X"33333333",
  16#165e# => X"77777733",
  16#165f# => X"00007777",
  16#1660# => X"00000000",
  16#1661# => X"33FFF800",
  16#1662# => X"33333333",
  16#1663# => X"33333333",
  16#1664# => X"3FFFF333",
  16#1665# => X"33333333",
  16#1666# => X"33333333",
  16#1667# => X"88883333",
  16#1668# => X"88888888",
  16#1669# => X"88888888",
  16#166a# => X"77777788",
  16#166b# => X"FFF80000",
  16#166c# => X"FFFFFFFF",
  16#166d# => X"555FFFFF",
  16#166e# => X"55555555",
  16#166f# => X"55FFFF55",
  16#1670# => X"55555555",
  16#1671# => X"55557755",
  16#1672# => X"55555555",
  16#1673# => X"77777775",
  16#1674# => X"55555555",
  16#1675# => X"F7055555",
  16#1676# => X"555555FF",
  16#1677# => X"55555555",
  16#1678# => X"55555555",
  16#1679# => X"FFFFFF55",
  16#167a# => X"555555FF",
  16#167b# => X"55555555",
  16#167c# => X"55555555",
  16#167d# => X"88888855",
  16#167e# => X"88888888",
  16#167f# => X"FF888888",
  16#1680# => X"FFFFFFFF",
  16#1681# => X"FFFFFFFF",
  16#1682# => X"FFFFFFFF",
  16#1683# => X"333333FF",
  16#1684# => X"33333333",
  16#1685# => X"33333333",
  16#1686# => X"77777773",
  16#1687# => X"00007777",
  16#1688# => X"00000000",
  16#1689# => X"33FFFF00",
  16#168a# => X"33333333",
  16#168b# => X"33333333",
  16#168c# => X"3FFFF333",
  16#168d# => X"33333333",
  16#168e# => X"33333333",
  16#168f# => X"88883333",
  16#1690# => X"88888888",
  16#1691# => X"88888888",
  16#1692# => X"77777788",
  16#1693# => X"FFF80000",
  16#1694# => X"FFFFFFFF",
  16#1695# => X"555FFFFF",
  16#1696# => X"55555555",
  16#1697# => X"55FFFF55",
  16#1698# => X"55555555",
  16#1699# => X"55557755",
  16#169a# => X"55555555",
  16#169b# => X"77777775",
  16#169c# => X"55555555",
  16#169d# => X"F8055555",
  16#169e# => X"55555FFF",
  16#169f# => X"55555555",
  16#16a0# => X"55555555",
  16#16a1# => X"FFFFFFFF",
  16#16a2# => X"555555FF",
  16#16a3# => X"55555555",
  16#16a4# => X"55555555",
  16#16a5# => X"88888855",
  16#16a6# => X"88888888",
  16#16a7# => X"FF888888",
  16#16a8# => X"FFFFFFFF",
  16#16a9# => X"FFFFFFFF",
  16#16aa# => X"FFFFFFFF",
  16#16ab# => X"333333FF",
  16#16ac# => X"33333333",
  16#16ad# => X"33333333",
  16#16ae# => X"77777773",
  16#16af# => X"00007777",
  16#16b0# => X"00000000",
  16#16b1# => X"33FFFF00",
  16#16b2# => X"33333333",
  16#16b3# => X"33333333",
  16#16b4# => X"3FFFF333",
  16#16b5# => X"33333333",
  16#16b6# => X"33333333",
  16#16b7# => X"88883333",
  16#16b8# => X"88888888",
  16#16b9# => X"88888888",
  16#16ba# => X"77777788",
  16#16bb# => X"FFF80000",
  16#16bc# => X"FFFFFFFF",
  16#16bd# => X"555FFFFF",
  16#16be# => X"55555555",
  16#16bf# => X"55FFFF55",
  16#16c0# => X"55555555",
  16#16c1# => X"55557755",
  16#16c2# => X"55555555",
  16#16c3# => X"77777775",
  16#16c4# => X"55555555",
  16#16c5# => X"F8055555",
  16#16c6# => X"5555FFFF",
  16#16c7# => X"55555555",
  16#16c8# => X"F5555555",
  16#16c9# => X"FFFFFFFF",
  16#16ca# => X"555555FF",
  16#16cb# => X"55555555",
  16#16cc# => X"55555555",
  16#16cd# => X"88888855",
  16#16ce# => X"88888888",
  16#16cf# => X"FF888888",
  16#16d0# => X"FFFFFFFF",
  16#16d1# => X"FFFFFFFF",
  16#16d2# => X"FFFFFFFF",
  16#16d3# => X"3333333F",
  16#16d4# => X"33333333",
  16#16d5# => X"33333333",
  16#16d6# => X"77777777",
  16#16d7# => X"00000777",
  16#16d8# => X"00000000",
  16#16d9# => X"33FFFF70",
  16#16da# => X"33333333",
  16#16db# => X"33333333",
  16#16dc# => X"3FFFF333",
  16#16dd# => X"33333333",
  16#16de# => X"33333333",
  16#16df# => X"88883333",
  16#16e0# => X"88888888",
  16#16e1# => X"88888888",
  16#16e2# => X"77777788",
  16#16e3# => X"FFF80000",
  16#16e4# => X"FFFFFFFF",
  16#16e5# => X"555FFFFF",
  16#16e6# => X"55555555",
  16#16e7# => X"55FFFF55",
  16#16e8# => X"55555555",
  16#16e9# => X"55557755",
  16#16ea# => X"55555555",
  16#16eb# => X"77777775",
  16#16ec# => X"55555555",
  16#16ed# => X"F8055555",
  16#16ee# => X"55FFFFFF",
  16#16ef# => X"55555555",
  16#16f0# => X"FFF55555",
  16#16f1# => X"FFFFFFFF",
  16#16f2# => X"555555FF",
  16#16f3# => X"55555555",
  16#16f4# => X"55555555",
  16#16f5# => X"88888855",
  16#16f6# => X"88888888",
  16#16f7# => X"FF888888",
  16#16f8# => X"FFFFFFFF",
  16#16f9# => X"FFFFFFFF",
  16#16fa# => X"FFFFFFFF",
  16#16fb# => X"33333333",
  16#16fc# => X"33333333",
  16#16fd# => X"33333333",
  16#16fe# => X"77777777",
  16#16ff# => X"00000777",
  16#1700# => X"00000000",
  16#1701# => X"33FFFF70",
  16#1702# => X"33333333",
  16#1703# => X"33333333",
  16#1704# => X"3FFFF333",
  16#1705# => X"33333333",
  16#1706# => X"33333333",
  16#1707# => X"88883333",
  16#1708# => X"88888888",
  16#1709# => X"88888888",
  16#170a# => X"77777788",
  16#170b# => X"FFFF0000",
  16#170c# => X"FFFFFFFF",
  16#170d# => X"FFFFFFFF",
  16#170e# => X"FFFFFFFF",
  16#170f# => X"FFFFFFFF",
  16#1710# => X"777FFFFF",
  16#1711# => X"77777777",
  16#1712# => X"77777777",
  16#1713# => X"77777777",
  16#1714# => X"77777777",
  16#1715# => X"FF777777",
  16#1716# => X"FFFFFFFF",
  16#1717# => X"5555555F",
  16#1718# => X"FFFFF555",
  16#1719# => X"FFFFFFFF",
  16#171a# => X"FFFFFFFF",
  16#171b# => X"000008FF",
  16#171c# => X"87777770",
  16#171d# => X"88888888",
  16#171e# => X"88888888",
  16#171f# => X"FF888888",
  16#1720# => X"FFFFFFFF",
  16#1721# => X"FFFFFFFF",
  16#1722# => X"3FFFFFFF",
  16#1723# => X"33333333",
  16#1724# => X"33333333",
  16#1725# => X"73333333",
  16#1726# => X"77777777",
  16#1727# => X"00000077",
  16#1728# => X"00000000",
  16#1729# => X"33FFFF70",
  16#172a# => X"33333333",
  16#172b# => X"33333333",
  16#172c# => X"3FFFF333",
  16#172d# => X"33333333",
  16#172e# => X"33333333",
  16#172f# => X"88883333",
  16#1730# => X"88888888",
  16#1731# => X"88888888",
  16#1732# => X"07777788",
  16#1733# => X"FFFF0000",
  16#1734# => X"FFFFFFFF",
  16#1735# => X"FFFFFFFF",
  16#1736# => X"FFFFFFFF",
  16#1737# => X"FFFFFFFF",
  16#1738# => X"777FFFFF",
  16#1739# => X"77777777",
  16#173a# => X"77777777",
  16#173b# => X"77777777",
  16#173c# => X"77777777",
  16#173d# => X"FF707777",
  16#173e# => X"FFFFFFFF",
  16#173f# => X"FFFFFFFF",
  16#1740# => X"FFFFFFFF",
  16#1741# => X"FFFFFFFF",
  16#1742# => X"FFFFFFFF",
  16#1743# => X"000007FF",
  16#1744# => X"87777770",
  16#1745# => X"88888888",
  16#1746# => X"88888888",
  16#1747# => X"FF888888",
  16#1748# => X"FFFFFFFF",
  16#1749# => X"FFFFFFFF",
  16#174a# => X"3FFFFFFF",
  16#174b# => X"33333333",
  16#174c# => X"33333333",
  16#174d# => X"73333333",
  16#174e# => X"77777777",
  16#174f# => X"00000077",
  16#1750# => X"00000000",
  16#1751# => X"33FFFF70",
  16#1752# => X"33333333",
  16#1753# => X"33333333",
  16#1754# => X"3FFFF333",
  16#1755# => X"33333333",
  16#1756# => X"33333333",
  16#1757# => X"88883333",
  16#1758# => X"88888888",
  16#1759# => X"88888888",
  16#175a# => X"07777778",
  16#175b# => X"FFFF7000",
  16#175c# => X"FFFFFFFF",
  16#175d# => X"FFFFFFFF",
  16#175e# => X"FFFFFFFF",
  16#175f# => X"FFFFFFFF",
  16#1760# => X"007FFFFF",
  16#1761# => X"77777777",
  16#1762# => X"77777777",
  16#1763# => X"77077777",
  16#1764# => X"77777777",
  16#1765# => X"FF707777",
  16#1766# => X"FFFFFFFF",
  16#1767# => X"FFFFFFFF",
  16#1768# => X"FFFFFFFF",
  16#1769# => X"FFFFFFFF",
  16#176a# => X"FFFFFFFF",
  16#176b# => X"000007FF",
  16#176c# => X"87777770",
  16#176d# => X"88888888",
  16#176e# => X"88888888",
  16#176f# => X"FF888888",
  16#1770# => X"FFFFFFFF",
  16#1771# => X"FFFFFFFF",
  16#1772# => X"33FFFFFF",
  16#1773# => X"33333333",
  16#1774# => X"33333333",
  16#1775# => X"77333333",
  16#1776# => X"77777777",
  16#1777# => X"00000077",
  16#1778# => X"00000000",
  16#1779# => X"33FFFF70",
  16#177a# => X"33333333",
  16#177b# => X"33333333",
  16#177c# => X"3FFFF333",
  16#177d# => X"33333333",
  16#177e# => X"33333333",
  16#177f# => X"88883333",
  16#1780# => X"88888888",
  16#1781# => X"88888888",
  16#1782# => X"07777778",
  16#1783# => X"FFFF7000",
  16#1784# => X"FFFFFFFF",
  16#1785# => X"FFFFFFFF",
  16#1786# => X"FFFFFFFF",
  16#1787# => X"FFFFFFFF",
  16#1788# => X"8888FFFF",
  16#1789# => X"FFFFFF88",
  16#178a# => X"FFFFFFFF",
  16#178b# => X"788888FF",
  16#178c# => X"77000077",
  16#178d# => X"FF707777",
  16#178e# => X"FFFFFFFF",
  16#178f# => X"FFFFFFFF",
  16#1790# => X"FFFFFFFF",
  16#1791# => X"FFFFFFFF",
  16#1792# => X"FFFFFFFF",
  16#1793# => X"000007FF",
  16#1794# => X"87777770",
  16#1795# => X"88888888",
  16#1796# => X"88888888",
  16#1797# => X"FF888888",
  16#1798# => X"FFFFFFFF",
  16#1799# => X"FFFFFFFF",
  16#179a# => X"33FFFFFF",
  16#179b# => X"33333333",
  16#179c# => X"33333333",
  16#179d# => X"77333333",
  16#179e# => X"77777777",
  16#179f# => X"00000007",
  16#17a0# => X"00000000",
  16#17a1# => X"33FFFF70",
  16#17a2# => X"33333333",
  16#17a3# => X"33333333",
  16#17a4# => X"3FFFF333",
  16#17a5# => X"33333333",
  16#17a6# => X"33333333",
  16#17a7# => X"88883333",
  16#17a8# => X"88888888",
  16#17a9# => X"88888888",
  16#17aa# => X"07777778",
  16#17ab# => X"FFFF7000",
  16#17ac# => X"FFFFFFFF",
  16#17ad# => X"FFFFFFFF",
  16#17ae# => X"FFFFFFFF",
  16#17af# => X"FFFFFFFF",
  16#17b0# => X"FFFFFFFF",
  16#17b1# => X"FFFFFFFF",
  16#17b2# => X"FFFFFFFF",
  16#17b3# => X"FFFFFFFF",
  16#17b4# => X"077888FF",
  16#17b5# => X"FF807700",
  16#17b6# => X"FFFFFFFF",
  16#17b7# => X"FFFFFFFF",
  16#17b8# => X"FFFFFFFF",
  16#17b9# => X"FFFFFFFF",
  16#17ba# => X"FFFFFFFF",
  16#17bb# => X"000007FF",
  16#17bc# => X"87777770",
  16#17bd# => X"88888888",
  16#17be# => X"88888888",
  16#17bf# => X"FF888888",
  16#17c0# => X"FFFFFFFF",
  16#17c1# => X"FFFFFFFF",
  16#17c2# => X"333FFFFF",
  16#17c3# => X"33333333",
  16#17c4# => X"33333333",
  16#17c5# => X"77733333",
  16#17c6# => X"77777777",
  16#17c7# => X"00000007",
  16#17c8# => X"00000000",
  16#17c9# => X"33FFFF80",
  16#17ca# => X"33333333",
  16#17cb# => X"33333333",
  16#17cc# => X"3FFFF333",
  16#17cd# => X"33333333",
  16#17ce# => X"33333333",
  16#17cf# => X"88883333",
  16#17d0# => X"88888888",
  16#17d1# => X"88888888",
  16#17d2# => X"00777778",
  16#17d3# => X"FFFF8000",
  16#17d4# => X"FFFFFFFF",
  16#17d5# => X"FFFFFFFF",
  16#17d6# => X"FFFFFFFF",
  16#17d7# => X"8FFFFFFF",
  16#17d8# => X"FFFF8788",
  16#17d9# => X"FFFFFFFF",
  16#17da# => X"FFFFFFFF",
  16#17db# => X"FFFFFFFF",
  16#17dc# => X"8FFFFFFF",
  16#17dd# => X"FF800778",
  16#17de# => X"FFFFFFFF",
  16#17df# => X"FFFFFFFF",
  16#17e0# => X"FFFFFFFF",
  16#17e1# => X"FFFFFFFF",
  16#17e2# => X"FFFFFFFF",
  16#17e3# => X"000000FF",
  16#17e4# => X"88777777",
  16#17e5# => X"88888888",
  16#17e6# => X"88888888",
  16#17e7# => X"87888888",
  16#17e8# => X"FFFFFF88",
  16#17e9# => X"FFFFFFFF",
  16#17ea# => X"333FFFFF",
  16#17eb# => X"33333333",
  16#17ec# => X"33333333",
  16#17ed# => X"77733333",
  16#17ee# => X"77777777",
  16#17ef# => X"00000000",
  16#17f0# => X"00000000",
  16#17f1# => X"33FFFF80",
  16#17f2# => X"33333333",
  16#17f3# => X"33333333",
  16#17f4# => X"3FFFF333",
  16#17f5# => X"33333333",
  16#17f6# => X"33333333",
  16#17f7# => X"88883333",
  16#17f8# => X"88888888",
  16#17f9# => X"88888888",
  16#17fa# => X"00777778",
  16#17fb# => X"FFFF8000",
  16#17fc# => X"FFFFFFFF",
  16#17fd# => X"FFFFFFFF",
  16#17fe# => X"FFFFFFFF",
  16#17ff# => X"777888FF",
  16#1800# => X"FFFF8070",
  16#1801# => X"FFFFFFFF",
  16#1802# => X"FFFFFFFF",
  16#1803# => X"FFFFFFFF",
  16#1804# => X"FFFFFFFF",
  16#1805# => X"FFF88FFF",
  16#1806# => X"FFFFFFFF",
  16#1807# => X"FFFFFFFF",
  16#1808# => X"FFFFFFFF",
  16#1809# => X"FFFFFFFF",
  16#180a# => X"FFFFFFFF",
  16#180b# => X"000000FF",
  16#180c# => X"88777777",
  16#180d# => X"88888888",
  16#180e# => X"88888888",
  16#180f# => X"00888888",
  16#1810# => X"88777000",
  16#1811# => X"887000FF",
  16#1812# => X"3333FFFF",
  16#1813# => X"33333333",
  16#1814# => X"33333333",
  16#1815# => X"77773333",
  16#1816# => X"77777777",
  16#1817# => X"00000000",
  16#1818# => X"00000000",
  16#1819# => X"33FFFF80",
  16#181a# => X"33333333",
  16#181b# => X"33333333",
  16#181c# => X"3FFFF333",
  16#181d# => X"33333333",
  16#181e# => X"33333333",
  16#181f# => X"88883333",
  16#1820# => X"88888888",
  16#1821# => X"88888888",
  16#1822# => X"00777777",
  16#1823# => X"FFFF8000",
  16#1824# => X"FFFFFFFF",
  16#1825# => X"FFFFFFFF",
  16#1826# => X"8FFFFFFF",
  16#1827# => X"77700778",
  16#1828# => X"FFFFF777",
  16#1829# => X"FFFFFFFF",
  16#182a# => X"FFFFFFFF",
  16#182b# => X"FFFFFFFF",
  16#182c# => X"FFFFFFFF",
  16#182d# => X"FFFFFFFF",
  16#182e# => X"FFFFFFFF",
  16#182f# => X"FFFFFFFF",
  16#1830# => X"FFFFFFFF",
  16#1831# => X"FFFFFFFF",
  16#1832# => X"FFFFFFFF",
  16#1833# => X"0000008F",
  16#1834# => X"88777777",
  16#1835# => X"88888888",
  16#1836# => X"88888888",
  16#1837# => X"00888888",
  16#1838# => X"00000000",
  16#1839# => X"00000000",
  16#183a# => X"33338877",
  16#183b# => X"33333333",
  16#183c# => X"33333333",
  16#183d# => X"77773333",
  16#183e# => X"77777777",
  16#183f# => X"00000000",
  16#1840# => X"00000000",
  16#1841# => X"33FFFF80",
  16#1842# => X"33333333",
  16#1843# => X"33333333",
  16#1844# => X"3FFFF333",
  16#1845# => X"33333333",
  16#1846# => X"33333333",
  16#1847# => X"88883333",
  16#1848# => X"88888888",
  16#1849# => X"88888888",
  16#184a# => X"00777777",
  16#184b# => X"FFFF8000",
  16#184c# => X"FFFFFFFF",
  16#184d# => X"FFFFFFFF",
  16#184e# => X"07788FFF",
  16#184f# => X"77777770",
  16#1850# => X"FFFFF777",
  16#1851# => X"FFFFFFFF",
  16#1852# => X"FFFFFFFF",
  16#1853# => X"FFFFFFFF",
  16#1854# => X"FFFFFFFF",
  16#1855# => X"87FFFFFF",
  16#1856# => X"FFFFFFF8",
  16#1857# => X"FFFFFFFF",
  16#1858# => X"FFFFFFFF",
  16#1859# => X"FFFFFFFF",
  16#185a# => X"FFFFFFFF",
  16#185b# => X"0000008F",
  16#185c# => X"88777777",
  16#185d# => X"88888888",
  16#185e# => X"88888888",
  16#185f# => X"00888888",
  16#1860# => X"00000000",
  16#1861# => X"77000000",
  16#1862# => X"33333007",
  16#1863# => X"33333333",
  16#1864# => X"33333333",
  16#1865# => X"77777333",
  16#1866# => X"07777777",
  16#1867# => X"00000000",
  16#1868# => X"00000000",
  16#1869# => X"33FFFF80",
  16#186a# => X"33333333",
  16#186b# => X"33333333",
  16#186c# => X"3FFFF333",
  16#186d# => X"33333333",
  16#186e# => X"33333333",
  16#186f# => X"88883333",
  16#1870# => X"88888888",
  16#1871# => X"88888888",
  16#1872# => X"00077777",
  16#1873# => X"FFFFF000",
  16#1874# => X"FFFFFFFF",
  16#1875# => X"FFFFFFFF",
  16#1876# => X"77777788",
  16#1877# => X"77777777",
  16#1878# => X"FFFFF777",
  16#1879# => X"FFFFFFFF",
  16#187a# => X"FFFFFFFF",
  16#187b# => X"FFFFFFFF",
  16#187c# => X"FFFFFFFF",
  16#187d# => X"008FFFFF",
  16#187e# => X"FFFF8870",
  16#187f# => X"FFFFFFFF",
  16#1880# => X"FFFFFFFF",
  16#1881# => X"FFFFFFFF",
  16#1882# => X"FFFFFFFF",
  16#1883# => X"7000008F",
  16#1884# => X"88877777",
  16#1885# => X"88888888",
  16#1886# => X"88888888",
  16#1887# => X"00888888",
  16#1888# => X"00000000",
  16#1889# => X"70000000",
  16#188a# => X"33333370",
  16#188b# => X"33333333",
  16#188c# => X"33333333",
  16#188d# => X"77777733",
  16#188e# => X"00777777",
  16#188f# => X"00000000",
  16#1890# => X"00000000",
  16#1891# => X"33FFFF80",
  16#1892# => X"33333333",
  16#1893# => X"33333333",
  16#1894# => X"3FFFF333",
  16#1895# => X"33333333",
  16#1896# => X"33333333",
  16#1897# => X"88883333",
  16#1898# => X"88888888",
  16#1899# => X"88888888",
  16#189a# => X"00077777",
  16#189b# => X"FFFFF000",
  16#189c# => X"FFFFFFFF",
  16#189d# => X"788FFFFF",
  16#189e# => X"77777770",
  16#189f# => X"77777777",
  16#18a0# => X"FFFFF807",
  16#18a1# => X"FFFFFFFF",
  16#18a2# => X"FFFFFFFF",
  16#18a3# => X"FFFFFFFF",
  16#18a4# => X"FFFFFFFF",
  16#18a5# => X"708FFFFF",
  16#18a6# => X"F8770077",
  16#18a7# => X"FFFFFFFF",
  16#18a8# => X"FFFFFFFF",
  16#18a9# => X"FFFFFFFF",
  16#18aa# => X"FFFFFFFF",
  16#18ab# => X"7000008F",
  16#18ac# => X"88877777",
  16#18ad# => X"88888888",
  16#18ae# => X"88888888",
  16#18af# => X"00888888",
  16#18b0# => X"00000000",
  16#18b1# => X"70000000",
  16#18b2# => X"33333377",
  16#18b3# => X"33333333",
  16#18b4# => X"33333333",
  16#18b5# => X"77777733",
  16#18b6# => X"00707777",
  16#18b7# => X"00000000",
  16#18b8# => X"00000000",
  16#18b9# => X"33FFFFF0",
  16#18ba# => X"33333333",
  16#18bb# => X"33333333",
  16#18bc# => X"3FFFF333",
  16#18bd# => X"33333333",
  16#18be# => X"33333333",
  16#18bf# => X"88883333",
  16#18c0# => X"88888888",
  16#18c1# => X"78888888",
  16#18c2# => X"00077777",
  16#18c3# => X"FFFFF700",
  16#18c4# => X"FFFFFFFF",
  16#18c5# => X"770788FF",
  16#18c6# => X"77777777",
  16#18c7# => X"77777777",
  16#18c8# => X"FFFFF807",
  16#18c9# => X"FFFFFFFF",
  16#18ca# => X"FFFFFFFF",
  16#18cb# => X"FFFFFFFF",
  16#18cc# => X"FFFFFFFF",
  16#18cd# => X"708FFFFF",
  16#18ce# => X"70077777",
  16#18cf# => X"FFFFFF88",
  16#18d0# => X"FFFFFFFF",
  16#18d1# => X"FFFFFFFF",
  16#18d2# => X"FFFFFFFF",
  16#18d3# => X"7000008F",
  16#18d4# => X"88877777",
  16#18d5# => X"88888888",
  16#18d6# => X"88888888",
  16#18d7# => X"00888888",
  16#18d8# => X"00000000",
  16#18d9# => X"70000000",
  16#18da# => X"33333337",
  16#18db# => X"33333333",
  16#18dc# => X"33333333",
  16#18dd# => X"77777773",
  16#18de# => X"00077777",
  16#18df# => X"00000000",
  16#18e0# => X"00000000",
  16#18e1# => X"33FFFFF7",
  16#18e2# => X"33333333",
  16#18e3# => X"33333333",
  16#18e4# => X"3FFFF333",
  16#18e5# => X"33333333",
  16#18e6# => X"33333333",
  16#18e7# => X"88883333",
  16#18e8# => X"88888888",
  16#18e9# => X"78888888",
  16#18ea# => X"00077777",
  16#18eb# => X"FFFFF700",
  16#18ec# => X"8FFFFFFF",
  16#18ed# => X"77770078",
  16#18ee# => X"77777777",
  16#18ef# => X"77777777",
  16#18f0# => X"FFFFFF77",
  16#18f1# => X"FFFFFFFF",
  16#18f2# => X"FFFFFFFF",
  16#18f3# => X"FFFFFFFF",
  16#18f4# => X"FFFFFFFF",
  16#18f5# => X"777FFFFF",
  16#18f6# => X"77777777",
  16#18f7# => X"FFF88700",
  16#18f8# => X"FFFFFFFF",
  16#18f9# => X"FFFFFFFF",
  16#18fa# => X"FFFFFFFF",
  16#18fb# => X"7000007F",
  16#18fc# => X"88877777",
  16#18fd# => X"88888888",
  16#18fe# => X"88888888",
  16#18ff# => X"00888888",
  16#1900# => X"00000000",
  16#1901# => X"77000000",
  16#1902# => X"33333337",
  16#1903# => X"33333333",
  16#1904# => X"33333333",
  16#1905# => X"77777773",
  16#1906# => X"00007777",
  16#1907# => X"00000000",
  16#1908# => X"00000000",
  16#1909# => X"33FFFFF7",
  16#190a# => X"33333333",
  16#190b# => X"33333333",
  16#190c# => X"3FFFF333",
  16#190d# => X"33333333",
  16#190e# => X"33333333",
  16#190f# => X"88883333",
  16#1910# => X"88888888",
  16#1911# => X"78888888",
  16#1912# => X"00007777",
  16#1913# => X"FFFFF800",
  16#1914# => X"778FFFFF",
  16#1915# => X"77777770",
  16#1916# => X"77777777",
  16#1917# => X"77777777",
  16#1918# => X"FFFFFF77",
  16#1919# => X"FFFFFFFF",
  16#191a# => X"FFFFFFFF",
  16#191b# => X"FFFFFFFF",
  16#191c# => X"FFFFFFFF",
  16#191d# => X"777FFFFF",
  16#191e# => X"77777777",
  16#191f# => X"F8770777",
  16#1920# => X"FFFFFFFF",
  16#1921# => X"FFFFFFFF",
  16#1922# => X"FFFFFFFF",
  16#1923# => X"7700007F",
  16#1924# => X"88887777",
  16#1925# => X"88888888",
  16#1926# => X"88888888",
  16#1927# => X"00888888",
  16#1928# => X"00000000",
  16#1929# => X"77000000",
  16#192a# => X"33333333",
  16#192b# => X"33333333",
  16#192c# => X"33333333",
  16#192d# => X"77777777",
  16#192e# => X"00000777",
  16#192f# => X"00000000",
  16#1930# => X"00000000",
  16#1931# => X"33FFFFF7",
  16#1932# => X"33333333",
  16#1933# => X"33333333",
  16#1934# => X"3FFFF333",
  16#1935# => X"33333333",
  16#1936# => X"33333333",
  16#1937# => X"88883333",
  16#1938# => X"88888888",
  16#1939# => X"77888888",
  16#193a# => X"00007777",
  16#193b# => X"FFFFF800",
  16#193c# => X"700788FF",
  16#193d# => X"77777777",
  16#193e# => X"77777777",
  16#193f# => X"77777777",
  16#1940# => X"FFFFFF70",
  16#1941# => X"FFFFFFFF",
  16#1942# => X"FFFFFFFF",
  16#1943# => X"FFFFFFFF",
  16#1944# => X"FFFFFFFF",
  16#1945# => X"7778FFFF",
  16#1946# => X"77777777",
  16#1947# => X"70077777",
  16#1948# => X"FFFFFF87",
  16#1949# => X"FFFFFFFF",
  16#194a# => X"FFFFFFFF",
  16#194b# => X"7700000F",
  16#194c# => X"88887777",
  16#194d# => X"88888888",
  16#194e# => X"88888888",
  16#194f# => X"00888888",
  16#1950# => X"00000000",
  16#1951# => X"77700000",
  16#1952# => X"33333333",
  16#1953# => X"33333333",
  16#1954# => X"33333333",
  16#1955# => X"77770788",
  16#1956# => X"00000777",
  16#1957# => X"00000000",
  16#1958# => X"00000000",
  16#1959# => X"33FFFFF8",
  16#195a# => X"33333333",
  16#195b# => X"33333333",
  16#195c# => X"3FFFF333",
  16#195d# => X"33333333",
  16#195e# => X"33333333",
  16#195f# => X"88883333",
  16#1960# => X"88888888",
  16#1961# => X"77888888",
  16#1962# => X"00007777",
  16#1963# => X"FFFFF800",
  16#1964# => X"77700078",
  16#1965# => X"77777777",
  16#1966# => X"77777777",
  16#1967# => X"77777777",
  16#1968# => X"FFFFFF80",
  16#1969# => X"FFFFFFFF",
  16#196a# => X"FFFFFFFF",
  16#196b# => X"FFFFFFFF",
  16#196c# => X"FFFFFFFF",
  16#196d# => X"7708FFFF",
  16#196e# => X"77777777",
  16#196f# => X"77777777",
  16#1970# => X"FFF88700",
  16#1971# => X"FFFFFFFF",
  16#1972# => X"FFFFFFFF",
  16#1973# => X"77700008",
  16#1974# => X"88887777",
  16#1975# => X"88888888",
  16#1976# => X"88888888",
  16#1977# => X"00888888",
  16#1978# => X"00000000",
  16#1979# => X"37700000",
  16#197a# => X"33333333",
  16#197b# => X"33333333",
  16#197c# => X"F3333333",
  16#197d# => X"00078FFF",
  16#197e# => X"00000077",
  16#197f# => X"00000000",
  16#1980# => X"00000000",
  16#1981# => X"33FFFFF8",
  16#1982# => X"33333333",
  16#1983# => X"33333333",
  16#1984# => X"3FFFF333",
  16#1985# => X"33333333",
  16#1986# => X"33333333",
  16#1987# => X"88883333",
  16#1988# => X"88888888",
  16#1989# => X"77888888",
  16#198a# => X"00000777",
  16#198b# => X"078FFF00",
  16#198c# => X"77777000",
  16#198d# => X"77777777",
  16#198e# => X"77777777",
  16#198f# => X"77777777",
  16#1990# => X"FFFFFF80",
  16#1991# => X"FFFFFFFF",
  16#1992# => X"FFFFFFFF",
  16#1993# => X"FFFFFFFF",
  16#1994# => X"FFFFFFFF",
  16#1995# => X"7708FFFF",
  16#1996# => X"77777777",
  16#1997# => X"77777777",
  16#1998# => X"F8700077",
  16#1999# => X"FFFFFFFF",
  16#199a# => X"FFFFFFFF",
  16#199b# => X"77700008",
  16#199c# => X"88888777",
  16#199d# => X"88888888",
  16#199e# => X"88888888",
  16#199f# => X"00888888",
  16#19a0# => X"00000000",
  16#19a1# => X"37770000",
  16#19a2# => X"33333333",
  16#19a3# => X"33333333",
  16#19a4# => X"F3333333",
  16#19a5# => X"788FFFFF",
  16#19a6# => X"00000700",
  16#19a7# => X"00000000",
  16#19a8# => X"00000000",
  16#19a9# => X"33FFFFF8",
  16#19aa# => X"33333333",
  16#19ab# => X"33333333",
  16#19ac# => X"3FFFF333",
  16#19ad# => X"33333333",
  16#19ae# => X"33333333",
  16#19af# => X"88883333",
  16#19b0# => X"88888888",
  16#19b1# => X"77888888",
  16#19b2# => X"00007777",
  16#19b3# => X"00078F87",
  16#19b4# => X"77770000",
  16#19b5# => X"77777777",
  16#19b6# => X"77777777",
  16#19b7# => X"77777777",
  16#19b8# => X"FFFFFFF7",
  16#19b9# => X"FFFFFFFF",
  16#19ba# => X"FFFFFFFF",
  16#19bb# => X"FFFFFFFF",
  16#19bc# => X"FFFFFFFF",
  16#19bd# => X"7777FFFF",
  16#19be# => X"77777777",
  16#19bf# => X"77777777",
  16#19c0# => X"70007777",
  16#19c1# => X"FFFFFF88",
  16#19c2# => X"FFFFFFFF",
  16#19c3# => X"77700008",
  16#19c4# => X"88888777",
  16#19c5# => X"88888888",
  16#19c6# => X"88888888",
  16#19c7# => X"00888888",
  16#19c8# => X"00000000",
  16#19c9# => X"33770000",
  16#19ca# => X"33333333",
  16#19cb# => X"33333333",
  16#19cc# => X"FF333333",
  16#19cd# => X"FFFFFFFF",
  16#19ce# => X"00000078",
  16#19cf# => X"00000000",
  16#19d0# => X"00000000",
  16#19d1# => X"33FFFFF8",
  16#19d2# => X"33333333",
  16#19d3# => X"33333333",
  16#19d4# => X"3FFFF333",
  16#19d5# => X"33333333",
  16#19d6# => X"33333333",
  16#19d7# => X"88883333",
  16#19d8# => X"88888888",
  16#19d9# => X"77788888",
  16#19da# => X"F8700077",
  16#19db# => X"000008FF",
  16#19dc# => X"77707000",
  16#19dd# => X"77777777",
  16#19de# => X"77777777",
  16#19df# => X"77777777",
  16#19e0# => X"FFFFFFF7",
  16#19e1# => X"FFFFFFFF",
  16#19e2# => X"FFFFFFFF",
  16#19e3# => X"FFFFFFFF",
  16#19e4# => X"FFFFFFFF",
  16#19e5# => X"7777FFFF",
  16#19e6# => X"77777777",
  16#19e7# => X"77777777",
  16#19e8# => X"00070077",
  16#19e9# => X"FFFF8700",
  16#19ea# => X"FFFFFFFF",
  16#19eb# => X"77700008",
  16#19ec# => X"88888777",
  16#19ed# => X"88888888",
  16#19ee# => X"88888888",
  16#19ef# => X"00888888",
  16#19f0# => X"00000000",
  16#19f1# => X"33770000",
  16#19f2# => X"33333333",
  16#19f3# => X"33333333",
  16#19f4# => X"FFF33333",
  16#19f5# => X"FFFFFFFF",
  16#19f6# => X"000788FF",
  16#19f7# => X"00000000",
  16#19f8# => X"00000000",
  16#19f9# => X"33FFFFFF",
  16#19fa# => X"33333333",
  16#19fb# => X"33333333",
  16#19fc# => X"3FFFF333",
  16#19fd# => X"33333333",
  16#19fe# => X"33333333",
  16#19ff# => X"88883333",
  16#1a00# => X"88888888",
  16#1a01# => X"77788888",
  16#1a02# => X"FFF88777",
  16#1a03# => X"000008FF",
  16#1a04# => X"77770000",
  16#1a05# => X"77777777",
  16#1a06# => X"77777777",
  16#1a07# => X"07777777",
  16#1a08# => X"FFFFFFF8",
  16#1a09# => X"FFFFFFFF",
  16#1a0a# => X"FFFFFFFF",
  16#1a0b# => X"FFFFFFFF",
  16#1a0c# => X"FFFFFFFF",
  16#1a0d# => X"77778FFF",
  16#1a0e# => X"77777777",
  16#1a0f# => X"77777777",
  16#1a10# => X"00000077",
  16#1a11# => X"F8770000",
  16#1a12# => X"FFFFFFFF",
  16#1a13# => X"77770007",
  16#1a14# => X"88888877",
  16#1a15# => X"88888888",
  16#1a16# => X"88888888",
  16#1a17# => X"00888888",
  16#1a18# => X"00000000",
  16#1a19# => X"33377000",
  16#1a1a# => X"33333333",
  16#1a1b# => X"33333333",
  16#1a1c# => X"FFF33333",
  16#1a1d# => X"FFFFFFFF",
  16#1a1e# => X"078FFFFF",
  16#1a1f# => X"00000000",
  16#1a20# => X"70000000",
  16#1a21# => X"33FFFFFF",
  16#1a22# => X"33333333",
  16#1a23# => X"33333333",
  16#1a24# => X"3FFFF333",
  16#1a25# => X"33333333",
  16#1a26# => X"33333333",
  16#1a27# => X"88883333",
  16#1a28# => X"88888888",
  16#1a29# => X"77778888",
  16#1a2a# => X"FFFFFF88",
  16#1a2b# => X"000008FF",
  16#1a2c# => X"77770000",
  16#1a2d# => X"77777777",
  16#1a2e# => X"77777777",
  16#1a2f# => X"77777777",
  16#1a30# => X"FFFFFFF8",
  16#1a31# => X"FFFFFFFF",
  16#1a32# => X"FFFFFFFF",
  16#1a33# => X"FFFFFFFF",
  16#1a34# => X"FFFFFFFF",
  16#1a35# => X"77708FFF",
  16#1a36# => X"77777777",
  16#1a37# => X"77777777",
  16#1a38# => X"00000077",
  16#1a39# => X"70000000",
  16#1a3a# => X"FFFFFFF8",
  16#1a3b# => X"77770007",
  16#1a3c# => X"88888877",
  16#1a3d# => X"88888888",
  16#1a3e# => X"88888888",
  16#1a3f# => X"00888888",
  16#1a40# => X"00000000",
  16#1a41# => X"33377000",
  16#1a42# => X"33333333",
  16#1a43# => X"33333333",
  16#1a44# => X"FFFF3333",
  16#1a45# => X"FFFFFFFF",
  16#1a46# => X"8FFFFFFF",
  16#1a47# => X"00000078",
  16#1a48# => X"70000000",
  16#1a49# => X"33FFFFFF",
  16#1a4a# => X"33333333",
  16#1a4b# => X"33333333",
  16#1a4c# => X"3FFFF333",
  16#1a4d# => X"33333333",
  16#1a4e# => X"33333333",
  16#1a4f# => X"88883333",
  16#1a50# => X"88888888",
  16#1a51# => X"F8887888",
  16#1a52# => X"FFFFFFFF",
  16#1a53# => X"000007FF",
  16#1a54# => X"77770000",
  16#1a55# => X"77777777",
  16#1a56# => X"77777777",
  16#1a57# => X"77777777",
  16#1a58# => X"FFFFFFF8",
  16#1a59# => X"FFFFFFFF",
  16#1a5a# => X"FFFFFFFF",
  16#1a5b# => X"FFFFFFFF",
  16#1a5c# => X"FFFFFFFF",
  16#1a5d# => X"77777FFF",
  16#1a5e# => X"77777777",
  16#1a5f# => X"77777777",
  16#1a60# => X"00000077",
  16#1a61# => X"00000000",
  16#1a62# => X"FFFF8700",
  16#1a63# => X"77777000",
  16#1a64# => X"88888887",
  16#1a65# => X"88888888",
  16#1a66# => X"88888888",
  16#1a67# => X"00888888",
  16#1a68# => X"00000000",
  16#1a69# => X"33337700",
  16#1a6a# => X"33333333",
  16#1a6b# => X"33333333",
  16#1a6c# => X"FFFF3333",
  16#1a6d# => X"FFFFFFFF",
  16#1a6e# => X"FFFFFFFF",
  16#1a6f# => X"333333FF",
  16#1a70# => X"33333333",
  16#1a71# => X"33FFFF33",
  16#1a72# => X"33333333",
  16#1a73# => X"33333333",
  16#1a74# => X"3FFFF333",
  16#1a75# => X"33333333",
  16#1a76# => X"33333333",
  16#1a77# => X"88883333",
  16#1a78# => X"88888888",
  16#1a79# => X"FFFF8888",
  16#1a7a# => X"FFFFFFFF",
  16#1a7b# => X"000007FF",
  16#1a7c# => X"77770000",
  16#1a7d# => X"77777777",
  16#1a7e# => X"77777777",
  16#1a7f# => X"77777777",
  16#1a80# => X"FFFFFFFF",
  16#1a81# => X"FFFFFFFF",
  16#1a82# => X"FFFFFFFF",
  16#1a83# => X"FFFFFFFF",
  16#1a84# => X"FFFFFFFF",
  16#1a85# => X"77777FFF",
  16#1a86# => X"77777777",
  16#1a87# => X"77777777",
  16#1a88# => X"00000077",
  16#1a89# => X"00000000",
  16#1a8a# => X"F8800000",
  16#1a8b# => X"77770078",
  16#1a8c# => X"88888887",
  16#1a8d# => X"88888888",
  16#1a8e# => X"88888888",
  16#1a8f# => X"00888888",
  16#1a90# => X"00000000",
  16#1a91# => X"33337700",
  16#1a92# => X"33333333",
  16#1a93# => X"33333333",
  16#1a94# => X"FFFFF333",
  16#1a95# => X"FFFFFFFF",
  16#1a96# => X"FFFFFFFF",
  16#1a97# => X"333333FF",
  16#1a98# => X"33333333",
  16#1a99# => X"3FFFFF33",
  16#1a9a# => X"33333333",
  16#1a9b# => X"33333333",
  16#1a9c# => X"3FFFF333",
  16#1a9d# => X"33333333",
  16#1a9e# => X"33333333",
  16#1a9f# => X"88888333",
  16#1aa0# => X"88888888",
  16#1aa1# => X"FFFFFFF8",
  16#1aa2# => X"FFFFFFFF",
  16#1aa3# => X"000000FF",
  16#1aa4# => X"77770000",
  16#1aa5# => X"77777777",
  16#1aa6# => X"77777777",
  16#1aa7# => X"77777777",
  16#1aa8# => X"FFFFFFFF",
  16#1aa9# => X"FFFFFFFF",
  16#1aaa# => X"FFFFFFFF",
  16#1aab# => X"FFFFFFFF",
  16#1aac# => X"FFFFFFFF",
  16#1aad# => X"777778FF",
  16#1aae# => X"77777777",
  16#1aaf# => X"77777777",
  16#1ab0# => X"00000077",
  16#1ab1# => X"00000000",
  16#1ab2# => X"F8000000",
  16#1ab3# => X"777788FF",
  16#1ab4# => X"88888887",
  16#1ab5# => X"88888888",
  16#1ab6# => X"88888888",
  16#1ab7# => X"00888888",
  16#1ab8# => X"00000000",
  16#1ab9# => X"33333770",
  16#1aba# => X"33333333",
  16#1abb# => X"33333333",
  16#1abc# => X"33333333",
  16#1abd# => X"33333333",
  16#1abe# => X"FFFFF333",
  16#1abf# => X"333333FF",
  16#1ac0# => X"33333333",
  16#1ac1# => X"3FFFFF33",
  16#1ac2# => X"33333333",
  16#1ac3# => X"33333333",
  16#1ac4# => X"3FFFF333",
  16#1ac5# => X"33333333",
  16#1ac6# => X"33333333",
  16#1ac7# => X"88888333",
  16#1ac8# => X"FF888888",
  16#1ac9# => X"FFFFFFFF",
  16#1aca# => X"FFFFFFFF",
  16#1acb# => X"0000008F",
  16#1acc# => X"77700000",
  16#1acd# => X"77777777",
  16#1ace# => X"77777777",
  16#1acf# => X"87777777",
  16#1ad0# => X"FFFFFFFF",
  16#1ad1# => X"FFFFFFFF",
  16#1ad2# => X"FFFFFFFF",
  16#1ad3# => X"FFFFFFFF",
  16#1ad4# => X"FFFFFFFF",
  16#1ad5# => X"777778FF",
  16#1ad6# => X"77777777",
  16#1ad7# => X"77777777",
  16#1ad8# => X"00000007",
  16#1ad9# => X"00000000",
  16#1ada# => X"F8000000",
  16#1adb# => X"888FFFFF",
  16#1adc# => X"88888888",
  16#1add# => X"88888888",
  16#1ade# => X"88888888",
  16#1adf# => X"00888888",
  16#1ae0# => X"00000000",
  16#1ae1# => X"33333770",
  16#1ae2# => X"33333333",
  16#1ae3# => X"33333333",
  16#1ae4# => X"33333333",
  16#1ae5# => X"33333333",
  16#1ae6# => X"FFFFF333",
  16#1ae7# => X"333333FF",
  16#1ae8# => X"33333333",
  16#1ae9# => X"3FFFFF33",
  16#1aea# => X"33333333",
  16#1aeb# => X"33333333",
  16#1aec# => X"33FF3333",
  16#1aed# => X"33333333",
  16#1aee# => X"33333333",
  16#1aef# => X"88888333",
  16#1af0# => X"FFFFFF88",
  16#1af1# => X"FFFFFFFF",
  16#1af2# => X"FFFFFFFF",
  16#1af3# => X"0000008F",
  16#1af4# => X"77700000",
  16#1af5# => X"77777777",
  16#1af6# => X"77777777",
  16#1af7# => X"87777777",
  16#1af8# => X"FFFFFFFF",
  16#1af9# => X"FFFFFFFF",
  16#1afa# => X"FFFFFFFF",
  16#1afb# => X"FFFFFFFF",
  16#1afc# => X"FFFFFFFF",
  16#1afd# => X"777777FF",
  16#1afe# => X"77777777",
  16#1aff# => X"77777777",
  16#1b00# => X"00000007",
  16#1b01# => X"00000000",
  16#1b02# => X"F8000000",
  16#1b03# => X"FFFFFFFF",
  16#1b04# => X"FFFF888F",
  16#1b05# => X"8888888F",
  16#1b06# => X"88888888",
  16#1b07# => X"00888888",
  16#1b08# => X"00000000",
  16#1b09# => X"33333377",
  16#1b0a# => X"33333333",
  16#1b0b# => X"33333333",
  16#1b0c# => X"33333333",
  16#1b0d# => X"33333333",
  16#1b0e# => X"FFFFF333",
  16#1b0f# => X"333333FF",
  16#1b10# => X"33333333",
  16#1b11# => X"3FFFFF33",
  16#1b12# => X"33333333",
  16#1b13# => X"33333333",
  16#1b14# => X"33FF3333",
  16#1b15# => X"33333333",
  16#1b16# => X"33333333",
  16#1b17# => X"F8888333",
  16#1b18# => X"FFFFFFFF",
  16#1b19# => X"FFFFFFFF",
  16#1b1a# => X"FFFFFFFF",
  16#1b1b# => X"0000007F",
  16#1b1c# => X"77000000",
  16#1b1d# => X"77777777",
  16#1b1e# => X"77777777",
  16#1b1f# => X"F7777777",
  16#1b20# => X"FFFFFFFF",
  16#1b21# => X"FFFFFFFF",
  16#1b22# => X"FFFFFFFF",
  16#1b23# => X"FFFFFFFF",
  16#1b24# => X"FFFFFFFF",
  16#1b25# => X"777777FF",
  16#1b26# => X"77777777",
  16#1b27# => X"77777777",
  16#1b28# => X"00000007",
  16#1b29# => X"00000000",
  16#1b2a# => X"FF700000",
  16#1b2b# => X"FFFFFFFF",
  16#1b2c# => X"FFFFFFFF",
  16#1b2d# => X"88FFFFFF",
  16#1b2e# => X"88888888",
  16#1b2f# => X"00888888",
  16#1b30# => X"00000000",
  16#1b31# => X"33333377",
  16#1b32# => X"33333333",
  16#1b33# => X"33333333",
  16#1b34# => X"33333333",
  16#1b35# => X"33333333",
  16#1b36# => X"FFFFF333",
  16#1b37# => X"333333FF",
  16#1b38# => X"33333333",
  16#1b39# => X"FFFFFF33",
  16#1b3a# => X"33333333",
  16#1b3b# => X"33333333",
  16#1b3c# => X"33333333",
  16#1b3d# => X"33333333",
  16#1b3e# => X"33333333",
  16#1b3f# => X"FFFFF833",
  16#1b40# => X"FFFFFFFF",
  16#1b41# => X"FFFFFFFF",
  16#1b42# => X"FFFFFFFF",
  16#1b43# => X"0000007F",
  16#1b44# => X"70000000",
  16#1b45# => X"77777777",
  16#1b46# => X"77777777",
  16#1b47# => X"F7777777",
  16#1b48# => X"FFFFFFFF",
  16#1b49# => X"FFFFFFFF",
  16#1b4a# => X"FFFFFFFF",
  16#1b4b# => X"FFFFFFFF",
  16#1b4c# => X"FFFFFFFF",
  16#1b4d# => X"7777778F",
  16#1b4e# => X"77777777",
  16#1b4f# => X"77777777",
  16#1b50# => X"00000000",
  16#1b51# => X"00000000",
  16#1b52# => X"FF700000",
  16#1b53# => X"FFFFFFFF",
  16#1b54# => X"FFFFFFFF",
  16#1b55# => X"FFFFFFFF",
  16#1b56# => X"888FFFFF",
  16#1b57# => X"00888888",
  16#1b58# => X"70000000",
  16#1b59# => X"33333377",
  16#1b5a# => X"33333333",
  16#1b5b# => X"33333333",
  16#1b5c# => X"33333333",
  16#1b5d# => X"33333333",
  16#1b5e# => X"FFFFF333",
  16#1b5f# => X"333333FF",
  16#1b60# => X"33333333",
  16#1b61# => X"FFF88733",
  16#1b62# => X"33333333",
  16#1b63# => X"33333333",
  16#1b64# => X"33333333",
  16#1b65# => X"33333333",
  16#1b66# => X"33333333",
  16#1b67# => X"FFFFFF33",
  16#1b68# => X"FFFFFFFF",
  16#1b69# => X"FFFFFFFF",
  16#1b6a# => X"FFFFFFFF",
  16#1b6b# => X"0000000F",
  16#1b6c# => X"70000000",
  16#1b6d# => X"77777777",
  16#1b6e# => X"77777777",
  16#1b6f# => X"F8777777",
  16#1b70# => X"FFFFFFFF",
  16#1b71# => X"FFFFFFFF",
  16#1b72# => X"FFFFFFFF",
  16#1b73# => X"FFFFFFFF",
  16#1b74# => X"FFFFFFFF",
  16#1b75# => X"7777778F",
  16#1b76# => X"77777777",
  16#1b77# => X"07777777",
  16#1b78# => X"00000000",
  16#1b79# => X"00000000",
  16#1b7a# => X"FF800000",
  16#1b7b# => X"FFFFFFFF",
  16#1b7c# => X"FFFFFFFF",
  16#1b7d# => X"FFFFFFFF",
  16#1b7e# => X"FFFFFFFF",
  16#1b7f# => X"00FFFFFF",
  16#1b80# => X"70000000",
  16#1b81# => X"33333377",
  16#1b82# => X"33333333",
  16#1b83# => X"33333333",
  16#1b84# => X"33333333",
  16#1b85# => X"33333333",
  16#1b86# => X"FFFFF333",
  16#1b87# => X"333333FF",
  16#1b88# => X"33333333",
  16#1b89# => X"88700033",
  16#1b8a# => X"33333338",
  16#1b8b# => X"33333333",
  16#1b8c# => X"33333333",
  16#1b8d# => X"33333333",
  16#1b8e# => X"33333333",
  16#1b8f# => X"FFFFFF33",
  16#1b90# => X"FFFFFFFF",
  16#1b91# => X"FFFFFFFF",
  16#1b92# => X"FFFFFFFF",
  16#1b93# => X"00000008",
  16#1b94# => X"00000000",
  16#1b95# => X"00777770",
  16#1b96# => X"77777770",
  16#1b97# => X"F8777777",
  16#1b98# => X"FFFFFFFF",
  16#1b99# => X"FFFFFFFF",
  16#1b9a# => X"FFFFFFFF",
  16#1b9b# => X"FFFFFFFF",
  16#1b9c# => X"FFFFFFFF",
  16#1b9d# => X"7777777F",
  16#1b9e# => X"77777777",
  16#1b9f# => X"07777777",
  16#1ba0# => X"00000000",
  16#1ba1# => X"00000000",
  16#1ba2# => X"FF800000",
  16#1ba3# => X"FFFFFFFF",
  16#1ba4# => X"FFFFFFFF",
  16#1ba5# => X"FFFFFFFF",
  16#1ba6# => X"FFFFFFFF",
  16#1ba7# => X"00FFFFFF",
  16#1ba8# => X"70000000",
  16#1ba9# => X"33333377",
  16#1baa# => X"33333333",
  16#1bab# => X"33333333",
  16#1bac# => X"33333333",
  16#1bad# => X"33333333",
  16#1bae# => X"FFFFF333",
  16#1baf# => X"333333FF",
  16#1bb0# => X"33333333",
  16#1bb1# => X"00070033",
  16#1bb2# => X"33333337",
  16#1bb3# => X"33333333",
  16#1bb4# => X"33333333",
  16#1bb5# => X"33333333",
  16#1bb6# => X"33333333",
  16#1bb7# => X"FFFFFFF3",
  16#1bb8# => X"FFFFFFFF",
  16#1bb9# => X"FFFFFFFF",
  16#1bba# => X"FFFFFFFF",
  16#1bbb# => X"00000008",
  16#1bbc# => X"00000000",
  16#1bbd# => X"00000000",
  16#1bbe# => X"77777700",
  16#1bbf# => X"FF777777",
  16#1bc0# => X"FFFFFFFF",
  16#1bc1# => X"FFFFFFFF",
  16#1bc2# => X"FFFFFFFF",
  16#1bc3# => X"FFFFFFFF",
  16#1bc4# => X"FFFFFFFF",
  16#1bc5# => X"77777778",
  16#1bc6# => X"77777777",
  16#1bc7# => X"07777777",
  16#1bc8# => X"00000000",
  16#1bc9# => X"00000000",
  16#1bca# => X"FFF00000",
  16#1bcb# => X"FFFFFFFF",
  16#1bcc# => X"FFFFFFFF",
  16#1bcd# => X"FFFFFFFF",
  16#1bce# => X"FFFFFFFF",
  16#1bcf# => X"00FFFFFF",
  16#1bd0# => X"70000000",
  16#1bd1# => X"33333377",
  16#1bd2# => X"33333333",
  16#1bd3# => X"33333333",
  16#1bd4# => X"33333333",
  16#1bd5# => X"33333333",
  16#1bd6# => X"FFFFF333",
  16#1bd7# => X"333333FF",
  16#1bd8# => X"33333333",
  16#1bd9# => X"77770033",
  16#1bda# => X"33333377",
  16#1bdb# => X"33333333",
  16#1bdc# => X"33333333",
  16#1bdd# => X"33333333",
  16#1bde# => X"33333333",
  16#1bdf# => X"FFFFFFFF",
  16#1be0# => X"FFFFFFFF",
  16#1be1# => X"FFFFFFFF",
  16#1be2# => X"FFFFFFFF",
  16#1be3# => X"00000007",
  16#1be4# => X"00000000",
  16#1be5# => X"00000000",
  16#1be6# => X"77777700",
  16#1be7# => X"FF777777",
  16#1be8# => X"FFFFFFFF",
  16#1be9# => X"FFFFFFFF",
  16#1bea# => X"FFFFFFFF",
  16#1beb# => X"FFFFFFFF",
  16#1bec# => X"FFFFFFFF",
  16#1bed# => X"77777778",
  16#1bee# => X"77777777",
  16#1bef# => X"00777777",
  16#1bf0# => X"00000000",
  16#1bf1# => X"00000000",
  16#1bf2# => X"FFF70000",
  16#1bf3# => X"FFFFFFFF",
  16#1bf4# => X"FFFFFFFF",
  16#1bf5# => X"FFFFFFFF",
  16#1bf6# => X"FFFFFFFF",
  16#1bf7# => X"00FFFFFF",
  16#1bf8# => X"77000000",
  16#1bf9# => X"33333377",
  16#1bfa# => X"33333333",
  16#1bfb# => X"33333333",
  16#1bfc# => X"33333333",
  16#1bfd# => X"33333333",
  16#1bfe# => X"FFFFF333",
  16#1bff# => X"333333FF",
  16#1c00# => X"33333333",
  16#1c01# => X"77707033",
  16#1c02# => X"33333777",
  16#1c03# => X"33333333",
  16#1c04# => X"33333333",
  16#1c05# => X"33333333",
  16#1c06# => X"F3333333",
  16#1c07# => X"FFFFFFFF",
  16#1c08# => X"FFFFFFFF",
  16#1c09# => X"FFFFFFFF",
  16#1c0a# => X"FFFFFFFF",
  16#1c0b# => X"00000007",
  16#1c0c# => X"00000000",
  16#1c0d# => X"00000000",
  16#1c0e# => X"77777700",
  16#1c0f# => X"FF877777",
  16#1c10# => X"FFFFFFFF",
  16#1c11# => X"FFFFFFFF",
  16#1c12# => X"FFFFFFFF",
  16#1c13# => X"FFFFFFFF",
  16#1c14# => X"FFFFFFFF",
  16#1c15# => X"77777777",
  16#1c16# => X"77777777",
  16#1c17# => X"00777777",
  16#1c18# => X"00000000",
  16#1c19# => X"00000000",
  16#1c1a# => X"FFF70000",
  16#1c1b# => X"FFFFFFFF",
  16#1c1c# => X"FFFFFFFF",
  16#1c1d# => X"FFFFFFFF",
  16#1c1e# => X"FFFFFFFF",
  16#1c1f# => X"00FFFFFF",
  16#1c20# => X"77000000",
  16#1c21# => X"33333377",
  16#1c22# => X"33333333",
  16#1c23# => X"33333333",
  16#1c24# => X"33333333",
  16#1c25# => X"33333333",
  16#1c26# => X"FFFFF333",
  16#1c27# => X"333333FF",
  16#1c28# => X"33333333",
  16#1c29# => X"77770733",
  16#1c2a# => X"33337777",
  16#1c2b# => X"33333333",
  16#1c2c# => X"33333333",
  16#1c2d# => X"33333333",
  16#1c2e# => X"F3333333",
  16#1c2f# => X"FFFFFFFF",
  16#1c30# => X"FFFFFFFF",
  16#1c31# => X"FFFFFFFF",
  16#1c32# => X"FFFFFFFF",
  16#1c33# => X"00000000",
  16#1c34# => X"00000000",
  16#1c35# => X"00000000",
  16#1c36# => X"77777770",
  16#1c37# => X"FF877777",
  16#1c38# => X"FFFFFFFF",
  16#1c39# => X"FFFFFFFF",
  16#1c3a# => X"FFFFFFFF",
  16#1c3b# => X"FFFFFFFF",
  16#1c3c# => X"FFFFFFFF",
  16#1c3d# => X"77777777",
  16#1c3e# => X"77777777",
  16#1c3f# => X"00077777",
  16#1c40# => X"00000000",
  16#1c41# => X"00000000",
  16#1c42# => X"FFFF0000",
  16#1c43# => X"FFFFFFFF",
  16#1c44# => X"FFFFFFFF",
  16#1c45# => X"FFFFFFFF",
  16#1c46# => X"FFFFFFFF",
  16#1c47# => X"00FFFFFF",
  16#1c48# => X"77000000",
  16#1c49# => X"33333377",
  16#1c4a# => X"33333333",
  16#1c4b# => X"33333333",
  16#1c4c# => X"33333333",
  16#1c4d# => X"33333333",
  16#1c4e# => X"FFFFF333",
  16#1c4f# => X"333333FF",
  16#1c50# => X"33333333",
  16#1c51# => X"77777033",
  16#1c52# => X"33377777",
  16#1c53# => X"33333333",
  16#1c54# => X"33333333",
  16#1c55# => X"33333333",
  16#1c56# => X"FFF33333",
  16#1c57# => X"FFFFFFFF",
  16#1c58# => X"FFFFFFFF",
  16#1c59# => X"FFFFFFFF",
  16#1c5a# => X"8FFFFFFF",
  16#1c5b# => X"00000000",
  16#1c5c# => X"00000000",
  16#1c5d# => X"00000000",
  16#1c5e# => X"77777770",
  16#1c5f# => X"FFF77777",
  16#1c60# => X"FFFFFFFF",
  16#1c61# => X"FFFFFFFF",
  16#1c62# => X"FFFFFFFF",
  16#1c63# => X"FFFFFFFF",
  16#1c64# => X"8FFFFFFF",
  16#1c65# => X"77777777",
  16#1c66# => X"77777777",
  16#1c67# => X"00077777",
  16#1c68# => X"00000000",
  16#1c69# => X"00000000",
  16#1c6a# => X"FFFF8000",
  16#1c6b# => X"FFFFFFFF",
  16#1c6c# => X"FFFFFFFF",
  16#1c6d# => X"FFFFFFFF",
  16#1c6e# => X"FFFFFFFF",
  16#1c6f# => X"00FFFFFF",
  16#1c70# => X"77000000",
  16#1c71# => X"33333377",
  16#1c72# => X"33333333",
  16#1c73# => X"33333333",
  16#1c74# => X"33333333",
  16#1c75# => X"33333333",
  16#1c76# => X"FFFFF333",
  16#1c77# => X"333333FF",
  16#1c78# => X"33333333",
  16#1c79# => X"77777033",
  16#1c7a# => X"33777777",
  16#1c7b# => X"33333333",
  16#1c7c# => X"33333333",
  16#1c7d# => X"33333333",
  16#1c7e# => X"FFFF3333",
  16#1c7f# => X"FFFFFFFF",
  16#1c80# => X"FFFFFFFF",
  16#1c81# => X"FFFFFFFF",
  16#1c82# => X"8FFFFFFF",
  16#1c83# => X"00000000",
  16#1c84# => X"00000000",
  16#1c85# => X"00000000",
  16#1c86# => X"77777777",
  16#1c87# => X"00087777",
  16#1c88# => X"FFFFFFF0",
  16#1c89# => X"FF00000F",
  16#1c8a# => X"000FFFFF",
  16#1c8b# => X"FFFFFFFF",
  16#1c8c# => X"8FFFF000",
  16#1c8d# => X"77000777",
  16#1c8e# => X"77777777",
  16#1c8f# => X"00007777",
  16#1c90# => X"00000000",
  16#1c91# => X"00000000",
  16#1c92# => X"FFFF8000",
  16#1c93# => X"FFFFFFFF",
  16#1c94# => X"FFFFFFFF",
  16#1c95# => X"FFFFFFFF",
  16#1c96# => X"FFFFFFFF",
  16#1c97# => X"00FFFFFF",
  16#1c98# => X"77700000",
  16#1c99# => X"33333377",
  16#1c9a# => X"33333333",
  16#1c9b# => X"33333333",
  16#1c9c# => X"33333333",
  16#1c9d# => X"33333333",
  16#1c9e# => X"FFFFF333",
  16#1c9f# => X"333333FF",
  16#1ca0# => X"33333333",
  16#1ca1# => X"77777733",
  16#1ca2# => X"77777777",
  16#1ca3# => X"33333333",
  16#1ca4# => X"33333333",
  16#1ca5# => X"33333333",
  16#1ca6# => X"FFFFF333",
  16#1ca7# => X"FFFFFFFF",
  16#1ca8# => X"FFFFFFFF",
  16#1ca9# => X"FFFFFFFF",
  16#1caa# => X"7FFFFFFF",
  16#1cab# => X"00000000",
  16#1cac# => X"00000000",
  16#1cad# => X"00000000",
  16#1cae# => X"77777777",
  16#1caf# => X"FFF00777",
  16#1cb0# => X"FFFFF00F",
  16#1cb1# => X"F0000000",
  16#1cb2# => X"00000FFF",
  16#1cb3# => X"00FFFF00",
  16#1cb4# => X"7FF00000",
  16#1cb5# => X"00000007",
  16#1cb6# => X"77777777",
  16#1cb7# => X"00007777",
  16#1cb8# => X"00000000",
  16#1cb9# => X"00000000",
  16#1cba# => X"FFFFF700",
  16#1cbb# => X"FFFFFFFF",
  16#1cbc# => X"FFFFFFFF",
  16#1cbd# => X"FFFFFFFF",
  16#1cbe# => X"FFFFFFFF",
  16#1cbf# => X"00FFFFFF",
  16#1cc0# => X"77770000",
  16#1cc1# => X"33333377",
  16#1cc2# => X"33333333",
  16#1cc3# => X"33333333",
  16#1cc4# => X"33333333",
  16#1cc5# => X"33333333",
  16#1cc6# => X"FFFFF333",
  16#1cc7# => X"333333FF",
  16#1cc8# => X"33333333",
  16#1cc9# => X"77777733",
  16#1cca# => X"87777777",
  16#1ccb# => X"33333388",
  16#1ccc# => X"33333333",
  16#1ccd# => X"33333333",
  16#1cce# => X"FFFFFFFF",
  16#1ccf# => X"FFFFFFFF",
  16#1cd0# => X"FFFFFFFF",
  16#1cd1# => X"FFFFFFFF",
  16#1cd2# => X"7FFFFFFF",
  16#1cd3# => X"00000000",
  16#1cd4# => X"00000000",
  16#1cd5# => X"00000000",
  16#1cd6# => X"77777777",
  16#1cd7# => X"FFF87077",
  16#1cd8# => X"0FFF0FFF",
  16#1cd9# => X"000FFF00",
  16#1cda# => X"0F000FFF",
  16#1cdb# => X"00FFFF00",
  16#1cdc# => X"78F000F0",
  16#1cdd# => X"00070007",
  16#1cde# => X"77777777",
  16#1cdf# => X"00000777",
  16#1ce0# => X"00000000",
  16#1ce1# => X"00000000",
  16#1ce2# => X"FFFFF700",
  16#1ce3# => X"FFFFFFFF",
  16#1ce4# => X"FFFFFFFF",
  16#1ce5# => X"FFFFFFFF",
  16#1ce6# => X"FFFFFFFF",
  16#1ce7# => X"00FFFFFF",
  16#1ce8# => X"77770000",
  16#1ce9# => X"77777777",
  16#1cea# => X"F8070077",
  16#1ceb# => X"FFFFFFFF",
  16#1cec# => X"FFFFFFFF",
  16#1ced# => X"FFFFFFFF",
  16#1cee# => X"FFFFFFFF",
  16#1cef# => X"FFFFFFFF",
  16#1cf0# => X"708FFFFF",
  16#1cf1# => X"77777770",
  16#1cf2# => X"87777777",
  16#1cf3# => X"33888888",
  16#1cf4# => X"33333333",
  16#1cf5# => X"F8833333",
  16#1cf6# => X"FFFFFFFF",
  16#1cf7# => X"FFFFFFFF",
  16#1cf8# => X"FFFFFFFF",
  16#1cf9# => X"FFFFFFFF",
  16#1cfa# => X"0FFFFFFF",
  16#1cfb# => X"00000000",
  16#1cfc# => X"00000000",
  16#1cfd# => X"70000000",
  16#1cfe# => X"77777777",
  16#1cff# => X"000F7707",
  16#1d00# => X"0FF0FFF0",
  16#1d01# => X"000FFF00",
  16#1d02# => X"FFF000FF",
  16#1d03# => X"000FF000",
  16#1d04# => X"78000FFF",
  16#1d05# => X"00777000",
  16#1d06# => X"77777770",
  16#1d07# => X"00000777",
  16#1d08# => X"00000000",
  16#1d09# => X"00000000",
  16#1d0a# => X"FFFFF807",
  16#1d0b# => X"FFFFFFFF",
  16#1d0c# => X"FFFFFFFF",
  16#1d0d# => X"FFFFFFFF",
  16#1d0e# => X"FFFFFFFF",
  16#1d0f# => X"00FFFFFF",
  16#1d10# => X"77000000",
  16#1d11# => X"77777777",
  16#1d12# => X"F8070000",
  16#1d13# => X"FFFFFFFF",
  16#1d14# => X"FFFFFFFF",
  16#1d15# => X"FFFFFFFF",
  16#1d16# => X"FFFFFFFF",
  16#1d17# => X"FFFFFFFF",
  16#1d18# => X"708FFFFF",
  16#1d19# => X"77777777",
  16#1d1a# => X"88777777",
  16#1d1b# => X"88888888",
  16#1d1c# => X"88888888",
  16#1d1d# => X"FF888888",
  16#1d1e# => X"FFFFFFFF",
  16#1d1f# => X"FFFFFFFF",
  16#1d20# => X"FFFFFFFF",
  16#1d21# => X"FFFFFFFF",
  16#1d22# => X"08FFFFFF",
  16#1d23# => X"00000000",
  16#1d24# => X"00000000",
  16#1d25# => X"70000000",
  16#1d26# => X"77777777",
  16#1d27# => X"FF008707",
  16#1d28# => X"FFF0FF00",
  16#1d29# => X"000FFFFF",
  16#1d2a# => X"FFF000FF",
  16#1d2b# => X"000FF000",
  16#1d2c# => X"77000FFF",
  16#1d2d# => X"00777000",
  16#1d2e# => X"77777770",
  16#1d2f# => X"00000077",
  16#1d30# => X"00000000",
  16#1d31# => X"70000000",
  16#1d32# => X"FFFFF877",
  16#1d33# => X"FFFFFFFF",
  16#1d34# => X"FFFFFFFF",
  16#1d35# => X"FFFFFFFF",
  16#1d36# => X"FFFFFFFF",
  16#1d37# => X"00FFFFFF",
  16#1d38# => X"00000000",
  16#1d39# => X"70007777",
  16#1d3a# => X"FF700000",
  16#1d3b# => X"FFFFFFFF",
  16#1d3c# => X"FFFFFFFF",
  16#1d3d# => X"FFFFFFFF",
  16#1d3e# => X"FFFFFFFF",
  16#1d3f# => X"FFFFFFFF",
  16#1d40# => X"707FFFFF",
  16#1d41# => X"77777777",
  16#1d42# => X"88777777",
  16#1d43# => X"88888888",
  16#1d44# => X"88888888",
  16#1d45# => X"FF888888",
  16#1d46# => X"FFFFFFFF",
  16#1d47# => X"FFFFFFFF",
  16#1d48# => X"FFFFFFFF",
  16#1d49# => X"FFFFFFFF",
  16#1d4a# => X"08FFFFFF",
  16#1d4b# => X"00000000",
  16#1d4c# => X"00000000",
  16#1d4d# => X"77000000",
  16#1d4e# => X"77777777",
  16#1d4f# => X"FFF00770",
  16#1d50# => X"FF0FFFFF",
  16#1d51# => X"000FFFFF",
  16#1d52# => X"FFF000FF",
  16#1d53# => X"000FF000",
  16#1d54# => X"77000FFF",
  16#1d55# => X"00777000",
  16#1d56# => X"77777770",
  16#1d57# => X"00000007",
  16#1d58# => X"00000000",
  16#1d59# => X"77000000",
  16#1d5a# => X"FFFFFF77",
  16#1d5b# => X"FFFFFFFF",
  16#1d5c# => X"FFFFFFFF",
  16#1d5d# => X"FFFFFFFF",
  16#1d5e# => X"FFFFFFFF",
  16#1d5f# => X"00FFFFFF",
  16#1d60# => X"00000000",
  16#1d61# => X"00000077",
  16#1d62# => X"FF700000",
  16#1d63# => X"FFFFFFFF",
  16#1d64# => X"FFFFFFFF",
  16#1d65# => X"FFFFFFFF",
  16#1d66# => X"FFFFFFFF",
  16#1d67# => X"FFFFFFFF",
  16#1d68# => X"707FFFFF",
  16#1d69# => X"77777777",
  16#1d6a# => X"88877777",
  16#1d6b# => X"88888888",
  16#1d6c# => X"88888888",
  16#1d6d# => X"FF888888",
  16#1d6e# => X"FFFFFFFF",
  16#1d6f# => X"FFFFFFFF",
  16#1d70# => X"FFFFFFFF",
  16#1d71# => X"FFFFFFFF",
  16#1d72# => X"07FFFFFF",
  16#1d73# => X"00000000",
  16#1d74# => X"00000000",
  16#1d75# => X"77000000",
  16#1d76# => X"77777777",
  16#1d77# => X"FFF00770",
  16#1d78# => X"FF0FFFFF",
  16#1d79# => X"F000FFFF",
  16#1d7a# => X"FFF000FF",
  16#1d7b# => X"000FF000",
  16#1d7c# => X"77000FFF",
  16#1d7d# => X"00777000",
  16#1d7e# => X"77777770",
  16#1d7f# => X"00000007",
  16#1d80# => X"00000000",
  16#1d81# => X"77000000",
  16#1d82# => X"FFFFFF87",
  16#1d83# => X"FFFFFFFF",
  16#1d84# => X"FFFFFFFF",
  16#1d85# => X"FFFFFFFF",
  16#1d86# => X"FFFFFFFF",
  16#1d87# => X"00FFFFFF",
  16#1d88# => X"00000000",
  16#1d89# => X"00000000",
  16#1d8a# => X"FF800000",
  16#1d8b# => X"FFFFFFFF",
  16#1d8c# => X"FFFFFFFF",
  16#1d8d# => X"FFFFFFFF",
  16#1d8e# => X"FFFFFFFF",
  16#1d8f# => X"FFFFFFFF",
  16#1d90# => X"7708FFFF",
  16#1d91# => X"77777777",
  16#1d92# => X"88887777",
  16#1d93# => X"88888888",
  16#1d94# => X"88888888",
  16#1d95# => X"FF888888",
  16#1d96# => X"FFFFFFFF",
  16#1d97# => X"FFFFFFFF",
  16#1d98# => X"FFFFFFFF",
  16#1d99# => X"FFFFFFFF",
  16#1d9a# => X"00FFFFFF",
  16#1d9b# => X"00000000",
  16#1d9c# => X"00000000",
  16#1d9d# => X"77000000",
  16#1d9e# => X"77777777",
  16#1d9f# => X"FFF00870",
  16#1da0# => X"FF0FFFFF",
  16#1da1# => X"F0000FFF",
  16#1da2# => X"FFF000FF",
  16#1da3# => X"000FF000",
  16#1da4# => X"77000FFF",
  16#1da5# => X"00070000",
  16#1da6# => X"77777770",
  16#1da7# => X"00000000",
  16#1da8# => X"00000000",
  16#1da9# => X"77700000",
  16#1daa# => X"FFFFFF87",
  16#1dab# => X"FFFFFFFF",
  16#1dac# => X"FFFFFFFF",
  16#1dad# => X"FFFFFFFF",
  16#1dae# => X"FFFFFFFF",
  16#1daf# => X"00FFFFFF",
  16#1db0# => X"00777770",
  16#1db1# => X"00000000",
  16#1db2# => X"FFF70000",
  16#1db3# => X"FFFFFFFF",
  16#1db4# => X"FFFFFFFF",
  16#1db5# => X"FFFFFFFF",
  16#1db6# => X"FFFFFFFF",
  16#1db7# => X"FFFFFFFF",
  16#1db8# => X"7708FFFF",
  16#1db9# => X"77777777",
  16#1dba# => X"88887777",
  16#1dbb# => X"88888888",
  16#1dbc# => X"88888888",
  16#1dbd# => X"FF888888",
  16#1dbe# => X"FFFFFFFF",
  16#1dbf# => X"FFFFFFFF",
  16#1dc0# => X"FFFFFFFF",
  16#1dc1# => X"FFFFFFFF",
  16#1dc2# => X"008FFFFF",
  16#1dc3# => X"00000000",
  16#1dc4# => X"00000000",
  16#1dc5# => X"77700000",
  16#1dc6# => X"77777777",
  16#1dc7# => X"FFF00870",
  16#1dc8# => X"FF0FFFFF",
  16#1dc9# => X"FF0000FF",
  16#1dca# => X"FFF000FF",
  16#1dcb# => X"000FF000",
  16#1dcc# => X"77000FFF",
  16#1dcd# => X"00000007",
  16#1dce# => X"07777770",
  16#1dcf# => X"00000000",
  16#1dd0# => X"00000000",
  16#1dd1# => X"77770000",
  16#1dd2# => X"FFFFFFF7",
  16#1dd3# => X"FFFFFFFF",
  16#1dd4# => X"FFFFFFFF",
  16#1dd5# => X"FFFFFFFF",
  16#1dd6# => X"FFFFFFFF",
  16#1dd7# => X"77FFFFFF",
  16#1dd8# => X"77000000",
  16#1dd9# => X"00000007",
  16#1dda# => X"FFF70000",
  16#1ddb# => X"FFFFFFFF",
  16#1ddc# => X"FFFFFFFF",
  16#1ddd# => X"FFFFFFFF",
  16#1dde# => X"FFFFFFFF",
  16#1ddf# => X"FFFFFFFF",
  16#1de0# => X"7707FFFF",
  16#1de1# => X"77777777",
  16#1de2# => X"88888777",
  16#1de3# => X"88888888",
  16#1de4# => X"88888888",
  16#1de5# => X"FFF88888",
  16#1de6# => X"FFFFFFFF",
  16#1de7# => X"FFFFFFFF",
  16#1de8# => X"FFFFFFFF",
  16#1de9# => X"FFFFFFFF",
  16#1dea# => X"008FFFFF",
  16#1deb# => X"00000000",
  16#1dec# => X"00000000",
  16#1ded# => X"77770000",
  16#1dee# => X"77777777",
  16#1def# => X"FF00F807",
  16#1df0# => X"FFF0FF00",
  16#1df1# => X"FFF0000F",
  16#1df2# => X"FFF000FF",
  16#1df3# => X"000FF000",
  16#1df4# => X"77000FFF",
  16#1df5# => X"00700077",
  16#1df6# => X"07777770",
  16#1df7# => X"00000000",
  16#1df8# => X"00000000",
  16#1df9# => X"77777000",
  16#1dfa# => X"FFFFFFF8",
  16#1dfb# => X"FFFFFFFF",
  16#1dfc# => X"FFFFFFFF",
  16#1dfd# => X"FFFFFFFF",
  16#1dfe# => X"FFFFFFFF",
  16#1dff# => X"FFFFFFFF",
  16#1e00# => X"00778888",
  16#1e01# => X"00000000",
  16#1e02# => X"FFF80000",
  16#1e03# => X"FFFFFFFF",
  16#1e04# => X"FFFFFFFF",
  16#1e05# => X"FFFFFFFF",
  16#1e06# => X"FFFFFFFF",
  16#1e07# => X"FFFFFFFF",
  16#1e08# => X"7777FFFF",
  16#1e09# => X"77777777",
  16#1e0a# => X"88888877",
  16#1e0b# => X"88888888",
  16#1e0c# => X"88888888",
  16#1e0d# => X"FFF88888",
  16#1e0e# => X"FFFFFFFF",
  16#1e0f# => X"FFFFFFFF",
  16#1e10# => X"FFFFFFFF",
  16#1e11# => X"FFFFFFFF",
  16#1e12# => X"007FFFFF",
  16#1e13# => X"00000000",
  16#1e14# => X"00000000",
  16#1e15# => X"77770000",
  16#1e16# => X"77777777",
  16#1e17# => X"000FFF07",
  16#1e18# => X"FFF0FFF0",
  16#1e19# => X"8888000F",
  16#1e1a# => X"88800088",
  16#1e1b# => X"000FF000",
  16#1e1c# => X"77000FFF",
  16#1e1d# => X"00777777",
  16#1e1e# => X"00777770",
  16#1e1f# => X"00000000",
  16#1e20# => X"00000000",
  16#1e21# => X"77777700",
  16#1e22# => X"FFFFFFF8",
  16#1e23# => X"FFFFFFFF",
  16#1e24# => X"FFFFFFFF",
  16#1e25# => X"FFFFFFFF",
  16#1e26# => X"FFFFFFFF",
  16#1e27# => X"FFFFFFFF",
  16#1e28# => X"88FFFFFF",
  16#1e29# => X"00000778",
  16#1e2a# => X"FFFF7000",
  16#1e2b# => X"FFFFFFFF",
  16#1e2c# => X"FFFFFFFF",
  16#1e2d# => X"FFFFFFFF",
  16#1e2e# => X"FFFFFFFF",
  16#1e2f# => X"FFFFFFFF",
  16#1e30# => X"77708FFF",
  16#1e31# => X"77777777",
  16#1e32# => X"88888877",
  16#1e33# => X"88888888",
  16#1e34# => X"88888888",
  16#1e35# => X"FFF88888",
  16#1e36# => X"FFFFFFFF",
  16#1e37# => X"FFFFFFFF",
  16#1e38# => X"FFFFFFFF",
  16#1e39# => X"FFFFFFFF",
  16#1e3a# => X"000FFFFF",
  16#1e3b# => X"00000000",
  16#1e3c# => X"00000000",
  16#1e3d# => X"77777000",
  16#1e3e# => X"77777777",
  16#1e3f# => X"FFFFF087",
  16#1e40# => X"77880FFF",
  16#1e41# => X"77777000",
  16#1e42# => X"07000777",
  16#1e43# => X"00777700",
  16#1e44# => X"77800080",
  16#1e45# => X"00077000",
  16#1e46# => X"00077777",
  16#1e47# => X"00000000",
  16#1e48# => X"00000000",
  16#1e49# => X"77777770",
  16#1e4a# => X"FFFFFFFF",
  16#1e4b# => X"FFFFFFFF",
  16#1e4c# => X"FFFFFFFF",
  16#1e4d# => X"FFFFFFFF",
  16#1e4e# => X"FFFFFFFF",
  16#1e4f# => X"FFFFFFFF",
  16#1e50# => X"FFFFFFFF",
  16#1e51# => X"07788FFF",
  16#1e52# => X"FFFF8000",
  16#1e53# => X"FFFFFFFF",
  16#1e54# => X"FFFFFFFF",
  16#1e55# => X"FFFFFFFF",
  16#1e56# => X"FFFFFFFF",
  16#1e57# => X"FFFFFFFF",
  16#1e58# => X"77708FFF",
  16#1e59# => X"77777777",
  16#1e5a# => X"88888887",
  16#1e5b# => X"88888888",
  16#1e5c# => X"88888888",
  16#1e5d# => X"FFF88888",
  16#1e5e# => X"FFFFFFFF",
  16#1e5f# => X"FFFFFFFF",
  16#1e60# => X"FFFFFFFF",
  16#1e61# => X"FFFFFFFF",
  16#1e62# => X"0008FFFF",
  16#1e63# => X"00000000",
  16#1e64# => X"00000000",
  16#1e65# => X"77777000",
  16#1e66# => X"77777777",
  16#1e67# => X"88F00F87",
  16#1e68# => X"07777007",
  16#1e69# => X"00000000",
  16#1e6a# => X"00000777",
  16#1e6b# => X"00777700",
  16#1e6c# => X"FFF00000",
  16#1e6d# => X"00000008",
  16#1e6e# => X"00077777",
  16#1e6f# => X"00000000",
  16#1e70# => X"00000000",
  16#1e71# => X"87777777",
  16#1e72# => X"FFFFFFFF",
  16#1e73# => X"FFFFFFFF",
  16#1e74# => X"FFFFFFFF",
  16#1e75# => X"FFFFFFFF",
  16#1e76# => X"FFFFFFFF",
  16#1e77# => X"FFFFFFFF",
  16#1e78# => X"FFFFFFFF",
  16#1e79# => X"8FFFFFFF",
  16#1e7a# => X"FFFF8078",
  16#1e7b# => X"FFFFFFFF",
  16#1e7c# => X"FFFFFFFF",
  16#1e7d# => X"FFFFFFFF",
  16#1e7e# => X"FFFFFFFF",
  16#1e7f# => X"FFFFFFFF",
  16#1e80# => X"77777FFF",
  16#1e81# => X"77777777",
  16#1e82# => X"88888888",
  16#1e83# => X"88888888",
  16#1e84# => X"88888888",
  16#1e85# => X"FFFF8888",
  16#1e86# => X"FFFFFFFF",
  16#1e87# => X"FFFFFFFF",
  16#1e88# => X"FFFFFFFF",
  16#1e89# => X"FFFFFFFF",
  16#1e8a# => X"0008FFFF",
  16#1e8b# => X"00000000",
  16#1e8c# => X"00000000",
  16#1e8d# => X"77777700",
  16#1e8e# => X"88877777",
  16#1e8f# => X"000788FF",
  16#1e90# => X"07777770",
  16#1e91# => X"00000000",
  16#1e92# => X"00077777",
  16#1e93# => X"77777777",
  16#1e94# => X"FFFFF000",
  16#1e95# => X"770000FF",
  16#1e96# => X"00077000",
  16#1e97# => X"00000000",
  16#1e98# => X"70000000",
  16#1e99# => X"87777777",
  16#1e9a# => X"FFFFFFFF",
  16#1e9b# => X"FFFFFFFF",
  16#1e9c# => X"FFFFFFFF",
  16#1e9d# => X"FFFFFFFF",
  16#1e9e# => X"FFFFFFFF",
  16#1e9f# => X"FF8FFFFF",
  16#1ea0# => X"FFFFFFFF",
  16#1ea1# => X"FFFFFFFF",
  16#1ea2# => X"FFFFFFFF",
  16#1ea3# => X"FFFFFFFF",
  16#1ea4# => X"FFFFFFFF",
  16#1ea5# => X"FFFFFFFF",
  16#1ea6# => X"FFFFFFFF",
  16#1ea7# => X"FFFFFFFF",
  16#1ea8# => X"777778FF",
  16#1ea9# => X"87777777",
  16#1eaa# => X"88888888",
  16#1eab# => X"88888888",
  16#1eac# => X"88888888",
  16#1ead# => X"FFFF8888",
  16#1eae# => X"FFFFFFFF",
  16#1eaf# => X"FFFFFFFF",
  16#1eb0# => X"FFFFFFFF",
  16#1eb1# => X"FFFFFFFF",
  16#1eb2# => X"0007FFFF",
  16#1eb3# => X"00000000",
  16#1eb4# => X"00000000",
  16#1eb5# => X"77777770",
  16#1eb6# => X"FFFF8877",
  16#1eb7# => X"7777778F",
  16#1eb8# => X"77777777",
  16#1eb9# => X"77777777",
  16#1eba# => X"77777777",
  16#1ebb# => X"77777777",
  16#1ebc# => X"FFFFFF87",
  16#1ebd# => X"FFFFFFFF",
  --Memory Dump Created by VOHDCA
      16#1f40# => "01010101010101010101010101010101",
      16#1f41# => "01010101010101010101010101010101",
      16#1f42# => "01010101010101010101010101010101",
      16#1f43# => "01010101010101010101010101010101",
      16#1f44# => "01010101010101010101010101010101",
      16#1f45# => "01010101010101010101010101010101",
      16#1f46# => "01010101010101010101010101010101",
      16#1f47# => "01010101010101010101010101010101",
      16#1f48# => "01010101010101010101010101010101",
      16#1f49# => "01010101010101010101010101010101",
      16#1f4a# => "01100110011001100110011001100110",
      16#1f4b# => "01100110011001100110011001100110",
      16#1f4c# => "01110111011101110111011101110111",
      16#1f4d# => "01110111011101110111011101110111",
      16#1f4e# => "01110111011101110111011101110111",
      16#1f4f# => "01110111011101110111011101110111",
      16#1f50# => "01110111011101110111011101110111",
      16#1f51# => "01110111011101110111011101110111",
      16#1f52# => "01110111011101110111011101110111",
      16#1f53# => "01110111011101110111011101110111",
      16#1f54# => "01100110011001100110011001100110",
      16#1f55# => "01100110011001100110011001100110",
      16#1f56# => "01010101010101010101010101010101",
      16#1f57# => "01010101010101010101010101010101",
      16#1f58# => "01010101010101010101010101010101",
      16#1f59# => "01010101010101010101010101010101",
      16#1f5a# => "01010101010101010101010101010101",
      16#1f5b# => "01010101010101010101010101010101",
      16#1f5c# => "01010101010101010101010101010101",
      16#1f5d# => "01010101010101010101010101010101",
      16#1f5e# => "01010101010101010101010101010101",
      16#1f5f# => "01010101010101010101010101010101",
      16#1f60# => "01010101010101010101010101010101",
      16#1f61# => "01010101010101010101010101010101",
      16#1f62# => "01010101010101010101010101010101",
      16#1f63# => "01010101010101010101010101010101",
      16#1f64# => "01010101010101010101010101010101",
      16#1f65# => "01010101010101010101010101010101",
      16#1f66# => "01010101010101010101010101010101",
      16#1f67# => "01010101010101010101010101010101",
      16#1f68# => "01010101010101010101010101010101",
      16#1f69# => "01010101010101010101010101010101",
      16#1f6a# => "00000000000000000000000000000000",
      16#1f6b# => "00000000000000000000000000000000",
      16#1f6c# => "01110111011101110111011101110111",
      16#1f6d# => "01110111011101110111011101110111",
      16#1f6e# => "01110111011101110111011101110111",
      16#1f6f# => "01110111011101110111011101110111",
      16#1f70# => "01110111011101110111011101110111",
      16#1f71# => "01110111011101110111011101110111",
      16#1f72# => "01110111011101110111011101110111",
      16#1f73# => "01110111011101110111011101110111",
      16#1f74# => "00000000000000000000000000000000",
      16#1f75# => "00000000000000000000000000000000",
      16#1f76# => "01010101010101010101010101010101",
      16#1f77# => "01010101010101010101010101010101",
      16#1f78# => "01010101010101010101010101010101",
      16#1f79# => "01010101010101010101010101010101",
      16#1f7a# => "01010101010101010101010101010101",
      16#1f7b# => "01010101010101010101010101010101",
      16#1f7c# => "01010101010101010101010101010101",
      16#1f7d# => "01010101010101010101010101010101",
      16#1f7e# => "01010101010101010101010101010101",
      16#1f7f# => "01010101010101010101010101010101",
      16#1f80# => "00000000000000000000000000010001",
      16#1f81# => "00000000000000000000000000001000",
      16#1f82# => "00000000000000000000000000101000",
      16#1f83# => "00000000000000000000000000010010",
      16#1f84# => "00000000000000000000000000010000",
      16#1f85# => "00000000000000000000000010100000",
      16#1f86# => "00000000000000000000000000010011",
      16#1f87# => "00000000000000000000000001100100",
      16#1f88# => "00000000000000000000000000010100",
      16#1f89# => "00000000000000000000000000010000",
      16#1f8a# => "00000000000000000000000001000110",
      16#1f8b# => "00000000000000000000000000010101",
      16#1f8c# => "00000000000000000000000000001000",
      16#1f8d# => "00000000000000000000000001111000",
      16#1f8e# => "00000000000000000000000000000001",
      16#1f8f# => "00000000000000000000000000000011",
      16#1f90# => "00000000000000000000000000000001",
      16#1f91# => "00000000000000000000000000000010",
      16#1f92# => "00000000000000000000000000000010",
      16#1f93# => "00000000000000000000000000000000",
      16#1f94# => "00000000000000000000000000001001",
      16#1f95# => "00000000000000000000000000000001",
      16#1f96# => "01110111011101110111011101110111",
      16#1f97# => "00000000000100110001001011010000",
      16#1f98# => "00000000000000000000000000000000",
      16#1f99# => "00000000000000000000000000000101",
      16#1f9a# => "00000000000000000000000000000000",
      16#1f9b# => "00000000000000000000000000000001",
      16#1f9c# => "00000000000000000000000000000000",
      16#1f9d# => "00000000000000000000000000000000",
      16#1f9e# => "00000000000000000000000000000101",
      16#1f9f# => "00000000000000000000000000000111",
      16#1fa0# => "00000000000000000000000000000000",
      16#1fa1# => "00000000000000000000000000000111",
      16#1fa2# => "00000000000000000000000000000111",
      16#1fa3# => "00000000000000000000000000001000",
      16#1fa4# => "00000000000000000000000000001010",
      16#1fa5# => "00000000000000000000000000000000",
	
	others => "11111111111111111111111111111111");
	attribute ram_style: string;
	attribute ram_style of ram: signal is "block";
	signal read_addra : std_logic_vector(width-1 downto 0);
	signal read_addrb : std_logic_vector(width-1  downto 0);
	signal addra : std_logic_vector(width-1 downto 0);
	signal addrb : std_logic_vector(width-1  downto 0);
	signal br_di, br_do: std_logic_vector (31 downto 0);
	signal br_en, br_we: std_logic;
	--signal delay: std_logic;
begin

  -- Glue logic...
	adressed <= '1'
	when vga_base_adr(31 downto width) = adr (31 downto width)
	else '0';

	addra <= adr(width-1 downto 0);
	addrb <= vga_adr(width-1 downto 0);
	br_di <= data when ascii_wr_in = '0' else ascii_data_in;
	data <= br_do when rd = '1' else "ZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZ";
	br_en <= rd or wr;
	br_we <= wr;
	
	process (clk)
	begin
		if (clk'event and clk = '1') then
			ready <= '0';
			if (br_en = '1') then
				if (br_we = '1') then
					RAM(conv_integer(addra)) <= br_di;
				end if;
				read_addra <= addra;
				ready <= '1';
			end if;
		end if;
	end process;

	process (clk_50m)
	begin
		if (clk_50m'event and clk_50m = '1') then
			read_addrb <= addrb;
		end if;
	end process;
	br_do <= RAM(conv_integer(read_addra));
	vga_out <= RAM(conv_integer(read_addrb));
end syn;
